# coding: utf8

from general_helpers import iso_to_gregorian
from general_helpers import NRtoDay


def schedule_get():
    '''
        Returns the schedule as XML or JSON depending on the extension used
        Variables required:
        - user: OpenStudio API user
        - key: Key for OpenStudio API user
        - year: Choose year to return schedule for
        - week: Chose week to return schedule for
    '''
    def get_day(weekday):
        '''
            Helper function that returns a dict containing a title for the 
            weekday, a date for the class and 
            a SQLFORM.grid for a selected day which is within 
            1 - 7 (ISO standard).
        '''
        startthisyear = datetime.date(year, 01, 01)
        date = iso_to_gregorian(int(year), int(week), int(weekday))
        holiday = False
        
        holiday_query = (db.school_holidays.Classes == True) & \
                        (db.school_holidays.Startdate <= date) & \
                        (db.school_holidays.Enddate >= date)
        holidays = db(holiday_query).count()
        if holidays > 0:
            classes = []
            holiday = True
        else:
            # get the classes
            where = '''classes.Week_day = '{weekday}' AND 
                       classes.Startdate <= '{date}' AND
                       (classes.Enddate >= '{enddate}' OR 
                        classes.Enddate IS NULL)'''.format(weekday=weekday,
                                                           date=date,
                                                           enddate=date)
            '''
                Rows are selected using .as_list(). Using a normal select the
                LocationID field would show up like this using XML:
                <LocationID><Name>Value</Name><id>NR</id></LocationID>
                Using the way below, JSON and XML are the same.
            '''
            fields = [db.classes.id,
                      db.classes.school_locations_id,
                      db.classes.Starttime,
                      db.classes.Endtime,
                      db.classes.school_classtypes_id,
                      db.classes.teachers_id,
                      db.classes.teachers_id2,
                      db.classes.school_levels_id,
                      db.classes_subteachers.teachers_id,
                      db.classes_subteachers.teachers_id2,
                      db.classes_cancelled.ClassDate]
                      
            query = """SELECT classes.id, 
                              classes.school_locations_id,
                              classes.Starttime,
                              classes.Endtime,
                              classes.school_classtypes_id,
                              classes.teachers_id,
                              classes.teachers_id2,
                              classes.school_levels_id,
                              st.teachers_id,
                              st.teachers_id2,
                              cc.ClassDate
                        FROM classes 
                        LEFT JOIN school_locations 
                            ON school_locations.id = classes.school_locations_id
                        LEFT JOIN 
                            (SELECT classes_id, 
                                    ClassDate,
                                    teachers_id,
                                    teachers_id2
                             FROM classes_subteachers
                             WHERE ClassDate = '{st_date}') st
                            ON st.classes_id = classes.id
                        LEFT JOIN 
                            (SELECT classes_id, ClassDate 
                             FROM classes_cancelled
                             WHERE ClassDate = '{c_date}') cc
                            ON cc.classes_id = classes.id
                        WHERE {where} 
                            AND classes.AllowAPI = 'T' 
                            AND school_locations.AllowAPI = 'T'""".format(\
                                                                st_date=date,
                                                                c_date=date,
                                                                where=where)
            
            classes = []
            rows = db.executesql(query, fields=fields).as_list()
            
            for row in rows:
                # general 
                locationID = row['classes']['school_locations_id']
                classtypeID = row['classes']['school_classtypes_id']
                teachers_id = row['classes']['teachers_id']
                teachers_id2 = row['classes']['teachers_id2']
                starttime = row['classes']['Starttime']
                endtime = row['classes']['Endtime']
                levelID = row['classes']['school_levels_id']
                
                # substitute teachers
                subteachers_id = row['classes_subteachers']['teachers_id']
                subteachers_id2 = row['classes_subteachers']['teachers_id2']
                
                subteacher = False
                if subteachers_id:
                    teachers_id = subteachers_id
                    subteacher = True
                if subteachers_id2:
                    teachers_id2 = subteachers_id2
                    subteacher = True
                
                # cancelled classes
                cancelled = False
                if date == row['classes_cancelled']['ClassDate']:
                    cancelled = True
                    
                # check for TeacherID
                if 'TeacherID' in request.vars:
                    TeacherID = int(request.vars['TeacherID'])
                    teacher_ids = [ teachers_id, teachers_id2 ]
                    if teachers_id not in teacher_ids:
                        continue # don't include this class
                        
                # check for ClassTypeID
                if 'ClassTypeID' in request.vars:
                    ClassTypeID = int(request.vars['ClassTypeID'])
                    if ClassTypeID != classtypeID:
                        continue # don't include this class
                        
                # check for LocationID
                if 'LocationID' in request.vars:
                    LocationID = int(request.vars['LocationID'])
                    if LocationID != locationID:
                        continue # don't include this class
                        
                # check for LevelID
                if 'LevelID' in request.vars:
                    LevelID = int(request.vars['LevelID'])
                    if LevelID != levelID:
                        continue # don't include this class
                        
                class_data = dict()
                class_data['LocationID'] = locationID
                class_data['Location'] = locations_dict.get(locationID, '')
                class_data['Starttime'] = starttime
                class_data['Endtime'] = endtime
                class_data['ClassTypeID'] = classtypeID
                class_data['ClassType'] = classtypes_dict.get(classtypeID, '')
                class_data['TeacherID'] = teachers_id
                class_data['TeacherID2'] = teachers_id2
                class_data['Teacher'] = teachers_dict.get(teachers_id, '')
                class_data['Teacher2'] = teachers_dict.get(teachers_id2, '')
                class_data['LevelID'] = levelID
                class_data['Level'] = levels_dict.get(levelID, '')
                class_data['Subteacher'] = subteacher
                class_data['Cancelled'] = cancelled
            
                classes.append(class_data)
        
        return dict(date=date, classes=classes, holiday=holiday)
    
    if request.extension == 'xml':
        response.view = 'generic.xml'
    elif request.extension == 'json':
        response.view == 'generic.json'
    else:
        return T("Extension error: please call with a .json or .xml extension.\
            Example: /api/schedule_get.json?user=user&key=key&year=2014&week=1")
    
    if 'user' in request.vars and 'key' in request.vars and \
        'year' in request.vars and 'week' in request.vars:
        try:
            user = request.vars['user']
            key = request.vars['key']
            year = int(request.vars['year'])
            week = int(request.vars['week'])
            
            # classes
            data = "Authentication error: please check user and key."
            # check authentication
            if user is None or user == "" or key is None or key == "":
                # no credentials specified
                pass
            else:
                query = (db.sys_api_users.Username==user)
                row = db(query).select(db.sys_api_users.ALL).first()
                if row is None: 
                    # Non existin user specified
                    pass 
                else:
                    if not row.ActiveUser or not row.APIKey == key: 
                        # Inactive user or wrong key
                        pass
                    else:
                        data = dict()
                        data['classes'] = dict()
                        for day in range(1, 8):
                            key = unicode(NRtoDay(day))
                            data['classes'][key] = get_day(day)
                            
                        # ClassTypes
                        rows = db().select(db.school_classtypes.ALL).as_list()
                        data['classtypes'] = rows
                        
                        # Teachers
                        teachers = []
                        rows = db().select(db.teachers.id, 
                                           db.teachers.FirstName,
                                           db.teachers.LastName)
                        for row in rows:
                            teachers.append(dict(id=row.id,
                                                  name=row.FirstName + ' ' + \
                                                  row.LastName))
                            
                        data['teachers'] = teachers
                        
                        # Locations
                        query = (db.school_locations.AllowAPI == True)
                        rows = db(query).select(db.school_locations.id, 
                                            db.school_locations.Name).as_list()
                        data['locations'] = rows
                        
                        # Practice levels
                        rows = db().select(db.school_levels.id,
                                           db.school_levels.Name).as_list()
                        data['levels'] = rows
                        
        except ValueError:
            data = T("Value error")
    else:
        data = T("Missing value: user, key, year and week are required values, \
                  one or more was missing in your request.")
        
    return dict(data=data)
