# coding: utf8
# local modules import
from general_helpers import datestr_to_python
from plugin_solidform import SOLIDFORM
from gluon.contrib.user_agent_parser import mobilize
# python general modules import
import openpyxl
import os

from general_helpers import get_button
from general_helpers import NRtoDay
from general_helpers import iso_to_gregorian
from general_helpers import get_classname
from general_helpers import get_button
from general_helpers import get_submenu
from general_helpers import create_teachers_dict
from general_helpers import create_locations_dict
from general_helpers import create_classtypes_dict
from general_helpers import max_string_length

# helper functions
    
@auth.requires_login()
def classes_get_menu(page, clsID, date_formatted):
    '''
        This function returns a menu for the manage page
        page is expected to be a function name showing a page
        class_id is expected to be the class ID
        class_date is expected to be the class date, formatted using session.date_format
    '''
    pages = []
    if auth.has_membership(group_id='Admins') or \
       auth.has_permission('read', 'schedule_manage'):
        pages.append([ 'manage', 
                       T('General'), 
                       URL('manage', args=[ clsID, date_formatted ]) ])
    if auth.has_membership(group_id='Admins') or \
       auth.has_permission('update', 'classes_attendance'):
        pages.append([ 'attendance', 
                       T('Attendance'), 
                       URL('attendance', args=[ clsID, date_formatted ]) ])
    if auth.has_membership(group_id='Admins') or \
       auth.has_permission('create', 'classes_attendance_override'):
        pages.append([ 'attendance_override', 
                       T("Attendance count"), 
                       URL('attendance_override',args=[ clsID, date_formatted ]) 
                     ]) 
    
    return get_submenu(pages, page)
    
    
def _check_attendance_remove(row):
    clsID = session.clsID
    date = session.attendance_classdate
    check = db.classes_attendance(customers_id=row.id, 
                                  classes_id=clsID, 
                                  ClassDate=date)
    if check == None:
        return A(T('Add'),_href=URL("attendance",vars=dict(customers_id=row.id, clsID=clsID, date=date))) # Add to attendance
        
def _check_attendance_dropin(row):
    clsID = session.clsID
    date = session.attendance_classdate
    return A(T('Drop In'),_href=URL("attendance",vars=dict(customers_id=row.id, clsID=clsID, date=date, attType=2, remove=False)))
    
def manage_registration_check_paused(row, value=None):
    if 'customers' in row:
        customers_id = row.customers.id
    else:
        customers_id = row.id
    if type(session.register_classdate) != datetime.date:
        date = datestr_to_python(session.date_format, session.register_classdate)
    else:
        date = session.register_classdate
    
    return_value = ''
    query = (db.customers_subscriptions.customers_id == customers_id) & \
            (db.customers_subscriptions.Startdate <= date) & \
            ((db.customers_subscriptions.Enddate >= date) | \
             (db.customers_subscriptions.Enddate == None))
    current_membership = \
        db(query).select(db.customers_subscriptions.ALL).first()
    if current_membership:
        query = (db.customers_subscriptions_paused.customers_subscriptions_id == current_membership.id) & \
                (db.customers_subscriptions_paused.Startdate <= date) & \
                ((db.customers_subscriptions_paused.Enddate >= date) | \
                 (db.customers_subscriptions_paused.Enddate == None))
        check = db(query).select(db.customers_subscriptions_paused.ALL).first()
        if check:
            return_value = T("Yes")
    return return_value
    
def _check_registration(row):
    clsID = session.clsID
    check = db.classes_registration(customers_id=row.id, classes_id=clsID)
    if check is None:
        return get_button('add', URL("register",vars={'cuID':row.id, 
                                                      'clsID':clsID}))
    else:
        return ""

def _check_waitinglist(row):
    clsID = session.clsID
    check = db.classes_waitinglist(customers_id=row.id, classes_id=clsID)
    if check == None:
        return A(SPAN(_class="buttontext button", _title=T("Add")), SPAN(_class="icon plus icon-plus"), " " + T("Add"), _class="button btn", \
            _href=URL("waitinglist",vars=dict(customers_id=row.id, clsID=clsID)))
    else:
        return ""
            
def attendance_expected_get_list(clsID, date, mobile=False):
    """
        return a tuple with 5 fields for every customer,
        firstname, lastname, dateofbirth, attendancetype and email.
    """
    def get_customer_details(customers_id, atttype=''):
        # helper function which adds a row to the model containing customer information
        query = (db.customers.id == customers_id)
        row = db(query).select(db.customers.ALL)
        row = row.render(0)
        return [row, atttype]
        
    # define list
    l = []
    # start with registered customers
    query = (db.classes_registration.classes_id==clsID)
    rows = db(query).select(db.classes_registration.customers_id)
    registered_customers = []
    for row in rows:
        l.append(get_customer_details(row.customers_id))
        registered_customers.append(row.customers_id)
    # add trialclass customers
    query = (db.classes_attendance.classes_id==clsID) & \
            (db.classes_attendance.ClassDate==date) & \
            (db.classes_attendance.AttendanceType==1) 
    rows = db(query).select(db.classes_attendance.customers_id)
    for row in rows:
        l.append(get_customer_details(row.customers_id, T("Trialclass")))
    # add drop-in customers             # 2 = drop-in # 3 = classcard
    query = (db.classes_attendance.classes_id==clsID) & \
            (db.classes_attendance.ClassDate==date) & \
            ((db.classes_attendance.AttendanceType==2) | \
             (db.classes_attendance.AttendanceType==3))
    rows = db(query).select(db.classes_attendance.customers_id)
    for row in rows:
        l.append(get_customer_details(row.customers_id, T("Drop-in")))
    ## add customers, who are attending the class with a membership, but not attending their regular class
    #query = (db.classes_attendance.classes_id==clsID)  & \
            #(db.classes_attendance.ClassDate==date) & \
            #(db.classes_attendance.AttendanceType==None)
    #rows = db(query).select(db.classes_attendance.customers_id)
    #for row in rows:
        ##print row
        #if row.customers_id not in registered_customers:
            #l.append(get_customer_details(row.customers_id))
    return l
    
def schedule_get_export_buttons(year, week):
    locations = A(T("By location"), 
                  _href=URL('schedule_export_excel', 
                            vars=dict(year=year, 
                                      week=week, 
                                      export='locations')), 
                  _style='width:132px', 
                  _class='left')
    teachers = A(T("By teacher"), 
                 _href=URL('schedule_export_excel', 
                           vars=dict(year=year, 
                                     week=week, 
                                     export='teachers')), 
                 _style='width:132px', 
                 _class='left')
        
    links = [ locations, teachers ]
    
    div = DIV( A(T('Export'), 
                 ' ',
                 SPAN(_class='caret'), 
                 _href='#', 
                 _class='btn dropdown-toggle', 
                 **{'_data-toggle':"dropdown"}),
              UL(*links, 
                 _class='dropdown-menu'),
              _class='btn-group',
              _style='padding-left:5px;')
        
    return div
    
    
def _generate_classdetails_searchform(clsID, date, name, function):
    date_formatted = date.strftime(session.date_format)
    form = SOLIDFORM.factory(
        Field('name', default=name, label=T("")),
        submit_button='Search',
        )
    form.element('input[name=name]')['_placeholder']='Search...'
    form.add_button(T('Show all'), URL(function, vars=dict(clsID=clsID, date=date_formatted)))
    return form
    
@auth.requires_login()
def classcard_count_classes(row):
    total = row.Classes
    query = (db.classes_attendance.customers_classcards_id == row.id)
    used = db(query).count()
    return T("Classes taken") + ": " + unicode(used) + "/" + unicode(total)

### helpers end ###
    
@auth.requires_login()
def attendance_count_classes(customers_id, date):
    try:
        date = datestr_to_python(session.date_format, date)
    except:
        pass
    iso_year = date.isocalendar()[0]
    iso_week = date.isocalendar()[1]
    weekstart = iso_to_gregorian(iso_year, iso_week, 1)
    weekend = iso_to_gregorian(iso_year, iso_week, 7)
    query = (db.classes_attendance.customers_id==customers_id) & \
            (db.classes_attendance.ClassDate>=weekstart) & \
            (db.classes_attendance.ClassDate<=weekend)
    return db(query).count()
    
@auth.requires_login()
def class_add():
    """
        This function shows an add page for a class
    """
    response.title = T("Add a new class")
    response.subtitle = T("")
    response.view = 'general/only_content.html'
    
    return_url = URL('schedule')
    
    crud.messages.submit_button = T("Add")
    crud.messages.record_created = T("Added class")
    crud.settings.create_next = return_url
    form = crud.create(db.classes)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires_login()
def class_edit():
    """
        This function shows an edit page for a class
        request.args[0] is expected to be the classes_id
    """
    response.title = T("Edit class")
    classes_id = request.args[0]
    classname = get_classname(classes_id)
    response.subtitle = classname
    response.view = 'general/only_content.html'
    
    return_url = URL('schedule')
    
    crud.messages.submit_button = T("Update")
    crud.messages.record_updated = T("Updated class")
    crud.messages.record_deleted = T('Deleted class') + ': ' + classname
    crud.settings.update_next = return_url
    form = crud.update(db.classes, classes_id)
    
    duplicate_class = A(SPAN(_class="buttontext button", _title=T("Duplicate")), SPAN(_class="icon retweet icon-retweet"), " " + T("Duplicate"), 
                _class="button btn", _href=URL('duplicate_class', args=[classes_id]), _title=T("Duplicate this class"))
    
    back = get_button('back', return_url)
    back = DIV(back, " ", duplicate_class)
    
    return dict(content=form, back=back)
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('update', 'classes'))
def class_delete():
    '''
        Removed the selected class and redirect to the manage page
        request.args[0] is expected to be the classes_id (clsID)
    '''
    clsID = request.args[0]
    
    query = (db.classes.id == clsID)
    result = db(query).delete()
    
    session.flash = T('Failed to delete class')
    if result == 1: # Success
        session.flash = T('Deleted class')
    
    redirect(URL('schedule'))
    
    
@auth.requires_login()
def duplicate_class():
    clsID = request.args[0]
    row = db.classes[clsID]
    
    id = db.classes.insert(school_locations_id=row.school_locations_id, 
                           school_classtypes_id=row.school_classtypes_id, 
                           Week_day=row.Week_day, 
                           Starttime=row.Starttime, 
                           Endtime=row.Endtime, 
                           Startdate=row.Startdate, 
                           Enddate=row.Enddate, 
                           teachers_id=row.teachers_id, 
                           teachers_id2=row.teachers_id2, 
                           Maxstudents=row.Maxstudents, 
                           DropInPrice=row.DropInPrice)
        
    session.flash = T("You are now editing the duplicated class")
    
    redirect(URL('class_edit', args=[id]))
    
@auth.requires_login()
def check_class(form):
    # check if teacher is marked for a class, if he/she isn't display a message.
    cltID = request.vars['school_classtypes_id']
    teID = request.vars['teachers_id']
    teID2 = request.vars['teachers_id2']
    
    te1_check = db.teachers_classtypes(school_classtypes_id=cltID, teachers_id=teID)
    if te1_check == None:
        session.flash = T("This teacher is not marked as a teacher for this type of class")
    te2_check = db.teachers_classtypes(school_classtypes_id=cltID, teachers_id=teID)
    if te2_check == None:
        session.flash = T("This teacher is not marked as a teacher for this type of class")
    
@mobilize
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'classes'))
def schedule():
    def return_lastweek(year):
        lastday = datetime.date(year,12,28) # the 28th of December is always in the last week... the 31st might be in week 1 according to the iso calendar
        return lastday.isocalendar()[1]
        
    def get_day(weekday):
        '''
            Helper function that returns a dict containing a title for the weekday,
            a date for the class and 
            a SQLFORM.grid for a selected day which is within 1 - 7 (ISO standard).
        '''
        title = NRtoDay(weekday)
        date = iso_to_gregorian(int(year), int(week), int(weekday))
        date_formatted = date.strftime(session.date_format)
        onclick_remove = "return confirm('" + \
                         T('Do you really want to delete this class?') + \
                         "');"
                         
        holiday_query = (db.school_holidays.Classes == True) & \
                        (db.school_holidays.Startdate <= date) & \
                        (db.school_holidays.Enddate >= date)
        holidays = db(holiday_query).count()
        if holidays > 0:
            div_classes = T("Holiday")
        else:
            query = (db.classes.Week_day == weekday) & \
                    (db.classes.Startdate <= date) & \
                    ((db.classes.Enddate >= date) | \
                     (db.classes.Enddate == None)) # can't use is here, query doesn't work
            query &= schedule_get_filter_query(query)
            
            table = TABLE(TR(TH('', _class='status_marker'), # status marker
                             TH(T('Location')),
                             TH(T('Class type')),
                             TH(T('Start')),
                             TH(T('End')),
                             TH(T('Teacher')),
                             TH(T('Level')),
                             TH(T('API')),
                             TH(T('Filled')),
                             TH(T('Trend')),
                             _class='os-table_header'),
                          _class='os-schedule')
                             
            #div_header = DIV(DIV(T("Location"), _class='location'),
                             #DIV(T("Class type"), _class='classtype'),
                             #DIV(T("Start"), _class='start'),
                             #DIV(T("End"), _class='end'),
                             #DIV(T("Teacher"), _class='teacher'),
                             #DIV(T("Level"), _class='level'),
                             #DIV(T("API"), _class='api'),
                             #DIV(T("Filled"), _class='filled'),
                             #DIV(T("Trend"), _class='trend'),
                             #_class='os-schedule_header')
                             
            #div_classes = DIV(div_header, _class='os-schedule_classes')
            
            status = session.schedule_filter_status
            
            if session.classes_schedule_sort == 'location':
                orderby = db.school_locations.Name|db.classes.Starttime
            elif session.classes_schedule_sort == 'starttime':
                orderby = db.classes.Starttime | db.school_locations.Name
            
            if not status is None and not status == '':
                # Open classes
                if status == 'open_class':
                    query &= (db.classes_open.ClassDate == date)
                    rows = db(query).select(db.classes.ALL,
                                        left=[db.school_locations.on(\
                                                     db.school_locations.id == \
                                                     db.classes.school_locations_id),
                                              db.classes_open.on(db.classes.id == \
                                                         db.classes_open.classes_id)],
                                        orderby=orderby)
                elif status == 'subteacher':
                    query &= (db.classes_subteachers.ClassDate == date)
                    rows = db(query).select(db.classes.ALL,
                                        left=[db.school_locations.on(\
                                                     db.school_locations.id == \
                                                     db.classes.school_locations_id),
                                              db.classes_subteachers.on(\
                                                         db.classes.id == \
                                                         db.classes_subteachers.classes_id)],
                                        orderby=orderby)
                elif status == 'cancelled':
                    query &= (db.classes_cancelled.ClassDate == date)
                    rows = db(query).select(db.classes.ALL,
                            left=[db.school_locations.on(\
                                             db.school_locations.id == \
                                             db.classes.school_locations_id),
                                  db.classes_cancelled.on(db.classes.id == \
                                             db.classes_cancelled.classes_id)],
                                    orderby=orderby)
            else:
                # Default query
                rows = db(query).select(db.classes.ALL,
                                        left=[db.school_locations.on(\
                                                    db.school_locations.id == \
                                                    db.classes.school_locations_id)],
                                        orderby=orderby)
                                            
            if len(rows) == 0:
                div_classes=DIV()
            else:
                # Generate list of classes
                for row in rows.render():
                    
                    clsID = row.id
                    result = status_get(clsID, date)
                    status = result['status']
                    status_marker = result['status_marker']
                    
                    location = max_string_length(row.school_locations_id, 15)
                    classtype = max_string_length(row.school_classtypes_id, 24)
                    start = row.Starttime
                    end = row.Endtime
                    level = max_string_length(row.school_levels_id, 12)
                    teacher = max_string_length(row.teachers_id, 20)
                    teacher2 = row.teachers_id2
                    
                    api = INPUT(value=row.AllowAPI,
                                _type='checkbox', 
                                _value='api',
                                _disabled='disabled')
                    
                    if status == 'subteacher':
                        record = db.classes_subteachers(classes_id=clsID, ClassDate=date)
                        teacher = teachers_dict.get(record.teachers_id)
                        teacher2 = teachers_dict.get(record.teachers_id2)
                    
                    filled = schedule_get_filled(row)
                        
                    attendance = A(T('Attendance'),
                                   _href=URL('attendance', 
                                             args = [row.id, date_formatted]))
                    edit = A(T('Edit'), 
                             _href=URL('class_edit', args=[row.id]))
                    manage = A(T('Manage'), 
                               _href=URL('manage', 
                                         args=[row.id, date_formatted]))
                    remove = A(T('Remove'), 
                               _href=URL('class_delete', args=[row.id]),
                               _onclick=onclick_remove)
                    tools = TD(_class='os-schedule_tools',
                               _colspan='10')
                    
                    if auth.has_membership(group_id='Admins') or \
                        auth.has_permission('update', 'classes-attendance'):
                        tools.append(attendance)
                        tools.append(' | ')
                    
                    if auth.has_membership(group_id='Admins') or \
                        auth.has_permission('read', 'schedule_manage'):
                        tools.append(manage)
                        tools.append(' | ')
                    if auth.has_membership(group_id='Admins') or \
                        auth.has_permission('update', 'classes'):
                        tools.append(edit)
                        tools.append(' | ')
                        tools.append(SPAN(remove, 
                                          _class='os-customer_tools_remove'))
                        #tools.append(' | ')
                        
                    trend = schedule_get_class_trend(row.id, date)
                        
                    row_class = TR(status_marker,
                                   location,
                                   classtype,
                                   start,
                                   end,
                                   teacher,
                                   level,
                                   api,
                                   filled,
                                   trend,
                                   _class='os-schedule_class')
                    row_tools = TR(tools,
                                   _class='os-schedule_tools',
                                   _id='class_' + unicode(clsID))
                        
                    table.append(row_class)
                    table.append(row_tools)
            
        date = date.strftime(session.date_format)
        
        return dict(title=title, date=date, grid=table)
        
    # used for expected attendance to know where to go back to.
    session.class_details_back = None 
    
    ## sort form begin ##
    if 'sort' in request.vars:
        session.classes_schedule_sort = request.vars['sort']
    elif session.classes_schedule_sort is None or \
         session.classes_schedule_sort == '':
        # check if we have a default value
        sys_property = 'classes_schedule_default_sort'
        row = db.sys_properties(Property=sys_property)
        if row:
            session.classes_schedule_sort = row.PropertyValue
        else:
            # default to sorting by location
            session.classes_schedule_sort = 'location'
            
    form_sort = schedule_get_form_sort()
    
    ## sort form end ##
        
    if 'schedule_year' in request.vars:
        year = int(request.vars['schedule_year'])
    elif not session.schedule_year is None:
        year = session.schedule_year
    else:
        today = datetime.date.today()
        year = datetime.date.today().year
    session.schedule_year = year
    if 'schedule_week' in request.vars:
        week = int(request.vars['schedule_week'])
    elif not session.schedule_week is None:
        week = session.schedule_week
    else:
        week = today.isocalendar()[1]
        if week == 0:
            week = 1
    session.schedule_week = week
    if session.schedule_show_date != None:
        date = datestr_to_python(session.date_format, session.schedule_show_date)
        year = date.year
        week = date.isocalendar()[1]
        session.schedule_show_date = None
        
    # if we used the jump date box to select a week
    if 'schedule_jump_date' in request.vars:
        jump_date = datestr_to_python(session.date_format,
                                      request.vars['schedule_jump_date'])
        year = jump_date.year
        week = jump_date.isocalendar()[1]
        session.schedule_year = year
        session.schedule_week = week
    else:
        jump_date = datetime.date.today()
        
    lastweek = return_lastweek(year)
        
    ## week selection
    form = FORM(INPUT(_name='schedule_year', 
                      requires=IS_NOT_EMPTY(), 
                      _type='number', 
                      _value=year,\
                      _min=datetime.date.today().year-25, 
                      _max=datetime.date.today().year+25,
                      _id='no_table_schedule_year'),
                INPUT(_name='schedule_week', 
                      requires=IS_NOT_EMPTY(), 
                      _type='number', 
                      _value=week,\
                      _min=1, 
                      _max=lastweek,
                      _id='no_table_schedule_week'),
                INPUT(_type='submit', 
                      _value=T('View'),
                      _id='no_table_schedule_submit'),
                _id='schedule_form_week')
    
    # jump to date
    print request.vars
    
    form_jump = SOLIDFORM.factory(
                Field('schedule_jump_date', 'date', 
                      requires=IS_DATE_IN_RANGE( \
                                format=session.date_format,
                                minimum=datetime.date(1900,1,1),
                                maximum=datetime.date(2999,1,1)), 
                      default=jump_date, 
                      label=T("")),
                submit_button=T('Jump to'),
                )
                
    form_jump = DIV(form_jump.custom.begin,
                    form_jump.custom.widget.schedule_jump_date,
                    form_jump.custom.submit,
                    form_jump.custom.end,
                    _id='form_jump')
                
    current_week = A(T('Current week'), _href=URL('schedule_current_week'))
    export = schedule_get_export_buttons(year, week)
                
    form = DIV(H4(T("Which week?")), 
               form, 
               #HR(),
               form_jump,
               current_week)
        
    ## week selection end ##
    
    # filter
    if 'classtype' in request.vars:
        location = request.vars['location']
        teacher = request.vars['teacher']
        classtype = request.vars['classtype']
        level = request.vars['level']
        status = request.vars['status']
        filter_form = _schedule_get_filter_form(request.vars['location'],
                                                request.vars['teacher'],
                                                request.vars['classtype'],
                                                request.vars['level'],
                                                request.vars['status'])
        session.schedule_filter_location = location
        session.schedule_filter_teacher = teacher
        session.schedule_filter_classtype = classtype
        session.schedule_filter_level = level
        session.schedule_filter_status = status
    elif not session.schedule_filter_location is None:
        filter_form = _schedule_get_filter_form(session.schedule_filter_location,
                                                session.schedule_filter_teacher,
                                                session.schedule_filter_classtype,
                                                session.schedule_filter_level,
                                                session.schedule_filter_status)
    else:
        filter_form = _schedule_get_filter_form()
        session.schedule_filter_location = None
        session.schedule_filter_teacher = None
        session.schedule_filter_classtype = None
        session.schedule_filter_level = None
        session.schedule_filter_status = None
    
    title = T('Schedule')
    response.title = T('Schedule')
    response.subtitle = unicode(year) + " " + T("week") + " " + unicode(week)

    db.classes.id.readable = False
    db.classes.teachers_id2.readable = False
    db.classes.Week_day.readable = False
    db.classes.DropInPrice.readable = False
    db.classes.Maxstudents.readable = False
    db.classes.TrialPrice.readable = False
    
    days = dict()
    for day in range(1, 8):
        days[day] = get_day(day)
        
    ## Schedule tools begin ##
    
    manage_classes = UL(_class='os-schedule_tools')
    
    # add new class
    permission = auth.has_membership(group_id='Admins') or \
                 auth.has_permission('create', 'classes')
    
    if permission:
        add_class = A(T("Add a new class"), 
                      _href=URL('class_add'),
                      _title=T("Add a new class"))
        manage_classes.append(add_class)
        
    # holidays
    permission = auth.has_membership(group_id='Admins') or \
                 auth.has_permission('read', 'school_holidays')
    
    if permission:
        holidays = A(T("School holidays"),
                     _href=URL('schedule', 'holidays'),
                     _title=T('List all holidays'))
        manage_classes.append(holidays)
    
    # teacher holidays
    permission = auth.has_membership(group_id='Admins') or \
                 auth.has_permission('read', 'teacher_holidays')
    
    if permission:
        teacher_holidays = A(T("Teacher holidays"),
                             _href=URL('teacher_holidays'),
                             _title=T('Open classes in a range of dates'))
        manage_classes.append(teacher_holidays)
        
    # List open classes
    permission = auth.has_membership(group_id='Admins') or \
                 auth.has_permission('read', 'classes_open')
    
    if permission:
        classes_open = A(T('All open classes'),
                        _href=URL('classes_open'),
                        _title=T('List open classes'))
        manage_classes.append(classes_open)
        
    # Export by teacher
    
    export_locations = A(T("Export by location"), 
                          _href=URL('schedule_export_excel', 
                                    vars=dict(year=year, 
                                              week=week, 
                                              export='locations')))
    export_teachers = A(T("Export by teacher"), 
                         _href=URL('schedule_export_excel', 
                                   vars=dict(year=year, 
                                             week=week, 
                                             export='teachers')))
    manage_classes.append(export_locations)
    manage_classes.append(export_teachers)
                
    ## Schedule tools end ##
    
    if request.user_agent()['is_mobile']:
        left_sidebar_enabled=True
        right_sidebar_enabled=False
    else:
        left_sidebar_enabled=False
        right_sidebar_enabled=True
        
        
        
    mobile_export_title = H4(T("Export"))
    mobile_classes_title = H4(T("Classes"))
    
    return dict(form=form, 
                filter_form=filter_form, 
                year=year, 
                title=title, 
                manage_classes=manage_classes, 
                days=days,
                export=export,
                form_sort=form_sort,
                mobile_export_title=mobile_export_title, 
                mobile_classes_title=mobile_classes_title,
                left_sidebar_enabled=left_sidebar_enabled, 
                right_sidebar_enabled=right_sidebar_enabled)
                
def schedule_get_class_trend(clsID, date):
    '''
        Generates a div that contains the trend for a class.
        Look at past 4 weeks and compare to the classes before it.
        Take cancelled classes into account
        Doesn't get executed during holidays
    '''
    def get_total_last_weeks(clsID, startdate, nr_weeks):
        '''
            Get attendance total of last nr_weeks weeks. 
        '''
        
        i = 0
        total = 0
        classes_counted = 0
        delta = datetime.timedelta(days=7)
        current_date = startdate - delta
        # max 4 weeks
        while i < nr_weeks and current_date >= class_info.Startdate:
            # check if cancelled
            cancelled = db.classes_cancelled(classes_id=clsID,
                                             ClassDate=current_date)
            # check holiday
            holiday = False
            holiday_query = (db.school_holidays.Classes == True) & \
                            (db.school_holidays.Startdate <= date) & \
                            (db.school_holidays.Enddate >= date)
            holidays = db(holiday_query).count()
            if holidays > 0:
                holiday = True
                
            if not cancelled and not holiday:
                # check for attendance override
                att_override = db.classes_attendance_override(
                                                    classes_id=clsID,
                                                    ClassDate=current_date)
                if att_override:
                    total += att_override.Amount
                else:
                    # count regular attendance
                    query = (db.classes_attendance.classes_id == clsID) & \
                            (db.classes_attendance.ClassDate == current_date)
                    total += db(query).count()
                classes_counted += 1
                
            i += 1
            current_date -= delta
        
        #print startdate
        #print total
        #print classes_counted
        
        try:
            average = float(total / classes_counted)
        except ZeroDivisionError:
            average = float(0)
            
        return average
    
    class_info = db.classes(clsID)
    result = get_total_last_weeks(clsID, date, 4)
    
    four_weeks_earlier = date - datetime.timedelta(days=28)
    previous_result = get_total_last_weeks(clsID, four_weeks_earlier, 4)
    
    div = DIV()
    try:
        value = int(round(float(result / previous_result)*100))
        if value > 100:
            value = unicode(value - 100) + '%'
            div = DIV(DIV(_class='os-trend_arrow_up'), 
                      value,
                      ' ',
                      SPAN(_class='icon user icon-user'))
        elif value < 100:
            value = unicode(100 - value) + '%'
            div = DIV(DIV(_class='os-trend_arrow_down'), 
                      value,
                      ' ',
                      SPAN(_class='icon user icon-user'))
    except ZeroDivisionError:
        pass
    
    return div

@auth.requires_login()
def schedule_get_sort_options():
    '''
        returns a list of sorting options for classes
    '''
    sort_options = [ ['location', T('Location')], 
                     ['starttime', T('Start time')] ]
                   
    return sort_options

@auth.requires_login()
def schedule_get_form_sort():
    '''
        Returns sort form for schedule
    '''
    sort_options = schedule_get_sort_options()
    
    sort = session.classes_schedule_sort
    
    form = SQLFORM.factory(
        Field('sort', 
            requires=IS_IN_SET(sort_options, 
                              zero=None), 
            default=sort, 
            label=""),
        formstyle='divs', 
        )
        
    # sumbit form on change
    selects = form.elements('select')
    for select in selects:
        select.attributes['_onchange'] = "this.form.submit();"
        
    # add permission check to allow setting of default sorting for schedule
    settings = ''
    permission = auth.has_membership(group_id='Admins') or \
                 auth.has_permission('update', 'schedule_set_default_sort')
    if permission:
        settings = A(SPAN(_class='icon cog icon-cog'),
                     _href=URL('schedule_set_sort_default'),
                     _title=T('Set default sorting for classes')
                    )
        
    form = TABLE(TR(TD(H4(T('Sort classes by'), _class='os-no_margin_top'),
                       # this is form.custom.begin with an extra class 
                       XML('<form action="#" enctype="multipart/form-data" \
                             method="post" class="os-no_margin_bottom">'),
                       form.custom.widget.sort,
                       form.custom.end),
                    TD(settings)),
                  _class='os-no_margin')
        
    return form
    
@auth.requires(auth.has_membership(group_id='Admins') or \
               auth.has_permission('update', 'schedule_set_default_sort'))
def schedule_set_sort_default():
    '''
        Displays a page to edit the default sorting order for the schedule
    '''
    response.title = T('Set default sort order for classes')
    response.subtitle = ''
    response.view = 'general/only_content.html'
    
    sort_options = schedule_get_sort_options()
    sort = session.classes_schedule_sort
    
    form = SQLFORM.factory(
        Field('sort', 
            requires=IS_IN_SET(sort_options, 
                              zero=None), 
            default=sort, 
            label=""),
        formstyle='divs', 
        submit_button=T('Update'),
        )
        
    title = H4(T('Sort classes by'))
    content = DIV(BR(), title, form)
    
    return_url = URL('schedule')
    
    if form.process().accepted:
        session.flash = T('Updated default sort order')
        sort = request.vars['sort']
        session.classes_schedule_sort = sort
        
        sys_property = 'classes_schedule_default_sort'
        row = db.sys_properties(Property=sys_property)
        if not row:
            db.sys_properties.insert(Property=sys_property, 
                                     PropertyValue=sort)
        else:
            row.PropertyValue=sort
            row.update_record()
        
        redirect(return_url)
    
    back = get_button('back', return_url)
    
    return dict(content=content,
                back=back)
    
@auth.requires_login()
def schedule_get_filled(row):
    total = unicode(row.Maxstudents)
    used = unicode(db(db.classes_registration.classes_id==row.id).count())
    return used + "/" + total
    
@auth.requires_login()
def schedule_current_week():
    session.schedule_week = None
    session.schedule_year = None
    
    redirect(URL('schedule'))
    
@auth.requires_login()
def _schedule_clear_filter():
    session.schedule_filter_location = None
    session.schedule_filter_teacher = None
    session.schedule_filter_classtype = None
    session.schedule_filter_level = None
    session.schedule_filter_status = None
    
    redirect(URL('schedule'))
    
@auth.requires_login()
def _schedule_get_filter_form(school_locations_id='', 
                               teachers_id='', 
                               school_classtypes_id='',
                               school_levels_id='',
                               status=''):
    
    form = SQLFORM.factory(
        Field('location', 
            requires=IS_IN_DB(db,'school_locations.id', '%(Name)s', 
                              zero=T('All locations')), 
            default=school_locations_id, 
            label=""),
        Field('teacher', 
            requires=IS_IN_DB(db,'teachers.id', 
                              '%(FirstName)s' + ' ' + '%(LastName)s', 
                              zero=T('All teachers')), 
            default=teachers_id, 
            label=""),
        Field('classtype', 
            requires=IS_IN_DB(db,'school_classtypes.id', '%(Name)s', 
                              zero=T('All classtypes')), 
            default=school_classtypes_id, 
            label=""),
        Field('level',
            requires=IS_IN_DB(db,'school_levels.id', '%(Name)s', 
                              zero=T('All levels')), 
            default=school_levels_id, 
            label=""),
        Field('status',
            requires=IS_IN_SET(session.class_status,
                               zero=T('All statuses')),
            default=status,
            label=''),
        submit_button=T('Go'),
        formstyle='divs', 
        )
        
    # sumbit form on change
    selects = form.elements('select')
    for select in selects:
        select.attributes['_onchange'] = "this.form.submit();"
        
    clear = A(T("All"), _class="button btn", \
                _href=URL('_schedule_clear_filter'),
                _title=T("Reset filter to default"))
    
    div = DIV(
        form.custom.begin, 
        form.custom.widget.location, 
        form.custom.widget.teacher, 
        form.custom.widget.classtype,
        form.custom.widget.level,
        form.custom.widget.status,
        form.custom.submit,
        clear,
        form.custom.end,
        _id='schedule_filter_form'
        )
        
    return div
    
def schedule_get_filter_query(query):
    '''
        This function takes a query and request.vars as arguments and returns a modified query
        based on the request.vars
    '''
    classtype = session.schedule_filter_classtype
    teacher = session.schedule_filter_teacher
    location = session.schedule_filter_location
    level = session.schedule_filter_level
    
    if not classtype is None and not classtype == '':
        query &= (db.classes.school_classtypes_id == classtype)
    if not teacher is None and not teacher == '':
        query &= (db.classes.teachers_id == teacher)
    if not location is None and not location == '':
        query &= (db.classes.school_locations_id == location)
    if not level is None and not level == '':
        query &= (db.classes.school_levels_id == level)
    
    return query
    

@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'classes'))
def schedule_export_excel():
    iso_year = request.vars['year']
    iso_week = request.vars['week']
    export = request.vars['export']
    
    def writer_location(locID=None):
        """
        This helper function writes the locations schedule
        """
        teachers_dict = create_teachers_dict()
        locations_dict = create_locations_dict()
        classtypes_dict = create_classtypes_dict()
        
        for day in range(1,8):
            date = iso_to_gregorian(iso_year, iso_week, day)
            col = day - 1
            cell = ws.cell(row = 2, column = col)
            cell.style.font.size = 10
            dayname = NRtoDay(day)
            cell.value = dayname + " \n" + str(date)
            cell.style.font.bold = True
            cell.style.alignment.wrap_text = True
            cell.style.alignment.vertical = "top"
            col_letter = openpyxl.cell.get_column_letter(day) 
            ws.column_dimensions[col_letter].width = 18 
            if locID == None:
                query = (db.classes.Week_day == day) & \
                        (db.classes.Startdate <= date) & \
                        ((db.classes.Enddate >= date) | \
                         (db.classes.Enddate == None))
            else:
                query = (db.classes.Week_day == day) & \
                        (db.classes.Startdate <= date) & \
                        ((db.classes.Enddate >= date) | \
                         (db.classes.Enddate == None)) & \
                        (db.classes.school_locations_id==locID)
            rows = db(query).select(db.classes.id, 
                                    db.classes.school_classtypes_id, 
                                    db.classes.Starttime, 
                                    db.classes.Endtime, 
                                    db.classes.teachers_id, 
                                    db.classes.teachers_id2, 
                                    db.classes.school_locations_id,
                                    orderby=db.classes.Starttime)

            r = 3
            for row in rows:
                location = locations_dict.get(row.school_locations_id, None)
                classtype = classtypes_dict.get(row.school_classtypes_id, None)
                teacher = teachers_dict.get(row.teachers_id, None)
                teacher2 = teachers_dict.get(row.teachers_id2, None)
                cell = ws.cell(row = r, column = col)
                cell.value = ""
                cell.style.font.size = 9
                cell.style.alignment.wrap_text = True
                cell.style.alignment.vertical = "top"
                cancelled = db.classes_cancelled(classes_id=row.id, ClassDate=date)
                if not cancelled is None:
                    cell.value = cell.value + T("CANCELLED") + "\n"
                    cell.style.font.strikethrough = True
                if locID is None:
                    cell.value += location + "\n" + row.Starttime.strftime('%H:%M') + "-" + row.Endtime.strftime('%H:%M') + " \n" + \
                    classtype + " \n" + \
                    teacher.decode('utf-8') + " \n" + teacher2.decode('utf-8') + " \n"
                else:
                    cell.value += row.Starttime.strftime('%H:%M') + "-" + row.Endtime.strftime('%H:%M') + " \n" + \
                    classtype + " \n" + \
                    teacher.decode('utf-8') + " \n" + teacher2.decode('utf-8') + " \n"
                r = r + 1
    def writer_teachers(teachers_id):
        """
        This helper function writes the teacher schedule
        """
        teachers_dict = create_teachers_dict()
        locations_dict = create_locations_dict()
        classtypes_dict = create_classtypes_dict()
        for day in range(1,8):
            date = iso_to_gregorian(iso_year, iso_week, day)
            col = day - 1
            cell = ws.cell(row = 2, column = col)
            cell.style.font.size = 10
            dayname = NRtoDay(day)
            cell.value = dayname + " \n" + unicode(date)
            cell.style.font.bold = True
            cell.style.alignment.wrap_text = True
            cell.style.alignment.vertical = "top"
            col_letter = openpyxl.cell.get_column_letter(day) 
            ws.column_dimensions[col_letter].width = 18 
            query = (db.classes.Week_day == day) & \
                    (db.classes.Startdate <= date) & \
                    ((db.classes.Enddate >= date) | \
                     (db.classes.Enddate == None)) & \
                    ((db.classes.teachers_id==teachers_id) | \
                    (db.classes.teachers_id==teachers_id))
            rows = db(query).select(db.classes.id, 
                                    db.classes.school_classtypes_id, 
                                    db.classes.Starttime, 
                                    db.classes.Endtime, 
                                    db.classes.teachers_id, 
                                    db.classes.teachers_id2, 
                                    db.classes.school_locations_id, 
                                    orderby=db.classes.Starttime)
                    
            r = 3
            for row in rows:
                location = locations_dict.get(row.school_locations_id, None)
                classtype = classtypes_dict.get(row.school_classtypes_id, None)
                teacher = teachers_dict.get(row.teachers_id, None)
                teacher2 = teachers_dict.get(row.teachers_id2, None)
                cell = ws.cell(row = r, column = col)
                cell.value = ""
                cell.style.font.size = 9
                cell.style.alignment.wrap_text = True
                cell.style.alignment.vertical = "top"
                check = db.classes_cancelled(classes_id=row.id, ClassDate=date)
                if not check is None:
                    cell.value = cell.value + "CANCELLED\n"
                    cell.style.font.strikethrough = True
                cell.value += location + " \n" + row.Starttime.strftime('%H:%M') + "-" + row.Endtime.strftime('%H:%M') + " \n" + \
                classtype + " \n" + \
                teacher.decode('utf-8') + " \n" + teacher2.decode('utf-8') + " \n"
                r = r + 1
                
    if export == 'locations':
        # Full schedule
        wb = openpyxl.workbook.Workbook()
        ws = wb.worksheets[0]
        ws.title = "ALL"
        cell = ws.cell(row = 0, column = 0)
        cell.value = T("Schedule week") + " " + unicode(iso_week)
        cell.style.font.size = 12
        cell.style.font.bold = True
        writer_location()
        # schedule by location
        rows = db().select(db.school_locations.id, 
                           db.school_locations.Name, 
                           orderby=db.school_locations.Name)
        for row in rows:
            ws = wb.create_sheet() 
            location = locations_dict.get(row.id, None)
            ws.title = location.decode('utf-8')
            cell = ws.cell(row = 0, column = 0)
            cell.value = T("Schedule") + " " + location + " " + T("week") + unicode(iso_week)
            cell.style.font.size = 12
            cell.style.font.bold = True
            writer_location(row.id)
            
        fname = T("Schedule Locations") + '.xlsx'
        filename = os.path.join(request.folder,('temp'),fname)
        wb.save(filename = filename)
        return response.stream(filename, attachment=True, filename=fname)
    elif export == 'teachers':
        # Full schedule
        wb = openpyxl.workbook.Workbook()
        ws = wb.worksheets[0]
        ws.title = "ALL"
        cell = ws.cell(row = 0, column = 0)
        cell.value = T("Schedule week") + " " + unicode(iso_week)
        cell.style.font.size = 12
        cell.style.font.bold = True
        writer_location()
        
        # schedule by teacher
        rows = db().select(db.teachers.id, db.teachers.FirstName, db.teachers.LastName, orderby=db.teachers.FirstName)
        for row in rows:
            ws = wb.create_sheet() 
            teacher = row.FirstName + " " + row.LastName
            ws.title = teacher.decode('utf-8')
            cell = ws.cell(row = 0, column = 0)
            cell.value = T("Schedule for") + " " + teacher + " " + T("week") + " " + str(iso_week)
            cell.style.font.size = 12
            cell.style.font.bold = True
            writer_teachers(row.id)
        
        fname = T("Schedule Teachers") + '.xlsx'
        filename = os.path.join(request.folder,('temp'),fname)
        wb.save(filename = filename)
        return response.stream(filename, attachment=True, filename=fname)
        
def status_get_return_url(clsID, date_formatted):
    '''
        Returns the return url for a status change from the manage page
    '''
    return URL('manage', args=[clsID, date_formatted])
        
def status_get(clsID, date):
    '''
        Returns the status of a class, possible options:
        normal
        subteacher
        open
        cancelled
    '''
    subteacher = db.classes_subteachers(classes_id=clsID, ClassDate=date)
    cancelled = db.classes_cancelled(classes_id=clsID, ClassDate=date)
    open_class = db.classes_open(classes_id=clsID, ClassDate=date)
    
    
    status = 'normal'
    status_marker = DIV(_class='status_marker_normal')
    if cancelled:
        status = 'cancelled'
        status_marker = DIV(_class='status_marker_cancelled')
    elif open_class:
        status = 'open'
        status_marker = DIV(_class='status_marker_open')
    elif subteacher:
        status = 'subteacher'
        status_marker = DIV(_class='status_marker_subteacher')
    
    return dict(status=status,
                  status_marker=status_marker)
    
def status_remove_cancelled(clsID, date):
    '''
        Removes a class from the cancelled classes table
        returns 1 for success and 0 for failure
    '''
    query = (db.classes_cancelled.classes_id==clsID) & \
            (db.classes_cancelled.ClassDate==date)
    result = db(query).delete()
    return result

def status_remove_subteacher(clsID, date):
    '''
        Removes a class from the classes_subteachers table
        returns 1 for success and 0 for failure
    '''
    query = (db.classes_subteachers.classes_id == clsID) & \
            (db.classes_subteachers.ClassDate == date)
    result = db(query).delete()
    return result

def status_remove_open(clsID, date):
    '''
        Removes a class from the classes_open table
        returns 1 for success and 0 for failure
    '''
    query = (db.classes_open.classes_id == clsID) & \
            (db.classes_open.ClassDate == date)
    result = db(query).delete()
    return result
        
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('update', 'class_status'))
def status_set_normal():
    '''
        Removes open, cancelled and subteacher statusses for a class
    '''
    clsID = request.args[0]
    date_formatted = request.args[1]
    date = datestr_to_python(session.date_format, date_formatted)
    
    status_remove_cancelled(clsID, date)
    status_remove_subteacher(clsID, date)
    status_remove_open(clsID, date)
    
    redirect(status_get_return_url(clsID, date_formatted))
    

@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('update', 'class_status'))
def status_set_open():
    '''
        This function cancels
    '''
    clsID = request.args[0]
    date_formatted = request.args[1]
    date = datestr_to_python(session.date_format, date_formatted)
    
    status_remove_cancelled(clsID, date)
    status_remove_subteacher(clsID, date)
    
    check = db.classes_open(classes_id=clsID, ClassDate=date)
    if check is None:
        db.classes_open.insert(classes_id=clsID, ClassDate=date)
        
    redirect(status_get_return_url(clsID, date_formatted))
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('update', 'class_status'))
def status_set_cancelled():
    '''
        This function cancels or uncancels a class
    '''
    clsID = request.args[0]
    date_formatted = request.args[1]
    date = datestr_to_python(session.date_format, date_formatted)
    
    check = db.classes_cancelled(classes_id=clsID, ClassDate=date)
    if check is None:
        db.classes_cancelled.insert(classes_id=clsID, ClassDate=date)
    else:
        session.flash = T('This class is already cancelled')
        
    redirect(status_get_return_url(clsID, date_formatted))
    
@auth.requires(auth.has_membership(group_id='Admins') or \
               auth.has_permission('update', 'class_status'))
def status_set_subteacher():
    '''
        This function allows adding and editing of substitute teachers
    '''
    clsID = request.args[0]
    date_formatted = request.args[1]
    date = datestr_to_python(session.date_format, date_formatted)
    
    response.title = T("Manage class")
    response.subtitle = get_classname(clsID) + ": " + date_formatted
    response.view = 'general/content_left_sidebar.html'
    
    return_url = URL('status_set_subteacher_clear_status',
                     args = [clsID, date_formatted])
    msg_success = T("Set substitute teachers")
    
    row = db.classes_subteachers(classes_id=clsID, ClassDate=date)
    if row is None:
        # add subteachers
        db.classes_subteachers.classes_id.default=clsID
        db.classes_subteachers.ClassDate.default=date
        
        crud.messages.submit_button = T("Add")
        crud.messages.record_created = msg_success
        crud.settings.showid = False
        crud.settings.create_next = return_url
        
        form = crud.create(db.classes_subteachers)
    else:
        # update subteachers
        crud.messages.submit_button = T("Update")
        crud.messages.record_updated = msg_success
        crud.settings.showid = False
        crud.settings.update_deletable = False
        crud.settings.update_next = return_url
        
        form = crud.update(db.classes_subteachers, row.id)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back, menu='',
        left_sidebar_enabled=True)
        
@auth.requires(auth.has_membership(group_id='Admins') or \
               auth.has_permission('update', 'class_status'))
def status_set_subteacher_clear_status():
    '''
        Clears the open or cancel status when accepted
    '''
    clsID = request.args[0]
    date_formatted = request.args[1]
    date = datestr_to_python(session.date_format, date_formatted)
    
    status_remove_open(clsID, date)
    status_remove_cancelled(clsID, date)
    
    redirect(status_get_return_url(clsID, date_formatted))
    
@auth.requires(auth.has_membership(group_id='Admins') or \
               auth.has_permission('create', 'classes_attendance_override'))
def attendance_override():
    '''
        This function shows a page that allows the customer count of attendance to be overridden
        request.args[0] is expected to be the classes_id
        request.args[1] is expected to be the data formatted using session.date_format
    '''
    clsID = request.args[0]
    date_formatted = request.args[1]
    date = datestr_to_python(session.date_format, date_formatted)
    response.title = T("Manage class")
    response.subtitle = get_classname(clsID) + ": " + date_formatted
    response.view = 'general/content_left_sidebar.html'
    
    return_url = URL('manage', args=[clsID, date_formatted])
    
    db.classes_attendance_override.classes_id.default = clsID
    db.classes_attendance_override.ClassDate.default = date
    
    fields = ['Amount']
    row = db.classes_attendance_override(classes_id=clsID, ClassDate=date)
    if not row: # Create new, else update current
        form = SOLIDFORM(db.classes_attendance_override, 
                         fields=fields, 
                         submit_button=T("Update"))
    else:
        form = SOLIDFORM(db.classes_attendance_override, 
                         row, 
                         fields=fields, 
                         showid=False, 
                         submit_button=T("Update"))
    
    if form.accepts(request.vars, session):
        if form.vars['Amount'] == None:
            query = (db.classes_attendance_override.classes_id==clsID) & \
                    (db.classes_attendance_override.ClassDate==date)
            db(query).delete()
        redirect(return_url)
        
    description = DIV( T("Here the attendance count from the attendance list can be overridden."), " ",
        T("By adding a value below, you can set the number of attending customers manually."), " ",
        T("This number will be used instead of the count from the attendance list for generating attendance statistics."))
        
    content = DIV(description, BR(), form)
    
    back = get_button('back', return_url)
    
    menu = classes_get_menu(request.function, clsID, date_formatted)
    
    return dict(content=content, menu=menu, back=back, 
        left_sidebar_enabled=True,
        right_sidebar_enabled=True)

@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'schedule_manage'))
def manage():
    '''
        This page shows an overview of properties of a class
        request.args[0] is expected to be the class ID
        request.args[1] is expected to be the date of the class, 
            formatted using session.date_format
    '''
    def get_spaces():
        spaces_total = db.classes(clsID).Maxstudents
        spaces_registered = db(db.classes_registration.classes_id==clsID).count()
        spaces_available = db.classes(clsID).Maxstudents - \
                           db(db.classes_registration.classes_id==clsID).count()
        spaces_attending = db(db.classes_attendance.classes_id==clsID, 
                              db.classes_attendance.ClassDate==date).count()
        
        spaces_data = [((T("Total: "), unicode(spaces_total))), 
        ((T("Registered: "), unicode(spaces_registered))), 
        ((T("Available: "), unicode(spaces_available)))
         ]
        #(T("Attending: "),unicode(spaces_attending)))
        
        spaces = TABLE(TR(TD(T("Spaces"), ': ', spaces_data[0][1], _width='33%'),
                          TD(spaces_data[1][0], spaces_data[1][1], _width='33%'),
                          TD(spaces_data[2][0], spaces_data[2][1], _width='33%')), _width="100%")
        return spaces
    
    clsID = request.args[0]
    date = datestr_to_python(session.date_format, request.args[1])
    date_formatted = date.strftime(session.date_format)
    
    session.register_classdate = date
    session.class_details_back = 'cd' # used for expected attendance to know where to go back to
    session.customers_back = 'classes_manage'
    
    response.title = T("Manage class")
    response.subtitle = get_classname(clsID) + ": " + date_formatted
    
    ## registrations
    title = H3(T("Registered"))
    
    query = (db.classes_registration.classes_id==clsID)
    rows = db(query).select(db.customers.id, 
                            db.customers.ThumbSmall, 
                            db.customers.ActiveMember, 
                            db.customers.FirstName, 
                            db.customers.LastName, 
        left = [(db.customers.on(db.classes_registration.customers_id==db.customers.id))], 
        orderby=db.customers.FirstName|db.customers.LastName
        )
    
    table = TABLE(_class="os-customer_table")
    for row in rows.render():
        paused = ''
        paused_check = manage_registration_check_paused(row)
        if paused_check:
            paused = T('Membership paused')
        table.append(TR(TD(row.ThumbSmall),
                        TD(SPAN(row.FirstName,
                                ' ',
                                row.LastName,
                                _class="os-customer_name"),
                            BR(), paused)))
    
    edit = A(T("Edit registrations"), 
             _href=URL('manage_registration', 
                       vars=dict(clsID=clsID, date=date_formatted)))
    
    if auth.has_membership(group_id='Admins') or \
        auth.has_permission('update', 'classes_registration'):
        spaces = get_spaces()
        registered = DIV(title, table, spaces, edit)
    else:
        registered = ''
        
    ## registrations end
    ## waitinglist
    title = H3(T("Waitinglist"))
    
    query = (db.classes_waitinglist.classes_id==clsID)
    rows = db(query).select(db.customers.id, 
                            db.customers.ActiveMember, 
                            db.customers.ThumbSmall, 
                            db.customers.FirstName, 
                            db.customers.LastName, 
        left = [(db.customers.on(db.classes_waitinglist.customers_id==db.customers.id))], 
        orderby=db.customers.FirstName|db.customers.LastName
        )
        
    table = TABLE(_class="os-customer_table")
    for row in rows.render():
        paused = ''
        paused_check = manage_registration_check_paused(row)
        if paused_check:
            paused = T('Membership paused')
        table.append(TR(TD(row.ThumbSmall),
                        TD(SPAN(row.FirstName,
                                ' ',
                                row.LastName,
                                _class="os-customer_name"),
                            BR(), paused)))
        
    edit = A(T("Edit waitinglist"), _href=URL('waiting_list', vars=dict(clsID=clsID, date=date_formatted)))
    
    if auth.has_membership(group_id='Admins') or auth.has_permission('update', 'classes_waitinglist'):
        waitinglist = DIV(title, table, edit)
    else:
        waitinglist = ''
        
    ## waitinglist end
    
    # subteachers begin 
    subcheck = db.classes_subteachers(classes_id=clsID, ClassDate=date)
    if not subcheck is None:
        teacher = db.teachers(subcheck.teachers_id)
        teacher2 = db.teachers(subcheck.teachers_id2)
        teacher_name = teacher.FirstName + " " + teacher.LastName
        if not teacher2 is None:
            teacher2_name = teacher2.FirstName + " " + teacher2.LastName
            subteachers = DIV(H4(T("Substitute teachers")), 
                              teacher_name, 
                              BR(), 
                              teacher2_name)
        else:
            subteachers = DIV(H4(T("Substitute teachers")), 
                              teacher_name)
    else:
        subteachers = ""
    
    # subteachers end
    
    # Status begin
    if auth.has_membership(group_id='Admins') or auth.has_permission('update', 'class_status'):
    
        result = status_get(clsID, date)
        status = result['status']
        
        status_normal = A(T("Normal"), 
                        _href=URL('status_set_normal', 
                                  args=[clsID, date_formatted]))
        status_cancelled = A(T('Cancelled'), 
                             _href=URL('status_set_cancelled',
                                       args=[clsID, date_formatted]))
        status_open = A(T("Open"), 
                        _href=URL('status_set_open',
                                  args=[clsID, date_formatted]))
        status_subteacher = A(T("Subteacher"), 
                              _href=URL('status_set_subteacher',
                                        args=[clsID, date_formatted]))
        
        status_marker = ''
        
        class_normal = ''
        class_subteacher = ''
        class_open = ''
        class_cancelled = ''
        
        if status == 'cancelled':
            status_cancelled = T("Cancelled")
            class_cancelled = 'web2py-menu-active'
        elif status == 'open':
            status_open = T("Open")
            class_open = 'web2py-menu-active'
        elif status == 'subteacher':
            status = 'subteacher'
            class_subteacher = 'web2py-menu-active'
        else:
            status = 'normal'
            status_normal = T("Normal")
            class_normal = 'web2py-menu-active'
        
        #status = session.class_status.get(status, T("Unknown..."))
        
        status_list = UL(LI(status_normal, _class=class_normal),
                         LI(status_subteacher, _class=class_subteacher),
                         LI(status_open, _class=class_open),
                         LI(status_cancelled, _class=class_cancelled),
                         _class='os-submenu-horizontal')
                         
        table_status = TABLE(TR(TD(P(T('Status')), status_marker),
                                TD(status_list)), 
                             _id='class_status',
                             _class=status)
                             
    else:
        table_status = ''
    
    # Status end
    
    # check if we're coming from a cancelled class link from the pinboard
    if request.vars['back'] == 'pinboard': 
        back = get_button('back', URL('default', 'pinboard'))
    else:
        session.schedule_show_date = date_formatted
        back = get_button('back' , URL('schedule'))
    
    if request.user_agent()['is_mobile']:
        content = DIV(registered, attendance, waitinglist)
    else:
        content = DIV(table_status,
                      DIV(registered,
                          _id='os-classes_left'),
                      DIV(waitinglist,
                          _id='os-classes_right'))
    
    menu = classes_get_menu(request.function, clsID, date_formatted)
    
    return dict(content=content, \
        back=back, menu=menu,
        subteachers=subteachers,
        left_sidebar_enabled=True)
    

@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('update', 'classes_attendance'))
def attendance_list():
    '''
        Lists customers and adds add buttons when a customer isn't attending yet
    '''
    response.title = T("Attendance")
    clsID = request.vars['clsID']
    date_formatted = request.vars['date']
    date = datestr_to_python(session.date_format, date_formatted)
    session.classes_attendance_back = 'attendance_list'
    response.subtitle = get_classname(clsID) + ": " + date_formatted
    
    name = ''
    query = (db.customers.ActiveMember == True) # generic show all active customers query
    if 'name' in request.vars: # check whether a search filter is in use
        if request.vars['name'] != '':
            name = request.vars['name'].strip()
            search_name = '%' + name + '%'
            query &= ((db.customers.FirstName.like(search_name)) | \
                      (db.customers.LastName.like(search_name)))
    
    form = _generate_classdetails_searchform(clsID, date, name, request.function)
    
    if request.user_agent()['is_mobile']:
        fields = [ db.customers.FirstName, db.customers.LastName ]
    else:
        fields = [ db.customers.FirstName, db.customers.LastName, db.customers.Gender, db.customers.DateOfBirth ]
    
    if session.show_location:
        db.customers.school_locations_id.readable = True
        db.customers.school_locations_id.writable = True
        if request.user_agent()['is_mobile']:
            pass
        else:
            fields.append(db.customers.school_locations_id)
    else:
        db.customers.school_locations_id.readable = False
        db.customers.school_locations_id.writable = False
        
    links = [lambda row: get_button('add', URL('attendance_process',
                                               vars={'cuID':    row.id, 
                                                     'clsID':   clsID,
                                                     'date':    date_formatted,
                                                     'remove':  False,
                                                     'atype':   None})),
            lambda row:A(SPAN(_class="buttontext button", 
                              _title=T("Trialclass")), 
                         SPAN(_class="icon leaf icon-leaf"), 
                         T("Trialclass"), 
                         _class="button btn", \
                         _href=URL('attendance_process',
                                   vars={'cuID':    row.id, 
                                         'clsID':   clsID,
                                         'date':    date_formatted,
                                         'remove':  False,
                                         'atype':   1})),
            lambda row: A(SPAN(_class="buttontext button", 
                               _title=T("Drop in")), 
                          SPAN(_class="icon arrow down icon-arrow-down"), 
                          T("Drop in"), 
                          _class="button btn", \
                          _href=URL('attendance_process',
                                   vars={'cuID':    row.id, 
                                         'clsID':   clsID,
                                         'date':    date_formatted,
                                         'remove':  False,
                                         'atype':   2})),
            lambda row: A(SPAN(_class="buttontext button", 
                               _title=T("Class card")), 
                          SPAN(_class="icon asterisk icon-asterisk"), 
                          T("Class pass"), 
                          _class="button btn", \
                          _href=URL('attendance_process',
                                    vars={'cuID':    row.id, 
                                          'clsID':   clsID,
                                          'date':    date_formatted,
                                          'remove':  False,
                                          'atype':   3}))]
    grid = SQLFORM.grid(query, 
                        fields=fields, 
                        editable=False, 
                        details=False, 
                        deletable=False, 
                        create=False, 
                        csv=False, 
                        searchable=False, 
                        links_placement='left', 
                        links=links, 
                        ui=session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    
    attending = []
    query = (db.classes_attendance.classes_id==clsID) & \
            (db.classes_attendance.ClassDate==date)
    rows = db(query).select(db.customers.FirstName, 
                            db.customers.LastName, 
                            db.classes_attendance.AttendanceType,
                            left = [(db.customers.on(\
                                    db.classes_attendance.customers_id==\
                                        db.customers.id))], 
                            orderby=db.customers.FirstName)
    for row in rows:
        if not row.classes_attendance.AttendanceType is None:
            attending.append(row.customers.FirstName + " " + \
                             row.customers.LastName + " - " + \
                             session.att_types_dict.get(\
                                row.classes_attendance.AttendanceType, ""))
        else:
            attending.append(row.customers.FirstName + " " + \
                             row.customers.LastName)
            
    # list of registered customers
    query = (db.classes_attendance.classes_id==clsID) & \
            (db.classes_attendance.ClassDate==date)
    rows = db(query).select(db.customers.id, 
                            db.customers.FirstName, 
                            db.customers.LastName, 
                            db.classes_attendance.AttendanceType,
                            left=[(db.customers.on(\
                                    db.classes_attendance.customers_id==\
                                    db.customers.id))], 
                            orderby=db.customers.FirstName)
    table = TABLE()
    for row in rows.render():
        table.append(TR(TD(row.customers.FirstName + ' ' + \
                             row.customers.LastName),
                        TD(row.classes_attendance.AttendanceType),
                        TD('[',
                           A('x', 
                             _href=URL('attendance_process',
                              vars={'cuID':row.customers.id,
                                    'clsID':clsID,
                                    'date':date_formatted,
                                    'remove':True})),
                            ']')))
    
    sidebar_title = H3(T("Attending"))
    sidebar_contents = DIV(sidebar_title, table)
    
    back = get_button('back', URL('attendance', args=[clsID, date_formatted]))
    
    return dict(back=back, grid=grid, form=form, sidebar_contents=sidebar_contents)
    
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('update', 'classes_registration'))
def manage_registration():
    response.title = T("Registration")
    session.classes_register_back = None
    clsID = request.vars['clsID']
    date = datestr_to_python(session.date_format, request.vars['date'])
    date_formatted = date.strftime(session.date_format)
    session.clsID = clsID
    session.register_classdate = date
    response.subtitle = get_classname(clsID) + ": " + date_formatted
    
    name = ''
    query = (db.customers.ActiveMember == True) # generic show all active customers query
    if 'name' in request.vars: # check whether a search filter is in use
        if request.vars['name'] != '':
            name = request.vars['name']
            search_name = '%' + request.vars['name'] + '%'
            query &= ((db.customers.FirstName.like(search_name)) | \
                      (db.customers.LastName.like(search_name)))
            
    form = _generate_classdetails_searchform(clsID, date, name, request.function)
    
    fields = [ db.customers.FirstName, 
               db.customers.LastName, 
               db.customers.Gender, 
               db.customers.DateOfBirth ]
    
    if session.show_location:
        db.customers.school_locations_id.readable = True
        db.customers.school_locations_id.writable = True
        fields.append(db.customers.school_locations_id)
    else:
        db.customers.school_locations_id.readable = False
        db.customers.school_locations_id.writable = False
        
    
    links = [dict(header=T('Paused'), body=manage_registration_check_paused), 
             dict(header=T(''), body=_check_registration)]
    grid = SQLFORM.grid(query, 
                        fields=fields, 
                        links=links,
                        editable=False, 
                        details=False, 
                        deletable=False, 
                        create=False, 
                        csv=False, 
                        searchable=False, 
                        ui=session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    
    # list of registered customers
    query = (db.classes_registration.classes_id==clsID)
    rows = db(query).select(db.customers.id, 
                            db.customers.FirstName, 
                            db.customers.LastName, 
                            left=[(db.customers.on(\
                                        db.classes_registration.customers_id==\
                                        db.customers.id))], 
                            orderby=db.customers.FirstName)
    table = TABLE()
    for row in rows.render():
        table.append(TR(TD(row.FirstName + ' ' + row.LastName),
                        TD('[',
                           A('x',
                             _href=URL('register',
                              vars=dict(customers_id=row.id,
                                         clsID=clsID))),
                            ']')))
    
    sidebar_title = H3(T("Registered"))
    registered = DIV(sidebar_title, table)
    
    back = get_button('back', URL('manage', args=[clsID, date_formatted]))
    
    return dict(grid=grid, 
                form=form, 
                sidebar_title=sidebar_title, 
                registered=registered, 
                back=back)
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('update', 'classes_waitinglist'))
def waiting_list():
    response.title = T("Waitinglist")
    clsID = request.vars['clsID']
    date = datestr_to_python(session.date_format, request.vars['date'])
    date_formatted = date.strftime(session.date_format)
    session.waitinglist_classdate = date
    session.clsID = clsID
    response.subtitle = get_classname(clsID)
    
    name = ''
    query = (db.customers.ActiveMember == True) # generic show all active customers query
    if 'name' in request.vars: # check whether a search filter is in use
        if request.vars['name'] != '':
            name = request.vars['name']
            search_name = '%' + request.vars['name'] + '%'
            query &= ((db.customers.FirstName.like(search_name)) | (db.customers.LastName.like(search_name)))
    
    form = _generate_classdetails_searchform(clsID, date, name, request.function)
    
    if request.user_agent()['is_mobile']:
        fields = [ db.customers.FirstName, db.customers.LastName ]
    else:
        fields = [ db.customers.FirstName, db.customers.LastName, db.customers.Gender, db.customers.DateOfBirth ]
    
    if session.show_location:
        db.customers.school_locations_id.readable = True
        db.customers.school_locations_id.writable = True
        if request.user_agent()['is_mobile']:
            pass
        else:
            fields.append(db.customers.school_locations_id)
    else:
        db.customers.school_locations_id.readable = False
        db.customers.school_locations_id.writable = False
    
    links = [ dict(header=T(''), body=_check_waitinglist)]
    grid = SQLFORM.grid(query, fields=fields, editable=False, details=False, deletable=False, create=False, csv=False, searchable=False, 
        links=links, ui = session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    
    # list of waiting customers
    query = (db.classes_waitinglist.classes_id==clsID)
    rows = db(query).select(db.customers.id, 
                            db.customers.FirstName, 
                            db.customers.LastName, 
                            left=[(db.customers.on(db.classes_waitinglist.customers_id==\
                                                   db.customers.id))], 
                            orderby=db.customers.FirstName)
                            
    table = TABLE()
    for row in rows.render():
        table.append(TR(TD(row.FirstName + ' ' + row.LastName),
                        TD('[',
                           A('x', 
                             _href=URL('waitinglist',
                              vars=dict(customers_id=row.id,
                                         clsID=clsID))),
                            ']')))
    
    sidebar_title = H3(T("Waitinglist"))
    onlist = DIV(sidebar_title, table)
    
    back = get_button('back', URL('manage', args=[clsID, date_formatted]))
    
    return dict(back=back, grid=grid, form=form, onlist=onlist, sidebar_title=sidebar_title)
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('update', 'attendance'))
def attendance_get_chart_title():
    '''
        This function returns the title for the attendance chart
    '''
    return dict(title=T("Attendance chart") + " " + 
                 unicode(session.stats_attendance_year))
                 
                 
@auth.requires_login()
def attendance_set_chart_year():
    '''
        This function is calles as json and sets the attendance year + or - 1 
        year. It's part of the back-end for the attendance chart buttons on the 
        attendance page.
    '''
    response.view = 'generic.json'
    try:
        session.stats_attendance_year += int(request.vars['year'])
        status = 'success'
        message = T('')
    except:
        status = 'fail'
        message = T('Error changing years')
    
    chart_title = attendance_get_chart_title()
    
    return dict(status=status,
                 message=message, 
                 chart_title=chart_title)
    
    

@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('update', 'attendance'))
def attendance():
    '''
        This function shows a page of people expected to attend a class
        request.args[0] is expected to be the classes_id
        request.args[1] is expected to be the class date
            formatted using session.date_format
    '''
    clsID = request.args[0]
    date = datestr_to_python(session.date_format, request.args[1])
    date_formatted = date.strftime(session.date_format)
    
    response.title = T("Manage class")
    response.subtitle = get_classname(clsID) + ": " + date_formatted
    
    # session variable to control redirect / back behaviour
    session.classes_attendance_back = None
    
    # session variables for stats/attendance
    session.stats_attendance_year = date.year
    session.stats_attendance_clsID = clsID
    
    ## Expected attendance ##
    title = H3(T("Expected Attendance"))
    
    query = (db.classes_attendance.classes_id == clsID) & \
            (db.classes_attendance.ClassDate == date)
    rows = db(query).select(db.classes_attendance.customers_id)
    attendance_list = []
    for row in rows:
        attendance_list.append(row.customers_id)
    
    mobile = False
    if request.user_agent()['is_mobile']:
        mobile = True
    expected_attendance_list = attendance_expected_get_list(clsID, 
                                                            date,
                                                            mobile) 
    table = TABLE(_class="os-customer_table")
    for item in expected_attendance_list:
        row = item[0]
        atttype = item[1]
        arrow = ''
        if not atttype and not row.id in attendance_list:
            arrow = TD(SPAN(_class='icon rightarrow icon-arrow-right'),
                      _class='add_attendance os-clickable')
        table.append(TR(TD(row.ThumbSmall, 
                           _class='os-customer_image_td'),
                        TD(SPAN(row.FirstName,
                                ' ',
                                row.LastName,
                                _class="os-customer_name"),
                            BR(), 
                            atttype),
                        arrow,
                        _id='expatt_' + unicode(row.id)))
    
    mail_teacher =  A(SPAN(_class="buttontext button", 
                           _title=T("List to teacher")), 
                      SPAN(_class="icon envelope icon-envelope"), 
                      " " + T("List to teacher"), 
                      _class="button btn", \
                      _href=URL('mail_teacher', 
                                vars=dict(clsID=clsID, date=date_formatted)))
                                
    expected_attendance = DIV(title, table)
    
    ## Expected attendance end ##
    
    ## attendance
    title = H3(T("Attendance"))
    query = (db.classes_attendance.classes_id==clsID) & \
            (db.classes_attendance.ClassDate==date)
    rows = db(query).select(db.customers.id, 
                            db.customers.ThumbSmall, 
                            db.customers.ActiveMember, 
                            db.customers.FirstName, 
                            db.customers.LastName,
                            db.classes_attendance.AttendanceType, 
                            db.classes_attendance.customers_classcards_id,
                            left = [(db.customers.on(\
                                        db.classes_attendance.customers_id==\
                                        db.customers.id))], 
                            orderby=db.customers.FirstName )
    
    table = TABLE(_class="os-customer_table")
    for row in rows.render():
        div_tools = DIV(SPAN(T('Remove'),
                             _class='os-customer_tools_remove'),
                        _class='os-customer_tools')
        table.append(TR(TD(row.customers.ThumbSmall,
                           _class='os-customer_image_td'),
                        TD(DIV(SPAN(row.customers.FirstName,
                                ' ',
                                row.customers.LastName,
                                _class="os-customer_name"),
                                BR(), row.classes_attendance.AttendanceType,
                                ' ', row.classes_attendance.customers_classcards_id,
                                div_tools,
                                _class='os-wrapper')),
                        _id='att_' + unicode(row.customers.id) + '_' + \
                                     unicode(clsID) + '_' + \
                                     date_formatted
                        ))
        
    edit = A(T("Edit attendance"), _href=URL('attendance_list',
                                             vars={'clsID':clsID,
                                                   'date':date_formatted}))
    
    if auth.has_membership(group_id='Admins') or \
        auth.has_permission('update', 'attendance'):
        attendance = DIV(title, table, edit)
    else:
        attendance = ''
    ## attendance end
    
    chart_title = DIV(SPAN(H3(attendance_get_chart_title()['title'])),
                      _class='left')
    chart_buttons = DIV(SPAN(SPAN(_class='icon arrow left icon-arrow-left'), 
                             _class='button btn',
                             _id='previous'),
                        SPAN(SPAN(_class='icon arrow right icon-arrow-right'), 
                             _class='button btn',
                             _id='next'),
                        _class='os-attendance_buttons right')
    chart_header = DIV(chart_title, chart_buttons, 
                       _class='os-attendance_chart_title')
    content = DIV(DIV(expected_attendance,
                      _id='os-classes_left'),
                  DIV(attendance,
                      _id='os-classes_right'),
                  chart_header,
                  )
    
    if session.class_details_back == 'cd':
        back_url = URL('manage', args=[clsID, date_formatted])
    else:
        session.schedule_show_date = date_formatted
        back_url = URL('schedule')
    back = get_button('back', back_url)
    
    menu = classes_get_menu(request.function, clsID, date_formatted)
    
    
    return dict(content=content, back=back, menu=menu, 
        mail_teacher=mail_teacher,
        left_sidebar_enabled=True)
    

@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('update', 'attendance'))
def mail_teacher():
    clsID = request.vars['clsID']
    date = datestr_to_python(session.date_format, request.vars['date'])
    date_formatted = date.strftime(session.date_format)
    teID = db.classes[clsID].teachers_id
    row = db.teachers[teID]
    
    # generate the message
    expected_attendance_list = attendance_expected_get_list(clsID, 
                                                             date,
                                                             False)
    table = TABLE(TR(TH(T("Name"))))
    for item in expected_attendance_list:
        customer = item[0]
        atttype = item[1]
        table.append(TR(TD(customer.FirstName + ' ' + customer.LastName),
                        TD(atttype)))
    
    message = "<html>\n	<head><STYLE TYPE='text/css'> <!-- TD{font-size: 9pt; align:left;} TH{font-size: 9pt; text-align:left;} --->  </STYLE></head>	\
        <body style=font-size:10pt; font-style:normal;'>"
    message +=  T("Dear") + " " + row.FirstName + ",<br><br>"
    message += T("Below is the expected attendance list for your class") + \
                " <i>" + get_classname(clsID) + "</i> " + T("on") + " <i>" + \
                unicode(date) + "</i>.<br><br>"
    message += unicode(table) + "</p><br><br>"
    if db.sys_properties(Property='smtp_signature'):
        smtp_signature = db.sys_properties(Property='smtp_signature').PropertyValue
    else:
        smtp_signature = ""
    message += smtp_signature    
    message += "<br><br><font size=2>" + T("This message is autogenerated by OpenStudio") + " - <a href='http://www.openstudioproject.com'>www.openstudioproject.com</a></font>"
    message += "</body></html>"
    # now send the mail
    subcheck = db.classes_subteachers(classes_id=clsID, ClassDate=date)
    if subcheck == None:
        email_teacher = row.Email
    else:
        teacher = db.teachers(subcheck.teachers_id)
        email_teacher = teacher.Email
    check = MAIL.send(to=[email_teacher],
          subject=T("Attendance list for your class on") + " " + unicode(date),
          # If reply_to is omitted, then mail.settings.sender is used
          reply_to=None,
          message=message)
          
    if check:
        session.flash = T("Successfully sent mail")
    else:
        session.flash = T("Unable to send mail")
    
    redirect(URL('attendance', args=[clsID, date_formatted]))
    

@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('update', 'classes_registration'))
def register():
    clsID = request.vars['clsID']
    customers_id = request.vars['cuID']
    
    if session.classes_register_back == "customers":
        next_page = URL('customers','classes_edit', vars={'cuID':customers_id})
    else:
        date = session.register_classdate
        date_formatted = date.strftime(session.date_format)
        next_page = URL('manage_registration', vars=dict(clsID=clsID, date=date_formatted))

    today = datetime.date.today()
    check = db.classes_registration(customers_id=customers_id, classes_id=clsID)
    if check is None:
        maxstudents = db.classes(clsID).Maxstudents
        registered = db(db.classes_registration.classes_id==clsID).count()
        if registered < maxstudents: # check if the maximum number of students hasn't been reached yet
            # now check for the membership
            query = (db.customers_subscriptions.customers_id==customers_id) & \
                    ((db.customers_subscriptions.Enddate>=today) | \
                     (db.customers_subscriptions.Enddate==None))
            subscriptions = db(query).select(orderby=db.customers_subscriptions.Startdate)
            latest_membership = subscriptions.first()
            if not latest_membership is None:
                allowed_classes = db.school_subscriptions(latest_membership.school_subscriptions_id).NRofClasses # get nrofclasses from membership
                query = (db.classes_registration.customers_id == customers_id)
                nrofclasses = db(query).count()
                if allowed_classes > nrofclasses or allowed_classes is None:
                     db.classes_registration.insert(customers_id=customers_id, 
                                                    classes_id=clsID)
                else:
                   session.flash = T("This customer is already added to the " + unicode(allowed_classes) + " class(es) allowed by his/her membership.")
            else:
                session.flash = T("This customer currently doesn´t have a membership.")
        else:
            session.flash = T("Maximum number of students reached for this class.")
    else:
        # unregister the customer
        query = (db.classes_registration.customers_id==customers_id) & \
                (db.classes_registration.classes_id==clsID)
        db(query).delete()
        session.flash = T("Unregistered customer")
        
    redirect(next_page)
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('update', 'classes_waitinglist'))
def waitinglist():
    clsID = request.vars['clsID']
    customers_id = request.vars['customers_id']
    date = session.waitinglist_classdate
    date_formatted = date.strftime(session.date_format)
    list_check = db.classes_waitinglist(customers_id=customers_id, classes_id=clsID)
    if list_check == None:
        db.classes_waitinglist.insert(customers_id=customers_id, classes_id=clsID)
    else:
        query = (db.classes_waitinglist.customers_id==customers_id) & \
                (db.classes_waitinglist.classes_id==clsID)
        db(query).delete()
    redirect(URL('waiting_list', vars=dict(clsID=clsID, date=date_formatted)))
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('update', 'attendance'))
def attendance_remove():
    '''
        Called as JSON, used to remove the attendance to a class for a customer
    '''
    response.view = 'generic.json'
    customers_id = request.vars['cuID']
    clsID = request.vars['clsID']
    date_formatted = request.vars['date']
    date = datestr_to_python(session.date_format, date_formatted)
    
    status = 'success'
    message = T('Successfully removed')
    
    query = (db.classes_attendance.customers_id==customers_id) & \
            (db.classes_attendance.classes_id==clsID) & \
            (db.classes_attendance.ClassDate==date)
    if not db(query).delete(): # returns 0 when it fails
        status = 'fail'
        message = T("Remove failed")
    
    return dict(status=status, message=message)
    
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('update', 'attendance'))
def attendance_process():
    '''
        This function is used to precess the attendance registrations for 
        classes.
    '''
    customers_id = request.vars['cuID']
    clsID = request.vars['clsID']
    date_formatted = request.vars['date']
    date = datestr_to_python(session.date_format, date_formatted)
    atype = request.vars['atype']
    if atype == '':
        atype = 'None'
    
    if session.classes_attendance_back == 'attendance_list':
        redirect_url = URL('attendance_list', vars={'clsID':clsID,
                                                    'date':date_formatted})
    else:
        redirect_url = URL('attendance', vars={'clsID':clsID,
                                               'date':date_formatted})

    bad_request = customers_id is None or \
                  clsID is None or \
                  date is None or \
                  atype is None and \
                  not request.vars['remove']
    if bad_request:
        session.flash = T("Error while adding attendance.")
        redirect(redirect_url)

    
    # attendance function used ajaj with attendance_remove function
    if request.vars['remove'] == 'True':
        query = (db.classes_attendance.customers_id==customers_id) & \
                (db.classes_attendance.classes_id==clsID) & \
                (db.classes_attendance.ClassDate==date)
        db(query).delete()
        redirect(URL('attendance_list', vars={'clsID':clsID,
                                              'date':date_formatted}))
        
    attending_check = db.classes_attendance(customers_id=customers_id, 
                                            classes_id=clsID, 
                                            ClassDate=date)
    if not attending_check is None:
        session.flash = T("This customer is already attending this class.")
        redirect(redirect_url)
    else:
        if atype == 'None': #regular attendance
            # register attendance in any case and show a message when something's strange
            db.classes_attendance.insert(customers_id=customers_id, 
                                         classes_id=clsID, 
                                         ClassDate=date, 
                                         AttendanceType=None)
            # get membership to check for allowed classes
            query = (db.customers_subscriptions.customers_id==customers_id) & \
                    (db.customers_subscriptions.Startdate<=date) & \
                    ((db.customers_subscriptions.Enddate>=date) | \
                     (db.customers_subscriptions.Enddate==None))
            ms = db(query).select().first() # first row of query results
            if ms != None:
                # check if the membership is paused
                paused_check = None
                query = (db.customers_subscriptions.customers_id == customers_id) & \
                        (db.customers_subscriptions.Startdate <= date) & \
                        ((db.customers_subscriptions.Enddate >= date) | \
                         (db.customers_subscriptions.Enddate == None))
                current_membership = db(query).select(db.customers_subscriptions.ALL).first()
                if current_membership:
                    query = (db.customers_subscriptions_paused.customers_subscriptions_id == \
                             current_membership.id) & \
                            (db.customers_subscriptions_paused.Startdate <= \
                             date) & \
                            ((db.customers_subscriptions_paused.Enddate >= \
                              date) | \
                             (db.customers_subscriptions_paused.Enddate == \
                              None))
                    paused_check = db(query).select(db.customers_subscriptions_paused.ALL).first()
                if paused_check is None:
                    allowed_classes = db.school_subscriptions(ms.school_subscriptions_id).NRofClasses
                    taken_classes = attendance_count_classes(customers_id, date)
                    #print taken_classes
                    #print allowed_classes
                    if allowed_classes is None or \
                       taken_classes <= allowed_classes:
                        redirect(redirect_url)
                    else:
                        session.flash = T("This customer has attended the maximum number \
                            of classes allowed by his/her membership this week")
                else:
                    session.flash = T("The membership for this customer is paused at this date")
            else:
                session.flash = T("This customer doesn't have a valid membership")
                
            redirect(redirect_url)
            
        elif atype == '3':
            redirect(URL('attendance_list_classcards', 
                         vars={'cuID':customers_id,
                               'clsID':clsID,
                               'date':date_formatted}))
        else:
            db.classes_attendance.insert(customers_id=customers_id, 
                                         classes_id=clsID, 
                                         ClassDate=date, 
                                         AttendanceType=atype)
            redirect(redirect_url)


@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('update', 'classes_attendance'))
def attendance_list_classcards():
    customers_id = request.vars['cuID']
    clsID = request.vars['clsID']
    date_formatted = request.vars['date']
    date = datestr_to_python(session.date_format, date_formatted)
    response.title = T("Which one...?")
    response.subtitle = T("Choose a class card")
    
    db.customers_classcards.customers_id.readable=False
    headers = {'customers_classcards.id':T("Card")}
    
    links = [dict(header=T('Number of classes'), body=classcard_count_classes), 
             lambda row: A(SPAN(_class="buttontext button", 
                           _title=T("Choose")), 
                           SPAN(_class="icon ok icon-ok"), 
                           " " + T("Choose"), 
                           _class="button btn", \
             _href=URL('attendance_book_classcard', 
                       vars={'clsID':clsID, 
                             'ccID':row.id, 
                             'cuID':customers_id,
                             'date':date_formatted}))]
    
    query = (db.customers_classcards.customers_id == customers_id)
    db.customers_classcards.customers_id.default = customers_id
    grid = SQLFORM.grid(query, headers=headers, links=links, details=False, csv=False, searchable=False, ui = session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    
    back = get_button('back', URL('attendance_list'))
        
    return dict(grid=grid, back=back)
    

@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('update', 'attendance'))
def attendance_book_classcard():
    customers_id = request.vars['cuID']
    clsID = request.vars['clsID']
    date_formatted = request.vars['date']
    date = datestr_to_python(session.date_format, date_formatted)
    ccID = request.vars['ccID']
    
    # check if the pass still has space for one more class
    row = db.customers_classcards[ccID]
    total = row.Classes
    query = (db.classes_attendance.customers_classcards_id == row.id)
    used = db(query).count()
    if used < total:
        db.classes_attendance.insert(customers_id=customers_id, 
                                     classes_id=clsID, 
                                     ClassDate=date, 
                                     AttendanceType=3, 
                                     customers_classcards_id=ccID)
        redirect(URL('attendance_list', vars={'clsID':clsID,
                                              'date':date_formatted}))
    else:
        session.flash = T("No more classes available on this pass")
        redirect(URL('attendance_list_classcards', 
                     vars={'cuID':customers_id,
                           'clsID':clsID,
                           'date':date_formatted}))
        
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'teacher_holidays'))
def teacher_holidays():
    '''
        Select a teacher and set all classes as open within a range of dates.
        response.flash allows you to flash a message to the user when the page 
        is returned. 
        Use session.flash instead of response.flash to display a message 
        after redirection.
    '''
    response.view = 'general/only_content.html'
    response.title = T('Teacher holiday')
    response.subtitle = ''
    
    explanation = DIV(T('This will change the status of all classes to open \
        for the selected teacher in the specified range of time.'))
    
    form = SQLFORM.factory(
        Field('teachers_id', db.teachers, 
            requires=IS_IN_DB(db,'teachers.id','%(FirstName)s %(LastName)s',
                              zero=(T('Select teacher...'))), \
            represent=lambda value, row: teachers_dict.get(value, None), 
            label=T("Teacher")),
        Field('Startdate', 'date', required=True, 
            requires=IS_DATE_IN_RANGE(format=session.date_format,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)), 
            label=T("From")),
        Field('Enddate', 'date', required=True, 
            requires=IS_DATE_IN_RANGE(format=session.date_format,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)), 
            label=T("Until")),
        submit_button = T('OK')
        )
    
    if form.process().accepted:
        teachers_id = request.vars['teachers_id']
        startdate = datestr_to_python(session.date_format, 
                                      request.vars['Startdate'])
        enddate = datestr_to_python(session.date_format,
                                    request.vars['Enddate'])
        if enddate < startdate:
            response.flash = \
                T('Enddate has to be equal to or later than start.')
        else:
            delta = datetime.timedelta(days=1)
            current_date = startdate
            changed = 0
            
            while current_date <= enddate:
                # find all classes for teacher and set them as open
                weekday = current_date.isoweekday()
                query = (db.classes.Week_day == weekday) & \
                        (db.classes.Startdate <= current_date) & \
                        ((db.classes.Enddate >= current_date) | \
                         (db.classes.Enddate == None)) & \
                        ((db.classes.teachers_id == teachers_id) | \
                         (db.classes.teachers_id2 == teachers_id))
                rows = db(query).select(db.classes.id)
                
                for row in rows:
                    check = db.classes_open(classes_id = row.id,
                                            ClassDate = current_date)
                    if not check:
                        db.classes_open.insert(classes_id = row.id,
                                               ClassDate = current_date)
                        changed += 1
                                           
                current_date += delta
                
            if changed == 0:
                session.flash = T('No classes found matching your criteria.')
            else:
                session.flash = T('Changed status for') + ' ' + \
                                 unicode(changed) + ' ' + \
                                 T('classes') + '.'
            redirect(URL('schedule'))
        
    elif form.errors:
        response.flash = T('Please check your input.')
        
    content = DIV(BR(), explanation, BR(), form)
        
    back = get_button('back', URL('schedule'))
    
    return dict(content=content,
                 back=back)
                 
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'classes_open'))
def classes_open():
    '''
        List all open classes. Initially list 25, list 25 more each time
        more is clicked
    '''
    response.title = T('Open classes')
    response.subtitle = ''
    
    today = datetime.date.today()
    
    div_header = DIV(DIV(T("Location"), _class='location'),
                     DIV(T("Class type"), _class='classtype'),
                     DIV(T('Date'), _class='date'),
                     DIV(T("Start"), _class='start'),
                     DIV(T("End"), _class='end'),
                     DIV(T("Teacher"), _class='teacher'),
                     DIV(T("Level"), _class='level'),
                     _class='os-schedule_header')
    div_classes = DIV(BR(), div_header, _class='os-schedule_classes')
    
    query = (db.classes_open.ClassDate >= today)
    rows = db(query).select(db.classes_open.ClassDate,
                            db.classes.ALL,
                            left=[db.classes.on(db.classes_open.classes_id == \
                                                db.classes.id)],
                            orderby=db.classes_open.ClassDate|\
                                    db.classes.Starttime)
    
    for row in rows.render():
        clsID = row.classes.id
        date = row.classes_open.ClassDate
        date_formatted = date.strftime(session.date_format)
        result = status_get(clsID, date)
        status = result['status']
        status_marker = result['status_marker']
        
        location = max_string_length(row.classes.school_locations_id, 15)
        classtype = max_string_length(row.classes.school_classtypes_id, 24)
        date = date_formatted
        start = row.classes.Starttime
        end = row.classes.Endtime
        level = max_string_length(row.classes.school_levels_id, 12)
        teacher = max_string_length(row.classes.teachers_id, 20)
        
        manage = A(T('Manage'), 
                   _href=URL('manage', 
                             args=[clsID, date_formatted]))
        tools = DIV(_class='os-customer_tools')
        
        if auth.has_membership(group_id='Admins') or \
            auth.has_permission('read', 'schedule_manage'):
            tools.append(manage)
        
        div_class = DIV(status_marker,
                DIV(location, _class='location'),
                DIV(classtype, _class='classtype'),
                DIV(date, _class='date'),
                DIV(start, _class='start'),
                DIV(end, _class='end'),
                DIV(teacher, _class='teacher'),
                DIV(level, _class='level'),
                tools,
                _class='os-schedule_class os-wrapper',
                _id=clsID)
                
        div_classes.append(div_class)
        
    back = get_button('back', URL('schedule'))
    
    return dict(content=div_classes,
                back=back)
    
    
