# coding: utf8
# plugin (local modules) import
from plugin_solidform import SOLIDFORM
from gluon.contrib.user_agent_parser import mobilize 
from general_helpers import datestr_to_python
from general_helpers import highlight_submenu
from general_helpers import get_button
from general_helpers import get_submenu
from general_helpers import get_last_day_month
from general_helpers import get_months_list
from general_helpers import get_paused_subscriptions
from general_helpers import Memo_links

from form_remodeller import remodel_memo_form

from os_storage import uploads_available_space

# python general modules import
import csv
import cStringIO
import os
import openpyxl
import calendar
import codecs


# helper functions

@auth.requires_login()
def memo_change_visibility():
    reminderID = request.vars['reminderID']
    row = db.memos[reminderID]
    row.Visible = not row.Visible
    row.update_record()
    if session.memos_back == 'pinboard':
        redirect(URL('pinboard'))
    else:
        redirect(URL('memos'))
    
@auth.requires_login()
def memo_change_announcement():
    reminderID = request.vars['reminderID']
    row = db.memos[reminderID]
    row.Announcement = not row.Announcement
    row.update_record()
    redirect(URL('memos'))
    
@auth.requires_login()
def memo_change_priority():
    reminderID = request.vars['reminderID']
    row = db.memos[reminderID]
    if row.Priority == 1:
        row.Priority = 3
    elif row.Priority == 2:
        row.Priority = 1
    elif row.Priority == 3:
        row.Priority = 2
    row.update_record()
    redirect(URL('memos'))
    
@auth.requires_login()
def memo_get_return_url(customers_id):
    if session.memos_back == 'pinboard': # check if we're coming from the pinboard page which sets session.memos_back.
        return URL('default', 'pinboard')
    elif session.memos_back == 'memos': # check if we're coming from the memos page in the default controller
        return URL('default', 'memos')
    else:
        return URL('memos', vars={'cuID':customers_id})
    
@auth.requires_login()
def _edit_check_picture(form):
    if form.vars['Picture'] == '':
        row = db.customers[form.vars['id']]
        row.Picture = None
        row.ThumbSmall = None
        row.ThumbLarge = None
        row.update_record()
        
@auth.requires_login()
def get_customer_name_subtitle(customers_id):
    row = db.customers[customers_id]
    return row.FirstName + " " + row.LastName
    
@auth.requires_login()
def classes_check_registration(row):
    '''
        Check if a customer is already registered for a class.
        If not, return an add button to use in the grid.
    '''
    customers_id = request.vars['cuID']
    clsID = row.id
    check = db.classes_registration(customers_id=customers_id, classes_id=clsID)
    if check is None:
        return get_button('add', URL("classes", "register",
                                     vars={'cuID':customers_id, 
                                           'clsID':clsID}))
                                           #'back'="customers_classes")
    else:
        return ""
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('update', 'alternativepayments'))
def alternativepayment_repeat():
    apID = request.vars['apID']
    row = db.alternativepayments[apID]
    
    year = row.PaymentYear
    month = row.PaymentMonth
    
    if month == 12:
        month = 1
        year += 1
    else:
        month += 1
    db.alternativepayments.insert(customers_id=row.customers_id, 
                                  PaymentYear=year, 
                                  PaymentMonth=month,
                                  payment_categories_id=row.payment_categories_id,
        Amount=row.Amount, Description=row.Description)
    
    redirect(URL("payments",vars={'cuID':row.customers_id}))
    
    
@auth.requires_login()
def _generate_export_buttons():
    mailinglist = A(T("Mailing list"), _href=URL('export_excel', vars=dict(export='mailing_list'), user_signature=True), _class='textalign_left')
    paymentsummary = A(T("Payment overview"), _href=URL('export_questions', vars=dict(export='payment_summary'), user_signature=True), _target='blank', _class='textalign_left')
    collection = A(T("Collection"), _href=URL('export_questions', vars=dict(export='collection'), user_signature=True), _target='blank', _class='textalign_left')
    payment = A(T("Payment"), _href=URL('export_questions', vars=dict(export='payment')), user_signature=True, _target='blank', _class='textalign_left')
    active_customers = A(T("Active customers"), _href=URL('export_excel', vars=dict(export='customers_list')), _class='textalign_left')
    current_subscriptions = A(T("Current subscriptions"), _href=URL('export_excel', vars=dict(export='subscription_list')), _class='textalign_left')
    
    export = DIV(
        A(T('Export'), SPAN(_class='caret'), _href='#', _class='btn dropdown-toggle', **{'_data-toggle':"dropdown"}),
        UL(collection, payment, mailinglist, paymentsummary, active_customers, current_subscriptions, _class='dropdown-menu' ),
        _class='btn-group right', _id='customers_export')
    
    return export

@auth.requires_login()
def _edit_clear_old_thumbs(form):
    """
        This function cleans up generated thumbnails of the old picture when a new picture is uploaded.
        It does nothing when no new picture is uploaded.
    """
    customers_id = session.customers_edit_clear_thumbs_customers_id
    row = db.customers[customers_id]
    
    if not form.vars['Picture'] is None:
        if not row.ThumbSmall is None:
            os.remove(request.folder + 'uploads/' + row.ThumbSmall)
        if not row.ThumbLarge is None:
            os.remove(request.folder + 'uploads/' + row.ThumbLarge)
    
@auth.requires_login()
def _edit_remodel_form(form, picture, change_picture, label_id, customers_id):
    """
        This function takes the default form and makes a nice looking desktop browser friendly form out of it.
    """
    
    div_picture = DIV( TABLE(
        TR( TD(form.custom.label.ActiveMember), 
            TD(form.custom.widget.ActiveMember), 
            TD(form.custom.label.Newsletter), 
            TD(form.custom.widget.Newsletter),
            ),
        TR( TD(picture, _colspan=4)),
        TR( TD(change_picture, _colspan=3),
            TD(label_id + customers_id)
            ),
        ),
        _class='left')
    
    div_basic_info = DIV( TABLE( 
            TR( TD(form.custom.label.FirstName, _class='cu_label'),
                TD(form.custom.widget.FirstName),
                ),
            TR( TD(form.custom.label.LastName, _class='cu_label'),
                TD(form.custom.widget.LastName)
                ),
            TR( TD(form.custom.label.DateOfBirth, _class='cu_label'),
                TD(form.custom.widget.DateOfBirth),
                ),
            TR( TD(form.custom.label.Gender, _class='cu_label'),
                TD(form.custom.widget.Gender),
                ),
            TR( TD(form.custom.label.Email, _class='cu_label'),
                TD(form.custom.widget.Email),
                ),
            TR( TD(form.custom.label.Telnr, _class='cu_label'),
                TD(form.custom.widget.Telnr),
                ),
            TR( TD(form.custom.label.Mobnr, _class='cu_label'),
                TD(form.custom.widget.Mobnr),
                ),
            TR( TD(form.custom.label.Company, _class='cu_label'),
                TD(form.custom.widget.Company),
                ),
            ), _class='customers_edit_basic_info')
            
            
    if session.show_location:
        location_label = form.custom.label.school_locations_id
        location_widget = form.custom.widget.school_locations_id
    else:
        location_label = ''
        location_widget = ''
    div_address = DIV( 
        H3(T("Address and studio info")),
        TABLE(
            TR( TD(form.custom.label.Address),
                TD(form.custom.widget.Address),
                TD(form.custom.label.PostalCode),
                TD(form.custom.widget.PostalCode),
                ),
            TR( TD(form.custom.label.City),
                TD(form.custom.widget.City),
                TD(form.custom.label.Country),
                TD(form.custom.widget.Country),
                ),
            TR( TD(form.custom.label.school_levels_id),
                TD(form.custom.widget.school_levels_id),
                TD(form.custom.label.school_discovery_id),
                TD(form.custom.widget.school_discovery_id),
                ),
            TR( TD(form.custom.label.Keynr),
                TD(form.custom.widget.Keynr),
                TD(form.custom.label.school_languages_id),
                TD(form.custom.widget.school_languages_id),
                ),
            TR( TD(location_label),
                TD(location_widget),
                TD(),
                TD(),
                ),
            
            TR( TD(form.custom.label.Comments),
                TD(form.custom.widget.Comments, _colspan=3)
                ),
            ), 
        _class='customers_edit_address_info')
    
    return DIV(
        form.custom.begin,
        div_picture,
        div_basic_info,
        div_address,
        form.custom.submit,
        form.custom.end
    , _class='customers_edit_container')
    
@auth.requires_login()
def show_all_customers():
    session.cu_mstype = None
    session.cu_msmonth = None
    session.cu_msyear = datetime.date.today().year
    session.cu_name = ''
    session.cu_active = ''
    
    redirect(URL('index'))
    
@auth.requires_login()
def index_get_active_query(query, active):
    if active == "True" or active:
        query &= (db.customers.ActiveMember == active)
    elif active == "False" or active == False:
        query &= (db.customers.ActiveMember == False)
    return query
        
@auth.requires_login()
def index_get_subscription_query(query, mstype, msmonth, msyear):
    msstartdate = datetime.date(msyear, msmonth, 1)
    next_month = msstartdate.replace(day=28) + datetime.timedelta(days=4)  # this will never fail
    msenddate = next_month - datetime.timedelta(days=next_month.day)
    if mstype != '':
        query &= (db.customers_subscriptions.Startdate <= msstartdate) & \
                 ((db.customers_subscriptions.Enddate >= msenddate) | \
                  (db.customers_subscriptions.Enddate == None)) & \
            (db.customers_subscriptions.customers_id == db.customers.id) & \
            (db.customers_subscriptions.school_subscriptions_id==mstype)
            # inner join for search :
            #(db.customers_subscriptions.customers_id == db.customers.id) & \
    return query
    
@auth.requires_login()
def index_get_name_query(query, name):
    if name != None and name.isdigit() == False:
        # we're searching for a name
        search_name = '%' + name.strip() + '%'
        query &= ((db.customers.FirstName.like(search_name)) | 
            (db.customers.LastName.like(search_name)) | 
            (db.customers_payment_info.AccountNumber.like(search_name)))
    else:
        # we're searching by ID
        query &= (db.customers.id==name)
    return query
    
@auth.requires_login()
def subscriptions_get_link_paused(row):
    '''
        This function returns a pause/resume button to use in the subscriptions table for a customer
    '''
    today = datetime.date.today()
    query = (db.customers_subscriptions_paused.customers_subscriptions_id == row.id) & \
            (db.customers_subscriptions_paused.Startdate <= today) & \
            ((db.customers_subscriptions_paused.Enddate >= today) | \
             (db.customers_subscriptions_paused.Enddate == None))
    paused_subscription = db(query).select(db.customers_subscriptions_paused.ALL).first()

    if paused_subscription:
        paused_text = T("Resume")
        paused = True
        icon = "icon play icon-play"
        
        return A(SPAN(_class="buttontext button", _title=paused_text), 
            SPAN(_class=icon), paused_text, _class="button btn", \
            _href=URL('subscription_pause', args=[row.id], vars=dict(resume=paused_subscription.id)), _style='width:70px')
    else:
        paused_text = T("Pause")
        paused = False
        icon = "icon pause icon-pause"
        
        return A(SPAN(_class="buttontext button", _title=paused_text), 
            SPAN(_class=icon), paused_text, _class="button btn", \
            _href=URL('subscription_pause', args=[row.id]), _style='width:70px')
        
    
### helpers end ###

@mobilize
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'customers'))
def index():
    def get_subscription(row):
        '''
            Gets the subscription for a customer, if any, from the dictionary in a 
            safe way
        '''
        subscription = ''
        if row.id in cu_mem_dict:
            subscription = cu_mem_dict[row.id][0]
        
        return subscription
        
    def get_subscription_start(row):
        '''
            Gets the subscription for a customer, if any, from the dictionary in a 
            safe way
        '''
        start = ''
        if row.id in cu_mem_dict:
            start = cu_mem_dict[row.id][1]
        
        return start
        
    def get_subscription_end(row):
        '''
            Gets the subscription for a customer, if any, from the dictionary in a 
            safe way
        '''
        end = ''
        if row.id in cu_mem_dict:
            end = cu_mem_dict[row.id][2]
        
        return end
    
    # make sure we're redirected back to the list from the edit page
    session.customers_back = None 
        
    # session.export_year and session.exportmonth are used to pass a year/month 
    # from the alternative payments overview page export buttons to 
    # the export_questions function
    session.export_year = None
    session.export_month = None
    
    query = (db.customers.id>0) # generic show all query
    today = datetime.date.today()
    
    # these two variables are used to determine which subscription to show in the list
    msstartdate = None
    msenddate = None
    # search variables
    mstype = None
    msmonth = None
    msyear = today.year
    name = ''
    active = ''
    if 'name' in request.vars: # check whether a search filter is in use
        response.subtitle = T("Showing search results")
        if 'active' in request.vars:
            active = request.vars['active']
            query = index_get_active_query(query, active)
            session.cu_active = active
        if request.vars['name'] != '':
            name = request.vars['name']
            query = index_get_name_query(query, name)
            session.cu_name = name
        if request.vars['msmonth'] != '' and request.vars['msyear'] != '' and \
            not request.vars['msmonth'] is None and not request.vars['msyear'] is None and \
            request.vars['mstype'] != '' and not request.vars['mstype'] is None:
            msmonth = int(request.vars['msmonth'])
            msyear = int(request.vars['msyear'])
            msstartdate = datetime.date(msyear, msmonth, 1)
            next_month = msstartdate.replace(day=28) + datetime.timedelta(days=4)  # this will never fail
            msenddate = next_month - datetime.timedelta(days=next_month.day) # last day of the month
            
            mstype = request.vars['mstype']
            query = index_get_subscription_query(query, mstype, msmonth, msyear)
            session.cu_mstype = mstype
            session.cu_msmonth = msmonth
            session.cu_msyear = msyear
        else:
            msyear = today.year # set the default value for the year
    elif (session.cu_name != '' and session.cu_name != None) or \
        (session.cu_active != '' and session.cu_name != None) or \
        (session.cu_msyear != None and session.cu_msmonth != None and session.cu_mstype != None):
        # persistent search until cleared manually
        response.subtitle = T("Showing search results")
        if session.cu_name != '':
            name = session.cu_name # has to be set to be used as a default value for the widgets
            query = index_get_name_query(query, name)
        if session.cu_active != '':
            active = session.cu_active # has to be set to be used as a default value for the widgets
            query = index_get_active_query(query, active)
        if not session.cu_msyear is None and not session.cu_msmonth is None and not session.cu_mstype is None:
            msmonth = session.cu_msmonth # has to be set to be used as a default value for the widgets
            msyear = session.cu_msyear # has to be set to be used as a default value for the widgets
            mstype = session.cu_mstype # has to be set to be used as a default value for the widgets
            msstartdate = datetime.date(msyear, msmonth, 1)
            next_month = msstartdate.replace(day=28) + datetime.timedelta(days=4)  # this will never fail
            msenddate = next_month - datetime.timedelta(days=next_month.day)
            query = index_get_subscription_query(query, mstype, msmonth, msyear)
    else:
        response.subtitle = T("Showing all customers")
                
    #print query
    #print db(query)._select()
                
    months = [(1,T("January")),(2,T("February")),(3,T("March")),(4,T("April")),(5,T("May")),(6,T("June")),(7,T("July")),\
    (8,T("August")),(9,T("September")),(10,T("October")),(11,T("November")),(12,T("December"))]
    active_set = [(True, T("Active")),(False,T("Inactive"))]
    
    # create dictionary to lookup latest subscription values
    expression = not msstartdate is None \
                 and not msenddate is None \
                 and msstartdate != '' \
                 and msenddate != ''
    if expression:
        # show the subscription for the selected month
        result = db.executesql("""SELECT cu.id, 
                                         mst.name, 
                                         cs.startdate, 
                                         cs.enddate 
                                  FROM customers cu 
            LEFT JOIN customers_subscriptions cs ON cs.customers_id = cu.id
            LEFT JOIN
            (SELECT id, name FROM school_subscriptions) ssu
            ON ssu.id = cs.school_subscriptions_id
            WHERE cs.startdate <= %s AND 
                  (cs.enddate >= %s OR cs.enddate IS NULL)""",
            (msstartdate, msenddate))
    else:
        # show the latest subscription
        result = db.executesql("""SELECT cu.id, ssu.name, cs.startdate, cs.enddate FROM customers cu 
            LEFT JOIN customers_subscriptions cs ON cs.customers_id = cu.id
            LEFT JOIN 
            (SELECT customers_id, school_subscriptions_id, max(startdate) as startdate, enddate 
            FROM customers_subscriptions GROUP BY customers_id) chk 
            ON cu.id = chk.customers_id
            LEFT JOIN
            (SELECT id, name FROM school_subscriptions) ssu
            ON ssu.id = cs.school_subscriptions_id
            WHERE cs.startdate = chk.startdate 
                  OR cs.startdate IS NULL """)
    cu_mem_dict = dict()
    for record in result:
        if record[1] is None:
            subscription = ""
        else:
            subscription = record[1]
        if record[2] is None:
            start = ""
        else:
            start = record[2].strftime(session.date_format)
        if record[3] is None:
            end = ""
        else:
            end = record[3].strftime(session.date_format)
        cu_mem_dict[record[0]]= [subscription, start, end]
        
    if request.user_agent()['is_mobile']:
        form = SOLIDFORM.factory(
                [Field('active', requires=IS_IN_SET(active_set, zero=T("All")), default=active, label=T("")),
                Field('name', default=name, label=T(""))],
                submit_button=T('Search'),
                )
                
        form.element('select[name=active]')['_style']='width:87px'
        form.element('input[name=name]')['_style']='width:175px'
        form.element('input[name=name]')['_placeholder']='Search...'
    else:
        form = SOLIDFORM.factory(
            [Field('active', 
                   requires=IS_IN_SET(active_set, zero=T("All")), 
                   default=active, 
                   label=T("")),
            Field('name', 
                  default=name, 
                  label=T("")),
            Field('mstype', 
                  db.school_subscriptions, 
                  default=mstype, 
                  requires=IS_EMPTY_OR(IS_IN_DB(db,
                                                'school_subscriptions.id',
                                                '%(Name)s', 
                                                zero=T(""))), 
                  label=T("Subscription filter")),
            Field('msmonth', requires=IS_IN_SET(months, zero=T("")), default=msmonth, label=T("")),
            Field('msyear', 'integer', default=msyear, label=T(""))], 
            submit_button=T('Search'),
            )
            
        form.element('select[name=active]')['_style']='width:87px'
        form.element('input[name=name]')['_style']='width:175px'
        form.element('input[name=name]')['_placeholder']='Search...'
        form.element('select[name=mstype]')['_style']='width:130px'
        form.element('select[name=msmonth]')['_style']='width:110px'
        form.element('input[name=msyear]')['_style']='width:50px'
        form.element('input[name=msyear]')['_type']='number'
    form.add_button(T('Show all'), URL('show_all_customers', user_signature=True))
    
    response.title = T("Customers")
    db.customers.id.readable = False
    db.customers.ActiveMember.readable = False
    db.customers.Address.readable = False
    db.customers.PostalCode.readable = False
    db.customers.Country.readable = False
    db.customers.Email.readable = False
    db.customers.Newsletter.readable = False
    db.customers.school_discovery_id.readable = False
    db.customers.school_levels_id.readable = False
    db.customers.Keynr.readable = False
    db.customers.Telnr.readable = False
    db.customers.Mobnr.readable = False
    db.customers.Comments.readable = False
    db.customers.ThumbSmall.label = T("Customer")
    db.customers_subscriptions.id.readable=False
    db.customers_subscriptions.customers_id.readable=False
    
    if request.user_agent()['is_mobile']:
        # mobile grid
        fields = [  db.customers.ActiveMember, 
                    db.customers.ThumbSmall, 
                    db.customers.FirstName, 
                    db.customers.LastName ]
        links = [dict(header=T('Subscription'), 
                      body=lambda row: cu_mem_dict[row.id][0])]
    else:
        # normal grid
        fields = [  db.customers.ActiveMember, 
                    db.customers.ThumbSmall, 
                    db.customers.FirstName, 
                    db.customers.LastName, 
                    db.customers.Gender, 
                    db.customers.DateOfBirth, 
                    db.customers.City,
                    db.customers.Email ]
        links = [dict(header=T('Subscription'), body=get_subscription),
            dict(header=T('Start'), body=get_subscription_start),
            dict(header=T('End'), body=get_subscription_end),
            lambda row: A(SPAN(_class="buttontext button", _title=T("Mail")), 
                           SPAN(_class="icon envelope icon-envelope"), " ", \
                           _class="button btn", 
                           _href='mailto:' + row.Email, 
                           _title=T("Mail customer"))]
    
    if session.show_location:
        if not request.user_agent()['is_mobile']:
            fields.append(db.customers.school_locations_id)
    
    headers = { 'customers.FirstName':T("First name"), 'customers.LastName':T("Last name") }

    grid = SQLFORM.grid(query, fields=fields, headers=headers, links=links, field_id=db.customers.id, ui=session.grid_ui,
        create=False,
        details=False, 
        searchable=False, 
        csv=False,
        paginate=15,
        left=[db.customers_payment_info.on(\
                                db.customers_payment_info.customers_id==\
                                db.customers.id)],
        orderby=~db.customers.id)
    
    # finding the back button, hack hack hack... but hey.. it works...
    if "Back" in unicode(grid.element('a', _class='button btn')):
        back = A(SPAN(_class="buttontext button", _title=T("Back")), SPAN(_class="icon leftarrow icon-arrow-left"), T("Back"), _class="button btn", \
            _href=URL('index'))
        grid.element('a', _class='button btn', replace=back)
    # remove text from delete button to save screen space
    grid.elements('span[title=Delete]', replace=None)
    
    add = A(SPAN(_class="buttontext button", _title=T("Add")), SPAN(_class="icon plus icon-plus"), " " + T("Add"), \
                _class="button btn", _href=URL('edit'), _title=T("Add a new customer"))
                
    if 'edit' not in request.args:
        console = grid.element('.web2py_console')
        console.insert(0, add)
      
    # use solid form pages instead of the default ones for a more compact page
    if 'edit' in request.args:
        customers_id = request.args[2]
        redirect(URL('edit', args=customers_id))
    else:
        right_sidebar_enabled=False
        sidebar_links=None
        
    sidebar_title = H3(T(""))
    export = _generate_export_buttons()
    mobile_export_title = H3(T("Export"))
    
    # align form fields
    style = STYLE(""" vertical-align:text-bottom; """)
    
    
    return dict(grid=grid, form=DIV(style, form), mobile_export_title=mobile_export_title, 
        right_sidebar_enabled=right_sidebar_enabled, sidebar_title=sidebar_title, sidebar_links=sidebar_links,
        export=export)
        
@auth.requires_login()
def _check_active(form):
    customers_id = form.vars['id']
    # Remove customer from classes and waitinglist if not active anymore
    if not form.vars['ActiveMember']:
        db(db.classes_registration.customers_id == customers_id).delete()
        db(db.classes_waitinglist.customers_id == customers_id).delete()
    
@mobilize
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('update', 'customers') or \
                auth.has_permission('create', 'customers'))
def edit():
    today = datetime.date.today()
    
    if not session.show_location:
        db.customers.school_locations_id.readable = False
        db.customers.school_locations_id.writable = False
        
    db.customers.Picture.readable=False
    db.customers.Picture.writable=False
    
    picture_class = 'customer_image_edit'
    picture = DIV(IMG(_src=URL('static', 'images/person.png'), _alt="Person picture"),
                      _class=picture_class)
    
    if not request.args:
        # new customer
        response.title = T("New customer")
        response.subtitle = T("")
        change_picture = ''
        customers_id = ''
        label_id = ''
        change_picture = ''
        menu = ''
        left_sidebar_enabled = False
        
        crud.messages.submit_button = T('Create')
        crud.messages.record_created = T('Created customer')
        
        form = crud.create(db.customers,
                           next='edit/[id]')
    else:
        # update customer
        customers_id = request.args[0]
        db.customers.id.label = T('Customer ID')
        
        row = db.customers[customers_id]
        response.title = T("Edit customer")
        response.subtitle = row.FirstName + " " + row.LastName
        menu = customers_get_menu(customers_id, 'general')
        left_sidebar_enabled = True
        label_id = T('ID') + ": "
        
        if row.Picture is None:
            change_picture_title = T("Add picture")
            picture = DIV(IMG(_src=URL('static', 'images/person.png'), 
                              _alt=row.FirstName + " " + row.LastName),
                              _class=picture_class)
        else:
            picture = DIV(IMG(_src=URL('default', 'download', 
                                       args=row.ThumbLarge), 
                              _alt=row.FirstName + " " + row.LastName),
                              _class=picture_class)
            change_picture_title = T("Change picture")
        change_picture = A(change_picture_title, 
                           _href=URL('edit_picture', args=[customers_id]))
        
        crud.settings.update_onaccept.customers.append(_check_active)
        crud.messages.submit_button = T('Update')
        crud.messages.record_updated = T('Updated ' + \
                                         row.FirstName + ' ' + row.LastName)
        
        form = crud.update(db.customers, customers_id)
        
    # remodel form for desktop users:
    if not request.user_agent()['is_mobile']:
        form = _edit_remodel_form(form, picture, change_picture, label_id, customers_id)
        
    back = edit_get_back()
    
    if request.args:
        # check if we're creating a new customer, if so, do nothing with alternative payments.
        row = db.customers(customers_id)
        email = row.Email
        mail_customer = A(SPAN(_class="buttontext button", 
                               _title=T('Send mail')), 
                          SPAN(_class='icon-envelope'), 
                          " " + T('Send mail'), 
                          _class="button btn", \
                          _href='mailto:' + email, 
                          _style='width:155px')
    else:
        mail_customer = ''
    
    tools = DIV(mail_customer, _class='customers_edit_tools')
    
    return dict(content=form, 
                back=back, 
                tools=tools, 
                menu=menu,
                left_sidebar_enabled=left_sidebar_enabled)
        
@auth.requires_login()
def edit_picture():
    """
        This function is called to show a page to change a customers' picture
        After successfully changing the picture, the user is redirected to the edit page.
    """
    response.title = T("Picture")
    response.view = 'general/only_content.html'
    customers_id = request.args[0]
    row = db.customers[customers_id]
    response.subtitle = row.FirstName + " " + row.LastName
    
    session.customers_edit_clear_thumbs_customers_id = customers_id
    
    for field in db.customers:
        field.writable = False
        field.readable = False
        
    db.customers.Picture.readable = True
    db.customers.Picture.writable = True
    db.customers.Picture.label = T("Picture")
    
    space = uploads_available_space(request.folder)
    if space['available'] > 1 or not row.Picture is None:
        crud = Crud(db)
        crud.settings.label_separator = ''
        crud.settings.update_deletable = False
        crud.settings.update_onvalidation.customers.append(_edit_clear_old_thumbs)
        crud.settings.update_onaccept.customers.append(_edit_check_picture)
        
        crud.messages.submit_button = T('Change picture')
        crud.messages.record_updated = T('Picture changed for') + " " + row.FirstName + " " + row.LastName
        
        form = crud.update(db.customers, customers_id, next='edit/[id]')
    
        msg_list = UL(T("Maximum dimensions: 1280*1280 pixels"), 
                      T("Maximum filesize: 500Kb"))
        
        content = DIV(BR(), msg_list, form)
    else:
        content = space['full_message']
    
    back = get_button('back', URL('edit', args=[customers_id]))
    
    return dict(content=content, back=back)
    
@auth.requires_login()
def edit_get_back():
    '''
        This function looks at the session variable
            session.customers_back
        to determine where the back button for customers edit pages should link to.
    '''
    if session.customers_back == 'keys':
        # check if we came from the edit button on the keys list page in school properties
        url = URL('school_properties', 'list_keys')
    elif session.customers_back == 'pinboard':
        # check if the birthday notification or a memo on the pinboard
        # referred to this page 
        url = URL('default', 'pinboard')
    elif session.customers_back == 'subscriptions_new' or \
       session.customers_back == 'subscriptions_stopped' or \
       session.customers_back == 'subscriptions_paused':
        # check if the we came from the default/subscriptions page
        url = URL('default', session.customers_back)
    elif session.customers_back == 'trialclasses':
        # check if the we came from the default/trialclasses page
        url = URL('default', 'trialclasses')
    elif session.customers_back == 'dropinclasses':
        # check if the we came from the default/dropinclasses page
        url = URL('default', 'dropinclasses')
    elif session.customers_back == 'classcards':
        # check if the we came from the default/classcards page
        url = URL('default', 'classcards')
    elif session.customers_back == 'alternativepayments':
        # check if the we came from the default/alternativepayments page
        url = URL('default', 'alternative_payments')
    elif session.customers_back == 'overduepayments':
        # check if the we came from the default/alternativepayments page
        url = URL('default', 'overdue_payments')
    elif session.customers_back == 'classes_manage':
        # check if the classes/manage page referred to this page
        url = URL('classes', 'manage')
    elif session.customers_back == 'workshops_attendance_add':
        # check if the workshops manage page referred to this page
        url = URL('workshops', 'attendance_add')
    elif session.customers_back == 'workshops_attendance':
        # check if the workshops attendance page referred to this page
        url = URL('workshops', 'attendance')
    elif session.customers_back == 'workshops_activity_attendance':
        # check if the workshops manage page referred to this page
        url = URL('workshops', 'activity_attendance')
    elif session.customers_back == 'workshops_activity_attendance_add':
        # check if the workshops manage page referred to this page
        url = URL('workshops', 'activity_attendance_add')
    else:
        url = URL('index')
        
    return get_button('back', url)
    
@auth.requires_login()
def _generate_breadcrumb(customers_id, function=None):
    record = db.customers[customers_id]
    urls = [A(T('Customers'), _href=URL('customers', 'index'))]
    urls.append(A(record.FirstName + " " + record.LastName, _href=URL('index', args=['edit', 'customers', customers_id], user_signature=True)))
    #urls.append(function)
    return DIV(XML(' > '.join(str(u) for u in urls)), BR(), BR())
        
@auth.requires_login()
def customers_get_menu(customers_id, page=None):
    pages = []
    pages.append(['general', T('General'), URL("customers","edit",
                                               args=[customers_id])])
    if auth.has_membership(group_id='admins') or \
       auth.has_permission('read', 'customers-subscriptions'):
        pages.append(['subscriptions', T("Subscriptions"), 
                      URL("customers","subscriptions",
                          vars={'cuID':customers_id})])
    if auth.has_membership(group_id='admins') or \
       auth.has_permission('read', 'customers_classcards'):
        pages.append(['classcards', T("Class cards"), 
                      URL("customers","classcards", vars={'cuID':customers_id})])
    if auth.has_membership(group_id='admins') or \
       auth.has_permission('read', 'classes_registration'):
        pages.append(['classes', T("Classes"), 
                      URL("customers","classes", vars={'cuID':customers_id})])
    if auth.has_membership(group_id='admins') or \
       auth.has_permission('read', 'workshops'):
        pages.append(['workshops', T("Workshops"), 
                      URL("customers","workshops", vars={'cuID':customers_id})])
    if auth.has_membership(group_id='admins') or \
       auth.has_permission('read', 'customers_payments'):
        pages.append(['payments', T("Payment info"), 
                      URL("customers","payments", vars={'cuID':customers_id})])
    if auth.has_membership(group_id='admins') or \
       auth.has_permission('read', 'customers_documents'):
        pages.append(['documents', T("Documents"), 
                      URL("customers","documents", vars={'cuID':customers_id})])
    if auth.has_membership(group_id='admins') or \
       auth.has_permission('read', 'memos'):
        pages.append(['memos', T("Memos"), 
                      URL("customers","memos", vars={'cuID':customers_id})])
    
    return get_submenu(pages, page)
    
@auth.requires_login()
def classcard_count_classes(row):
    total = row.Classes
    query = (db.classes_attendance.customers_classcards_id == row.id)
    used = db(query).count()
    return A(T("Classes taken") + ": " + \
                unicode(used) + "/" + unicode(total), 
             _href=URL('classcard_classes', vars=dict(ccID=row.id)))
    
@auth.requires_login()
def export_questions():
    response.title = T("Export")
    export_type = request.vars['export']
    today = datetime.date.today()
    
    # pick up values from alternative payments overview page
    if session.export_year != None:
        year = int(session.export_year)
        session.export_year = None
    else:
        year = today.year
    if session.export_month != None:
        month = int(session.export_month)
        session.export_month = None
    else:
        month = today.month
    
    if export_type == 'payment_summary': 
        response.subtitle = T("Payment overview preferences")
        form = FORM(T('Choose Year'), BR(),
                INPUT(_name='year', requires=IS_NOT_EMPTY(), _type='number', _value=year, _style='width:75px',\
                    _min=datetime.date.today().year-25, _max=datetime.date.today().year+25),
                "", BR(),
                INPUT(_type='submit', _value=T('Export')))
    elif export_type == 'collection' or 'payment':
        if export_type == 'collection':
            response.subtitle = T("Collection")
            categories = [(0, T("Default"))]
            query = (db.payment_categories.CategoryType==0)
            rows = db(query).select(db.payment_categories.id, 
                                    db.payment_categories.Name, 
                                    orderby=db.payment_categories.Name)
        else:
            response.subtitle = T("Payment")
            categories = []
            query = (db.payment_categories.CategoryType==1)
            rows = db(query).select(db.payment_categories.id, 
                                    db.payment_categories.Name, 
                                    orderby=db.payment_categories.Name)
        for row in rows:
            categories.append((row.id, row.Name))
            
        if 'collection_type' in request.vars:
            col_type = request.vars['collection_type']
        else:
            col_type = 'csv'
        if 'month' in request.vars:
            month = int(request.vars['month'])
        if 'year' in request.vars:
            year = int(request.vars['year'])
        if 'exdate' in request.vars and request.vars['exdate'] != "":
            execution_date = datestr_to_python(session.date_format, request.vars['exdate'])
        else:
            execution_date = datetime.date(year, month, 1)
        if 'description' in request.vars:
            description = request.vars['description']
        else:
            description =  ""
        if 'category' in request.vars:
            paycatID = request.vars['category']
        else:
            paycatID = 0 # Default category
        if 'location' in request.vars:
            locationID = request.vars['location']
        else:
            locationID = 0 # Default location
        if 'include_0' in request.vars:
            include_0 = request.vars['include_0']
        else:
            include_0 = ''
            
        collection_date = datetime.date(year, month, 1)
            
        types = [('csv',T('csv (Excel compatible)'))]
        months = [(1,T("January")),(2,T("February")),(3,T("March")),(4,T("April")),(5,T("May")),(6,T("June")),(7,T("July")),\
            (8,T("August")),(9,T("September")),(10,T("October")),(11,T("November")),(12,T("December"))]
            
        if session.show_location:
            rows = db().select(db.school_locations.id, 
                               db.school_locations.Name, 
                               orderby=db.school_locations.Name)
            locations = [(0, T("All"))]
            for row in rows:
                locations.append((row.id, row.Name))
                
            form = SOLIDFORM.factory(
                Field('collection_type', 
                      requires=IS_IN_SET(types, zero=T("Please select...")), 
                      default=col_type, 
                      label=T("Export type")), 
                [Field('month', 
                        requires=IS_IN_SET(months), 
                        default=month, 
                        label=T("Month")),
                 Field('year', 'integer', 
                       requires=IS_INT_IN_RANGE(1000,9999, 
                                    error_message=T("Please enter a year.")), 
                       default=year, 
                       label=T(""))],
                Field('exdate', 'date', 
                      required=True, 
                      requires=IS_DATE(format=session.date_format), 
                      default=execution_date, 
                      label=T("Execution date")),
                Field('description', length=32, 
                      default=description, 
                      label=T("Default description")),
                Field('category', 
                      requires=IS_IN_SET(categories, 
                                         zero=T("Please select...")), 
                      default=paycatID, 
                      label=T("Category")),
                Field('location', 
                      requires=IS_IN_SET(locations, zero=T("Please select...")), 
                      default=locationID, 
                      label=T("Location")),
                Field('include_0', 'boolean', 
                      default=include_0, 
                      label=T("Include rows with amount 0")),
                submit_button=T("Export"))
        else:
            form = SOLIDFORM.factory(
                Field('collection_type', 
                      requires=IS_IN_SET(types, zero=T("Please select...")), 
                      default=col_type, 
                      label=T("Export type")), 
                [Field('month', 
                       requires=IS_IN_SET(months), 
                       default=month, 
                       label=T("Month")),
                 Field('year', 'integer', 
                       requires=IS_INT_IN_RANGE(1000,9999, 
                                error_message=T("Please enter a year.")), 
                       default=year, 
                       label=T(""))],
                Field('exdate', 'date', required=True, 
                      requires=IS_DATE(format=session.date_format), 
                      default=execution_date, 
                      label=T("Execution date")),
                Field('description', length=32, 
                      default=description, 
                      label=T("Default description on bank statement")),
                Field('category', 
                      requires=IS_IN_SET(categories, 
                                         zero=T("Please select...")), 
                      default=paycatID, 
                      label=T("Category")),
                Field('include_0', 'boolean', 
                      default=include_0, 
                      label=T("Include rows with amount 0")),
                submit_button=T("Export"))
                
        
        form.element('input[name=year]')['_style']='width:60px'
        form.element('input[name=year]')['_type']='number'
    
    if form.accepts(request.vars,session):
        if export_type == 'payment_summary':
            redirect(URL('export_excel', vars=dict(export=export_type, year=year), user_signature=True))
        elif export_type == 'collection':
            # check if csv and redirect accordingly
            if request.vars['collection_type'] == 'csv':
                redirect(URL('export_csv', vars=dict(export="collection", collection_date=collection_date.strftime(session.date_format), execution_date=execution_date.strftime(session.date_format),
                    description=description, paycatID=paycatID, locationID=locationID, include_0=include_0)))
        elif export_type == 'payment':
            if request.vars['collection_type'] == 'csv':
                redirect(URL('export_csv', vars=dict(export="payment", collection_date=collection_date.strftime(session.date_format), execution_date=execution_date.strftime(session.date_format),
                    description=description, paycatID=paycatID, locationID=locationID, include_0=include_0)))
            
        
    close = A(SPAN(_class="buttontext button", _title=T("Close")), SPAN(_class="icon remove icon-remove"), T("Close"), _class="button btn", \
        _href="JavaScript:window.close()")
        
    return dict(form=form, close=close)
    
@auth.requires_login()
def export_csv():
    """
        Valid export types are: collection, payment
    """
    export_type=request.vars['export']
    collection_date=datestr_to_python(session.date_format, request.vars['collection_date'])
    execution_date=datestr_to_python(session.date_format, request.vars['execution_date'])
    description=request.vars['description']
    paycatID=request.vars['paycatID']
    locationID=request.vars['locationID']
    include_0=request.vars['include_0']
    
    
    error = False
    message = ""
    
    description = description or ""
    if (db.sys_properties(Property='Currency')):
        currency = db.sys_properties(Property='Currency').PropertyValue
    else:
        currency = "EUR"
    
    if export_type.lower() == "collection":
        lastdaythismonth = get_last_day_month(collection_date)
        firstdaythismonth = datetime.date(collection_date.year,collection_date.month,1)
        
        fname = T("Collection") + "_" + unicode(collection_date.year) + "_" + unicode(collection_date.month) + ".csv"
        filename = os.path.join(request.folder,('temp'),fname)
        stream = cStringIO.StringIO()
        writer = csv.writer(stream)
        
        # write the header
        writer.writerow(['customers_id'] + \
                        ['Account holder'] + \
                        ['Bank Location'] + \
                        ['Currency'] + \
                        ['Amount'] + \
                        ['Account number'] + \
                        ['BIC'] + \
                        ['Mandate Signature Date'] + \
                        ['Description'] + \
                        ['Execution Date'] + \
                        ['Location'])
                        
        year = collection_date.year
        month = collection_date.month
        
        if paycatID == "0":
            # Export default
            query = (db.customers_subscriptions.Startdate <= lastdaythismonth) & \
                    ((db.customers_subscriptions.Enddate >= firstdaythismonth) | \
                     (db.customers_subscriptions.Enddate == None)) & \
                    (db.customers_payment_info.Payment == 3) # 3 = Direct Debit
        
            if ( session.show_location and 
                (
                   locationID != '' and \
                   locationID != '0' and \
                   not locationID is None 
                )
               ):
                query &= (db.customers.school_locations_id==locationID)
            
            rows = db(query).select(db.customers_subscriptions.id, 
                                    db.customers_subscriptions.customers_id, 
                                    db.customers_subscriptions.school_subscriptions_id, 
                                    db.school_subscriptions.MonthlyFee, 
                                    db.customers.FirstName, 
                                    db.customers.LastName, 
                                    db.school_locations.Name,
                                    db.customers_payment_info.ALL,
                left= [ db.customers.on(db.customers_subscriptions.customers_id==\
                                        db.customers.id),
                        db.customers_payment_info.on(\
                                        db.customers_payment_info.customers_id==\
                                        db.customers_subscriptions.customers_id),
                        db.school_subscriptions.on(\
                                db.customers_subscriptions.school_subscriptions_id==\
                                db.school_subscriptions.id),
                        db.school_locations.on(db.customers.school_locations_id==\
                                               db.school_locations.id)])
            
            paused_subscriptions = get_paused_subscriptions(firstdaythismonth)
            
            for row in rows:
                if row.customers_subscriptions.id in paused_subscriptions:
                    continue # skip paused subscriptions
                    
                customers_id = row.customers_subscriptions.customers_id
                mstypeID = row.customers_subscriptions.school_subscriptions_id
                
                if (    row.customers_payment_info.AccountNumber is None or 
                        row.customers_payment_info.AccountNumber == "" ):
                    error = True
                    message += T("Please add a bank account number for") + \
                               " " + \
                                  row.customers.FirstName + " " + \
                               row.customers.LastName + "\n"
                elif ( row.customers_payment_info.AccountHolder is None or 
                        row.customers_payment_info.AccountHolder == "" ):
                    error = True
                    message += T("Please add a bank account holder for") + \
                               " " + \
                               row.customers.FirstName + " " + \
                               row.customers.LastName + "\n"
                else:
                    # We're good, continue export.
                    location = row.school_locations.Name
                    holder = row.customers_payment_info.AccountHolder
                    banklocation = row.customers_payment_info.BankLocation
                    accountnr = row.customers_payment_info.AccountNumber.strip()
                    mandate_signature_date = \
                        row.customers_payment_info.MandateSignatureDate
                    desc = description
                    if not row.customers_payment_info.BIC is None:
                        bic = row.customers_payment_info.BIC.strip()
                    else:
                        bic = ""
                        
                    # Check for alternative paymetns in default category
                    altpay = db.alternativepayments(customers_id=customers_id, 
                                                    PaymentYear=year, \
                                                    PaymentMonth=month, 
                                                    payment_categories_id=None)
                    if altpay:
                        if altpay.Amount == 0 and include_0 == '':
                            ''' When there's an alternative payment set to 0, 
                                skip the customer this run. '''
                            continue
                        else: # Write alternative payment
                            amount = altpay.Amount
                            if ( not altpay.Description is None or 
                                      altpay.Description != "") :
                                desc = altpay.Description
                    else: # no alternative payment
                        if ( row.school_subscriptions.MonthlyFee == 0 and 
                             include_0 == '' ):
                            continue
                        else: # nothing unusual, write the regular fees
                            amount = row.school_subscriptions.MonthlyFee
    
                    writer.writerow([customers_id] + \
                        [holder] + \
                        [banklocation] + \
                        [currency] + \
                        [amount] + \
                        [accountnr] + \
                        [bic] + \
                        [mandate_signature_date] + \
                        [desc] + \
                        [execution_date] + \
                        [location])
                            
        else:
            # Export by category
            query = (db.alternativepayments.payment_categories_id == paycatID) & \
                    (db.alternativepayments.PaymentYear == year) & \
                    (db.alternativepayments.PaymentMonth == month) #& \
                    #(db.customers_payment_info.Payment == 3) # 3 = Direct debit
                    
            if ( session.show_location and 
                (
                   locationID != '' and \
                   locationID != '0' and \
                   not locationID is None 
                )
               ):
                query &= (db.customers.school_locations_id==locationID)
                    
            left = [ db.customers.on(db.customers.id == \
                                     db.alternativepayments.customers_id),
                     db.school_locations.on(db.school_locations.id == \
                                            db.customers.school_locations_id),
                     db.customers_payment_info.on(\
                                       db.customers_payment_info.customers_id == \
                                       db.alternativepayments.customers_id) ]
                    
            rows = db(query).select(db.alternativepayments.Amount,
                                    db.alternativepayments.Description,
                                    db.alternativepayments.customers_id,
                                    db.customers.FirstName, 
                                    db.customers.LastName, 
                                    db.school_locations.Name,
                                    db.customers_payment_info.ALL,
                             left=left)
            
            for row in rows:
                if (    row.customers_payment_info.AccountNumber is None or 
                        row.customers_payment_info.AccountNumber == "" ):
                    error = True
                    message += T("Please add a bank account number for") + \
                               " " + \
                                  row.customers.FirstName + " " + \
                               row.customers.LastName + "\n"
                elif ( row.customers_payment_info.AccountHolder is None or 
                        row.customers_payment_info.AccountHolder == "" ):
                    error = True
                    message += T("Please add a bank account holder for") + \
                               " " + \
                               row.customers.FirstName + " " + \
                               row.customers.LastName + "\n"
                else:
                    customers_id = row.alternativepayments.customers_id
                    location = row.school_locations.Name
                    holder = row.customers_payment_info.AccountHolder
                    banklocation = row.customers_payment_info.BankLocation
                    accountnr = row.customers_payment_info.AccountNumber.strip()
                    if not row.customers_payment_info.BIC is None:
                        bic = row.customers_payment_info.BIC.strip()
                    else:
                        bic = ""
                    mandate_signature_date = \
                        row.customers_payment_info.MandateSignatureDate
                    amount = row.alternativepayments.Amount
                    if ( not row.alternativepayments.Description is None and 
                              row.alternativepayments.Description != "") :
                        desc = row.alternativepayments.Description
                    else:
                        desc = description
                        
                    writer.writerow([customers_id] + \
                        [holder] + \
                        [banklocation] + \
                        [currency] + \
                        [amount] + \
                        [accountnr] + \
                        [bic] + \
                        [mandate_signature_date] + \
                        [desc] + \
                        [execution_date] + \
                        [location])
            
    elif export_type.lower() == "payment":
        year = collection_date.year
        month = collection_date.month
        # check if we need to filter by location
        if not session.show_location or \
           locationID == '' or \
           locationID == '0' or \
           locationID == None:
            query = (db.alternativepayments.PaymentYear == year) & \
                    (db.alternativepayments.PaymentMonth == month) & \
                    (db.alternativepayments.payment_categories_id == paycatID)
        else:
            query = (db.alternativepayments.PaymentYear == year) & \
                    (db.alternativepayments.PaymentMonth == month) & \
                    (db.alternativepayments.payment_categories_id == paycatID) & \
                    (db.customers.school_locations_id == locationID)
        rows = db(query).select(db.alternativepayments.customers_id, 
                                db.alternativepayments.Amount, 
                                db.alternativepayments.Description,
                                db.customers.id, 
                                db.customers.FirstName, 
                                db.customers.LastName, 
                                db.school_locations.Name,
                                db.customers_payment_info.ALL,
            left = [db.customers.on(db.alternativepayments.customers_id==\
                                    db.customers.id),
                    db.customers_payment_info.on(\
                                      db.alternativepayments.customers_id==\
                                      db.customers_payment_info.customers_id),
                    db.school_locations.on(db.customers.school_locations_id==\
                                           db.school_locations.id)])
        
        fname = T("Payment") + "_" + \
                unicode(collection_date.year) + "_" + \
                unicode(collection_date.month) + ".csv"
        filename = os.path.join(request.folder,('temp'),fname)
        stream = cStringIO.StringIO()
        writer = csv.writer(stream)
        
        # write the header!
        writer.writerow(['customers_id'] + \
                        ['Account holder'] + \
                        ['Bank Location'] + \
                        ['Currency'] + \
                        ['Amount'] + \
                        ['Account number'] + \
                        ['BIC'] + \
                        ['Mandate Signature Date'] + \
                        ['Description'] + \
                        ['Execution Date'] + \
                        ['Location'])
        
        for row in rows:
            # get the objects and other vars we need
            location = row.school_locations.Name
            customers_id = row.alternativepayments.customers_id
            holder = row.customers_payment_info.AccountHolder
            banklocation = row.customers_payment_info.BankLocation
            accountnr = row.customers_payment_info.AccountNumber
            mandate_signature_date = \
                row.customers_payment_info.MandateSignatureDate
            if row.customers_payment_info.BIC != None:
                bic = row.customers_payment_info.BIC.strip()
            else:
                bic = ""
            if not row.alternativepayments.Description is None and \
               row.alternativepayments.Description != "":
                desc = row.alternativepayments.Description
            else:
                desc = description
            
            if row.customers_payment_info.AccountNumber is None or \
               row.customers_payment_info.AccountNumber == "":
                error = True
                message += T("Please add a bank account number for") + " " + \
                           row.customers.FirstName + " " + \
                           row.customers.LastName + "\n"
            elif row.customers_payment_info.AccountHolder is None or \
                  row.customers_payment_info.AccountHolder == "":
                error = True
                message += T("Please add a bank account holder for") + " " + \
                            row.customers.FirstName + " " + \
                            row.customers.LastName + "\n"
            else:
                if row.alternativepayments.Amount == 0 and include_0 == '':
                    continue
                else:
                    accountnr = accountnr.strip()
                    writer.writerow([customers_id] + \
                                    [holder] + \
                                    [banklocation] + \
                                    [currency] + \
                                    [row.alternativepayments.Amount] + \
                                    [accountnr] + \
                                    [bic] + \
                                    [mandate_signature_date] + \
                                    [desc] + \
                                    [execution_date] + \
                                    [location])
    
    #print "Buffer end: \n"
    #print stream.getvalue()
    if error:
        stream.truncate(0) # empty all written rows
        stream.write("Errors were encountered while exporting:\n\n")
        stream.write(message) # write errors instead
        fname = 'export_errors.txt'
        response.headers['Content-Type']='text;charset=utf-8'
    else:
        response.headers['Content-Type']='application/vnd.ms-excel'
        
    response.headers['Content-disposition']='attachment; filename=' + fname
    return stream.getvalue()
    
@auth.requires_login()
def export_excel():
    """
    valid export_types include: payment_summary, mailinglist, attendance_list, customers_list, subscription_list
    """
    export_type = request.vars['export']
    def write_cell(r, c, data, fontsize, bold=False, wrap_text=True):
        cell = ws.cell(row = r, column = c)
        cell.value = ""
        cell.style.font.bold = bold
        cell.style.font.size = fontsize
        cell.style.alignment.wrap_text = wrap_text
        cell.style.alignment.vertical = "top"
        cell.style.alignment.horizontal = "left"
        cell.value = data
        
    if export_type.lower() == "customers_list" or export_type.lower() == "subscription_list":
        # create dictionary to lookup latest subscription values
        if export_type.lower() == 'customers_list':
            result = db.executesql("""SELECT cu.id, mst.name, cs.startdate, cs.enddate FROM customers cu 
                LEFT JOIN customers_subscriptions cs ON mem.customers_id = cu.id
                LEFT JOIN 
                (SELECT customers_id, school_subscriptions_id, max(startdate) as startdate, enddate 
                FROM customers_subscriptions GROUP BY customers_id) chk 
                ON cu.id = chk.customers_id
                LEFT JOIN
                (SELECT id, name FROM school_subscriptions) ssu
                ON ssu.id = cs.school_subscriptions_id
                WHERE cs.startdate = chk.startdate OR cs.startdate IS NULL """)
                
            fname = T("Customers list") + '.xlsx'
            title = 'Customers list'
        elif export_type.lower() == 'subscription_list':
            today = datetime.date.today()
            result = db.executesql("""SELECT cs.customers_id, mst.name, cs.startdate, cs.enddate FROM customers_subscriptions cs
                LEFT JOIN
                (SELECT id, name FROM school_subscriptions) ssu
                ON ssu.id = cs.school_subscriptions_id
                WHERE cs.startdate <= %s AND (cs.enddate > %s OR cs.enddate IS NULL) 
                GROUP BY cs.customers_id """, (today, today))
            fname = T("Subscription list") + '.xlsx'
            title = 'subscription list'
        cu_mem_dict = dict()
        for record in result:
            subscription = record[1] or ""
            start = record[2] or ""
            end = record[3] or ""
            cu_mem_dict[record[0]] = [subscription, start, end]
        
        bd_dict = dict()
        types_dict = {1:"Cash",2:"Wire transfer",3:"Direct debit"}
        rows = db().select(db.customers_payment_info.ALL)
        for row in rows:
            payment = types_dict.get(row.Payment, None)
            bd_dict[row.customers_id] = [ payment, row.AccountNumber, row.AccountHolder, row.BankName, row.BankLocation ]
        
        # Create the workbook
        wb = openpyxl.workbook.Workbook()
        ws = wb.worksheets[0]
        ws.title = title
        headers = [ "First name", 
                    "Last name", 
                    "Date of birth",
                    "Gender",
                    "Address",
                    "Postal code",
                    "City",
                    "Country",
                    "Email",
                    "Newsletter",
                    "Telephone",
                    "Mobile",
                    "Key",
                    "Location",
                    "Subscription",
                    "Startdate",
                    "Enddate",
                    "Payment",
                    "AccountNR",
                    "AccountHolder",
                    "BankName",
                    "BankLocation",
                    "Comments"]
        column = 0
        for header in headers:
            cell = ws.cell(row = 0, column = column)
            cell.value = header
            cell.style.font.size = 12
            cell.style.font.bold = True
            column += 1
            
        r = 1
        query = (db.customers.ActiveMember == True)
        rows = db(query).select(db.customers.ALL, 
                                db.school_locations.Name,
                left=[db.school_locations.on(db.customers.school_locations_id==\
                                             db.school_locations.id)])
        for row in rows:
            customers_id = row.customers.id
            if export_type.lower() == 'subscription_list' and \
                not cu_mem_dict.has_key(customers_id): 
                # subscription list, if no subscription --> check the next customer.
                continue
            else:
                write_cell(r, 0, row.customers.FirstName, 10, False, True)
                write_cell(r, 1, row.customers.LastName, 10, False, True)
                write_cell(r, 2, row.customers.DateOfBirth, 10, False, True)
                write_cell(r, 3, row.customers.Gender, 10, False, True)
                write_cell(r, 4, row.customers.Address, 10, False, True)
                write_cell(r, 5, row.customers.PostalCode, 10, False, True)
                write_cell(r, 6, row.customers.City, 10, False, True)
                write_cell(r, 7, row.customers.Country, 10, False, True)
                write_cell(r, 8, row.customers.Email, 10, False, True)
                write_cell(r, 9, row.customers.Newsletter, 10, False, True)
                write_cell(r, 10, row.customers.Telnr, 10, False, True)
                write_cell(r, 11, row.customers.Mobnr, 10, False, True)
                write_cell(r, 12, row.customers.Keynr, 10, False, True)
                write_cell(r, 13, row.school_locations.Name, 10, False, True) 
                write_cell(r, 14, cu_mem_dict[customers_id][0], 10, False, True)
                write_cell(r, 15, cu_mem_dict[customers_id][1], 10, False, True)
                write_cell(r, 16, cu_mem_dict[customers_id][2], 10, False, True)
                if bd_dict.get(customers_id, None) != None:
                    write_cell(r, 17, bd_dict[customers_id][0], 10, False, True)
                    write_cell(r, 18, bd_dict[customers_id][1], 10, False, True)
                    write_cell(r, 19, bd_dict[customers_id][2], 10, False, True)
                    write_cell(r, 20, bd_dict[customers_id][3], 10, False, True)
                    write_cell(r, 21, bd_dict[customers_id][4], 10, False, True)
                write_cell(r, 22, row.customers.Comments, 10, False, True)
                r += 1
            
        filename = os.path.join(request.folder,('temp'),fname)
        wb.save(filename = filename)
        return response.stream(filename, attachment=True, filename=fname)
        redirect(URL('index'))
    
    if export_type.lower() == "payment_summary":
        year = request.vars['year'] or datetime.date.today().year
        year = int(year)
        wb = openpyxl.workbook.Workbook()
        ws = wb.worksheets[0]
        ws.title = unicode(year)
        cell = ws.cell(row = 0, column = 0)
        cell.value = "Payment summary for " + unicode(year)
        cell.style.font.size = 12
        cell.style.font.bold = True
        
        # write the header
        field = [ "First name", "Last name", "Date of birth", "Jan", "Feb", 
        "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" ]
        row = 2
        for c in range(0, 15):
            result = write_cell(row, c, field[c], 10, True, False)
        # fill the columns with information
        rows = db((db.paymentsummary.PaymentYear==year) | (db.paymentsummary.PaymentYear==None)).select(
        db.customers.id, db.customers.FirstName, db.customers.LastName, db.customers.DateOfBirth,
        db.paymentsummary.M01, db.paymentsummary.M02, db.paymentsummary.M03, db.paymentsummary.M04, db.paymentsummary.M05, db.paymentsummary.M06,
        db.paymentsummary.M07, db.paymentsummary.M08, db.paymentsummary.M09, db.paymentsummary.M10, db.paymentsummary.M11, db.paymentsummary.M12,
        left=db.paymentsummary.on(db.paymentsummary.customers_id==db.customers.id),
        orderby=db.customers.FirstName)
        
        r = 3 
        for row in rows:
            write_cell(r, 0, row.customers.FirstName, 10)
            write_cell(r, 1, row.customers.LastName, 10)
            write_cell(r, 2, row.customers.DateOfBirth, 10)
            
            months = [ row.paymentsummary.M01, row.paymentsummary.M02, row.paymentsummary.M03, row.paymentsummary.M04, row.paymentsummary.M05,
            row.paymentsummary.M06, row.paymentsummary.M07, row.paymentsummary.M08, row.paymentsummary.M09, row.paymentsummary.M10, 
            row.paymentsummary.M11, row.paymentsummary.M12]
            
            if months[0] != None:
                m = 1
                for month in months:
                    amount = ""
                    if month:
                        firstday = datetime.date(year, m, 1)
                        lastday = datetime.date(year, m, calendar.monthrange(year, m)[1])
                        query = (db.customers_subscriptions.customers_id==\
                                 row.customers.id) & \
                                (db.customers_subscriptions.Startdate <= \
                                 firstday) & \
                                ((db.customers_subscriptions.Enddate >= \
                                  firstday) | \
                                 (db.customers_subscriptions.Enddate == None))
                        subscription = db(query).select(
                                    db.customers_subscriptions.customers_id, 
                                    db.customers_subscriptions.school_subscriptions_id, 
                                    db.customers_subscriptions.Startdate, 
                                    db.customers_subscriptions.Enddate, 
                                    db.school_subscriptions.MonthlyFee,
                            left=db.school_subscriptions.on(\
                                db.customers_subscriptions.school_subscriptions_id==\
                                db.school_subscriptions.id)).first()
                        if not subscription is None:
                            amount = subscription.school_subscriptions.MonthlyFee
                        
                        ap = db((db.alternativepayments.customers_id==row.customers.id) & \
                                (db.alternativepayments.PaymentYear==year) & \
                                (db.alternativepayments.PaymentMonth==m) & \
                                (db.alternativepayments.payment_categories_id==None)).select(
                        db.alternativepayments.Amount).first()
                        if not ap is None:
                            amount = ap.Amount
                    
                    write_cell(r, m+2, amount, 10)
                    m += 1
            r += 1
        fname = T("Payment Overview") + '.xlsx'
        filename = os.path.join(request.folder,('temp'),fname)
        wb.save(filename = filename)
        return response.stream(filename, attachment=True, filename=fname)
        redirect(URL('index'))

    elif export_type.lower() == "mailing_list":
        wb = openpyxl.workbook.Workbook()
        wsnr = 0
        # write the sheet for all mail addresses
        ws = wb.worksheets[wsnr]
        today = datetime.date.today()
        ws.title = "All customers"
        rows = db(db.customers.ActiveMember == True).select(db.customers.Email)
        r = 0
        for row in rows:
            write_cell(r, 0, row.Email, 10, False, False)
            r += 1
        wsnr += 1
        # All newsletter
        ws = wb.create_sheet() 
        ws = wb.worksheets[wsnr]
        today = datetime.date.today()
        ws.title = "Newsletter"
        rows = db((db.customers.ActiveMember == True) & (db.customers.Newsletter==True)).select(db.customers.Email)
        r = 0
        for row in rows:
            write_cell(r, 0, row.Email, 10, False, False)
            r += 1
        
        # write the sheets sorted by classtype
        result = db.executesql("""SELECT distinct cu.email, clt.name FROM classtypes clt 
            LEFT JOIN
            (SELECT id, school_classtypes_id FROM classes) cla
            on cla.school_classtypes_id = clt.id
            LEFT JOIN
            (SELECT id, customers_id, classes_id FROM classes_registration) clr
            ON clr.classes_id = cla.id
            LEFT JOIN
            (SELECT customers_id, classes_id FROM classes_attendance  WHERE classdate >= %s) att
            ON att.classes_id = cla.id
            LEFT JOIN
            (SELECT id, email FROM customers) cu
            ON clr.customers_id = cu.id
            OR att.customers_id = cu.id
            ORDER BY clt.name, cu.email""", (datetime.date.today()))
        r = 0
        previous_classtype = "None"
        address_list = list()
        for record in result:
            current_classtype = record[1]
            if current_classtype != previous_classtype:
                # another classtype so reset some values and create a new worksheet
                r = 0
                ws = wb.create_sheet() 
                wsnr += 1
                ws = wb.worksheets[wsnr]
                ws.title = current_classtype.replace('/', '').replace('\\', '')[:30]
            result = write_cell(r, 0, record[0], 10, False, False)
            r +=1
            previous_classtype = record[1]
            
        fname = T("Mailinglist") + '.xlsx'
        filename = os.path.join(request.folder,('temp'),fname)
        wb.save(filename = filename)
        return response.stream(filename, attachment=True, filename=fname)

    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'customers_classcards'))
def classcards():
    response.view = 'customers/edit.html'
    customers_id = request.vars['cuID']
        
    row = db.customers[customers_id]
    response.title = T("Class cards")
    response.subtitle = row.FirstName + " " + row.LastName
    # set the default value in the grid
    query = (db.customers_classcards.customers_id == customers_id)
    db.customers_classcards.customers_id.default = customers_id
    db.customers_classcards.id.label = T("Pass number")
    
    links = [dict(header=T('Number of classes'), body=classcard_count_classes)]
    if request.user_agent()['is_mobile']:
        fields = [ db.customers_classcards.id, 
                   db.customers_classcards.Startdate, 
                   db.customers_classcards.Classes, 
                   db.customers_classcards.Price ]
        headers = { 'customers_classcards.id':T("Pass") }
        links_in_grid = False
    else:
        fields = [ db.customers_classcards.id, 
                   db.customers_classcards.Startdate, 
                   db.customers_classcards.Enddate, 
                   db.customers_classcards.Classes, 
                   db.customers_classcards.Price ]
        headers = { 'customers_classcards.id':T("Pass") }
        links_in_grid = True
        
    grid = SQLFORM.grid(query, 
                        fields=fields, 
                        headers=headers, 
                        links=links, 
                        links_in_grid=links_in_grid, 
                        create=False, 
                        details=False, 
                        csv=False, 
                        searchable=False, 
                        ui=session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    
    add_url = URL('classcard_add', args=[customers_id])
    add = get_button('add', add_url, T("Add a new class card"))
    
    if 'edit' not in request.args:
        console = grid.element('.web2py_console')
        console.insert(0, add)
    else:
        redirect(URL('classcard_edit', args=[customers_id, 
                                             request.args[2]]))
    
    back = edit_get_back()
    
    menu = customers_get_menu(customers_id, request.function)
    
    return dict(content=grid, menu=menu, back=back, left_sidebar_enabled=True)
    
@auth.requires_login()
def classcards_get_return_url(customers_id):
    '''
        returns the return url for class cards
    '''
    return URL('classcards', vars={'cuID':customers_id})
    
@auth.requires_login()
def classcard_add():
    """
        This function shows an add page for a classcard
        request.args[0] is expected to be the customers_id
    """
    customers_id = request.args[0]
    response.title = T("New Class card")
    response.subtitle = get_customer_name_subtitle(customers_id)
    response.view = 'general/only_content.html'
    
    db.customers_classcards.customers_id.default = customers_id
    
    return_url = classcards_get_return_url(customers_id)
    
    crud.messages.submit_button = T("Add")
    crud.messages.record_created = T("Added classcard")
    crud.settings.create_next = return_url
    form = crud.create(db.customers_classcards)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires_login()
def classcard_edit():
    """
        This function shows an edit page for a classcard
        request.args[0] is expected to be the customers_id
        request.args[1] is expected to be the classcardID
    """
    customers_id = request.args[0]
    classcardID = request.args[1]
    response.title = T("Edit Class card") + " " + unicode(classcardID)
    response.subtitle = get_customer_name_subtitle(customers_id)
    response.view = 'general/only_content.html'
    
    db.customers_classcards.customers_id.default = customers_id
    
    return_url = classcards_get_return_url(customers_id)
    
    crud.messages.submit_button = T("Update")
    crud.messages.record_updated = T("Updated class card")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.customers_classcards, classcardID)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires_login()
def classcard_classes():
    response.title = T("Class cards")
    ccID = request.vars['ccID']
    back = request.vars['back']
    response.subtitle = T("Classes taken using pass") + " " + ccID
    response.view = 'general/only_content.html'
    row = db.customers_classcards[ccID]
    customers_id = row.customers_id
    
    query = (db.classes_attendance.customers_classcards_id == ccID)
    rows = db(query).select(db.classes_attendance.ClassDate, 
                            db.classes.school_locations_id, 
                            db.classes.school_classtypes_id, 
                            db.classes.Starttime, 
                            db.classes.teachers_id,
                            left = [(db.classes.on(\
                                     db.classes_attendance.classes_id==\
                                     db.classes.id))], 
                            orderby=db.classes_attendance.ClassDate|\
                                    db.classes.school_locations_id)
    
    table = TABLE()
    for row in rows.render():
        table.append(TR(TD(row.attendance.ClassDate),
                        TD(row.classes.school_locations_id),
                        TD(row.classes.school_classtypes_id),
                        TD(row.classes.Starttime),
                        TD(row.classes.teachers_id)))
    
    back = get_button('back', URL('classcards', vars={'cuID':customers_id}))
    
    return dict(content=table, back=back)
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'classes_registration'))
def classes():
    '''
        List classes, attendance and waitinglist entries for customers
    '''
    response.title = T("Classes")
    response.view = 'customers/edit.html'
    
    customers_id = request.vars['cuID']
    
    row = db.customers[customers_id]
    response.subtitle = row.FirstName + " " + row.LastName
    function = request.function
    # Back button
    back = edit_get_back()
    
    ## Registration ##
    query = (db.classes_registration.customers_id == customers_id)
    db.classes_registration.customers_id.default = customers_id
    fields = [ db.classes.Week_day,
               db.classes.school_locations_id, 
               db.classes.school_classtypes_id, 
               db.classes.Starttime, 
               db.classes.teachers_id ]
    reg_grid = SQLFORM.grid(query, fields, 
                            create=False, 
                            details=False, 
                            editable=False, 
                            searchable=False, 
                            csv=False,
                            user_signature=False,
                            left=[db.classes.on(\
                                    db.classes_registration.classes_id==\
                                    db.classes.id)],
                            ui=session.grid_ui)
    reg_grid.element('.web2py_counter', replace=None) # remove the counter
    # remove text from delete button
    reg_grid.elements('span[title=Delete]', replace=None)
    reg_title = H3(T("Registration"))
    reg_edit = A(T("Add more classes"), 
                 _href=URL('classes_edit', vars={'cuID':customers_id}))
    reg_div=DIV(reg_title, reg_edit, reg_grid)
    
    ## Waitinglist ##
    query = (db.classes_waitinglist.customers_id == customers_id)
    db.classes_waitinglist.customers_id.default = customers_id
    fields = [ db.classes.Week_day, 
               db.classes.school_locations_id, 
               db.classes.school_classtypes_id, 
               db.classes.Starttime, 
               db.classes.teachers_id ]
    wai_grid = SQLFORM.grid(query, fields, 
                            create=False, 
                            details=False, 
                            editable=False, 
                            csv=False, 
                            searchable=False, 
                            user_signature=False,
                            left=db.classes.on(\
                                    db.classes_waitinglist.classes_id==\
                                    db.classes.id), 
                            ui = session.grid_ui)
    wai_grid.element('.web2py_counter', replace=None) # remove the counter
    # remove text from delete button
    wai_grid.elements('span[title=Delete]', replace=None)
    wai_title = H3(T("Waitinglist"))
    wai_div=DIV(wai_title, wai_grid)
    
    ## Attendance ##
    query = (db.classes_attendance.customers_id == customers_id)
    db.classes_attendance.customers_id.default = customers_id
    fields = [ db.classes_attendance.ClassDate, 
               db.classes.Week_day, 
               db.classes.school_locations_id, 
               db.classes.school_classtypes_id, 
               db.classes.Starttime, 
               db.classes.teachers_id,
               db.classes_attendance.AttendanceType, 
               db.classes_attendance.customers_classcards_id ]
    att_grid = SQLFORM.grid(query, fields, 
        create=False, 
        details=False, 
        editable=False, 
        csv=False, 
        searchable=False, 
        user_signature=False,
        left=db.classes.on(db.classes_attendance.classes_id==db.classes.id),
        orderby=~db.classes_attendance.ClassDate, 
        ui = session.grid_ui)
    att_grid.element('.web2py_counter', replace=None) # remove the counter
    att_grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    att_title = H3(T("Attendance"))
    att_div=DIV(att_title, att_grid)
    
    
    menu = customers_get_menu(customers_id, request.function)
    
    content = DIV(reg_div, wai_div, att_div)
    
    return dict(content=content, back=back, menu=menu, left_sidebar_enabled=True)
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('update', 'classes_registration'))
def classes_edit():
    customers_id = request.vars['cuID']
    session.classes_register_back = 'customers'
    response.view = 'general/one_grid_with_sidebar.html'
    response.title = T("Edit class registration")
    row = db.customers[customers_id]
    response.subtitle = row.FirstName + " " + row.LastName
    
    query = (db.classes)
    links = [classes_check_registration]
    fields = [ db.classes.Week_day, 
               db.classes.school_locations_id, 
               db.classes.school_classtypes_id, 
               db.classes.Starttime, 
               db.classes.teachers_id ]
    grid = SQLFORM.grid(query, 
                        fields, 
                        create=False, 
                        details=False, 
                        deletable=False, 
                        editable=False, 
                        searchable=False, 
                        csv=False,
                        links=links,
                        paginate=50,
                        orderby=db.classes.Week_day|\
                                db.classes.school_locations_id|\
                                db.classes.Starttime,
                        ui=session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
        
    back = get_button('back', URL('classes', vars={'cuID':customers_id}))
    
    return dict(grid=grid, back=back)
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('update', 'customers_subscriptions'))
def subscription_pause_history():
    '''
        This function shows a page which lists all pauses for a subscription
        request.vars['cusID'] is expected to be the subscription ID
    '''
    response.title = T("Pause history")
    response.subtitle = T("")
    response.view = 'general/only_content.html'
    cusID = request.vars['cusID']
    row = db.customers_subscriptions(cusID)
    customers_id = row.customers_id
    db.customers_subscriptions_paused.id.readable = False
    
    if 'resume' in request.vars:
        return_url = URL('subscription_pause', 
                         args=[cusID], 
                         vars=dict(resume=request.vars['resume']))
    else:
        return_url = URL('subscription_pause', args=[cusID])
    
    query = (db.customers_subscriptions_paused.customers_subscriptions_id == cusID)
    if not db(query).count() > 0:
        grid = DIV(BR(), BR(), T("This subscription hasn't been paused before."))
    else:    
        grid = SQLFORM.grid(query, 
            create=False, 
            details=False, 
            editable=False, 
            searchable=False, 
            csv=False,
            paginate=50,
            orderby=db.customers_subscriptions_paused.Startdate,
            ui = session.grid_ui)
        grid.element('.web2py_counter', replace=None) # remove the counter
        grid.elements('span[title=Delete]', replace=None) # remove text from delete button
            
    content = grid
    
    back = get_button('back', return_url)
    
    return dict(content=content, back=back)
    
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('update', 'customers_subscriptions'))
def subscription_pause():
    """
        This function shows a page to select to pause a subscription
        for one month or multiple months.
        request.args[0] is expected to be the subscriptionID
        In case request.vars['resume'] is set, a page to view the history 
        and resume the subscription will be shown.
    """
    response.title = T("Pause subscription")
    response.subtitle = T("")
    response.view = 'general/only_content.html'
    cusID = request.args[0]
    
    row = db.customers_subscriptions(cusID)
    customers_id = row.customers_id
    
    return_url = URL('subscriptions', vars=dict(customers_id=customers_id))
    
    history_vars = dict(cusID=cusID)
    content = UL(_class='os-multiple_choice')
    if 'resume' in request.vars:
        pcusID = request.vars['resume']
        content.insert(0, LI(A(T("Resume"), _href=URL('subscription_resume', args=[pcusID])), BR(), P(T("Resume this subscription from the current month"))))
        history_vars['resume'] = pcusID
    else:
        content.insert(0, LI(A(T("Current month"), _href=URL('subscription_pause_current_month', args=[cusID])), BR(), P(T("Pause this subscription for the current month"))))
        content.insert(1, LI(A(T("Multiple months"), _href=URL('subscription_pause_multiple_months', args=[cusID])), BR(), P(T("Pause this subscription for more than one month"))))
        
    content.append(LI(A(T("History"), _href=URL('subscription_pause_history', vars=history_vars)), BR(), P(T("View pause history for this subscription"))))
    
    back = get_button('back', return_url)
    
    return dict(content=content, back=back)
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('update', 'customers_subscriptions'))
def subscription_resume():
    '''
        This function resumes the paused subscription starting from the current month
        request.args[0] is expected to te the paused subscriptionID
    '''
    pcusID = request.args[0]
    row = db.customers_subscriptions_paused(pcusID)
    subscription_row = db.customers_subscriptions(row.customers_subscriptions_id)
    customers_id = subscription_row.customers_id
    
    today = datetime.date.today()
    first_day_this_month = datetime.date(today.year, today.month, 1)
    
    if row.Startdate == first_day_this_month:
        '''
            If the subscription was only paused this month, delete the entry
        '''
        query = (db.customers_subscriptions_paused.id == pcusID)
        db(query).delete()
    else:
        '''
            Else set the end date to the end of last month
        '''
        year = today.year
        if today.month == 1:
            year = year - 1
            month = 12
        else:
            month =  today.month - 1
        
        last_month_date = datetime.date(year, month, 1)
        enddate = get_last_day_month(last_month_date)
        row.Enddate = enddate
        row.update_record()
        
    redirect(URL('subscriptions', vars={'cuID':customers_id}))
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('update', 'customers_subscriptions'))
def subscription_pause_current_month():
    '''
        This function pauses a subscription for the current month
        request.args[0] is expected to be the subscriptionID
    '''
    response.title = T("Pause subscription")
    response.subtitle = T("")
    response.view = 'general/only_content.html'
    cusID = request.args[0]
    row = db.customers_subscriptions(cusID)
    customers_id = row.customers_id
    
    return_url = URL('subscriptions', vars={'cuID':customers_id})
    
    today = datetime.date.today()
    start = datetime.date(today.year, today.month, 1)
    end = get_last_day_month(start)
    
    db.customers_subscriptions_paused.customers_subscriptions_id.default = cusID
    db.customers_subscriptions_paused.customers_subscriptions_id.writable = False
    db.customers_subscriptions_paused.customers_subscriptions_id.readable = False
    db.customers_subscriptions_paused.Startdate.readable = False
    db.customers_subscriptions_paused.Startdate.writable = False
    db.customers_subscriptions_paused.Startdate.default = start
    db.customers_subscriptions_paused.Enddate.readable = False
    db.customers_subscriptions_paused.Enddate.writable = False
    db.customers_subscriptions_paused.Enddate.default = end
    
    form = SQLFORM(db.customers_subscriptions_paused, 
                   separator='', 
                   submit_button=T("Update"),
                   formstyle='table2cols')
                   
    if form.process().accepted:
        redirect(return_url)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('update', 'customers_subscriptions'))
def subscription_pause_multiple_months():
    '''
        This function shows a page to allow a user to pause a subscription for multiple months
        request.args[0] is expected to be the subscriptionID
    '''
    response.title = T("Pause subscription")
    response.subtitle = T("")
    response.view = 'general/only_content.html'
    cusID = request.args[0]
    row = db.customers_subscriptions(cusID)
    
    today = datetime.date.today()
    return_url = URL('subscriptions', vars={'cuID':row.customers_id})
    
    months = get_months_list()
    
    form = SQLFORM.factory(
        Field('from_month', 'integer', requires=IS_IN_SET(months), default=today.month),
        Field('from_year', 'integer', default=today.year),
        Field('until_month', 'integer', requires=IS_IN_SET(months)),
        Field('until_year', 'integer', default=today.year),
        Field('description'),
        separator = '',
        submit_button = T("Update")
        )
    
    # change type to number from the default text
    form.element('#no_table_from_year').attributes['_type'] = 'number'
    form.element('#no_table_from_year').attributes['_class'] += ' os-input_year'
    form.element('#no_table_from_month').attributes['_class'] += ' os-input_month'
    form.element('#no_table_until_year').attributes['_type'] = 'number'
    form.element('#no_table_until_year').attributes['_class'] += ' os-input_year'
    form.element('#no_table_until_month').attributes['_class'] += ' os-input_month'
    form.element('#no_table_description').attributes['_placeholder'] = T("Description")
    
    if form.process().accepted:
        start = datetime.date(int(form.vars.from_year), int(form.vars.from_month), 1)
        end = get_last_day_month(datetime.date(int(form.vars.until_year), int(form.vars.until_month), 1))
        description = form.vars.description
        db.customers_subscriptions_paused.insert(customers_subscriptions_id = cusID,
                                                 Startdate = start,
                                                 Enddate = end,
                                                 Description = description)
        redirect(return_url)
    
    content = DIV(
        BR(),
        form.custom.begin,
        TABLE(
            TR(T("From"), form.custom.widget.from_month, form.custom.widget.from_year), 
            TR(T("Until"), form.custom.widget.until_month, form.custom.widget.until_year)), 
        form.custom.widget.description, 
        BR(),
        form.custom.submit,
        form.custom.end)
    
    back = get_button('back', return_url)
    
    return dict(content=content, back=back)
    
def subscriptions_get_return_url(customers_id):
    '''
        Returns return URL for subscriptions
    '''
    return URL('subscriptions', vars={'cuID':customers_id})
    
@auth.requires_login()
def subscription_add():
    """
        This function shows an add page for a subscription
        request.args[0] is expected to be the customers_id
    """
    customers_id = request.args[0]
    response.view = 'general/only_content.html'
    response.title = T("New subscription")
    response.subtitle = get_customer_name_subtitle(customers_id)
    
    db.customers_subscriptions.customers_id.default = customers_id
    
    return_url = subscriptions_get_return_url(customers_id)
    
    crud.messages.submit_button = T("Add")
    crud.messages.record_created = T("Added subscription")
    crud.settings.create_next = return_url
    form = crud.create(db.customers_subscriptions)
    
    back = get_button("back", return_url)
    
    return dict(content=form, back=back)

@auth.requires_login()
def subscription_edit():
    """
        This function shows an edit page for a subscription
        request.args[0] is expected to be the customers_id
        request.args[1] is expected to be the subscriptionID
    """
    response.view = 'general/only_content.html'
    request.title = T("Edit subscription")
    customers_id = request.args[0]
    subscriptionID = request.args[1]
    response.subtitle = get_customer_name_subtitle(customers_id)
    
    db.customers_subscriptions.customers_id.default = customers_id
    
    return_url = subscriptions_get_return_url(customers_id)
    
    crud.messages.submit_button = T("Update")
    crud.messages.record_updated = T("Updated subscription")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.customers_subscriptions, subscriptionID)
    
    back = get_button("back", return_url)
    
    return dict(content=form, back=back)

@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'customers_subscriptions'))
def subscriptions():
    '''
        Lists subscriptions for a customer
        request.vars['cuID'] is expected to be the customersID
    '''
    customers_id = request.vars['cuID']
    response.view = 'customers/edit.html'
    
    row = db.customers[customers_id]
    response.title = T("Subscriptions")
    response.subtitle = row.FirstName + " " + row.LastName
    
    db.customers_subscriptions.id.readable=False
    # set the default value in the grid
    query = (db.customers_subscriptions.customers_id == customers_id)
    db.customers_subscriptions.customers_id.default = customers_id
    
    maxtextlengths = {'customers_subscriptions.school_subscriptions_id' : 30}
    
    links = [ subscriptions_get_link_paused ]
    
    fields = [db.customers_subscriptions.school_subscriptions_id, \
              db.customers_subscriptions.Startdate, \
              db.customers_subscriptions.Enddate]
    grid = SQLFORM.grid(query, 
                        fields=fields, 
                        links=links,
                        details=False, 
                        searchable=False, 
                        csv=False, 
                        create=False,
                        maxtextlengths=maxtextlengths,
                        orderby=~db.customers_subscriptions.Startdate, 
                        ui = session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    
    add_url = URL('subscription_add', args=[customers_id])
    add = get_button('add', add_url, T("Add a new subscription"))
                
    if 'edit' not in request.args:
        console = grid.element('.web2py_console')
        console.insert(0, add)
    else:
        redirect(URL('subscription_edit', args=[customers_id, 
                                                request.args[2]]))
    
    back = edit_get_back()
    menu = customers_get_menu(customers_id, request.function)
    
    return dict(content=grid, menu=menu, back=back, left_sidebar_enabled=True)
    
@auth.requires_login()
def payments_delete_payment_info(form):
    page = redirect(URL('payments', vars={'cuID':customers_id}))
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'customers_payments'))
def payments():
    response.title = T("Payment info")
    response.view = 'customers/edit.html'
    
    customers_id = request.vars['cuID']
    row = db.customers[customers_id]
    response.subtitle = row.FirstName + " " + row.LastName
    function = request.function
    
    # back button
    back = edit_get_back()
    
    # payment_info
    db.customers_payment_info.id.readable=False
    db.customers_payment_info.customers_id.readable=False
    query = (db.customers_payment_info.customers_id == customers_id)
    db.customers_payment_info.customers_id.default = customers_id
    fields = [ db.customers_payment_info.payment_methods_id, 
               db.customers_payment_info.AccountNumber, 
               db.customers_payment_info.AccountHolder, 
               db.customers_payment_info.BIC,
               db.customers_payment_info.BankName, 
               db.customers_payment_info.BankLocation ]
    links = [lambda row: get_button('edit', 
                                    URL('payment_info_edit', 
                                        args=[customers_id, row.id])) ]
    bd_grid = SQLFORM.grid(query, 
                           fields=fields, 
                           links=links, 
                           details=False, 
                           editable=False, 
                           csv=False, 
                           searchable=False, 
                           ui=session.grid_ui)
    if db(query).count() == 0:
        add = get_button('add', URL('payment_info_add', args=[customers_id]))
    else:
        add = None
    bd_grid.element('a', _class='button btn', replace=add)
    bd_grid.element('.web2py_counter', replace=None) # remove the counter
    bd_grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    bd_title = H3(T("Payment information"))
    
    # alternative payments
    db.alternativepayments.id.readable=False
    db.alternativepayments.customers_id.readable=False
    query = (db.alternativepayments.customers_id == customers_id)
    db.alternativepayments.customers_id.default = customers_id
    
    fields = [ db.alternativepayments.PaymentYear, 
               db.alternativepayments.PaymentMonth, 
               db.alternativepayments.Amount, 
               db.alternativepayments.payment_categories_id, 
               db.alternativepayments.Description ] 
    
    links = [lambda row: get_button('edit', 
                                    URL('alternativepayment_edit', 
                                        args=[customers_id, row.id])),
            lambda row: A(SPAN(_class="buttontext button", _title=T("Repeat")), 
                          SPAN(_class="icon repeat icon-repeat"), 
                          " ", T("Repeat"), 
                          _class="button btn", \
                          _href=URL('alternativepayment_repeat', 
                                    vars=dict(apID=row.id)), 
                          _title=T("Repeat")) ]
    maxtextlengths = {'overduepayments.Description' : 30}
    ap_grid = SQLFORM.grid(query, 
                           fields=fields, 
                           links=links, 
                           maxtextlengths=maxtextlengths, 
                           details=False, 
                           editable=False, 
                           csv=False, 
                           searchable=False, 
                           ui=session.grid_ui,
                           orderby=~db.alternativepayments.PaymentYear|\
                                   ~db.alternativepayments.PaymentMonth)
    add = get_button('add', URL('alternativepayment_add', args=[customers_id]))
    ap_grid.element('a', _class='button btn', replace=add)
    ap_grid.element('.web2py_counter', replace=None) # remove the counter
    ap_grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    ap_title = H3(T("Alternative payments"))
    
    # overdue payments 
    db.overduepayments.id.readable=False
    db.overduepayments.customers_id.readable=False
    query = (db.overduepayments.customers_id == customers_id)
    db.overduepayments.customers_id.default = customers_id
    
    fields = [ db.overduepayments.PaymentDate,
               db.overduepayments.Amount, 
               db.overduepayments.Description, 
               db.overduepayments.Paid ]
    
    links = [lambda row: get_button('edit', 
                                    URL('overduepayment_edit', 
                                        args=[customers_id, row.id]))]
    maxtextlengths = {'overduepayments.Description' : 30}
    op_grid = SQLFORM.grid(query, 
                           fields=fields, 
                           links=links, 
                           maxtextlengths=maxtextlengths, 
                           details=False, 
                           editable=False, 
                           csv=False, 
                           searchable=False, 
                           orderby=db.overduepayments.Paid|\
                                   db.overduepayments.PaymentDate,
                           ui=session.grid_ui)
    add = get_button('add', URL('overduepayment_add', args=[customers_id]))
    op_grid.element('a', _class='button btn', replace=add)
    op_grid.element('.web2py_counter', replace=None) # remove the counter
    op_grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    op_title = H3(T("Overdue payments"))
    
    # payment summary (payment overview)
    db.paymentsummary.id.readable=False
    db.paymentsummary.customers_id.readable=False
    
    if request.user_agent()['is_mobile']:
        db.paymentsummary.M01.label = T("Jan")
        db.paymentsummary.M02.label = T("Feb")
        db.paymentsummary.M03.label = T("Mar")
        db.paymentsummary.M04.label = T("Apr")
        db.paymentsummary.M05.label = T("May")
        db.paymentsummary.M06.label = T("Jun")
        db.paymentsummary.M07.label = T("Jul")
        db.paymentsummary.M08.label = T("Aug")
        db.paymentsummary.M09.label = T("Sep")
        db.paymentsummary.M10.label = T("Oct")
        db.paymentsummary.M11.label = T("Nov")
        db.paymentsummary.M12.label = T("Dec")
    
    query = (db.paymentsummary.customers_id == customers_id)
    db.paymentsummary.customers_id.default = customers_id
    links = [lambda row: get_button('edit', 
                                    URL('paymentoverview_edit', 
                                        args=[customers_id, row.id]))]
    ps_grid = SQLFORM.grid(query, 
                           links=links, 
                           details=False, 
                           editable=False, 
                           csv=False, 
                           searchable=False, 
                           ui=session.grid_ui)
    add = get_button('add', URL('paymentoverview_add', args=[customers_id]))
    ps_grid.element('a', _class='button btn', replace=add)
    ps_grid.element('.web2py_counter', replace=None) # remove the counter
    # remove text from delete button
    ps_grid.elements('span[title=Delete]', replace=None)
    ps_title = H3(T("Payment overview"))
    
    menu = customers_get_menu(customers_id, request.function)
    
    content = DIV(bd_title, bd_grid, 
                  ap_title, ap_grid, 
                  op_title, op_grid, 
                  ps_title, ps_grid)
    
    return dict(content=content, 
                menu=menu, 
                back=back, 
                left_sidebar_enabled=True)

@auth.requires_login()
def memos():
    '''
        Lists all memos for a customer
    '''
    customers_id = request.vars['cuID']
    
    memo_links = Memo_links()
    # reset variable if we visited the pinboard before, otherwise we might get redirected there again after adding or editing a memo.
    session.memos_back = None 
    # now continue settings things as usual
    response.title = T("Memos")
    response.view = 'customers/edit.html'
    row = db.customers[customers_id]
    response.subtitle = get_customer_name_subtitle(customers_id)
    
    db.memos.id.readable = False
    db.memos.customers_id.default = customers_id
    
    fields = [ db.memos.Visible, 
               db.memos.Announcement, 
               db.memos.Title, 
               db.memos.Msg, 
               db.memos.Duedate, 
               db.memos.Startdate, 
               db.memos.Enddate, 
               db.memos.Priority ]
               
    if 'edit' in request.args:
        links = []
    else:
        links = [ dict(header=T("Priority"), 
                       body=memo_links.memo_link_priority), 
                  dict(header=T("Visible"), 
                       body=memo_links.memo_link_visible), 
                  dict(header=T("Top"), 
                       body=memo_links.memo_link_announcement) ]
    
    maxtextlengths = {'memos.Msg' : 30}
    query = (db.memos.customers_id == customers_id)
    grid = SQLFORM.grid(query, 
                        fields=fields, 
                        links=links, 
                        details=False, 
                        csv=False, 
                        searchable=False, 
                        create=False,
                        maxtextlengths=maxtextlengths, 
                        ui=session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    
    add_url = URL('memo_add', args=[customers_id])
    add = get_button('add', add_url, T("Add a new memo"))
                
    if 'edit' not in request.args:
        console = grid.element('.web2py_console')
        console.insert(0, add)
    else:
        redirect(URL('memo_edit', args=[request.args[2]]))
        
    back = edit_get_back()
    menu = customers_get_menu(customers_id, request.function)
    
    return dict(content=grid, menu=menu, back=back, left_sidebar_enabled=True)
    
@auth.requires_login()
def memo_add():
    """
        This function shows an add page for a memo
        request.args[0] is expected to be the customers_id
    """
    
    response.title = T("New memo")
    response.view = 'general/only_content.html'
    if request.args[0] == "None":
        customers_id = None
        db.memos.customers_id.default = None
        response.subtitle = T('')
    else:
        customers_id = request.args[0]
        db.memos.customers_id.default = customers_id
        response.subtitle = get_customer_name_subtitle(customers_id)
        
    return_url = memo_get_return_url(customers_id)
    
    crud.messages.submit_button = T("Create")
    crud.messages.record_created = T("Added memo")
    crud.settings.create_next = return_url
    form = crud.create(db.memos)
    
    content = remodel_memo_form(form)
    
    back = get_button('back', return_url)
    
    return dict(content=content, back=back)
    
@auth.requires_login()
def memo_edit():
    """
        This function shows an edit page for a memo
        request.args[0] is expected to be the memoID
    """
    response.view = 'general/only_content.html'
    response.title = T("Edit memo")
    memoID = request.args[0]
    customers_id = db.memos(memoID).customers_id
    if customers_id is None:
        db.memos.customers_id.default = None
        response.subtitle = T('')
    else:
        db.memos.customers_id.default = customers_id
        response.subtitle = get_customer_name_subtitle(customers_id)
    
    return_url = memo_get_return_url(customers_id)
    
    crud.messages.submit_button = T("Update")
    crud.messages.record_updated = T("Updated memo")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.memos, memoID)
    
    content = remodel_memo_form(form)
    
    back = get_button('back', return_url)
    
    return dict(content=content, back=back)
    
def payment_info_get_returl_url(customers_id):
    '''
        Returns the return url for payment_info_add and payment_info_edit
    '''
    return URL('payments', vars={'cuID':customers_id})
    
@auth.requires_login()
def payment_info_add():
    """
        This function shows an add page for payment_info
        request.args[0] is expected to be the customers_id
    """
    response.title = T("Payment info")
    customers_id = request.args[0]
    row = db.customers[customers_id]
    response.subtitle = T("Payment info") + " " + \
                        get_customer_name_subtitle(customers_id)
    response.view = 'general/only_content.html'
    
    return_url = payment_info_get_returl_url(customers_id)
    
    query = (db.customers_payment_info.customers_id == customers_id)
    if db(query).count() < 1:
        db.customers_payment_info.customers_id.default = customers_id
        
        crud.messages.submit_button = T("Add")
        crud.messages.record_created = T("Added payment info")
        crud.settings.create_next = return_url
        form = crud.create(db.customers_payment_info)
    else:
        form = T("Payment info already entered for this customer.")
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires_login()
def payment_info_edit():
    """
        This function shows an edit page for the payment_info of a customer
        request.args[0] is expected to be the customers_id
        request.args[1] is expected to be the payment_infoID (bdID)
    """
    response.title = T("Payment info")
    customers_id = request.args[0]
    bdID = request.args[1]
    row = db.customers[customers_id]
    response.subtitle = T("Payment info") + " " + \
                        get_customer_name_subtitle(customers_id)
    response.view = 'general/only_content.html'
    
    db.customers_payment_info.customers_id.default = customers_id
    
    return_url = payment_info_get_returl_url(customers_id)
    
    crud.messages.submit_button = T("Update")
    crud.messages.record_updated = T("Updated payment info")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.customers_payment_info, bdID)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
def alternativepayment_get_return_url(customers_id):
    '''
        Returns return url for alternative payments
    '''
    return URL('payments', vars={'cuID':customers_id})
    
@auth.requires_login()
def alternativepayment_add():
    """
        This function shows an add page for alternative payments
        request.args[0] is expected to be the customers_id
    """
    customers_id = request.args[0]
    response.title = T("Payment info")
    row = db.customers[customers_id]
    response.subtitle = T("Alternative payments") + " " + \
                          get_customer_name_subtitle(customers_id)
    response.view = 'general/only_content.html'
    
    db.alternativepayments.customers_id.default = customers_id
    
    return_url = alternativepayment_get_return_url(customers_id)
    
    crud.messages.submit_button = T("Add")
    crud.messages.record_created = T("Added alternative payment")
    crud.settings.create_next = return_url
    form = crud.create(db.alternativepayments)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires_login()
def alternativepayment_edit():
    """
        This function shows an add page for alternative payments
        request.args[0] is expected to be the customers_id
        request.args[1] is expected to be the alternativepaymentsID
    """
    response.title = T("Payment info")
    customers_id = request.args[0]
    apID = request.args[1]
    row = db.customers[customers_id]
    response.subtitle = T("Alternative payments") + " " + \
                          get_customer_name_subtitle(customers_id)
    response.view = 'general/only_content.html'
    
    db.alternativepayments.customers_id.default = customers_id
    
    return_url = alternativepayment_get_return_url(customers_id)
    
    crud.messages.submit_button = T("Update")
    crud.messages.record_updated = T("Updated alternative payment")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.alternativepayments, apID)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires_login()
def alternativepayments_check(form):
    month = form.vars['PaymentMonth']
    year = form.vars['PaymentYear']
    categoryID = form.vars['PaymentCategoryID']
    
    query = (db.alternativepayments.customers_id==customers_id) & \
            (db.alternativepayments.PaymentYear==year) & \
            (db.alternativepayments.PaymentMonth==month) & \
            (db.alternativepayments.payment_categories_id==categoryID)
    count = db(query).count()
    if count > 1:
        session.flash = T("Only one alternative payment per category per month is allowed.")
        db.rollback() # undo changes, we don't want this data in the db to prevent export issues.
        
def paymentoverview_get_return_url(customers_id):
    '''
        Returns the return url for payment overview
    '''
    return URL('payments', vars={'cuID':customers_id})
    
@auth.requires_login()
def paymentoverview_add():
    '''
        This function shows an add page for a payment overview
        request.args[0] is expected to be the customers_id
    '''
    customers_id = request.args[0]
    response.title = T("Payment info")
    row = db.customers[customers_id]
    response.subtitle = T("Payment overview") + " " + \
                        get_customer_name_subtitle(customers_id)
    response.view = 'general/only_content.html'
    
    db.paymentsummary.customers_id.default = customers_id
    
    return_url = paymentoverview_get_return_url(customers_id)
    
    crud.messages.submit_button = T("Add")
    crud.messages.record_created = T("Added payment overview")
    crud.settings.create_next = return_url
    form = crud.create(db.paymentsummary)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires_login()
def paymentoverview_edit():
    '''
        This function shows an edit page for a payment overview
        request.args[0] is expected to be the customers_id
        request.args[1] is expected to be the paymentsummaryID (psID)
    '''
    response.title = T("Payment info")
    customers_id = request.args[0]
    psID = request.args[1]
    row = db.customers[customers_id]
    response.subtitle = T("Payment overview") + " " + \
                        get_customer_name_subtitle(customers_id)
    response.view = 'general/only_content.html'
    
    db.paymentsummary.customers_id.default = customers_id
    
    return_url = paymentoverview_get_return_url(customers_id)
    
    crud.messages.submit_button = T("Update")
    crud.messages.record_updated = T("Updated payment overview")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.paymentsummary, psID)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
def overduepayment_get_return_url(customers_id):
    '''
        Returns the return url for the overdue add and edit pages
    '''
    return URL('payments', vars={'cuID':customers_id})
    
@auth.requires_login()
def overduepayment_add():
    """
        This function shows an add page for a overdue payment
        Request.args[0] is expected to be the customers_id
    """    
    customers_id = request.args[0]
    response.title = T("Payment info")
    row = db.customers[customers_id]
    response.subtitle = T("Overdue payments") + " " + \
                          get_customer_name_subtitle(customers_id)
    response.view = 'general/only_content.html'
    
    db.overduepayments.customers_id.default = customers_id
    
    return_url = overduepayment_get_return_url(customers_id)
    
    crud.messages.submit_button = T("Add")
    crud.messages.record_created = T("Added overdue payment")
    crud.settings.create_next = return_url
    form = crud.create(db.overduepayments)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires_login()
def overduepayment_edit():
    """
        This function shows an edit page for a overdue payment
        request.args[0] is expected to be the customers_id
        request.args[1] is expected to be the overduepaymentsID (opID)
    """    
    response.title = T("Payment info")
    customers_id = request.args[0]
    opID = request.args[1]
    row = db.customers[customers_id]
    response.subtitle = T("Overdue payments") + " " + \
                        get_customer_name_subtitle(customers_id)
    response.view = 'general/only_content.html'
    
    db.overduepayments.customers_id.default = customers_id
    
    return_url = overduepayment_get_return_url(customers_id)
    
    crud.messages.submit_button = T("Update")
    crud.messages.record_updated = T("Updated overdue payment")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.overduepayments, opID)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'workshops'))
def workshops():
    '''
        This function shows a page which lists the workshop attendance for a 
        customer.
    '''
    customers_id = request.vars['cuID']
    
    session.workshops_payment_back = 'customer'
    response.title = T("Workshops")
    row = db.customers[customers_id]
    response.subtitle = get_customer_name_subtitle(customers_id)
    #response.view = 'customers/edit.html'
    
    query = (db.workshops_registration.customers_id == customers_id)
    rows = db(query).select(
        db.workshops_registration.id, 
        db.workshops_registration.workshops_activities_id,
        db.workshops_payments.id,
        db.workshops.Name, 
        db.workshops.Startdate,
        db.workshops_activities.Activity,
        db.workshops_activities.Activitydate,
        db.workshops_activities.Starttime,
        db.workshops_payments.Amount, 
        db.workshops_payments.PaymentDate, 
        left = [db.workshops_payments.on(\
                    db.workshops_registration.id == \
                    db.workshops_payments.workshops_registration_id),
                db.workshops_activities.on(\
                    db.workshops_activities.id == \
                    db.workshops_registration.workshops_activities_id),
                db.workshops.on(db.workshops.id == \
                    db.workshops_registration.workshops_id),
                ], 
        orderby=db.workshops.Startdate|db.workshops_activities.Activitydate)
        
    even_row = True
    table = TABLE(_class="os-customer_table")
    for row in rows.render():
        # set the class for the row
        if even_row:
            row_class = "os-customer_table_row_even"
        else:
            row_class = "os-customer_table_row_odd"
        even_row = not even_row
        
        # get payment info
        if row.workshops_payments.Amount is None:
            add_url = URL('workshops', 'payment_add', 
                          args=[row.workshops_registration.id])
            payment = A(T("Add"), _href=add_url)
        else:
            payment_amount = format(row.workshops_payments.Amount, '.2f')
            payment_date = \
                row.workshops_payments.PaymentDate.strftime(session.date_format)
            edit_url = URL('workshops', 'payment_edit', 
                           args=[row.workshops_payments.id])
            payment = A(payment_amount + ' on ' + payment_date, 
                      _href=edit_url)
        # date
        if 'workshops_activities_id' in row.workshops_registration:
            date = row.workshops_activities.Activitydate
        if date is None:
            date = row.workshops.Startdate
        date = date.strftime(session.date_format)
        
        # get attendance info form
        hidden_field_id = INPUT(_type="hidden", 
                                _name="id",
                                _value=row.workshops_registration.id)
        form = SQLFORM(db.workshops_registration, 
            row.workshops_registration.id)
        
        payment_table = DIV( form.custom.begin,
            TABLE(TR(TD(T("Payment")),
                     TD(payment)),
                  TR(TD(form.custom.label.PaymentConfirmation),
                     TD(form.custom.widget.PaymentConfirmation)),
                  TR(TD(form.custom.label.Info),
                     TD(form.custom.widget.Info)),
                  _class='os-attendance_info_table'),
            hidden_field_id,
            form.custom.end,
        _class="left")
        
        delete = SPAN(_class="icon remove icon-remove",
                      _id=row.workshops_registration.id)

        table.append(TR(TD(SPAN(row.workshops.Name, _class="bold"),
                           BR(),
                           row.workshops_activities.Activity or \
                                T("Full Workshop"),
                           BR(),
                           date,
                            _width="40%"),
                        TD(payment_table, DIV(delete, _class="right")),
                        _class=row_class)
                        
                    )
    
    # back button
    back = edit_get_back()
    menu = customers_get_menu(customers_id, request.function)
    
    return dict(content=table, menu=menu, back=back, 
        left_sidebar_enabled=True)
        

@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'customers_documents'))
def documents():
    '''
        This function shows a list of documents uploaded for a customer
    '''
    customers_id = request.vars['cuID']
    response.view = 'customers/edit.html'
    row = db.customers[customers_id]
    response.title = T("Documents")
    response.subtitle = row.FirstName + " " + row.LastName
    
    db.customers_documents.id.readable=False
    db.customers_documents.UploadDateTime.readable=True
    # set the default value in the grid
    links = [lambda row: get_button('edit', 
                                     URL('document_edit', 
                                         args=[customers_id, row.id]),
                                     T("Edit document description"))]
    
    query = (db.customers_documents.customers_id == customers_id)
    maxtextlengths = {'customers_documents.Description' : 30}
    headers = {'customers_documents.DocumentFile': T("File")}
    
    fields = [db.customers_documents.UploadDateTime,
              db.customers_documents.Description,
              db.customers_documents.DocumentFile]
    grid = SQLFORM.grid(query, fields=fields, links=links,
        headers=headers,
        details=False, 
        searchable=False, 
        csv=False, 
        create=False,
        editable=False,
        maxtextlengths=maxtextlengths,
        orderby=~db.customers_documents.UploadDateTime, 
        ui = session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    
    add_url = URL('document_add', args=[customers_id])
    add = get_button('add', add_url, T("Add a new document"))
    console = grid.element('.web2py_console')
    console.insert(0, add)
    
    back = edit_get_back()
    
    menu = customers_get_menu(customers_id, request.function)
    
    return dict(content=grid, menu=menu, back=back, left_sidebar_enabled=True)

def documents_get_return_url(customers_id):
    '''
        Returns the return url for documents add and edit
    '''
    return URL('documents', vars={'cuID':customers_id})
    
@auth.requires_login()
def document_add():
    """
        This function shows an add page for a document
        request.args[0] is expected to be the customers_id
    """
    customers_id = request.args[0]
    response.view = 'general/only_content.html'
    response.title = T("Upload document")
    response.subtitle = get_customer_name_subtitle(customers_id)
    
    db.customers_documents.customers_id.default = customers_id
    
    return_url = documents_get_return_url(customers_id)
    
    space = uploads_available_space(request.folder)
    if space['available'] > 4:
        crud.messages.submit_button = T("Add")
        crud.messages.record_created = T("Uploaded document")
        crud.settings.create_next = return_url
        form = crud.create(db.customers_documents)
        content = form
    else:
        content = space['full_message']
    
    back = get_button("back", return_url)
    
    return dict(content=content, back=back)
    
@auth.requires_login()
def document_edit():
    """
        This function shows an edit page for a document
        request.args[0] is expected to be the customers_id
        request.args[1] is expected to be the customers_documentID (cudID)
    """
    response.view = 'general/only_content.html'
    request.title = T("Edit Document")
    customers_id = request.args[0]
    cudID = request.args[1]
    response.subtitle = get_customer_name_subtitle(customers_id)
    
    db.customers_documents.customers_id.default = customers_id
    db.customers_documents.DocumentFile.readable = False
    db.customers_documents.DocumentFile.writable = False
    
    return_url = documents_get_return_url(customers_id)
    
    crud.messages.submit_button = T("Update")
    crud.messages.record_updated = T("Updated document")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.customers_documents, cudID)
    
    back = get_button("back", return_url)
    
    return dict(content=form, back=back)

