# -*- coding: utf-8 -*-
# this file is released under the gplv2 (or later at your choice) license

#########################################################################
## - index is the default action of any application
## - user is required for authentication and authorization
## - download is for downloading files uploaded in the db (does streaming)
## - call exposes all registered services (none by default)
#########################################################################

from general_helpers import datestr_to_python
from general_helpers import get_button
from general_helpers import get_submenu
from general_helpers import User_helpers
from general_helpers import Memo_links

from gluon.contrib.user_agent_parser import mobilize 
from plugin_solidform import SOLIDFORM

import datetime

# helpers start

@auth.requires_login()
def _generate_welcome_message():
    message = DIV(H3(T("Welcome to OpenStudio")), T("To get started quickly first add membership types, class types and locations \
        in the"), " ", A(T("school properties"), _href=URL('school_properties', 'index', user_signature=True)), " ", T("pages"), ".", 
        BR(),
        T("Then add"), " ", A(T("teachers"), _href=URL('teachers', 'index')), " ",  T("and after that add"), " ",
        A(T("classes"), _href=URL('classes', 'schedule', user_signature=True)), " " , T("to the schedule"), ". ", 
        T("Now you're ready to start adding"), " ", A(T("customers"), _href=URL('customers', 'index')), ". ", 
        BR(),
        T("If you have any questions, please have a look at the"), " ", A(T("manual"), _href='http://www.openstudioproject.com/content/manual', _target="_blank")," ", 
        ".", BR(),
        T("When using OpenStudio for the first time, please have look at the"), " ",
        A(T("license"), _href=URL('help', 'about', user_signature=True)), ".",
        BR(), BR(), 
        "[", A(T("hide this message"), _href=URL('hide_welcome', user_signature=True)), "]"
        )
    
    return message
    
@auth.requires_login()
def subscriptions_get_menu(page=None):
    pages = [
        (['subscriptions_new', T('New subscriptions'), URL("default","subscriptions_new")]),
        (['subscriptions_stopped', T('Stopped subscriptions'), URL("default","subscriptions_stopped")]),
        (['subscriptions_paused', T('Paused subscriptions'), URL("default","subscriptions_paused")]),
        (['subscriptions_overview', T('Subscriptions overview'), URL("default","subscriptions_overview")]),
        ]
    
    horizontal = True
    if request.user_agent()['is_mobile']:
        horizontal = False
    
    return get_submenu(pages, 
                        page, 
                        horizontal=horizontal,
                        _id='os_overview-subscriptions')
    
# helpers end


@auth.requires_login()
def hide_welcome():
    row = db.sys_properties(Property='ShowWelcomeMessage')
    if row:
        row.PropertyValue='off'
        row.update_record()
    else:
        db.sys_properties.insert(Property='ShowWelcomeMessage', PropertyValue='off')
    
    redirect(URL('pinboard', user_signature=True))
    
@auth.requires_login()
def _generate_form_subtitle(month, year, function):
    months = [(1,T("January")),
              (2,T("February")),
              (3,T("March")),
              (4,T("April")),
              (5,T("May")),
              (6,T("June")),
              (7,T("July")),
              (8,T("August")),
              (9,T("September")),
              (10,T("October")),
              (11,T("November")),
              (12,T("December"))]
    
    for m in months:
        if m[0] == month:
            month_title = m[1]
    subtitle = T("Overview ") + month_title + " " + unicode(year)
        
    form = SOLIDFORM.factory(
        [Field('month', 
               requires=IS_IN_SET(months, zero=T("Please select...")), 
               default=month, 
               label=T("")),
        Field('year', 'integer', 
              default=year, 
              label=T(""))],
        submit_button=T("Show")
        )
    form.attributes['_name'] = 'form_select_date'
        
    form.element('select[name=month]')['_style']='width:110px'
    form.element('input[name=year]')['_style']='width:50px'
    form.element('input[name=year]')['_type']='number'
    form.element('input[type=submit]')['_style']='width:80px'
    
    if function == 'subscriptions_new' or \
            function == 'subscriptions_paused' or \
            function == 'subscriptions_stopped' or \
            function == 'subscriptions_overview':
        form.add_button(T('Show current'), URL('subscriptions_show_current'))
    elif function == 'dropinclasses':
        form.add_button(T('Show current'), URL('dropinclasses_show_current'))
    elif function == 'trialclasses':
        form.add_button(T('Show current'), URL('trialclasses_show_current'))
    elif function == 'classcards':
        form.add_button(T('Show current'), URL('classcards_show_current'))
    elif function == 'alternative_payments':
        form.add_button(T('Show current'), 
                        URL('alternative_payments_show_current'))
                        
    
    
    title = H4(T("Which month?"))
    
    return dict(form=form, subtitle=subtitle, title=title)
    
@auth.requires_login()
def _get_pinboard_sorting(sort):
    if sort == 'pa':
        return db.memos.Priority|db.memos.Duedate
    elif sort == 'pd':
        return ~db.memos.Priority|db.memos.Duedate
    elif sort == 'da':
        return ~db.memos.Duedate|db.memos.Priority
    elif sort == 'dd':
        return db.memos.Duedate|db.memos.Priority
        
@auth.requires_login()
def _memos_show_all():
    session.me_name = None
    session.me_date_from = None
    session.me_date_until = None
    redirect(URL('memos'))
        
@auth.requires_login()
def _get_memo_edit_button(row):
    edit = DIV(A(SPAN(_class="buttontext button", _title=T("Edit notification")), SPAN(_class="icon pencil icon-pencil"), " " + T(""), \
        _class="button btn", _href=URL('customers', 'memo_edit', args=[row.memos.id])), _class='right', _title=T("Edit this memo"))
    done = DIV(A(SPAN(_class="buttontext button", _title=T("Hide")), SPAN(_class="icon ok icon-ok"), " " + T(""), \
        _class="button btn", _href=URL('memo_change_visibility',  vars=dict(reminderID=row.memos.id), user_signature=True)), _class='right', _title=T("Hide this memo"))
    if row.customers.Email != None:
        mail = DIV(A(SPAN(_class="buttontext button", _title=T("Mail customer")), SPAN(_class="icon envelope icon-envelope"), " " + T(""), \
        _class="button btn", _href='mailto:' + row.customers.Email), _class='right', _title=T("Mail customer"))
    else:
        mail = ""
    return DIV(done, edit, mail)
    
@auth.requires_login()
def _get_memo_link_edit(row):
    return get_button('edit', URL('customers', 'memo_edit', 
                            args=[row.memos.id]))
    
@auth.requires_login()
def _get_memo_subtitle(row):
    if not row.memos.Duedate is None:
        duedate = I(row.memos.Duedate.strftime(session.date_format))
    else:
        duedate = ""
    priority = I(_NRtoPriority(row.memos.Priority))
    if not row.memos.customers_id is None:
        extra = A(DIV(B(I(row.customers.FirstName + " " + row.customers.LastName)), _class='right'),
                  _href=URL('customers', 'edit', args=[row.memos.customers_id]))
    elif not row.memos.workshops_id is None:
        extra = DIV(B(I(row.workshops.Name)), _class='right')
    else:
        extra = ""
    return DIV(duedate, " ", priority, " ", extra)
    
@auth.requires_login()
def _get_memo_query_name(query, name):
    if name != None and name.isdigit() == False:
        # we're searching for a name
        search_name = '%' + name.strip() + '%'
        query &= ((db.customers.FirstName.like(search_name)) | (db.customers.LastName.like(search_name)) | \
            (db.memos.Title.like(search_name)))
    return query
    
@auth.requires_login()
def _get_memo_query_date_from(query, date):
    if date != None and date != '':
        query &= ((db.memos.Duedate >= date) | (db.memos.Startdate >= date))
    return query
    
@auth.requires_login()
def _get_memo_query_date_until(query, date):
    if date != None and date != '':
        query &= ((db.memos.Duedate <= date) | (db.memos.Startdate <= date))
    return query
    
@auth.requires_login()
def _NRtoPriority(value, row=None):
    value = int(value)
    priorities = [(3,T("Low")),(2,T("Normal")),(1,T("High"))]
    for p in priorities:
        if p[0] == value:
            return p[1]
        
@auth.requires_login()
def memo_change_visibility():
    reminderID = request.vars['reminderID']
    row = db.memos[reminderID]
    row.Visible = not row.Visible
    row.update_record()
    if session.memos_back == 'pinboard':
        redirect(URL('pinboard'))
    else:
        redirect(URL('memos'))
    
@auth.requires_login()
def memo_change_announcement():
    reminderID = request.vars['reminderID']
    row = db.memos[reminderID]
    row.Announcement = not row.Announcement
    row.update_record()
    redirect(URL('memos'))
    
@auth.requires_login()
def memo_change_priority():
    reminderID = request.vars['reminderID']
    row = db.memos[reminderID]
    if row.Priority == 1:
        row.Priority = 3
    elif row.Priority == 2:
        row.Priority = 1
    elif row.Priority == 3:
        row.Priority = 2
    row.update_record()
    redirect(URL('memos'))
        
@auth.requires_login()
def blank():
    '''
        Landing page for users who aren't allowed to see the pinboard
    '''
    response.subtitle = T("")
    response.view = 'general/only_content.html'
    
    return dict(content='')

@auth.requires_login()
def index():
    if not request.user_agent()['is_mobile']:
        session.flash = T("Welcome to OpenStudio!")
    user_helpers = User_helpers()
    if user_helpers.check_read_permission('pinboard', auth.user.id):
        redirect(URL('pinboard'))
    else:
        redirect(URL('blank'))
    
@mobilize
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'default_pinboard'))
def pinboard():
    response.google_analytics_id=session.google_analytics_id
    response.title = T("Pin board")
    response.subtitle = T("")
    
    session.memos_back = 'pinboard'
    session.customers_back = 'pinboard'
    
    if 'sort' in request.vars:
        sort = request.vars['sort']
        session.pinboard_sort = sort
        sorting = _get_pinboard_sorting(sort)

    else:
        # check for session sorting
        try: 
            sort = session.pinboard_sort
            sorting = _get_pinboard_sorting(sort)
        except:
            # default sort by duedate
            sorting = db.memos.Duedate|~db.memos.Startdate
    
    content = []
    
    if db.sys_properties(Property='ShowWelcomeMessage') == None or db.sys_properties(Property='ShowWelcomeMessage').PropertyValue == 'on':
        content.append(_generate_welcome_message())
    
    today = datetime.date.today()
    # cancelled classes
    delta =  datetime.timedelta(days=7)
    query = (db.classes_cancelled.ClassDate >= today) & \
            (db.classes_cancelled.ClassDate <= (today + delta))
    rows = db(query).select(db.classes_cancelled.classes_id, 
                            db.classes_cancelled.ClassDate, 
                            db.school_locations.Name, 
                            db.school_classtypes.Name, 
                            db.classes.Starttime, 
                            db.teachers.FirstName, 
                            db.teachers.LastName, 
                            orderby=db.classes_cancelled.ClassDate|\
                                    ~db.classes.school_locations_id|\
                                    db.classes.Starttime,
        left = [db.classes.on(db.classes_cancelled.classes_id==db.classes.id),
                db.school_locations.on(db.classes.school_locations_id==\
                                       db.school_locations.id),
                db.school_classtypes.on(db.classes.school_classtypes_id==\
                                        db.school_classtypes.id),
                db.teachers.on(db.classes.teachers_id==db.teachers.id) ]
        )
    if rows.first() != None:
        cancelled_classes = DIV(H3(T("Cancelled classes")))
        goto = []
        for row in rows:
            class_date = row.classes_cancelled.ClassDate
            clsID = row.classes_cancelled.classes_id
            classname = row.school_locations.Name + " " + \
                        row.school_classtypes.Name + " " + \
                        row.classes.Starttime.strftime('%H:%M') + " " + \
                        T("by") + " "  + row.teachers.FirstName + " " + \
                        row.teachers.LastName
            goto = DIV(class_date, " ",
                       A(classname, 
                         _href=URL('classes', 'class_details', 
                                   vars=dict(clsID=claID,
                                             date=today,
                                             back='pinboard'))))
            cancelled_classes.append(goto)
            cancelled_classes.append(BR())
        content.append(cancelled_classes)
        
    # Announcements (shown at top)
    entries = [] # [0] = message, [1] = edit_button
    query = (db.memos.Visible==True) & (db.memos.Announcement==True) & \
        (db.memos.Startdate <= today) & ((db.memos.Enddate >= today) | (db.memos.Enddate == None))
    rows = db(query).select(db.memos.ALL, db.customers.FirstName, db.customers.LastName, db.customers.Email,
        db.workshops.Name,
        left=[db.customers.on(db.memos.customers_id==db.customers.id),
              db.workshops.on(db.memos.workshops_id==db.workshops.id)],
        orderby=sorting)
    for row in rows:
        subtitle = _get_memo_subtitle(row)
        entry = DIV(H3(row.memos.Title), subtitle, P(), row.memos.Msg)
        edit_button = _get_memo_edit_button(row)
        entries.append([entry, edit_button])
        
    anouncements_table = TABLE(_id="pinboard_anouncements")
    while len(entries) > 0:
        element = entries.pop(0)
        anouncements_table.append(TR(TD(element[0])))
        anouncements_table.append(TR(TD(element[1]), _class='buttons'))
    content.append(DIV(anouncements_table))
    content.append(BR())
    content.append(BR())
    
    # Memos
    entries = []
    query = (db.memos.Visible==True) & (db.memos.Announcement==False) & \
        (db.memos.Startdate <= today) & ((db.memos.Enddate >= today) | (db.memos.Enddate == None))
    rows = db(query).select(db.memos.ALL, db.customers.FirstName, db.customers.LastName, db.customers.Email,
        db.workshops.Name,
        left=[db.customers.on(db.memos.customers_id==db.customers.id),
              db.workshops.on(db.memos.workshops_id==db.workshops.id)],
        orderby=sorting)
    for row in rows:
        subtitle = _get_memo_subtitle(row)
        edit_button = _get_memo_edit_button(row)
        entry = DIV(H3(row.memos.Title), subtitle, P(), row.memos.Msg)
        entries.append([entry, edit_button])

    memos_table = TABLE(_id="pinboard_memos")
    if request.user_agent()['is_mobile']:
        while len(entries) > 0:
            element = entries.pop(0)
            memos_table.append(TR(TD(element[0])))
            memos_table.append(TR(TD(element[1]), _class='buttons'))
    else:
        while len(entries) > 0:
            if len(entries) > 1:
                element_l = entries.pop(0)
                element_r = entries.pop(0)
                memos_table.append(TR(TD(element_l[0], _style='width:50%'), TD(element_r[0], _style='width:50%')))
                memos_table.append(TR(TD(element_l[1]), TD(element_r[1]), _class='buttons'))
            else:
                element = entries.pop(0)
                memos_table.append(TR(TD(element[0], _style='width:50%'), TD("", _style='width:50%')))
                memos_table.append(TR(TD(element[1]), TD(""), _class='buttons'))
    content.append(memos_table)
    
    add_reminder = get_button('add', URL('customers', 'memo_add', args=[None]), T('Add a new memo'))
    find_memos = A(SPAN(_class="buttontext button", 
                            _title=T("Find memos")), 
                       SPAN(_class="icon search icon-search"), 
                       B(" " + T("Find")), \
                       _class="button btn", 
                       _href=URL('memos'))
    
    # birthdays
    birthdays = get_birthdays()
    
    sorting = DIV(
          A( SPAN(_class="icon list icon-list"), B(" ", T("Sort") , " "), 
             SPAN(_class='caret'),
             _href='#',
             _class='btn dropdown-toggle',
             **{'_data-toggle':"dropdown"}
            ),
          UL(A(T("Priority "), SPAN(_class="icon arrow up icon-arrow-up"), _href=URL(request.function, vars=dict(sort='pa'))),
             A(T("Priority "), SPAN(_class="icon arrow down icon-arrow-down"), _href=URL(request.function, vars=dict(sort='pd'))),
             A(T("Duedate "), SPAN(_class="icon arrow up icon-arrow-up"), _href=URL(request.function, vars=dict(sort='da'))),
             A(T("Duedate "), SPAN(_class="icon arrow down icon-arrow-down"), _href=URL(request.function, vars=dict(sort='dd'))),
             _class='dropdown-menu'
            ),
        _class='btn-group'
        )
    
    manage_memos = DIV(H3(T("Memos")), 
                       add_reminder, " ", 
                       find_memos, " ", 
                       sorting, 
                       BR(), BR())
    
    left_sidebar_enabled=False
    right_sidebar_enabled=True
    
    return dict(content=content, 
                 add_reminder=add_reminder, 
                 edit_memos=find_memos, 
                 birthdays=birthdays, 
                 sorting=sorting, 
                 manage_memos=manage_memos,
                 left_sidebar_enabled=left_sidebar_enabled, 
                 right_sidebar_enabled=right_sidebar_enabled)
    
@auth.requires_login()
def get_birthdays():
    today = datetime.date.today()
    birthday_check = datetime.date(1900, today.month, today.day)
    
    query = (db.customers.Birthday >= birthday_check) & \
            (db.customers.Birthday <= birthday_check + datetime.timedelta(days=7)) 
    rows = db(query).select(db.customers.id, 
                            db.customers.FirstName, 
                            db.customers.LastName, 
                            db.customers.Birthday, 
                            orderby=db.customers.Birthday)
    table = TABLE()
    for row in rows.render():
        table.append(TR(TD(row.Birthday,
                           _class='os-date_month_name'),
                        TD(A(row.FirstName + ' ' + row.LastName,
                             _href=URL('customers', 'edit', args=[row.id])))))
                        
                        
    table_customers = table
        
    query = (db.teachers.Birthday >= birthday_check) & \
            (db.teachers.Birthday <= birthday_check + \
                                     datetime.timedelta(days=7)) 
    teacher_rows = db(query).select(db.teachers.id, 
                                    db.teachers.FirstName, 
                                    db.teachers.LastName, 
                                    db.teachers.Birthday, 
                                    orderby=db.teachers.Birthday)
                                    
    table = TABLE()
    for row in rows.render():
        table.append(TR(TD(row.Birthday),
                        TD(A(row.FirstName + ' ' + row.LastName,
                             _href=URL('teachers', 'techer_edit', 
                                       args=[row.id])))))
    table_teachers = table
    
    birthdays = DIV(H3(T("Birthdays this week")))
    birthdays.append(table_customers)
    if teacher_rows.first() != None:
        birthdays.append(H5(T("Teachers")))
        birthdays.append(table_teachers)
            
    return birthdays
    
@auth.requires_login()
def memos():
    response.title = T("Memos")
    
    back = get_button('back', URL('pinboard'))
        
    memo_links = Memo_links()
        
    db.customers.id.readable = False
    db.memos.id.readable = False
    db.memos.customers_id.default = None
    
    session.memos_back = 'memos'
    
    query = (db.memos.id > 0)
    
    # search variables
    name = None
    date_from = None
    date_until = None
    
    if 'name' in request.vars: # check if a search filter is in use
        response.subtitle = T("Showing search results")
        if request.vars['name'] != "":
            name = request.vars['name']
            query = _get_memo_query_name(query, name)
            session.me_name = name
        if request.vars['date_from'] != "":
            date_from = request.vars['date_from']
            date_from = datestr_to_python(session.date_format, date_from)
            query = _get_memo_query_date_from(query, date_from)
            session.me_date_from = date_from
        if request.vars['date_until'] != "":
            date_until = request.vars['date_until']
            date_until = datestr_to_python(session.date_format, request.vars['date_until'])
            query = _get_memo_query_date_until(query, date_until)
            session.me_date_until = date_until
    elif (session.me_name != None and session.me_name != '') or \
         (session.date_from != None and session.me_date_from != '') or \
         (session.date_until != None and session.me_date_until != ''):
            response.subtitle = T("Showing search results")
            if session.me_name != '':
                name = session.me_name
                query = _get_memo_query_name(query, name)
            if session.me_date_from != '':
                date_from = session.me_date_from
                query = _get_memo_query_date_from(query, date_from)
            if session.me_date_until != '':
                date_until = session.me_date_until
                query = _get_memo_query_date_until(query, date_until)
    else:
        response.subtitle = T("Showing all memos")
    
    links = [_get_memo_link_edit]
    if request.user_agent()['is_mobile']:
        fields = [ db.memos.id, db.memos.Title, db.memos.Startdate, db.customers.id ]
    else:
        fields = [ db.memos.id, db.memos.Visible, db.memos.Announcement, db.memos.Title, db.memos.Msg, 
            db.memos.customers_id, db.memos.Duedate, db.memos.Startdate, db.memos.Enddate, db.memos.Priority, 
            db.customers.id ]
        links.insert(0, dict(header=T("Priority"), body=memo_links.memo_link_priority))
        links.insert(1, dict(header=T("Visible"), body=memo_links.memo_link_visible))
        links.insert(2, dict(header=T("Top"), body=memo_links.memo_link_announcement))
    
    grid = SQLFORM.grid(query, fields=fields, create=False, editable=False, details=False, searchable=False, csv=False, 
        orderby=~db.memos.Startdate,
        links=links,
        user_signature=False,
        left = [db.customers.on(db.memos.customers_id==db.customers.id)],
        ui = session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    
    # a listing is requested.. so we don't want to alter the back button to point to pinboard
    db.memos.customers_id.readable = True
    form = SOLIDFORM.factory(
        [Field('name', default=name, label=T("")),
        Field('date_from', 'date', 
            default=date_from, 
            requires=IS_EMPTY_OR(IS_DATE(format=session.date_format)), 
            label=T("From")),
        Field('date_until', 'date', 
            default=date_until, 
            requires=IS_EMPTY_OR(IS_DATE(format=session.date_format)), 
            label=T("Until"))],
        submit_button=T('Search')
        )
    form.add_button(T('Show all'), URL('_memos_show_all'))
    
    form.element('input[name=name]')['_placeholder']='Search...'
    form.element('input[name=name]')['_style']='width:200px'
    form.element('input[name=date_from]')['_style']='width:90px'
    form.element('input[name=date_until]')['_style']='width:90px'
    
    # remove the defeault add button
    add = grid.element('a', _class='button btn')
    buttons = DIV(back, " ", add)
    grid.element('a', _class='button btn', replace=None)
        
    content = DIV(back, form, grid)
        
    return dict(content=content, right_sidebar_enabled=False)
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'default_overduepayments'))
def overdue_payments():
    def get_table(rows):
        '''
            Helper to render a table from rows. 
        '''
        table = TABLE(_class="os-customer_table")
        table.append(TR(TH(), # image
                        TH(), # name,
                        TH(db.overduepayments.PaymentDate.label,
                           _class='os-table_date'),
                        TH(db.overduepayments.Amount.label),
                        TH(db.overduepayments.Description.label, 
                            _class='os-customer_table_description'),
                        _class='os-table_header'))
                     
        for row in rows.render():
            table.append(TR(TD(row.customers.ThumbSmall,
                               _class='os-customer_image_td'),
                            TD(DIV(row.customers.FirstName,
                                    ' ',
                                    row.customers.LastName,
                                    _class="os-customer_name"),
                                BR()),
                            TD(row.overduepayments.PaymentDate),
                            TD(row.overduepayments.Amount),
                            TD(row.overduepayments.Description),
                            _id=row.customers.id,
                            _class='os-clickable'))
        return table
    
    response.title = T("Overdue payments")
    response.subtitle = T("")
    style = STYLE('.italic {font-style: italic;} .bold {font-weight: bold;}')
    
    session.customers_back = 'overduepayments'
    
    # general variables
    today = datetime.date.today()
    
    sum = db.overduepayments.Amount.sum()
    # overdue
    query = (db.overduepayments.Paid == 0) & \
            (db.overduepayments.PaymentDate < today)    
    rows = db(query).select(db.customers.id, 
                            db.customers.ActiveMember, 
                            db.customers.ThumbSmall,
                            db.customers.FirstName, 
                            db.customers.LastName, 
                            db.customers.DateOfBirth, 
                            db.overduepayments.PaymentDate, 
                            db.overduepayments.Amount, 
                            db.overduepayments.Description,
        left=db.customers.on(db.customers.id==db.overduepayments.customers_id), 
        orderby=db.overduepayments.PaymentDate|db.customers.FirstName)
        
    sum_total = db(query).select(sum).first()[sum]
    if sum_total == None:
        sum_total = '0'
    else:
        sum_total = unicode(sum_total)
    overdue_total = DIV(T("Total Amount") + ": " + sum_total, _class='right')
    overdue_total_customers = DIV(T("Total customers" + ": " + unicode(len(rows))), _class='left')
    
    table = get_table(rows)
    overdue = DIV(table, style)
    
    # due this week
    enddate = today + datetime.timedelta(days=7)
    query = (db.overduepayments.Paid == 0) & (db.overduepayments.PaymentDate >= today) & \
        (db.overduepayments.PaymentDate <= enddate)
    rows = db(query).select(db.customers.id, db.customers.ActiveMember, db.customers.ThumbSmall,
        db.customers.FirstName, db.customers.LastName, db.customers.DateOfBirth, 
        db.overduepayments.PaymentDate, db.overduepayments.Amount, db.overduepayments.Description, 
        left=db.customers.on(db.customers.id==db.overduepayments.customers_id), 
        orderby=db.overduepayments.PaymentDate|db.customers.FirstName)
        
    sum_total = db(query).select(sum).first()[sum]
    if sum_total == None:
        sum_total = '0'
    else:
        sum_total = unicode(sum_total)
    this_week_total = DIV(T("Total Amount") + ": " + sum_total, _class='right')
    this_week_total_customers = DIV(T("Total customers" + ": " + unicode(len(rows))), _class='left')
    
    table = get_table(rows)
    this_week = DIV(table, style)
    
    # due later
    enddate = today + datetime.timedelta(days=7)
    query = (db.overduepayments.Paid == 0) & (db.overduepayments.PaymentDate > enddate) 
    rows = db(query).select(db.customers.id, db.customers.ActiveMember, db.customers.ThumbSmall,
        db.customers.FirstName, db.customers.LastName, db.customers.DateOfBirth, 
        db.overduepayments.PaymentDate, db.overduepayments.Amount, db.overduepayments.Description, 
        left=db.customers.on(db.customers.id==db.overduepayments.customers_id), 
        orderby=db.overduepayments.PaymentDate|db.customers.FirstName)
        
    sum_total = db(query).select(sum).first()[sum]
    if sum_total == None:
        sum_total = '0'
    else:
        sum_total = unicode(sum_total)
    later_total = DIV(T("Total Amount") + ": " + sum_total, _class='right')
    later_total_customers = DIV(T("Total customers" + ": " + unicode(len(rows))), _class='left')
    
    table = get_table(rows)
    later = DIV(table, style)
    
    overdue_title = T("Overdue")
    this_title = T("Due this week")
    later_title = T("Due later")
    
    if request.user_agent()['is_mobile']:
        right_sidebar_enabled=False
    else:
        right_sidebar_enabled=True
    
    return dict(overdue=overdue, overdue_total=overdue_total, overdue_total_customers=overdue_total_customers, overdue_title=overdue_title,
        this_week=this_week, this_week_total=this_week_total, this_week_total_customers=this_week_total_customers, this_title=this_title,
        later_total=later_total, later_total_customers=later_total_customers, later=later, later_title=later_title,
        right_sidebar_enabled=right_sidebar_enabled)
        
@auth.requires_login()
def trialclasses_show_current():
    session.overview_tc_year = None
    session.overview_tc_month = None
    
    redirect(URL('trialclasses'))

@mobilize
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'default_trialclasses'))
def trialclasses():
    response.title = T("Trial classes")
    session.customers_back = 'trialclasses'
    
    today = datetime.date.today()
    if 'year' in request.vars:
        year = int(request.vars['year'])
    elif not session.overview_tc_year is None:
        year = session.overview_tc_year
    else:
        year = today.year
    session.overview_tc_year = year
    if 'month' in request.vars:
        month = int(request.vars['month'])
    elif not session.overview_tc_month is None:
        month = session.overview_tc_month
    else:
        month = today.month
    session.overview_tc_month = month
        
    style = STYLE('.italic {font-style: italic;} .bold {font-weight: bold;}')
        
    date = datetime.date(year,month,1)
    firstdaythismonth = date
    next_month = date.replace(day=28) + datetime.timedelta(days=4)  # this will never fail
    lastdaythismonth = next_month - datetime.timedelta(days=next_month.day)
    
    form_subtitle = _generate_form_subtitle(month, year, request.function)
    response.subtitle = form_subtitle['subtitle']
    form = form_subtitle['form']
    form_title = form_subtitle['title']
    
    fields = [ db.customers.id, 
               db.customers.ActiveMember, 
               db.customers.ThumbSmall,
               db.classes_attendance.ClassDate, 
               db.classes.TrialPrice, 
               db.customers.FirstName, 
               db.customers.LastName, 
               db.classes_attendance.classes_id, 
               db.customers_subscriptions.Startdate ]
    rows = db.executesql("""SELECT cu.id, 
                                   cu.ActiveMember, 
                                   cu.ThumbSmall, 
                                   att.ClassDate, 
                                   cla.trialprice, 
                                   cu.firstname, 
                                   cu.lastname, 
                                   att.classes_id, 
                                   chk.startdate
        FROM customers cu 
        LEFT JOIN classes_attendance att ON att.customers_id = cu.id 
        LEFT JOIN classes cla ON att.classes_id = cla.id 
        LEFT JOIN
        (SELECT min(startdate) startdate, 
                customers_id 
         FROM customers_subscriptions GROUP BY customers_id) chk
        ON chk.customers_id = cu.id
        WHERE att.attendancetype = 1
        AND att.ClassDate >= %s AND att.ClassDate <= %s 
        ORDER BY att.ClassDate, att.classes_id, cu.firstname, cu.lastname DESC""", 
        (firstdaythismonth, lastdaythismonth), fields=fields)
    
    ms_total = 0
    trialprice_total = 0
    for row in rows:
        if row.customers_subscriptions.Startdate != None:
            ms_total += 1
        if row.classes.TrialPrice != None:
            trialprice_total += row.classes.TrialPrice
            
    ms_total = DIV(T("Total with membership") + ": " + unicode(ms_total), _class='left')
    trialprice_total = DIV(T("Total amount") + ": " + unicode(trialprice_total), _class='left')
    trial_total = DIV(T("Total: " + unicode(len(rows))), _class='right')
    
    table = TABLE(_class="os-customer_table")
    table.append(TR(TH(), # image
                    TH(), # name,
                    TH(T("Start membership")),
                    TH(T("Price")),
                    _class='os-table_header'))
    for row in rows.render():
        table.append(TR(TD(row.customers.ThumbSmall,
                           _class='os-customer_image_td'),
                        TD(DIV(row.customers.FirstName,
                                ' ',
                                row.customers.LastName,
                                _class="os-customer_name"),
                            row.classes_attendance.ClassDate,
                            ' ', row.classes_attendance.classes_id),
                        TD(row.customers_subscriptions.Startdate),
                        TD(row.classes.TrialPrice),
                        _id=row.customers.id,
                        _class='os-clickable'))    
    
    trialclasses = DIV(table, style)
    
    if request.user_agent()['is_mobile']:
        left_sidebar_enabled=True
        right_sidebar_enabled=False
    else:
        left_sidebar_enabled=False
        right_sidebar_enabled=True
    
    return dict(form=form, form_title=form_title, trialclasses=trialclasses, trial_total=trial_total, ms_total=ms_total, 
        trialprice_total=trialprice_total,
        left_sidebar_enabled=left_sidebar_enabled, right_sidebar_enabled=right_sidebar_enabled)
        
@auth.requires_login()
def dropinclasses_show_current():
    session.overview_dic_year = None
    session.overview_dic_month = None
    
    redirect(URL('dropinclasses'))
        
@mobilize
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'default_dropinclasses'))
def dropinclasses():
    response.title = T("Drop in classes")
    session.customers_back = 'dropinclasses'

    today = datetime.date.today()
    if 'year' in request.vars:
        year = int(request.vars['year'])
    elif not session.overview_dic_year is None:
        year = session.overview_dic_year
    else:
        year = today.year
    session.overview_dic_year = year
    if 'month' in request.vars:
        month = int(request.vars['month'])
    elif not session.overview_dic_month is None:
        month = session.overview_dic_month
    else:
        month = today.month
    session.overview_dic_month = month
        
    style = STYLE('.italic {font-style: italic;} .bold {font-weight: bold;}')
    
    date = datetime.date(year,month,1)
    firstdaythismonth = date
    next_month = date.replace(day=28) + datetime.timedelta(days=4)  # this will never fail
    lastdaythismonth = next_month - datetime.timedelta(days=next_month.day)
    
    form_subtitle = _generate_form_subtitle(month, year, request.function)
    response.subtitle = form_subtitle['subtitle']
    form = form_subtitle['form']
    form_title = form_subtitle['title']
    
    query = (db.classes_attendance.ClassDate >= firstdaythismonth) & \
            (db.classes_attendance.ClassDate <= lastdaythismonth) & \
            (db.classes_attendance.AttendanceType == 2)
    
    rows = db(query).select(db.customers.id, 
                            db.customers.ActiveMember, 
                            db.customers.ThumbSmall,
                            db.customers.FirstName, 
                            db.customers.LastName, 
                            db.customers.DateOfBirth, 
                            db.classes_attendance.classes_id, 
                            db.classes_attendance.ClassDate, 
                            db.classes.DropInPrice,
        left=[db.customers.on(db.customers.id==db.classes_attendance.customers_id), 
              db.classes.on(db.classes.id==db.classes_attendance.classes_id)],
        orderby=db.classes_attendance.ClassDate | \
                db.classes_attendance.classes_id | \
                db.customers.FirstName)
        
    total_price = 0
    for row in rows:
        if row.classes.DropInPrice != None:
            total_price += row.classes.DropInPrice
    
    price_total = DIV(T("Total amount") + ": " + \
        unicode(total_price), _class='right')
    classes_total = DIV(T("Total classes") + ": " + \
        unicode(len(rows)), _class='left')
    
    table = TABLE(_class="os-customer_table")
    table.append(TR(TH(), # image
                    TH(), # name,
                    TH(db.classes_attendance.ClassDate.label),
                    TH(db.classes_attendance.classes_id.label),
                    TH(db.classes.DropInPrice.label),
                    _class='os-table_header'))
    for row in rows.render():
        table.append(TR(TD(row.customers.ThumbSmall,
                           _class='os-customer_image_td'),
                        TD(DIV(row.customers.FirstName,
                                ' ',
                                row.customers.LastName,
                                _class="os-customer_name")),
                        TD(row.classes_attendance.ClassDate),
                        TD(row.classes_attendance.classes_id),
                        TD(row.classes.DropInPrice),
                        _id=row.customers.id,
                        _class='os-clickable'))
        
    dropinclasses = DIV(table, style)
    
    if request.user_agent()['is_mobile']:
        left_sidebar_enabled=True
        right_sidebar_enabled=False
    else:
        left_sidebar_enabled=False
        right_sidebar_enabled=True
    
    return dict(form=form, form_title=form_title, dropinclasses=dropinclasses, price_total=price_total, classes_total=classes_total, 
        right_sidebar_enabled=right_sidebar_enabled, left_sidebar_enabled=left_sidebar_enabled)
        
@auth.requires_login()
def classcards_show_current():
    session.overview_cc_year = None
    session.overview_cc_month = None
    
    redirect(URL('classcards'))
    
@mobilize
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'default_classcards'))
def classcards():
    response.title = T("Class cards")
    session.customers_back = 'classcards'
    
    today = datetime.date.today()
    if 'year' in request.vars:
        year = int(request.vars['year'])
    elif not session.overview_cc_year is None:
        year = session.overview_cc_year
    else:
        year = today.year
    session.overview_cc_year = year
    if 'month' in request.vars:
        month = int(request.vars['month'])
    elif not session.overview_cc_month is None:
        month = session.overview_cc_month
    else:
        month = today.month
    session.overview_cc_month = month
        
    style = STYLE('.italic {font-style: italic;} .bold {font-weight: bold;}')
        
    date = datetime.date(year,month,1)
    firstdaythismonth = date
    next_month = date.replace(day=28) + datetime.timedelta(days=4)  # this will never fail
    lastdaythismonth = next_month - datetime.timedelta(days=next_month.day)
    
    form_subtitle = _generate_form_subtitle(month, year, request.function)
    response.subtitle = form_subtitle['subtitle']
    form = form_subtitle['form']
    form_title = form_subtitle['title']
    
    query = (db.customers_classcards.Startdate >= firstdaythismonth) & \
            (db.customers_classcards.Startdate <= lastdaythismonth)
    
    rows = db(query).select(db.customers.id, 
                            db.customers.ActiveMember, 
                            db.customers.ThumbSmall,
                            db.customers.FirstName, 
                            db.customers.LastName, 
                            db.customers.DateOfBirth, 
                            db.customers_classcards.id,
                            db.customers_classcards.Startdate, 
                            db.customers_classcards.Enddate, 
                            db.customers_classcards.Classes, 
                            db.customers_classcards.Price, 
        left=db.customers.on(db.customers.id==\
                             db.customers_classcards.customers_id), 
        orderby=db.customers_classcards.Startdate|~db.customers.FirstName)
        
    total_price = 0
    for row in rows:
        total_price += row.customers_classcards.Price
    
    price_total = DIV(T("Total amount") + ": " + \
        unicode(total_price), _class='right')
    cards_total = DIV(T("Total cards") + ": " + \
        unicode(len(rows)), _class='left')
    
    table = TABLE(_class="os-customer_table")
    table.append(TR(TH(), # image
                    TH(), # name,
                    TH(db.customers_classcards.Startdate.label),
                    TH(db.customers_classcards.Enddate.label),
                    TH(db.customers_classcards.Price.label),
                    _class='os-table_header'))
                 
    for row in rows.render():
        table.append(TR(TD(row.customers.ThumbSmall,
                           _class='os-customer_image_td'),
                        TD(DIV(row.customers.FirstName,
                                ' ',
                                row.customers.LastName,
                                _class="os-customer_name"),
                            T("Card"), ' ', 
                            unicode(row.customers_classcards.id)),
                        TD(row.customers_classcards.Startdate),
                        TD(row.customers_classcards.Enddate),
                        TD(row.customers_classcards.Price),
                        _id=row.customers.id,
                        _class='os-clickable'))
    
    classcards = DIV(table, style)
    
    if request.user_agent()['is_mobile']:
        left_sidebar_enabled=True
        right_sidebar_enabled=False
    else:
        left_sidebar_enabled=False
        right_sidebar_enabled=True
    
    return dict(form=form, 
                form_title=form_title, 
                classcards=classcards, 
                cards_total=cards_total, 
                price_total=price_total, 
                left_sidebar_enabled=left_sidebar_enabled, 
                right_sidebar_enabled=right_sidebar_enabled)
        
@auth.requires_login()
def subscriptions_count_totals(rows):
    # totals table begin
    index = 0
    table_row = 0
    previous_membership = None
    current_total = 1
    totals_names = []
    totals = []
    for row in rows.render():
        current_membership = row.customers_subscriptions.school_subscriptions_id
        if previous_membership == current_membership or index == 0:
            current_total += 1
        else: # all other runs
            totals.append([previous_membership, current_total])
            current_total = 1
            table_row += 1
        
        previous_membership = current_membership
        index += 1
        
    totals = sorted(totals, key=lambda tup: tup[1], reverse=True)
    
    totals_table = TABLE(_class='os-overview_subscriptions_totals')
    for total in totals:
        totals_table.append(TR(TD(total[1], _class='total'), TD(total[0])))
        
    title = H4(T("Totals"))
        
    return DIV(HR(), title, totals_table)
        
@auth.requires_login()
def subscriptions_show_current():
    '''
        Reset the session variables for month and year to show the current 
        month.
    '''
    session.overview_subscriptions_year = None
    session.overview_subscriptions_month = None
    
    redirect(URL(session.overview_subscriptions))
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'default_subscriptions'))
def subscriptions_new():
    response.title = T("New subscriptions")
    session.customers_back = 'subscriptions_new'
    response.view = 'default/subscriptions.html'
    
    # set the session vars for year/month
    subscriptions_set_date()
    
    style = STYLE('.italic {font-style: italic;} .bold {font-weight: bold;}')
    
    date = datetime.date(session.overview_subscriptions_year,
                         session.overview_subscriptions_month,
                         1)
    firstdaythismonth = date
    next_month = date.replace(day=28) + datetime.timedelta(days=4)  # this will never fail
    lastdaythismonth = next_month - datetime.timedelta(days=next_month.day)
    
    form_subtitle = _generate_form_subtitle(session.overview_subscriptions_month, 
                                            session.overview_subscriptions_year, 
                                            request.function)
    response.subtitle = form_subtitle['subtitle']
    form = form_subtitle['form']
    form_title = form_subtitle['title']

    fields = [ db.customers.id, 
               db.customers.ActiveMember, 
               db.customers.ThumbSmall, 
               db.customers.FirstName, 
               db.customers.LastName, 
               db.customers.DateOfBirth, 
               db.customers_subscriptions.school_subscriptions_id, 
               db.customers_subscriptions.Startdate ]
    rows = db.executesql("""SELECT cu.id, 
                                   cu.activemember, 
                                   cu.thumbsmall, 
                                   cu.firstname, 
                                   cu.lastname, 
                                   cu.dateofbirth, 
                                   ms.school_subscriptions_id, 
                                   ms.startdate 
                            FROM customers cu
                            LEFT JOIN
                                (SELECT customers_id, 
                                        startdate, 
                                        enddate, 
                                        school_subscriptions_id 
                                 FROM customers_subscriptions 
                                 GROUP BY customers_id) ms
                            ON cu.id = ms.customers_id
                            LEFT JOIN school_subscriptions ssu
                            ON ssu.id = ms.school_subscriptions_id
                            , 
                            (SELECT min(startdate) startdate, 
                                                   customers_id 
                             FROM customers_subscriptions 
                             GROUP BY customers_id) chk
                            WHERE chk.startdate = ms.startdate AND 
                                  chk.customers_id = ms.customers_id AND 
                                  ms.startdate >= %s AND ms.startdate <= %s 
                            ORDER BY ssu.Name,
                                     cu.firstname, 
                                     cu.lastname 
                                     DESC""", \
            (firstdaythismonth, lastdaythismonth), fields=fields)

    total = T("Total: " + unicode(len(rows)))
    
    table = TABLE(_class="os-customer_table")
    table.append(TR(TH(), # image
                    TH(), # name,
                    TH(db.customers_subscriptions.Startdate.label),
                    TH(db.customers_subscriptions.school_subscriptions_id.label),
                    _class='os-table_header'))
                    
    for row in rows.render():
        table.append(TR(TD(row.customers.ThumbSmall,
                           _class='os-customer_image_td'),
                        TD(DIV(row.customers.FirstName,
                                ' ',
                                row.customers.LastName,
                                _class="os-customer_name")),
                        TD(row.customers_subscriptions.Startdate),
                        TD(row.customers_subscriptions.school_subscriptions_id),
                        _id=row.customers.id,
                        _class='os-clickable'))
        
    # totals table begin
    totals_table = subscriptions_count_totals(rows)
    # totals table end
        
    content = DIV(table, style, DIV(total, _class='right'))
    form = DIV(form_title, form)
    menu = subscriptions_get_menu(request.function)
    
    return dict(content=content, totals_table=totals_table, 
                 form=form, menu=menu)
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'default_subscriptions'))
def subscriptions_stopped():
    response.title = T("Stopped subscriptions")
    session.customers_back = 'subscriptions_stopped'
    response.view = 'default/subscriptions.html'
    
    # set the session vars for year/month
    subscriptions_set_date()
    
    style = STYLE('.italic {font-style: italic;} .bold {font-weight: bold;}')
    
    
    date = datetime.date(session.overview_subscriptions_year,
                         session.overview_subscriptions_month,
                         1)
                         
    firstdaythismonth = date
    next_month = date.replace(day=28) + datetime.timedelta(days=4)  # this will never fail
    lastdaythismonth = next_month - datetime.timedelta(days=next_month.day)
    
    form_subtitle = _generate_form_subtitle(session.overview_subscriptions_month, 
                                            session.overview_subscriptions_year, 
                                            request.function)
    response.subtitle = form_subtitle['subtitle']
    form = form_subtitle['form']
    form_title = form_subtitle['title']
    
    fields = [ db.customers.id, 
               db.customers.ActiveMember, 
               db.customers.ThumbSmall, 
               db.customers.FirstName, 
               db.customers.LastName, 
               db.customers.DateOfBirth, 
               db.customers_subscriptions.school_subscriptions_id, 
               db.customers_subscriptions.Enddate ]
    rows = db.executesql("""SELECT cu.id, 
                                   cu.activemember, 
                                   cu.thumbsmall, 
                                   cu.firstname, 
                                   cu.lastname, 
                                   cu.dateofbirth, 
                                   ms.school_subscriptions_id, 
                                   ms.enddate
                            FROM customers cu
                            LEFT JOIN
                                (SELECT customers_id, 
                                        startdate, 
                                        enddate, 
                                        school_subscriptions_id 
                                 FROM customers_subscriptions) ms
                            ON cu.id = ms.customers_id
                            LEFT JOIN school_subscriptions ssu
                            ON ssu.id = ms.school_subscriptions_id, 
                                (SELECT max(startdate) startdate, 
                                        customers_id 
                                 FROM customers_subscriptions 
                                 GROUP BY customers_id) chk
                            WHERE chk.startdate = ms.startdate AND 
                                  ms.customers_id = chk.customers_id AND 
                                  ms.enddate >= %s AND ms.enddate <= %s 
                                  AND ms.enddate IS NOT NULL 
                            ORDER BY ssu.Name,
                                     cu.firstname, 
                                     cu.lastname 
                                     DESC""", \
                        (firstdaythismonth, lastdaythismonth), fields=fields)

    total = T("Total: " + unicode(len(rows)))
    
    table = TABLE(_class="os-customer_table")
    table.append(TR(TH(), # image
                    TH(), # name,
                    TH(db.customers_subscriptions.Enddate.label),
                    TH(db.customers_subscriptions.school_subscriptions_id.label),
                    _class='os-table_header'))
    for row in rows.render():
        table.append(TR(TD(row.customers.ThumbSmall,
                           _class='os-customer_image_td'),
                        TD(DIV(row.customers.FirstName,
                                ' ',
                                row.customers.LastName,
                                _class="os-customer_name")),
                        TD(row.customers_subscriptions.Enddate),
                        TD(row.customers_subscriptions.school_subscriptions_id),
                        _id=row.customers.id,
                        _class='os-clickable'))
        
    # totals table begin
    totals_table = subscriptions_count_totals(rows)
    # totals table end
        
    content = DIV(table, style, DIV(total, _class='right'))
    menu = subscriptions_get_menu(request.function)
    form = DIV(form_title, form)
    
    return dict(content=content, totals_table=totals_table,
                 form=form, menu=menu)
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'default_subscriptions'))
def subscriptions_paused():
    response.title = T("Paused subscriptions")
    session.customers_back = 'subscriptions_paused'
    response.view = 'default/subscriptions.html'
    
    # set the session vars for year/month
    subscriptions_set_date()
    
    style = STYLE('.italic {font-style: italic;} .bold {font-weight: bold;}')
    
    date = datetime.date(session.overview_subscriptions_year,
                         session.overview_subscriptions_month,
                         1)
    firstdaythismonth = date
    next_month = date.replace(day=28) + datetime.timedelta(days=4)  # this will never fail
    lastdaythismonth = next_month - datetime.timedelta(days=next_month.day)
    
    form_subtitle = _generate_form_subtitle(session.overview_subscriptions_month, 
                                            session.overview_subscriptions_year, 
                                            request.function)
    response.subtitle = form_subtitle['subtitle']
    form = form_subtitle['form']
    form_title = form_subtitle['title']
    
    # Paused subscriptions
    query = (db.customers_subscriptions_paused.Startdate <= \
                                               lastdaythismonth) & \
            ((db.customers_subscriptions_paused.Enddate >= \
                                                firstdaythismonth) | \
             (db.customers_subscriptions_paused.Enddate == None))
    rows = db(query).select(db.customers.id, 
                            db.customers.ActiveMember, 
                            db.customers.ThumbSmall, 
                            db.customers.FirstName, 
                            db.customers.LastName, 
                            db.customers_subscriptions_paused.Description, 
        left=[ db.customers_subscriptions.on(\
                    db.customers_subscriptions.id==\
                    db.customers_subscriptions_paused.customers_subscriptions_id), 
               db.customers.on(db.customers.id==\
                               db.customers_subscriptions.customers_id)])
    total = T("Total: " + unicode(len(rows)))
    
    table = TABLE(_class="os-customer_table")
    table.append(TR(TH(), # image
                    TH(), # name,
                    TH(db.customers_subscriptions_paused.Description.label),
                    _class='os-table_header'))
    for row in rows.render():
        table.append(TR(TD(row.customers.ThumbSmall,
                           _class='os-customer_image_td'),
                        TD(DIV(row.customers.FirstName,
                                ' ',
                                row.customers.LastName,
                                _class="os-customer_name")),
                        TD(row.customers_subscriptions_paused.Description),
                        _id=row.customers.id,
                        _class='os-clickable'))
    paused_subscriptions = DIV(table, style)
        
    content = DIV(table, style, DIV(total, _class='right'))
    menu = subscriptions_get_menu(request.function)
    form = DIV(form_title, form)
        
    return dict(content=content, form=form, menu=menu)
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'default_subscriptions'))
def subscriptions_overview():
    '''
        Lists current subscriptions by type
    '''
    response.title = T("Subscriptions overview")
    response.view = 'default/subscriptions.html'
    
    # set the session vars for year/month
    subscriptions_set_date()
    
    # get first and last day of this month
    date = datetime.date(session.overview_subscriptions_year,
                         session.overview_subscriptions_month,
                         1)
    firstdaythismonth = date
    next_month = date.replace(day=28) + datetime.timedelta(days=4)  # this will never fail
    lastdaythismonth = next_month - datetime.timedelta(days=next_month.day)
    
    # get month/year selection form and subtitle
    form_subtitle = _generate_form_subtitle(session.overview_subscriptions_month, 
                                            session.overview_subscriptions_year, 
                                            request.function)
    response.subtitle = form_subtitle['subtitle']
    form = form_subtitle['form']
    form_title = form_subtitle['title']
    form = DIV(form_title, form)
    
    # check if we need a location filter
    
    if not session.show_location:
        location_filter = ''
    else:
        # add location filter form
        if 'locationID' in request.vars:
            locationID = request.vars['locationID']
        else:
            locationID = ''
        
        location_filter = SQLFORM.factory(
            Field('locationID', 
                  requires=IS_IN_DB(db,'school_locations.id', '%(Name)s',
                                    zero=T('All locations')),
                  default=locationID,
                  label=T('')))
                  
        # sumbit form on change
        selects = location_filter.elements('select')
        for select in selects:
            select.attributes['_onchange'] = "this.form.submit();"
            
        location_filter = DIV(location_filter.custom.begin,
                              location_filter.custom.widget.locationID,
                              location_filter.custom.end)
    
    
    count = db.customers_subscriptions.school_subscriptions_id.count()
    query = (db.customers_subscriptions.Startdate <= lastdaythismonth) & \
            ((db.customers_subscriptions.Enddate >= firstdaythismonth) | \
             (db.customers_subscriptions.Enddate == None))
             
    if session.show_location:
        if locationID:
            query &= (db.customers.school_locations_id == locationID)
    
    rows = db(query).select(db.customers_subscriptions.ALL, 
                            db.school_subscriptions.ALL,
                            count,
                            left=[db.school_subscriptions.on(
                                db.customers_subscriptions.school_subscriptions_id == \
                                db.school_subscriptions.id),
                               db.customers.on(
                                db.customers_subscriptions.customers_id == \
                                db.customers.id)],
                            groupby=db.customers_subscriptions.school_subscriptions_id,
                            orderby=~count)
    
    revenue_total = 0
    subscriptions_total = 0
    table = TABLE(TR(TH(T('Membership')), 
                     TH(T('Monthly fee'), _class='text-right'),
                     TH(T('Customers'), _class='text-right'),
                     TH(T('Revenue'), _class='text-right')))
    for row in rows:
        monthly_fee = format(row.school_subscriptions.MonthlyFee, '.2f')
        revenue = row.school_subscriptions.MonthlyFee * row[count]
        
        table.append(TR(TD(row.school_subscriptions.Name),
                        TD(monthly_fee,
                           _class='text-right'),
                        TD(row[count],
                           _class='text-right'),
                        TD(format(revenue, '.2f'),
                           _class='text-right')))
                           
        subscriptions_total += row[count]
        revenue_total += revenue
    
    table.append(TR(TD(T('Total')),
                    TD(),
                    TD(subscriptions_total,
                       _class='text-right'),
                    TD(format(revenue_total, '.2f'),
                       _class='text-right'),
                 _class='total'))
                    
    
    content = DIV(HR(), location_filter, table)
                        
    menu = subscriptions_get_menu(request.function)
                        
    return dict(content=content,
                form=form,
                menu=menu)
    
@auth.requires_login()
def subscriptions_set_date():
    '''
        This function takes the request.vars as a argument and 
    '''
    today = datetime.date.today()
    if 'year' in request.vars:
        year = int(request.vars['year'])
    elif not session.overview_subscriptions_year is None:
        year = session.overview_subscriptions_year
    else:
        year = today.year
    session.overview_subscriptions_year = year
    if 'month' in request.vars:
        month = int(request.vars['month'])
    elif not session.overview_subscriptions_month is None:
        month = session.overview_subscriptions_month
    else:
        month = today.month
    session.overview_subscriptions_month = month
    
    session.overview_subscriptions = request.function
    
@auth.requires_login()
def alternative_payments_show_current():
    session.overview_ap_year = None
    session.overview_ap_month = None
    
    redirect(URL('alternative_payments'))
    
@mobilize
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'default_alternativepayments'))
def alternative_payments():
    response.title = T("Alternative payments")
    session.customers_back = 'alternativepayments'
    
    today = datetime.date.today()
    if 'year' in request.vars:
        year = int(request.vars['year'])
    elif not session.overview_ap_year is None:
        year = session.overview_ap_year
    else:
        year = today.year
    session.overview_ap_year = year
    if 'month' in request.vars:
        month = int(request.vars['month'])
    elif not session.overview_ap_month is None:
        month = session.overview_ap_month
    else:
        month = today.month
    session.overview_ap_month = month
        
    style = STYLE('.italic {font-style: italic;} .bold {font-weight: bold;}')
    
    date = datetime.date(year,month,1)
    
    form_subtitle = _generate_form_subtitle(month, year, request.function)
    response.subtitle = form_subtitle['subtitle']
    form = form_subtitle['form']
    form_title = form_subtitle['title']
    
    # collection
    query = (db.alternativepayments.PaymentYear==date.year) & \
            (db.alternativepayments.PaymentMonth==date.month) & \
            ((db.payment_categories.CategoryType != 1) | \
             (db.payment_categories.CategoryType == None))
    rows = db(query).select(db.customers.id, 
                            db.customers.ActiveMember, 
                            db.customers.ThumbSmall,
                            db.customers.FirstName, 
                            db.customers.LastName, 
                            db.customers.DateOfBirth, 
                            db.alternativepayments.Amount, 
                            db.alternativepayments.payment_categories_id, 
                            db.alternativepayments.Description, 
        left=[db.customers.on(db.customers.id==db.alternativepayments.customers_id), 
            db.payment_categories.on(db.payment_categories.id==\
                db.alternativepayments.payment_categories_id)],
        orderby=db.alternativepayments.payment_categories_id|\
                db.customers.FirstName|db.customers.LastName)
                
    col_total = DIV(T("Total: " + unicode(len(rows))), _class='right')
    
    table = TABLE(_class="os-customer_table")
    table.append(TR(TH(), # image
                    TH(), # name,
                    TH(db.alternativepayments.Amount.label),
                    TH(db.alternativepayments.payment_categories_id.label),
                    TH(db.alternativepayments.Description.label),
                    _class='os-table_header'))
                 
    for row in rows.render():
        table.append(TR(TD(row.customers.ThumbSmall,
                           _class='os-customer_image_td'),
                        TD(DIV(row.customers.FirstName,
                                ' ',
                                row.customers.LastName,
                                _class="os-customer_name"),
                            BR()),
                        TD(row.alternativepayments.Amount),
                        TD(row.alternativepayments.payment_categories_id),
                        TD(row.alternativepayments.Description),
                        _id=row.customers.id,
                        _class='os-clickable'))
    
    collection = DIV(table, style)
    # payments
    query = (db.alternativepayments.PaymentYear==date.year) & \
            (db.alternativepayments.PaymentMonth==date.month) & \
            (db.payment_categories.CategoryType == 1)
    rows = db(query).select(db.customers.id, 
                            db.customers.ActiveMember, 
                            db.customers.ThumbSmall,
                            db.customers.FirstName, 
                            db.customers.LastName, 
                            db.customers.DateOfBirth, 
                            db.alternativepayments.Amount, 
                            db.alternativepayments.payment_categories_id, 
                            db.alternativepayments.Description, 
        left=[db.customers.on(db.customers.id==db.alternativepayments.customers_id), 
            db.payment_categories.on(db.payment_categories.id==\
                db.alternativepayments.payment_categories_id)], 
        orderby=db.alternativepayments.payment_categories_id|\
            db.customers.FirstName|db.customers.LastName)
    pay_total = DIV(T("Total: " + unicode(len(rows))), _class='right')
    
    table = TABLE(_class="os-customer_table")
    table.append(TR(TH(), # image
                    TH(), # name,
                    TH(db.alternativepayments.Amount.label),
                    TH(db.alternativepayments.payment_categories_id.label),
                    TH(db.alternativepayments.Description.label),
                    _class='os-table_header'))
                 
    for row in rows.render():
        table.append(TR(TD(row.customers.ThumbSmall,
                           _class='os-customer_image_td'),
                        TD(DIV(row.customers.FirstName,
                                ' ',
                                row.customers.LastName,
                                _class="os-customer_name"),
                            BR()),
                        TD(row.alternativepayments.Amount),
                        TD(row.alternativepayments.payment_categories_id),
                        TD(row.alternativepayments.Description),
                        _id=row.customers.id,
                        _class='os-clickable'))
                        
    payments = DIV(table, style)
    
    pay_title = T("Payments")
    col_title = T("Collection")
    export_title = T("Export")
    form_title = T("Which month?")
    
    ### export buttons begin
    session.export_year = year
    session.export_month = month
    col_export = A(SPAN(_class="buttontext button", _title=T("Collection")), SPAN(_class="icon download icon-download"), " " + T("Collection"), \
    _class="button btn", _href=URL('customers', 'export_questions', vars=dict(export='collection'), user_signature=True), _target='blank')
    pay_export = A(SPAN(_class="buttontext button", _title=T("Payments")), SPAN(_class="icon download icon-download"), " " + T("Payments"), \
    _class="button btn", _href=URL('customers', 'export_questions', vars=dict(export='payment'), user_signature=True), _target='blank')
    
    ### export buttons end

    if request.user_agent()['is_mobile']:
        left_sidebar_enabled=True
        right_sidebar_enabled=False
    else:
        left_sidebar_enabled=False
        right_sidebar_enabled=True
    
    return dict(form=form, collection=collection, col_total=col_total, col_title=col_title, col_export=col_export, 
        payments=payments, pay_total=pay_total, pay_title=pay_title, pay_export=pay_export, 
        export_title=export_title, form_title=form_title, 
        left_sidebar_enabled=left_sidebar_enabled, right_sidebar_enabled=right_sidebar_enabled)

####### Don't mess with the functions below ##########

def user():
    """
    exposes:
    http://..../[app]/default/user/login
    http://..../[app]/default/user/logout
    http://..../[app]/default/user/register
    http://..../[app]/default/user/profile
    http://..../[app]/default/user/retrieve_password
    http://..../[app]/default/user/change_password
    use @auth.requires_login()
        @auth.requires_membership('group name')
        @auth.requires_permission('read','table name',record_id)
    to decorate functions that need access control
    """
    response.title = T("OpenStudio")
    response.subtitle = T("")
    
    if request.args[0] == 'profile' and not db.auth_user(auth.user.id).teachers_id is None:
        redirect(URL('profile_teacher'))
    
    return dict(form=auth())
    
def profile_teacher():
    '''
        Teachers can change their own info in the teachers table. 
    '''
    response.subtitle = T("")
    user_id = auth.user.id
    teacher_id = db.auth_user(user_id).teachers_id
    
    response.view = 'general/only_content.html'
    
    user_helpers= User_helpers()
    
    crud.messages.submit_button = T("Appy changes")
    crud.messages.record_updated = T("Updated profile")
    crud.settings.update_onaccept = [user_helpers.teacher_edit_auth_user_sync]
    crud.settings.update_deletable = False
    form = crud.update(db.teachers, teacher_id)
    
    content = DIV(H2(T("Profile")), form)
    
    return dict(content=content)
    

    return(locals())

def user():
    """
    exposes:
    http://..../[app]/default/user/login
    http://..../[app]/default/user/logout
    http://..../[app]/default/user/register
    http://..../[app]/default/user/profile
    http://..../[app]/default/user/retrieve_password
    http://..../[app]/default/user/change_password
    http://..../[app]/default/user/manage_users (requires membership in
    use @auth.requires_login()
        @auth.requires_membership('group name')
        @auth.requires_permission('read','table name',record_id)
    to decorate functions that need access control
    """
    return dict(form=auth())


@cache.action()
def download():
    """
    allows downloading of uploaded files
    http://..../[app]/default/download/[filename]
    """
    return response.download(request, db)


def call():
    """
    exposes services. for example:
    http://..../[app]/default/call/jsonrpc
    decorate with @services.jsonrpc the functions to expose
    supports xml, json, xmlrpc, jsonrpc, amfrpc, rss, csv
    """
    return service()


#@auth.requires_login() 
#def api():
    #"""
    #this is example of API with access control
    #WEB2PY provides Hypermedia API (Collection+JSON) Experimental
    #"""
    #from gluon.contrib.hypermedia import Collection
    #rules = {
        #'<tablename>': {'GET':{},'POST':{},'PUT':{},'DELETE':{}},
        #}
    #return Collection(db).process(request,response,rules)
