# coding: utf8

'''
    This controller contains functions shared by the classes, employees and 
    workshop schedules.
'''

from general_helpers import get_button


@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'school_holidays'))
def holidays():
    '''
        List holidays
    '''
    response.title = T("Schedule")
    response.subtitle = T("Holidays")
    response.view = 'general/one_grid_with_sidebar.html'
    db.school_holidays.id.readable=False
    fields = [db.school_holidays.Description,
              db.school_holidays.Startdate,
              db.school_holidays.Enddate]
    links = [ lambda row: get_button('edit', 
                                      URL('holiday_edit', args=[row.id]), 
                                      T("Edit holiday")) ]
    grid = SQLFORM.grid(db.school_holidays, fields=fields, links=links,
        create=False,
        editable=False,
        details=False, 
        searchable=False, 
        csv=False, 
        ui = session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    
    add_url = URL('holiday_add')
    add = get_button('add', add_url, T("Add a holiday"))
    
    back = get_button('back', URL('classes', 'schedule'))
    console = grid.element('.web2py_console')
    console.insert(0, back)
    console.insert(1, add)
    
    return dict(grid=grid, right_sidebar_enabled=True)
    
@auth.requires_login()
def holiday_add():
    """
        This function shows an add page for a holiday
    """
    response.title = T("New holiday")
    response.subtitle = T('')
    response.view = 'general/only_content.html'
    
    db.school_holidays.Classes.readable = False
    db.school_holidays.Classes.writable = False
    db.school_holidays.Classes.default = True
    
    return_url = URL('holidays')
    
    crud.messages.submit_button = T("Add")
    crud.messages.record_created = T("Added holiday")
    crud.settings.create_next = return_url
    form = crud.create(db.school_holidays)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires_login()
def holiday_edit():
    """
        This function shows an edit page for a holiday
        request.args[0] is expected to be the holidayID (hoID)
    """
    hoID = request.args[0]
    response.title = T("Edit holiday")
    response.subtitle = T('')
    response.view = 'general/only_content.html'
    
    db.school_holidays.Classes.readable = False
    db.school_holidays.Classes.writable = False
    
    db.school_holidays.Classes.default = True
    
    return_url = URL('holidays')
    
    crud.messages.submit_button = T("Update")
    crud.messages.record_updated = T("Updated holiday")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.school_holidays, hoID)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
