# coding: utf8
    
from plugin_solidform import SOLIDFORM
from gluon.contrib.user_agent_parser import mobilize 

from general_helpers import get_button

@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'schoolproperties'))
def index():
    response.title = T("School")
    response.subtitle = T("")
    response.view = 'general/only_content.html'
    teachers = DIV(A(H3(T("Teachers")), 
        _href=URL('teachers', 'index')), 
        T("Here you can manage the list of teachers for your school, add their \
        contact information and keep track of who is able to teach what kind of classes."))
    mstypes = DIV(A(H3(T("Subscriptions")), 
        _href=URL('subscriptions')), 
        T("Here you can define which subscriptions your studio offers."))
    discovery = DIV(A(H3(T("Discovery")),
        _href=URL('discovery')), 
        T("Discovery allows you to keep track of how customers found your school. \
        The list you create here appears on the add and edit customer pages. Later you can get an overview "), 
        A(T("here"), _href=URL('stats', 'discovery')), ".")
    plevels = DIV(A(H3(T("Practice levels")), 
        _href=URL('school_levels')), 
        T("Here you can create a list of practice levels and \
        this list will appear on the add and edit customer pages in a drop down list."))
    classtypes = DIV(A(H3(T("Class types")), 
        _href=URL('classtypes')), 
        T("Class types are used when adding classes to the schedule, \
        it defines what kind of class will be taught. It's also used to assign teachers to certain types of classes. When a teacher is marked \
        as a teacher for a class type he or she isn't assigned to teach, a notification will be shown."))
    locations = DIV(A(H3(T("Locations")), 
        _href=URL('locations')), 
        T("Locations define locations for classes. These are also used when adding \
        classes to the schedule. This way external classes, eg. company classes etc. can also be shown in the schedule or it can be used for \
        schools with more than one room, etc."))
    keys = DIV(A(H3(T("Keys")), _href=URL('list_keys')), T("When keys, swipecards etc. are being distributed to customers, you can get an overview \
        here of who has which key. It's searchable by name and key number."))
    languages = DIV(A(H3(T("Languages")), _href=URL('languages')), 
        T("Languages are used with customers to keep track of the primary \
        language of a customer. This way you can always communicate using the \
        right language."))
    
    if request.user_agent()['is_mobile']:
        table = TABLE(TR(teachers), 
                      TR(mstypes), 
                      TR(discovery), 
                      TR(plevels), 
                      TR(classtypes), 
                      TR(locations), 
                      TR(keys),
                      TR(languages))
    else:
        table = TABLE(TR(TD(teachers, _width="33%"), TD(mstypes, _width="33%"), TD(plevels, _width="33%")), 
                    TR(TD(discovery, _width="33%"), TD(classtypes), TD(locations)), 
                    TR(TD(keys), TD(languages)),
                    _class="os-index_table")
    
    return dict(content=table)

@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'school_classtypes'))
def classtypes():
    response.title = T("School")
    response.subtitle = T("Class types")
    response.view = 'general/one_grid_with_sidebar.html'
    db.school_classtypes.id.readable=False
    fields = [db.school_classtypes.Name]
    links = [ lambda row: get_button('edit', 
                                     URL('classtype_edit', args=[row.id]), 
                                     T("Edit the name of this classtype")) ]
    maxtextlengths = {'school_classtypes.Name' : 50}
    grid = SQLFORM.grid(db.school_classtypes, 
                        maxtextlengths=maxtextlengths, 
                        fields=fields, 
                        links=links,
                        create=False,
                        editable=False, 
                        details=False, 
                        searchable=False, 
                        csv=False, 
                        orderby=db.school_classtypes.Name,
                        ui = session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    
    add_url = URL('classtype_add')
    add = get_button('add', add_url, T("Add a new class type"))
    
    back = get_button('back', URL('index'))
    console = grid.element('.web2py_console')
    console.insert(0, back)
    console.insert(1, add)
    
    return dict(grid=grid, right_sidebar_enabled=True)
    
@auth.requires_login()
def classtype_add():
    """
        This function shows an add page for a classtype
    """
    response.title = T("New class type")
    response.subtitle = T('')
    response.view = 'general/only_content.html'
    
    return_url = URL('classtypes')
    
    crud.messages.submit_button = T("Add")
    crud.messages.record_created = T("Added new class type")
    crud.settings.create_next = return_url
    form = crud.create(db.school_classtypes)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires_login()
def classtype_edit():
    """
        This function shows an edit page for a classtype
        request.args[0] is expected to be the school_classtypes_id (ctID)
    """
    ctID = request.args[0]
    response.title = T("Edit class type")
    response.subtitle = T('')
    response.view = 'general/only_content.html'
    
    return_url = URL('classtypes')
    
    crud.messages.submit_button = T("Update")
    crud.messages.record_updated = T("Updated class type")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.school_classtypes, ctID)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'school_discovery'))
def discovery():
    response.title = T("School")
    response.subtitle = T("Discovery")
    response.view = 'general/one_grid_with_sidebar.html'
    db.school_discovery.id.readable=False
    maxtextlengths = {'school_discovery.Name' : 60}
    fields = [db.school_discovery.Name]
    links = [ lambda row: get_button('edit', 
                                URL('discovery_edit', args=[row.id]), 
                                T("Edit the name of this way of discovery")) ]
    grid = SQLFORM.grid(db.school_discovery, fields=fields, links=links, 
        create=False,
        editable=False, 
        details=False, 
        searchable=False, 
        csv=False,
        maxtextlengths=maxtextlengths,
        orderby=db.school_discovery.Name,
        ui = session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    
    add_url = URL('discovery_add')
    add = get_button('add', add_url, T("Add a new way of discovery"))
    
    back = get_button('back', URL('index'))
    console = grid.element('.web2py_console')
    console.insert(0, back)
    console.insert(1, add)
    
    return dict(grid=grid, right_sidebar_enabled=True)
    
@auth.requires_login()
def discovery_add():
    """
        This function shows an add page for a way of discovery
    """
    response.title = T("New way of discovery")
    response.subtitle = T('')
    response.view = 'general/only_content.html'
    
    return_url = URL('discovery')
    
    crud.messages.submit_button = T("Add")
    crud.messages.record_created = T("Added new way of discovery")
    crud.settings.create_next = return_url
    form = crud.create(db.school_discovery)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires_login()
def discovery_edit():
    """
        This function shows an edit page for a way of discovery
        request.args[0] is expected to be the discoveryID
    """
    discoveryID = request.args[0]
    response.title = T("Edit way of discovery")
    response.subtitle = T('')
    response.view = 'general/only_content.html'
    
    return_url = URL('discovery')
    
    crud.messages.submit_button = T("Update")
    crud.messages.record_updated = T("Updated way of discovery")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.school_discovery, discoveryID)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'school_locations'))
def locations():
    response.title = T("School")
    response.subtitle = T("Locations")
    response.view = 'general/one_grid_with_sidebar.html'
    db.school_locations.id.readable=False
    fields = [db.school_locations.Name,
              db.school_locations.AllowAPI]
    headers = {'school_locations.AllowAPI': T("API")}
    links = [ lambda row: get_button('edit', 
                                      URL('location_edit', args=[row.id]), 
                                      T("Edit the name of this location")) ]
    grid = SQLFORM.grid(db.school_locations, fields=fields, links=links,
        headers=headers,
        create=False,
        editable=False,
        details=False, 
        searchable=False, 
        csv=False, 
        ui = session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    
    add_url = URL('location_add')
    add = get_button('add', add_url, T("Add a location"))
    
    back = get_button('back', URL('index'))
    console = grid.element('.web2py_console')
    console.insert(0, back)
    console.insert(1, add)
    
    return dict(grid=grid, right_sidebar_enabled=True)
    
@auth.requires_login()
def location_add():
    """
        This function shows an add page for a location
    """
    response.title = T("New location")
    response.subtitle = T('')
    response.view = 'general/only_content.html'
    
    return_url = URL('locations')
    
    crud.messages.submit_button = T("Add")
    crud.messages.record_created = T("Added location")
    crud.settings.create_next = return_url
    form = crud.create(db.school_locations)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires_login()
def location_edit():
    """
        This function shows an edit page for a location
        request.args[0] is expected to be the locationID (locID)
    """
    locID = request.args[0]
    response.title = T("Edit location")
    response.subtitle = T('')
    response.view = 'general/only_content.html'
    
    return_url = URL('locations')
    
    crud.messages.submit_button = T("Update")
    crud.messages.record_updated = T("Updated location")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.school_locations, locID)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'school_levels'))
def school_levels():
    response.title = T("School")
    response.subtitle = T("Practice levels")
    response.view = 'general/one_grid_with_sidebar.html'
    db.school_levels.id.readable=False
    fields = [db.school_levels.Name]
    links = [ lambda row: get_button('edit', URL('level_edit', args=[row.id]), T("Edit this practice level")) ]
    grid = SQLFORM.grid(db.school_levels, fields=fields, links=links,
        create=False,
        editable=False,
        details=False, 
        searchable=False, 
        csv=False, 
        orderby = db.school_levels.Name, 
        ui = session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    
    add_url = URL('level_add')
    add = get_button('add', add_url, T("Add a new practice level"))
    
    back = get_button('back', URL('index'))
    console = grid.element('.web2py_console')
    console.insert(0, back)
    console.insert(1, add)
    
    return dict(grid=grid, right_sidebar_enabled=True)
    
@auth.requires_login()
def level_add():
    """
        This function shows an add page for a practice level
    """
    response.title = T("New practice level")
    response.subtitle = T('')
    response.view = 'general/only_content.html'
    
    return_url = URL('school_levels')
    
    crud.messages.submit_button = T("Add")
    crud.messages.record_created = T("Added practice level")
    crud.settings.create_next = return_url
    form = crud.create(db.school_levels)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires_login()
def level_edit():
    """
        This function shows an edit page for a practice level
        request.args[0] is expected to be the school levelID (plID)
    """
    slID = request.args[0]
    response.title = T("Edit practice level")
    response.subtitle = T('')
    response.view = 'general/only_content.html'
    
    return_url = URL('school_levels')
    
    crud.messages.submit_button = T("Update")
    crud.messages.record_updated = T("Updated practice level")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.school_levels, slID)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'school_subscriptions'))
def subscriptions_get_filter_form(filter_option):
    filter_options = [('current', T('Current')), ('archived', T('Archived'))]
    
    form = FORM(_action='#', 
                _method='post', 
                _class='right', 
                _id='school_subscriptions_show_filter', 
                _style='width:300px;')
    select = SELECT(value=filter_option, 
                    _name='filter', 
                    _class='generic-widget', 
                    _onchange="this.form.submit();", 
                    _style='vertical-align:text;')
    for option in filter_options:
        selected = False
        if filter_option == option[0]:
            selected = True
        select.append(OPTION(option[1], _value=option[0], _selected=selected))
        
    submit = INPUT(_type='submit', _value=T("Go"))
    form.insert(0, select)
    form.append(submit)
        
    return form
    
@auth.requires(auth.has_membership(group_id='admins') or \
               auth.has_permission('read', 'school_subscriptions'))
def subscriptions():
    '''
        This function shows a page to list subscriptions.
    '''
    response.title = T("School")
    response.subtitle = T("Subscriptions")
    response.view = 'general/one_grid_with_sidebar.html'
    
    filter_option = 'current'
    query = (db.school_subscriptions.Archived == False)
    
    if 'filter' in request.vars:
        filter_option = request.vars['filter']
        session.mstype_filter_option = filter_option
        if filter_option == 'current':
            query = (db.school_subscriptions.Archived == False)
        elif filter_option == 'archived':
            query = (db.school_subscriptions.Archived == True)
    elif session.mstype_filter_option == 'archived':
            query = (db.school_subscriptions.Archived == True)
    else:
        session.mstype_filter_option = filter_option
    
    db.school_subscriptions.id.readable=False
    fields = [db.school_subscriptions.Name, 
              db.school_subscriptions.MonthlyFee, 
              db.school_subscriptions.NRofClasses]
    links = [ lambda row: get_button('edit', 
                                     URL('subscription_edit', args=[row.id]), 
                                     T("Edit this subscription type")) ]
    maxtextlengths = {'school_subscriptions.Name' : 50}
    grid = SQLFORM.grid(query, fields=fields, links=links,
        maxtextlengths=maxtextlengths, 
        create=False,
        editable=False,
        deletable=False,
        details=False, 
        searchable=False, 
        csv=False, 
        orderby = db.school_subscriptions.Name, 
        ui = session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    
    add_url = URL('subscription_add')
    add = get_button('add', add_url, T("Add a new subscription"))
    
    back = get_button('back', URL('index'))
    console = grid.element('.web2py_console')
    console.insert(0, back)
    console.insert(1, add)
    console.append(subscriptions_get_filter_form(session.mstype_filter_option)) 
    
    return dict(grid=grid, right_sidebar_enabled=True)
    
@auth.requires_login()
def subscription_add():
    """
        This function shows an add page for a subscription
    """
    response.title = T("New subscription")
    response.subtitle = T('')
    response.view = 'general/only_content.html'
    
    db.school_subscriptions.Archived.readable=False
    db.school_subscriptions.Archived.writable=False
    
    return_url = URL('subscriptions')
    
    crud.messages.submit_button = T("Add")
    crud.messages.record_created = T("Added subscription")
    crud.settings.create_next = return_url
    form = crud.create(db.school_subscriptions)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires_login()
def subscription_edit():
    """
        This function shows an edit page for a subscription
        request.args[0] is expected to be the subscriptionID (ssuID)
    """
    ssuID = request.args[0]
    response.title = T("Edit subscription")
    response.subtitle = T('')
    response.view = 'general/only_content.html'
    
    return_url = URL('subscriptions')
    
    crud.messages.submit_button = T("Update")
    crud.messages.record_updated = T("Updated subscription")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.school_subscriptions, ssuID)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'schoolproperties_keys'))
def list_keys():
    response.title = T("School")
    response.subtitle = T("Keys")
    
    session.customers_back = 'keys'
    
    search = ''
    query = (db.customers.Keynr != None) & (db.customers.Keynr != '') # generic show all active customers query
    if 'search' in request.vars: # check whether a search filter is in use
        if request.vars['search'] != '':
            search = request.vars['search']
            search_name = '%' + request.vars['search'] + '%'
            query &= ((db.customers.FirstName.like(search_name)) | (db.customers.LastName.like(search_name)) | (db.customers.Keynr.like(search_name)))
            
    form = SOLIDFORM.factory(
        Field('search', default=search, label=T("")),
        submit_button='Search',
        )
    form.element('input[name=search]')['_placeholder']='Search...'
    form.add_button(T('Show all'), URL(request.function))
    
    if 'edit' in request.args:
        customerID = request.args[2]
        redirect(URL('customers', 'edit', args=[customerID]))
    
    if request.user_agent()['is_mobile']:
        fields = [ db.customers.Keynr, db.customers.FirstName, db.customers.LastName ]
    else:
        fields = [ db.customers.Keynr, db.customers.FirstName, db.customers.LastName, db.customers.DateOfBirth ]
    grid = SQLFORM.grid(query, fields=fields, create=False, details=False, deletable=False, searchable=False, csv=False, ui=session.grid_ui,
        orderby = db.customers.Keynr)
    grid.element('.web2py_counter', replace=None) # remove the counter
    
    back = A(SPAN(_class="buttontext button", _title=T("Back")), SPAN(_class="icon leftarrow icon-arrow-left"), T("Back"), _class="button btn", \
        _href=URL('index'))
    
    return dict(grid=grid, back=back, form=form)

@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'school_languages'))
def languages():
    response.title = T("School")
    response.subtitle = T("Languages")
    response.view = 'general/one_grid_with_sidebar.html'
    db.school_languages.id.readable=False
    fields = [db.school_languages.Name]
    links = [ lambda row: get_button('edit', URL('language_edit', args=[row.id]), T("Edit this language")) ]
    grid = SQLFORM.grid(db.school_languages, fields=fields, links=links,
        create=False,
        editable=False,
        details=False, 
        searchable=False, 
        csv=False, 
        orderby = db.school_languages.Name, 
        ui = session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    
    add_url = URL('language_add')
    add = get_button('add', add_url, T("Add a new language"))
    
    back = get_button('back', URL('index'))
    console = grid.element('.web2py_console')
    console.insert(0, back)
    console.insert(1, add)
    
    return dict(grid=grid, right_sidebar_enabled=True)
    
@auth.requires_login()
def language_add():
    """
        This function shows an add page for a language
    """
    response.title = T("New language")
    response.subtitle = T('')
    response.view = 'general/only_content.html'
    
    return_url = URL('languages')
    
    crud.messages.submit_button = T("Add")
    crud.messages.record_created = T("Added language")
    crud.settings.create_next = return_url
    form = crud.create(db.school_languages)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires_login()
def language_edit():
    """
        This function shows an edit page for a language
        request.args[0] is expected to be the language id (lID)
    """
    lID = request.args[0]
    response.title = T("Edit language")
    response.subtitle = T('')
    response.view = 'general/only_content.html'
    
    return_url = URL('languages')
    
    crud.messages.submit_button = T("Update")
    crud.messages.record_updated = T("Updated languages")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.school_languages, lID)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)

