# coding: utf8

import string
import random
import os_storage

from plugin_solidform import SOLIDFORM
from general_helpers import highlight_submenu
from general_helpers import get_button
from general_helpers import get_submenu
from general_helpers import User_helpers


# helpers begin

@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def _get_user_group_form(selected=None):
    '''
        This function returns the roles of the auth_group table as a list
        of radio buttons. 
    '''
    form = FORM()
    
    # show a no group option only when editing
    if not selected is None:
        form.append(LABEL(INPUT(_type="radio", _name="group_id", _value=0, value=0), " ", T("No group")))
    
    query = ~(db.auth_group.role.contains('user'))
    rows = db(query).select(db.auth_group.ALL, orderby=db.auth_group.role)
    for row in rows:
        form.append(LABEL(INPUT(_type="radio", _name="group_id", _value=row.id, value=selected), " ", row.role))
    form.append(INPUT(_type="submit", _value=T("Update")))
    
    
    return form
    
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def _get_users_groups_menu(page):
    '''
        This function returns the submenu used for the users/groups pages.
        It takes on argument, page, which is used to highlight the active element.
    '''
    pages = ( ['users', T("Users"), URL("users")], ['groups', T("Groups"), URL('groups')] )
    
    menu = []
    for p in pages:
        active = False
        if p[0] == page:
            active = True
        menu.append([p[1], active, p[2]])
        
    return MENU(menu, li_first='', li_last='', _class="os-submenu")
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def _get_users_group(row):
    '''
        This function returns the group a user belongs to and shows it as a link
        to a page which allows users to change it.
    '''
    if row.id == 1:
        return "Admins"
    else: # check if the user had a group
        if db(db.auth_membership.user_id == row.id).count() > 1: # change group
            query = (db.auth_membership.user_id == row.id)
            rows = db(query).select(db.auth_group.ALL,
                                   db.auth_membership.ALL,
                                   left= [ db.auth_group.on(db.auth_group.id == db.auth_membership.group_id) ])
            for query_row in rows:
                role = query_row.auth_group.role
                if 'user' not in role:
                    return A(role, _href=URL("user_group_edit", args=[query_row.auth_membership.id]))
        else: # no group added yet
            return A(T("No group"), _href=URL('user_group_add', args=[row.id]))
            
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def _get_manage_users_form(page):
    pages = [('general', T('General')), ('teachers', T('Teachers'))]
    
    form = FORM(_action='#', _method='post', _class='right', _id='preferences_users_filter', _style='width:300px;')
    select = SELECT(value=page, _name='manage', _class='generic-widget', _onchange="this.form.submit();", _style='vertical-align:text;')
    for p in pages:
        selected = False
        if page == p[0]:
            selected = True
        select.append(OPTION(p[1], _value=p[0], _selected=selected))
        
    submit = INPUT(_type='submit', _value=T("Go"))
    form.insert(0, select)
    form.append(submit)
        
    return form
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def _get_general_settings():
    general_settings = []
    if db.sys_properties(Property='Customer_ShowLocation'):
        separate = db.sys_properties(Property='Customer_ShowLocation').PropertyValue or "off"
    else:
        separate = "off"
    general_settings.append((separate, T("Separate customers by location")))
    if db.sys_properties(Property='ShowWelcomeMessage'):
        show_welcome = db.sys_properties(Property='ShowWelcomeMessage').PropertyValue or 'off'
    else:
        show_welcome = 'on'
    general_settings.append((show_welcome, T("Show welcome message")))
    if db.sys_properties(Property='Currency'):
        currency = db.sys_properties(Property='Currency').PropertyValue
    else:
        currency = "EUR"
    general_settings.append((currency, T("Currency (3 letters)")))
    if db.sys_properties(Property='CurrencySymbol'):
        currency_symbol = db.sys_properties(Property='CurrencySymbol').PropertyValue
    else:
        currency_symbol = '€'
    general_settings.append((currency_symbol, T("Currency Symbol")))
    if db.sys_properties(Property='DateFormat'):
        date_format = db.sys_properties(Property='DateFormat').PropertyValue
        for f in session.date_formats:
            if date_format == f[0]:
                date_format = f[1]
    else:
        date_format = 'yyyy-mm-dd'
    general_settings.append((date_format, T("Date format")))

    return general_settings

@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def _get_mail_settings(edit=False):
    mail_settings = []
    if db.sys_properties(Property='from_name'):
        from_name = db.sys_properties(Property='from_name').PropertyValue
    else:
        from_name = ""
    mail_settings.append((from_name, T("From name")))
    if db.sys_properties(Property='from_address'):
        from_address = db.sys_properties(Property='from_address').PropertyValue
    else:
        from_address = ""
    mail_settings.append((from_address, T("From address")))
    if db.sys_properties(Property='smtp_server'):
        smtp_server = db.sys_properties(Property='smtp_server').PropertyValue
    else:
        smtp_server = ""
    mail_settings.append((smtp_server, T("SMTP server")))
    if db.sys_properties(Property='smtp_port'):
        smtp_port = db.sys_properties(Property='smtp_port').PropertyValue
    else:
        smtp_port = ""
    mail_settings.append((smtp_port, T("SMTP port")))
    if db.sys_properties(Property='smtp_tls'):
        smtp_tls = db.sys_properties(Property='smtp_tls').PropertyValue
    else:
        smtp_tls = ""
    mail_settings.append((smtp_tls, T("SMTP authentication")))
    if db.sys_properties(Property='smtp_username'):
        smtp_username = db.sys_properties(Property='smtp_username').PropertyValue
    else:
        smtp_username = ""
    mail_settings.append((smtp_username, T("SMTP username")))
    if db.sys_properties(Property='smtp_password'):
        if edit==False:
            smtp_password = "******"
        else:
            smtp_password = db.sys_properties(Property='smtp_password').PropertyValue
    else:
        smtp_password = ""
    mail_settings.append((smtp_password, T("SMTP password")))
    if db.sys_properties(Property='smtp_signature'):
        smtp_signature = db.sys_properties(Property='smtp_signature').PropertyValue
    else:
        smtp_signature = ""
    mail_settings.append((smtp_signature, T("Email signature")))

    return mail_settings

# helpers end

@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def index():
    response.title = T("Settings")
    response.subtitle = T("")
    response.view = 'general/only_content.html'
    general = DIV(A(H3(T("General")), _href=URL('general')), 
        T("Edit the general settings, here you can cet the default currency"), ", ", T("tell OpenStudio which server to use when sending e-mails "), ", ", 
    T("specify whether you'd like to separate customers by location"), " ", T('and set the date format.'))
    users = DIV(A(H3(T("Users & groups")), _href=URL('users')), T("Manage OpenStudio user accounts, groups and permissions."))
    api_users = DIV(A(H3(T("OpenStudio API")), _href=URL('api_users')), 
        T("Manage OpenStudio API Access."))
    storage = DIV(A(H3(T("Storage")), _href=URL('storage')), 
        T("Manage OpenStudio file storage."))
    payment_categories = DIV(A(H3(T("Payment categories")), _href=URL('payment_categories')), 
        T("Manage Payment categories to be used in alternative payments and exports on the customers pages."))
    payment_methods = DIV(A(H3(T("Payment methods")), 
        _href=URL('payment_methods')),
        T("Here you can add payment methods to the methods predefined \
           by OpenStudio."))
    about = DIV(A(H3(T("About")), _href=URL('help', 'about')), T("View the OpenStudio version and license."))
    creds = DIV(A(H3(T("Credits")), _href=URL('help', 'credits')), T("OpenStudio has used some resources under a creative commons license, view the credits on this page."))
    
    if request.user_agent()['is_mobile']:
        table = TABLE(TR(general), 
                      TR(users), 
                      TR(api_users),
                      TR(storage),
                      TR(payment_categories), 
                      TR(payment_methods), 
                      TR(about), 
                      TR(creds))
    else:
        table = TABLE(TR(TD(general, _width="50%"), TD(users, _width="50%")), 
                      TR(TD(api_users), TD(storage)),
                      TR(TD(payment_categories), TD(payment_methods)),
                      TR(TD(about), TD(creds)),
                      _class='os-index_table')
    
    return dict(content=table)
    

@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def users():
    '''
        This function shows a page which lists OpenStudio users
    '''
    response.title = T("Settings")
    response.subtitle = T("Manage OpenStudio users")
    response.view = 'settings/users_groups.html'
    
    db.auth_user.id.readable = False
    
    if request.user_agent()['is_mobile']:
        db.auth_user.email.readable=False
        
    db.auth_user.teachers_id.readable=False
    db.auth_user.teachers_id.writable=False
    db.auth_user.teachers_id.default=None
    maxtextlengths = {'auth_user.email' : 35}
    
    sidebar_content = ""
        
    page = 'general'
    links = [ lambda row: get_button('edit', URL('user_edit', args=[row.id])) ]
    query = (db.auth_user.teachers_id == None)
    create = True
    
    if 'manage' in request.vars:
        page = request.vars['manage']
        session.page = page
        if page == 'teachers':
            session.teachers_back = 'users'
            links = [ lambda row: get_button('edit', URL('teachers', 'teacher_edit', args=[row.teachers_id]), T("Edit this teacher")) ]
            db.auth_user.first_name.writable = False
            db.auth_user.last_name.writable = False
            db.auth_user.email.writable = False
            query = (db.auth_user.teachers_id != None)
    elif session.page == 'teachers':
            session.teachers_back = 'users'
            links = [ lambda row: get_button('edit', URL('teachers', 'teacher_edit', args=[row.teachers_id]), T("Edit this teacher")) ]
            db.auth_user.first_name.writable = False
            db.auth_user.last_name.writable = False
            db.auth_user.email.writable = False
            query = (db.auth_user.teachers_id != None)
    else:
        session.page = page
        session.teachers_back = None
    
    links.append(dict(header=T('Group'), body=_get_users_group))
    user_helpers = User_helpers()
    grid = SQLFORM.grid(query, links=links, user_signature=False, 
        create=False,
        editable=False,
        details=False, 
        csv=False, 
        searchable=False, 
        ondelete=user_helpers.on_user_delete,
        maxtextlengths=maxtextlengths,
        orderby=db.auth_user.first_name,
        ui = session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    
    back = get_button('back', URL('index'))
    
    add_url = URL('user_add')
    add = get_button('add', add_url, T("Add a new user"))
    
    # Add back button to web2py_console div at top of the grid
    console = grid.element('.web2py_console')
    console.insert(0, add)
    console.append(_get_manage_users_form(session.page)) 
    
    menu = _get_users_groups_menu(request.function)
    
    return dict(content=grid, menu=menu, back=back,
                left_sidebar_enabled=True, 
                right_sidebar_enabled=False)
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def user_add():
    '''
        This function shows a page to add a new user
    '''
    response.title = T("New user")
    response.subtitle = T("")
    response.view = 'general/only_content.html'
    
    db.auth_user.teachers_id.default=None
    
    return_url = URL('users')
    user_helpers = User_helpers()
    
    crud.messages.submit_button = T("Add")
    crud.messages.record_created = T("Added user")
    crud.settings.create_next = return_url
    crud.settings.create_onaccept = [ user_helpers.on_user_create ]
    form = crud.create(db.auth_user)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def user_edit():
    """
        This function shows an edit page for a user
        request.args[0] is expected to be the userID
    """
    userID = request.args[0]
    response.title = T("Edit user")
    response.subtitle = T("")
    response.view = 'general/only_content.html'
    
    db.auth_user.teachers_id.default=None
    
    return_url = URL('users')
    
    crud.messages.submit_button = T("Update")
    crud.messages.record_updated = T("Updated user")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.auth_user, userID)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def user_group_add():
    '''
        This function shows a page that allows setting of a group for a user
        request.args[0] is expected to be the user id
    '''
    response.title = T("Assign group")
    user_id = request.args[0]
    row = db.auth_user(user_id)
    response.subtitle = row.first_name + " " + row.last_name
    response.view = 'general/only_content.html'
    
    form = _get_user_group_form()
    
    if form.process().accepted:
        group_id = form.vars.group_id
        group_name = db.auth_group(group_id).role
        auth.add_membership(group_id, user_id)
        session.flash = T("Added") + " " + row.first_name + " " + row.last_name + " " + T("to group") + ": " + group_name
        redirect(URL('users'))
    
    content = DIV(form, _class="os-user_group_form")
    back = get_button("back", URL('users'))
    
    return dict(content=content, back=back)
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def user_group_edit():
    '''
        This function shows a page that allows setting of a group for a user
        request.args[0] is expected to be the auth_membership id (am_id)
    '''
    response.title = T("Assign group")
    am_id = request.args[0]
    am_row = db.auth_membership(am_id)
    user_id = am_row.user_id
    row = db.auth_user(user_id)
    response.subtitle = row.first_name + " " + row.last_name
    response.view = 'general/only_content.html'
    
    form = _get_user_group_form(am_row.group_id)
    
    if form.process().accepted:
        db(db.auth_membership.id == am_id).delete()
        group_id = form.vars.group_id
        if group_id != '0':
            group_name = db.auth_group(group_id).role
            message =  T("Added") + " " + row.first_name + " " + row.last_name + " " + T("to group") + ": " + group_name
            auth.add_membership(group_id, user_id)
        else:
            message = T("Removed") + " " + row.first_name + " " + row.last_name + " " + T('from all groups with permissions')
        session.flash = message
        redirect(URL('users'))
    
    content = DIV(form, _class="os-user_group_form")
    back = get_button("back", URL('users'))
    
    return dict(content=content, back=back)
    
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def groups():
    '''
        This function shows a page which lists all user groups
    '''
    def get_edit_button(row):
        if not row.id == 1:
            return get_button('edit', URL('group_edit', args=[row.id]))
    def get_permissions_button(row):
        if not row.id == 1:
            return  A(T("Permissions"), _href=URL("group_permissions", args=[row.id]))
    response.title = T("Settings")
    response.subtitle = T("Manage OpenStudio user groups")
    response.view = 'settings/users_groups.html'
    
    db.auth_group.id.readable = False
    
    links = [ get_permissions_button, 
              get_edit_button ]
    maxtextlengths = {'auth_group.role' : 30, 'auth_group.description' : 50}
    query = ~(db.auth_group.role.contains('user')) # '~' is the NOT operator
    grid = SQLFORM.grid(query, links=links, user_signature=False, 
        create=False,
        editable=False,
        details=False, 
        csv=False, 
        searchable=False, 
        maxtextlengths=maxtextlengths,
        orderby=db.auth_group.role,
        ui = session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
        
    back = get_button('back', URL('index'))
    
    add_url = URL('group_add')
    add = get_button('add', add_url, T("Add a new user group"))
    
    # Add back button to web2py_console div at top of the grid
    console = grid.element('.web2py_console')
    console.insert(0, add)
    
    menu = _get_users_groups_menu(request.function)
    
    return dict(content=grid, menu=menu, back=back,
                left_sidebar_enabled=True, 
                right_sidebar_enabled=False)
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def group_add():
    '''
        This function shows a page to add a new user group
    '''
    response.title = T("New user group")
    response.subtitle = T("")
    response.view = 'general/only_content.html'
    
    return_url = URL('groups')
    
    crud.messages.submit_button = T("Add")
    crud.messages.record_created = T("Added user group")
    crud.settings.create_next = return_url
    form = crud.create(db.auth_group)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def group_edit():
    """
        This function shows an edit page for a user group
        request.args[0] is expected to be the groupID
    """
    groupID = request.args[0]
    response.title = T("Edit user group")
    response.subtitle = T("")
    response.view = 'general/only_content.html'
    
    db.auth_user.teachers_id.default=None
    
    return_url = URL('groups')
    
    crud.messages.submit_button = T("Update")
    crud.messages.record_updated = T("Updated user group")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.auth_group, groupID)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def group_permissions():
    '''
        This page allows the user to configure the permissions of a group
        request.args[0] is expected to the the id from auth_group (gID)
    '''
    def append_table(permissions_list, parent=None, depth=0):
        '''
            This is a helper function to make a nice overview from the permissions list.
            Layout permissions list: [ name, label, bold(boolean), [optional submenu list] ]
        '''
        table = TABLE(TR(TH(T('Permission')), 
                         TH()),
                      _class='unselectable',
                      _id=permissions_list[1])
        odd = False
        
        for field in permissions_list[0]:
            ''' 
                check if we're at the top level, if so set an id for the 
                container div 
            '''
            
            # process permissions list
            value = check_permission(group_id, field[0])
            row = TR(TD(field[1], 
                        _class="level_" + unicode(depth)),
                     TD(INPUT(_type="checkbox", 
                                      _name=field[0], 
                                      value=value),
                        _class='checkbox'))
            table.append(row)
            if parent is None:
                parent = table
            else:
                if depth != 0:
                    parent.append(row)
            try:
                if isinstance(field[2], list):
                    append_table([ field[2], None ], parent, depth+1)
            except IndexError:
                pass
        return parent
        
    def check_permission(group_id, permission):
        '''
            This function checks whether a group has a certain permission
        '''
        (obj, name) = permission.split('-')
        row = db.auth_permission(group_id=group_id, name=name, table_name=obj)
        if row:
            return True
        else:
            return False
    
    group_id = request.args[0]
    response.title = T("Settings")
    row = db.auth_group(group_id)
    response.subtitle = T("Edit permissions for") + " " + row.role
    
    default_permissions = [
        [ 'default-read', T("View overview"), [
            [ 'default_pinboard-read', T('View pin board'), [
                ['memos-create', T("Add memo")],
                ['memos-update', T("Edit memo") ]]],
            [ 'default_subscriptions-read', T("View subscriptions")],
            [ 'default_dropinclasses-read', T("View drop in classes")],
            [ 'default_trialclasses-read', T("View trialclasses")],
            [ 'default_classcards-read', T("View class cards")],
            [ 'default_alternativepayments-read', T("View alternative Payments")],
            [ 'default_overduepayments-read', T("View overdue payments")],
            ]]
        ]
        
    customers_permissions = [
        [ 'customers-read', T("View customers"), [
            [ 'customers-create', T("Add customers")],
            [ 'customers-update', T("Edit customers' general info"), [
                [ 'customers_subscriptions-read', T("View subscriptions"), [
                    [ 'customers_subscriptions-create', T("Add subscriptions") ],
                    [ 'customers_subscriptions-update', T("Edit subscriptions") ]]],
                [ 'customers_classcards-read', T("View class cards"), [
                    [ 'customers_classcards-create', T("Add class cards") ],
                    [ 'customers_classcards-update', T("Edit class cards")]]],
                [ 'classes_registration-read', T("View classes"), [
                    [ 'classes_registration-create', T("Add class registrations") ],
                    [ 'classes_registration-update', T("Edit class registrations")]]],
                [ 'customers_documents-read', T("View documents"), [
                    [ 'customers_documents-create', T("Upload new documents") ],
                    [ 'customers_documents-update', T("Edit uploaded documents") ]]],
                [ 'customers_payments-read', T("View payment info"), [
                    [ 'customers_payment_info-create', T("Add payment info") ],
                    [ 'customers_payment_info-update', T("Edit payment info")],
                    [ 'paymentsummary-create', T("Add payments overview") ],
                    [ 'paymentsummary-update', T("Edit payments overview")],
                    [ 'alternativepayments-create', T("Add alternative payments") ],
                    [ 'alternativepayments-update', T("Edit alternative payments")],
                    [ 'overduepayments-create', T("Add overdue payments") ],
                    [ 'overduepayments-update', T("Edit overdue payments")]]],
                ]]]],
        ]
    
    schedule_permissions = [
        [ 'classes-read', T("Schedule"), [
            [ 'classes-create', T("Add classes"), ],
            [ 'classes-update', T("Edit classes"), [
                [ 'schedule_manage-read', T("Manage class"), [
                    [ 'classes_registration-update', T("Edit registrations") ],
                    [ 'classes_attendance-update', T("Edit attendance") ],
                    [ 'classes_waitinglist-update', T("Edit waitinglist") ],
                    [ 'classes_attendance_override-create', T("Add attendance count") ],
                    [ 'classes_attendance_override-update', T("Edit attendance count") ],
                    [ 'class_status-update', T("Edit class status") ]]]]],
            [ 'school_holidays-read', T('School holidays'), [
                [ 'school_holidays-create', T('Add holidays') ],
                [ 'school_holidays-update', T('Edit holidays') ]]],
            [ 'teacher_holidays-read', T('Teacher holidays') ],
            [ 'classes_open-read', T('View all open classes') ],
            [ 'schedule_set_default_sort-update', T('Set default sorting of classes') ],
            ]],
        ]
        
    workshops_permissions = [
         [ 'workshops-read', T("Workshops"), [
                [ 'workshops-create', T("Add workshops")],
                [ 'workshops-update', T("Edit workshops"), [
                    [ 'workshops_activities-create', T("Add activities") ],
                    [ 'workshops_activities-update', T("Edit activities") ]]],
                ]],
        ]

    school_permissions = [
        [ 'schoolproperties-read', T("School properties"), [
            [ 'teachers-read', T("Teachers"), [
                [ 'teachers-create', T("Add teachers")],
                [ 'teachers-update', T("Edit teachers")]]],
            [ 'school_subscriptions-read', T("Membership types"), [
                [ 'school_subscriptions-create', T("Add membership types")],
                [ 'school_subscriptions-update', T("Edit membership types")]]],
            [ 'school_levels-read', T("Practice levels"), [
                [ 'school_levels-create', T("Add practice levels")],
                [ 'school_levels-update', T("Edit practice levels")]]],
            [ 'school_discovery-read', T("Discovery"), [
                [ 'school_discovery-create', T("Add discovery")],
                [ 'school_discovery-update', T("Edit discovery")]]],
            [ 'school_classtypes-read', T("Class types"), [
                [ 'school_classtypes-create', T("Add classtypes")],
                [ 'school_classtypes-update', T("Edit classtypes")]]],
            [ 'school_locations-read', T("Locations"), [
                [ 'school_locations-create', T("Add locations") ],
                [ 'school_locations-update', T("Edit locations") ]]],
            [ 'schoolproperties_keys-read', T("Keys") ],
            [ 'school_languages-read', T("Languages"), [
                [ 'school_languages-create', T("Add languages") ],
                [ 'school_languages-update', T("Edit languages") ]]]]],
        ]
        
    other_permissions = [
        [ 'stats-read', T("Stats") ],
        [ 'settings-read', T("Settings") ],
    ]
        
    permissions_list = [ [ default_permissions, 'overview' ],
                         [ customers_permissions, 'customers' ],
                         [ schedule_permissions, 'schedule' ],
                         [ workshops_permissions, 'workshops' ],
                         [ school_permissions, 'school' ],
                         [ other_permissions, 'other' ] ]
    
    form = FORM()
    
    for permission in permissions_list:
        form.append(append_table(permission))
    
    #form.append(append_div(permission_list))
    form.append(BR())
    form.append(INPUT(_type="submit", _value=T("Set permissions")))
    
    if form.process().accepted:
        for key in form.vars:
            (obj, name) = key.split('-')
            if form.vars[key] is None:
                if obj == 'class_status': # subteachers used a crud form
                    auth.del_permission(group_id, 'create', 'classes_subteachers', 0)
                    auth.del_permission(group_id, 'update', 'classes_subteachers', 0)
                
                auth.del_permission(group_id, name, obj, 0) # 0 means all records
            else:
                if obj == 'class_status': # subteachers used a crud form
                    auth.add_permission(group_id, 'create', 'classes_subteachers', 0)
                    auth.add_permission(group_id, 'update', 'classes_subteachers', 0)
                
                auth.add_permission(group_id, name, obj, 0) # 0 means all records
        session.flash = T("Updated permissions")
        redirect(URL('groups'))
        
    submenu = UL(_class='os-submenu-horizontal',
                 _id='os-menu-settings-permissions')
        
    menu_items = [ [ 'overview', T('Overview'), True],
                   [ 'customers', T('Customers'), False],
                   [ 'schedule', T('Classes'), False],
                   [ 'workshops', T('Workshops'), False],
                   [ 'school', T('School'), False],
                   [ 'other', T('Other'), False],
                 ]
                 
    for item in menu_items:
        _class = ''
        active = item[2]
        if active:
            _class = 'web2py-menu-active'
        submenu.append(LI(A(item[1], _href='#'), 
                          _class=_class,
                          _id='tab-' + item[0]))
        
    content = DIV(form, 
                  _id='os-permissions')
    
    back = DIV(get_button('back', URL('groups'), _class='left'), 
               ' ',
               submenu)
    
    return dict(content=content, back=back)

@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def payment_categories():
    response.title = T("Settings")
    response.subtitle = T("Payment categories")
    response.view = 'general/one_grid_with_sidebar.html'
    db.payment_categories.id.readable=False
    fields = [db.payment_categories.Name, 
              db.payment_categories.CategoryType]
    links = [ lambda row: get_button('edit', URL('paymentcategory_edit', args=[row.id])) ]
    maxtextlengths = {'payment_categories.Name' : 50}
    grid = SQLFORM.grid(db.payment_categories, fields=fields, links=links,
        create=False,
        editable=False,
        details=False, 
        searchable=False, 
        csv=False,
        maxtextlengths=maxtextlengths, ui=session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    
    back = get_button('back', URL('index'))
    add = get_button('add', URL('paymentcategory_add'))
    
    console = grid.element('.web2py_console')
    console.insert(0, back)
    console.insert(1, add)
    
    return dict(grid=grid)
    
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def paymentcategory_add():
    """
        This function shows an add page for a payment category
    """
    response.title = T("New payment category")
    response.subtitle = T('')
    response.view = 'general/only_content.html'
    
    return_url = URL('payment_categories')
    
    crud.messages.submit_button = T("Add")
    crud.messages.record_created = T("Added payment category")
    crud.settings.create_next = return_url
    form = crud.create(db.payment_categories)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def paymentcategory_edit():
    """
        This function shows an edit page for a payment category
        request.args[0] is expected to be the paymentcategoryID (pcID)
    """
    pcID = request.args[0]
    response.title = T("Edit payment category")
    response.subtitle = T('')
    response.view = 'general/only_content.html'
    
    return_url = URL('payment_categories')
    
    crud.messages.submit_button = T("Update")
    crud.messages.record_updated = T("Updated payment category")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.payment_categories, pcID)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'settings'))
def payment_methods():
    response.title = T("Settings")
    response.subtitle = T("Payment methods")
    response.view = 'general/one_grid_with_sidebar.html'
    db.payment_methods.id.readable=False
    fields = [db.payment_methods.Name]
    links = [ payment_methods_get_link_edit, payment_methods_get_link_delete ]
    
    maxtextlengths = {'payment_methods.Name' : 50}
    grid = SQLFORM.grid(db.payment_methods, fields=fields, links=links,
        maxtextlengths=maxtextlengths, 
        create=False,
        editable=False,
        deletable=False,
        details=False, 
        searchable=False, 
        csv=False, 
        orderby = db.payment_methods.id, 
        ui = session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    
    add_url = URL('payment_method_add')
    add = get_button('add', add_url, T("Add a new payment method"))
    
    back = get_button('back', URL('index'))
    console = grid.element('.web2py_console')
    console.insert(0, back)
    console.insert(1, add)
    
    return dict(grid=grid, right_sidebar_enabled=True)
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def payment_method_add():
    """
        This function shows an add page for a payment method
    """
    response.title = T("New payment method")
    response.subtitle = T('')
    response.view = 'general/only_content.html'
    
    return_url = URL('payment_methods')
    
    crud.messages.submit_button = T("Add")
    crud.messages.record_created = T("Added payment method")
    crud.settings.create_next = return_url
    form = crud.create(db.payment_methods)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def payment_method_edit():
    """
        This function shows an edit page for a payment method
        request.args[0] is expected to be the payment_methodID (pmID)
    """
    pmID = request.args[0]
    response.title = T("Edit payment method")
    response.subtitle = T('')
    response.view = 'general/only_content.html'
    
    return_url = URL('payment_methods')
    
    crud.messages.submit_button = T("Update")
    crud.messages.record_updated = T("Updated payment method")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.payment_methods, pmID)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def payment_method_delete():
    """
        This function deletes a payment method
        request.args[0] is expected to be the payment_methodID (pmID)
    """
    pmID = request.args[0]
    payment_method = db.payment_methods(pmID).Name
    query = (db.payment_method.id == pmID)
    db(query).delete()
    
    return_url = URL('payment_methods')
    session.flash = T('Removed payment method') + ' ' + payment_method
    
    redirect(redirect_url)
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'settings'))
def payment_methods_get_link_edit(row):
    '''
        This function returns the edit link for a payment method, 
        if the id is > 3. The first 3 are reserved for OpenStudio defined
        methods.
    '''
    edit = ''
    if row.id > 3:
        edit = get_button('edit', 
                           URL('payment_method_edit', args=[row.id]), 
                           T("Edit this payment method"))
    
    return edit
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'settings'))
def payment_methods_get_link_delete(row):
    '''
        This function returns the delete link for a payment method, 
        if the id is > 3. The first 3 are reserved for OpenStudio defined
        methods.
    '''
    delete = ''
    if row.id > 3:
        onclick = "return confirm('" + \
                   T('Do you really want to delete this method?') + \
                   "');"
        delete = get_button('delete_notext', 
                           URL('payment_method_delete', args=[row.id]), 
                           T("Edit this payment method"),
                           onclick=onclick)
    
    return delete
    
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def general():
    response.title = T("Settings")
    response.subtitle = T("General")
    
    # general settings
    general_title = H3(T("General settings"))
    general_settings = _get_general_settings()
    
    edit = A(T("Edit general settings"), _href=URL('edit_general', user_signature=True))
        
    # mail settings
    mail_title = H3(T("Email settings"))
    mail_settings = _get_mail_settings()
    mail_edit = A(T("Edit email settings"), _href=URL('edit_mail', user_signature=True))
    
    back = get_button('back', URL('index'))
    
    return dict(edit=edit, mail_edit=mail_edit,
        general_settings=general_settings, general_title=general_title, 
        mail_settings=mail_settings, mail_title=mail_title,
        right_sidebar_enabled=True, back=back)
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def edit_general():
    response.title = T("Settings")
    response.subtitle = T("General")
    response.view = 'general/one_form.html'
    
    general_settings = _get_general_settings()
    separate = general_settings[0][0]
    show_welcome = general_settings[1][0]
    currency = general_settings[2][0]
    currency_symbol = general_settings[3][0]
    date_format = general_settings[4][0]
    for f in session.date_formats:
        if f[1] == date_format:
            date_format = f[0]
        
    form = SOLIDFORM.factory(
        Field('separate', 'boolean', 
            default=separate, 
            label=general_settings[0][1]), 
        Field('show_welcome', 'boolean', 
            default=show_welcome, 
            label=general_settings[1][1]), 
        Field('currency', length=3, 
            requires=IS_LENGTH(3,3, 
                        error_message=T('Enter 3 letters representing the \
                                         currency, eg. "EUR" or "USD"')), 
            default=currency, 
            label=general_settings[2][1]), 
        Field('currency_symbol', length=1, 
            requires=IS_LENGTH(1,1, 
                        error_message=T('Enter only the currency symbol')), 
            default=currency_symbol, 
            label=general_settings[3][1]),
        Field('date_format', 
            requires=IS_IN_SET(session.date_formats, 
                               zero=T("Please select...")), 
            default=date_format, 
            label=general_settings[4][1]),
        submit_button=T("Update"),
        separator=' '
        )
        
    if form.accepts(request.vars, session):
        # check separate by location (possible values are None and "on")
        separate = request.vars['separate']
        row = db.sys_properties(Property='Customer_ShowLocation')
        if not row:
            db.sys_properties.insert(Property='Customer_ShowLocation', PropertyValue=separate)
        else:
            row.PropertyValue=separate
            row.update_record()
            
        # update session variable
        if separate == "on":
            session.show_location = True
        else:
            session.show_location = False
            
        # check show welcome message
        show_welcome = request.vars['show_welcome']
        row = db.sys_properties(Property='ShowWelcomeMessage')
        if not row:
            db.sys_properties.insert(Property='ShowWelcomeMessage', PropertyValue=show_welcome)
        else:
            row.PropertyValue=show_welcome
            row.update_record()
        
        # check currency
        currency = request.vars['currency'].upper() # make sure it's upper case.
        row = db.sys_properties(Property='Currency')
        if not row:
            db.sys_properties.insert(Property='Currency', PropertyValue=currency)
        else:
            row.PropertyValue=currency
            row.update_record()
        
        # check currency symbol
        currency_symbol = request.vars['currency_symbol']
        row = db.sys_properties(Property='CurrencySymbol')
        if not row:
            db.sys_properties.insert(Property='CurrencySymbol', PropertyValue=currency_symbol)
        else:
            row.PropertyValue=currency_symbol
            row.update_record()
            
        # check date_format
        date_format = request.vars['date_format']
        row = db.sys_properties(Property='DateFormat')
        if not row:
            db.sys_properties.insert(Property='DateFormat', PropertyValue=date_format)
        else:
            row.PropertyValue=date_format
            row.update_record()
        session.date_format = date_format

        session.flash = T('Updated general settings')
        redirect(URL('general')) # reload so the user sees how the values are stored in the db now
        
    back = A(SPAN(_class="buttontext button", _title=T("Back")), SPAN(_class="icon leftarrow icon-arrow-left"), T("Back"), _class="button btn", \
        _href=URL('general', user_signature=True))
    
    return dict(form=form, back=back)
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def edit_mail():
    response.title = T("Settings")
    response.subtitle = T("Email settings")
    response.view = 'general/one_form.html'
    
    # set default values
    mail_settings = _get_mail_settings(True)
    from_name = mail_settings[0][0]
    from_address = mail_settings[1][0]
    smtp_server = mail_settings[2][0]
    smtp_port = mail_settings[3][0]
    smtp_tls = mail_settings[4][0]
    smtp_username = mail_settings[5][0]
    smtp_password = mail_settings[6][0]
    smtp_signature = mail_settings[7][0]
    
    form = SOLIDFORM.factory(
        Field("from_name", required=True, default=from_name, label=T("From name")),
        Field("from_address", required=True, default=from_address,label=T("From address")),
        Field("smtp_server", required=True, default=smtp_server, label=T("SMTP server")),
        Field("smtp_port", required=True, default=smtp_port, label=T("SMTP port")),
        Field("smtp_tls", 'boolean', default=smtp_tls, label=T("SMTP authentication")),
        Field("smtp_username", default=smtp_username, label=T("SMTP username")),
        Field("smtp_password", default=smtp_password, label=T("SMTP password")),
        Field("smtp_signature", 'text', default=smtp_signature, label=T("Email signature")),
        submit_button=T("Update"),
        separator=' '
        )
        
    if form.accepts(request.vars, session):
        # check from_name
        setting = request.vars['from_name']
        row = db.sys_properties(Property='from_name')
        if not row:
            db.sys_properties.insert(Property='from_name', PropertyValue=setting)
        else:
            row.PropertyValue=setting
            row.update_record()
        # check from_address
        setting = request.vars['from_address']
        row = db.sys_properties(Property='from_address')
        if not row:
            db.sys_properties.insert(Property='from_address', PropertyValue=setting)
        else:
            row.PropertyValue=setting
            row.update_record()
        # check smtp_server
        setting = request.vars['smtp_server']
        row = db.sys_properties(Property='smtp_server')
        if not row:
            db.sys_properties.insert(Property='smtp_server', PropertyValue=setting)
        else:
            row.PropertyValue=setting
            row.update_record()
        # check smtp_port
        setting = request.vars['smtp_port']
        row = db.sys_properties(Property='smtp_port')
        if not row:
            db.sys_properties.insert(Property='smtp_port', PropertyValue=setting)
        else:
            row.PropertyValue=setting
            row.update_record()
        # check smtp_tls
        setting = request.vars['smtp_tls']
        if setting == None:
            setting = "off"
        row = db.sys_properties(Property='smtp_tls')
        if not row:
            db.sys_properties.insert(Property='smtp_tls', PropertyValue=setting)
        else:
            row.PropertyValue=setting
            row.update_record()
        # check smtp_username
        setting = request.vars['smtp_username']
        row = db.sys_properties(Property='smtp_username')
        if not row:
            db.sys_properties.insert(Property='smtp_username', PropertyValue=setting)
        else:
            row.PropertyValue=setting
            row.update_record()
        # check smtp_password
        setting = request.vars['smtp_password']
        row = db.sys_properties(Property='smtp_password')
        if not row:
            db.sys_properties.insert(Property='smtp_password', PropertyValue=setting)
        else:
            row.PropertyValue=setting
            row.update_record()
        # check smtp_signature
        setting = request.vars['smtp_signature']
        row = db.sys_properties(Property='smtp_signature')
        if not row:
            db.sys_properties.insert(Property='smtp_signature', PropertyValue=setting)
        else:
            row.PropertyValue=setting
            row.update_record()

            
        session.flash = T('Updated email settings')
        redirect(URL('general')) # reload so the user sees how the values are stored in the db now
        
    back = get_button('back', URL('general'))
    
    return dict(form=form, back=back)
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def api_users():
    '''
        This page shows an overview of current API users
    '''
    response.title = T("Settings")
    response.subtitle = T("API users")
    response.view = 'general/only_content.html'
    
    db.sys_api_users.id.readable = False
    maxtextlengths = {'sys_api_users.APIKey' : 30,
                      'sys_api_users.Description' : 40}
                      
    new_key_onclick_msg = T('Do you really want to renew this key?') + ' ' + \
                          T('The old key wil STOP working!')
    new_key_onclick = "return confirm('" + new_key_onclick_msg + "');"
    links = [lambda row: A(T("Generate new key"), 
                            _href=URL('api_user_generate_new_key', 
                                      args=[row.id]),
                            _onclick=new_key_onclick),
             api_user_link_active,]
    
    query = (db.sys_api_users.id > 0)
    grid = SQLFORM.grid(query, links=links, user_signature=False, 
        create=False,
        editable=False,
        details=False, 
        csv=False, 
        searchable=False, 
        maxtextlengths=maxtextlengths,
        ondelete=user_helpers.on_user_delete,
        orderby=db.sys_api_users.Username,
        ui = session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    
    back = get_button('back', URL('index'))
    
    add_url = URL('api_user_add')
    add = get_button('add', add_url, T("Add a new api user"))
    
    # Add back button to web2py_console div at top of the grid
    console = grid.element('.web2py_console')
    console.insert(0, add)
    console.insert(0, back)
        
    return dict(content=grid)
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def api_user_add():
    '''
        Shows a page to add a new api user
    '''
    response.title = T("New API user")
    response.subtitle = T('')
    response.view = 'general/only_content.html'
    
    db.sys_api_users.ActiveUser.readable=False
    db.sys_api_users.APIKey.readable = False
    db.sys_api_users.APIKey.default = api_user_generate_key()
    
    return_url = URL('api_users')
    
    crud.messages.submit_button = T("Add")
    crud.messages.record_created = T("Added new api user")
    crud.settings.create_next = return_url
    form = crud.create(db.sys_api_users)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def api_user_generate_new_key():
    '''
        Sets a new API key for an api user.
        request.args[0] is expected to be the sys_api_users id (suaID)
    '''
    suaID = request.args[0]
    key = api_user_generate_key()
    
    record = db.sys_api_users(suaID)
    record.APIKey = key
    record.update_record()
    
    redirect(URL('api_users'))
    
def api_user_generate_key(length=30):
    '''
        Function to generate a key for an api user.
        Keys are auto generated to increase security, it's impossible now to
        add weak keys.
    '''
    chars = string.ascii_uppercase + string.ascii_lowercase + string.digits
    key = ''.join(random.SystemRandom().choice(chars) for _ in xrange(length))
    
    return key

@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def api_user_change_active_state():
    '''
        Enables or disables an api user.
        request.args[0] is expected to be the sys_api_users id (suaID)
    '''
    suaID = request.args[0]
    record = db.sys_api_users(suaID)
    
    # invert active state
    active = not record.ActiveUser
    record.ActiveUser = not record.ActiveUser
    record.update_record()
    
    redirect(URL('api_users'))

@auth.requires_login()
def api_user_link_active(row):
    '''
        Returns a link which allows (de)activating an api user.
    '''
    record = db.sys_api_users(row.id)
    active = record.ActiveUser
    if active:
        link_text = T("Deactivate")
    else:
        link_text = T("Activate")
        
    link = A(link_text, _href=URL('api_user_change_active_state',
                                  args=[row.id]))
        
    return link
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'settings'))
def storage():
    '''
        Shows a page with the used and available storage
    '''
    response.title = T("Settings")
    response.subtitle = T("Storage")
    
    row = db.sys_properties(Property='storage_allowed_space')
    allowed_space = int(row.PropertyValue)
    
    uploads_dir = os.path.join(request.folder, 'uploads')
    used_space = os_storage.get_size(uploads_dir)
    available_space = allowed_space - used_space
    
    data = [ available_space, used_space ]
    labels = [ T('Available'), T('Used') ]
    json_data = []
    color = dict(red=82, green=136, blue=154)
    i = 0
    for item in data:
        current_color = 'rgb(' + unicode(color['red']) + ',' + \
                                 unicode(color['green']) + ',' + \
                                 unicode(color['blue']) + ')'
        json_data.append(dict(label=labels[i], 
                               value=data[i],
                               color=current_color,
                               highlight="rgb(17,160,94)"))
        color['red'] += 20
        color['green'] += 20
        color['blue'] += 20
        
        i += 1
            
    data_table = TABLE(TR(TH(T("Space")), TH(T("MB"))), 
        _class="os-stats_data_table")
    i = 0
    for value in data:
        data_table.append(TR(labels[i], value))
        i += 1
    
    if auth.user.id == 1:
        edit = A(T("Set allowed space"), _href=URL('storage_set_limit'))
    else:
        edit = ""
    
    back = get_button('back', URL('index'))
    
    return dict(data_table=data_table,
                  json_data=json_data,
                  edit=edit, 
                  back=back)
    
@auth.requires(auth.user_id==1)
def storage_set_limit():
    '''
        page to set the storage limit for this OpenStudio installation
    '''
    response.title = T("Settings")
    response.subtitle = T("Set storage limit")
    response.view = 'general/only_content.html'
    
    row = db.sys_properties(Property='storage_allowed_space')
    allowed_space = row.PropertyValue
    
    form = SOLIDFORM.factory(
        Field('space', 'integer',
            default=allowed_space, 
            label=T("Allowed space in MB"),),
        submit_button=T("Update"),
        separator=' '
        )
        
    if 'space' in request.vars:
        row = db.sys_properties(Property='storage_allowed_space')
        row.PropertyValue = request.vars['space']
        row.update_record()
        
        redirect(URL('storage'))
        
    back = get_button('back', URL('storage'))
        
    return dict(content=form, back=back)
        
    
    
    
