# coding: utf8
import datetime
from gluon.contrib.user_agent_parser import mobilize 

from general_helpers import iso_to_gregorian
from general_helpers import NRtoDay
from general_helpers import get_paused_subscriptions
from general_helpers import get_button

#########################        
### Helper functions ####
#########################
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'stats'))
def _first_day_of_next_month(date):
    next_month = date.replace(day=28) + datetime.timedelta(days=4)  # this will never fail
    last_day = next_month - datetime.timedelta(days=next_month.day)
    return last_day + datetime.timedelta(days=1)
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'stats'))

@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'stats'))
def get_first_monday(year):
    date = datetime.date(year, 1, 4) 
    delta = datetime.timedelta(days=date.weekday())
    return date - delta

@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'stats'))
def generate_form(year):
    form = FORM(INPUT(_name='year', requires=IS_NOT_EMPTY(), _type='number', _value=year,\
                _min=datetime.date.today().year-25, _max=datetime.date.today().year+25),
            " ",  
            INPUT(_type='submit', _value=T('View')),
            _class="os-stats_year_form")
    form.element('input[name=year]')['_style']='width:70px'
    
    return form

#@auth.requires(auth.has_membership(group_id='admins') or \
                #auth.has_permission('read', 'stats'))
#def generate_barchart(title, chart_data, max_value, axis_bottom, nr_bars):
    ## for neat lines in the chart matching the numbers on the axis, make sure the max_value is divisible by 5
    #if int(max_value)%5!=0:
        #while int(max_value)%5 != 0:
            #max_value += 1
        
    #width = 900
    #height = 300
        
    #chart = GroupedVerticalBarChart(width, height, y_range=(0, max_value))
    #chart.set_bar_width((width/nr_bars)-4)
    #chart.set_bar_spacing(0)
    #chart.set_group_spacing(4)
    #chart.set_colours(['52889A', 'ff0000'])
    #chart.add_data(chart_data)
    #step = int(max_value/5)
    #if step == 0:
        #step = 1
    #y_step = max_value/step*4
    #chart.set_grid(0, y_step, 5, 5)
    #chart.set_title(title)
    #left_axis = list(range(0, int(max_value) + 1, step))
    #left_axis[0] = ''
    #chart.set_axis_labels(Axis.LEFT, left_axis)
    #chart.set_axis_labels(Axis.BOTTOM, axis_bottom)
##    chart.set_legend(['Legend'])
    #try:
        #url = chart.get_url()
        #barchart = IMG(_src=url)
    #except:
        #barchart = "No data available for this year"
    #return barchart



@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'stats'))
def index():
    response.title = T("Statistics")
    response.subtitle = T("")
    response.view = 'general/only_content.html'
    discovery = DIV(A(H3(T("Discovery")), _href=URL('discovery')), T("The discovery statistics show an easy to read chart of how customers found your school. \
        This overview is based on the discovery field in the edit customer pages."))
    revenue = DIV(A(H3(T("Revenue")), 
                  _href=URL('revenue')), 
                  T("Generated revenue shown in charts."))
    attendance = DIV(A(H3(T("Attendance")), 
                     _href=URL('attendance')), 
                     T("Attendance charts for classes and several totals for a selected year that might help to \
                        discover trends. These charts are based on the attendance entered on the manage class pages."))
    geography = DIV(A(H3(T("Geography")), _href=URL('geography')), T("Shows the top 10 cities and countries based on customer information."))
        
    if request.user_agent().is_mobile:
        table = TABLE(TR(discovery), TR(attendance), TR(revenue), TR(geography))
    else:
        table = TABLE(TR(TD(discovery, _width="33%"), TD(attendance, _width="33%"), TD(revenue, _width="33%")),
                    TR(TD(geography)))
    
    return dict(content=table)
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'stats'))
def geography():
    if request.extension == 'json':
        response.view = 'generic.json'
    
    response.title = T("Statistics")
    response.subtitle = T("Geography")
    
    # chart datasets for chart.js
    fillColor = "rgba(151,187,205,0.5)"
    strokeColor = "rgba(151,187,205,0.8)"
    highlightFill = "rgba(151,187,205,0.75)"
    highlightStroke = "rgba(151,187,205,1)"
    json_data = dict(city=dict(labels=[], 
                                datasets=[dict(label="city_data", 
                                           fillColor=fillColor,
                                           strokeColor=strokeColor,
                                           highlightFill=highlightFill,
                                           highlightStroke= highlightStroke,
                                           data=[])]), 
                     country=dict(labels=[], 
                                  datasets=[dict(label="country_data", 
                                           fillColor=fillColor,
                                           strokeColor=strokeColor,
                                           highlightFill=highlightFill,
                                           highlightStroke= highlightStroke,
                                           data=[])]) 
                     )
    
    # City
    count = db.customers.City.count()
    rows = db().select(db.customers.City, count, 
        groupby=db.customers.City, 
        orderby=~count|db.customers.City, 
        limitby=(0,10))
    
    city_data = list()
    city_labels = list()
    
    for row in rows:
        value = row[count]
        label = row.customers.City
        
        city_data.append(value)
        city_labels.append(label)
        
        json_data['city']['labels'].append(label)
        json_data['city']['datasets'][0]['data'].append(value)
        
    
    city_title = H2(T("Top 10 Cities"))
    city_data_table = TABLE(TR(TH(T("City")), TH(T("Customers"))),
        _class="os-stats_data_table")
    i = 0
    for value in city_data:
        city_data_table.append(TR(city_labels[i], value))
        i += 1
    
    # Country
    count = db.customers.Country.count()
    rows = db().select(db.customers.Country, count, 
        groupby=db.customers.Country, 
        orderby=~count|db.customers.Country, 
        limitby=(0,10))
    
    country_data = list()
    country_labels = list()
    
    for row in rows:
        value = row[count]
        label = row.customers.Country
        
        country_data.append(value)
        country_labels.append(label)
        
        json_data['country']['labels'].append(label)
        json_data['country']['datasets'][0]['data'].append(value)

    
    country_headers = [ T("Country"), T("Customers") ]
    country_title = H2(T("Top 10 Countries"))
    
    country_data_table = TABLE(TR(TH(T("Country")), TH(T("Customers"))), 
        _class="os-stats_data_table")
    i = 0
    for value in country_data:
        country_data_table.append(TR(country_labels[i], value))
        i += 1
    
    back = get_button('back', URL('index'))
    
    return dict(city_data_table=city_data_table, city_title=city_title,
        country_data_table=country_data_table, country_title=country_title,
        json_data=json_data,
        back=back)

@mobilize
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'stats'))
def attendance_time_year():
    try:
        year = int(request.vars['year'])
    except:
        year = datetime.date.today().year
        
    response.title = T("Statistics")
    title = T("Total attendance by time") + " " + unicode(year)
    response.subtitle = title
    
    form = generate_form(year)
    
    yearstart = datetime.date(year, 1, 1)
    yearend = datetime.date(year, 12, 31)
    
    result = db.executesql("""SELECT cla.id, cla.StartTime, att.count, atto.amount FROM classes cla
            LEFT JOIN
            (SELECT id, COUNT(classes_id) AS count, classes_id FROM classes_attendance WHERE ClassDate>=%s AND ClassDate <= %s group by classes_id) att 
            ON att.classes_id = cla.id
            LEFT JOIN
            (SELECT SUM(Amount) AS amount, classes_id FROM classes_attendance_override WHERE ClassDate>=%s AND ClassDate <= %s GROUP BY classes_id) atto
            ON atto.classes_id = cla.id
            ORDER BY cla.StartTime;""",
            (yearstart, yearend, yearstart, yearend))
            
    nr_times = 0
    previous_time = None
    time_dict = dict()
    for record in result:
        if record[1] != previous_time:
            time = (datetime.datetime.min + record[1]).time()
            time_name = unicode(time.strftime('%H:%M'))
            previous_time = record[1]
            nr_times += 1
        if record[3] != None:
            try:
                time_dict[time_name] += int(record[3])
            except:
                time_dict[time_name] = int(record[3])
        elif record[2] != None:
            try:
                time_dict[time_name] += int(record[2])
            except:
                time_dict[time_name] = int(record[2])
        
                
    rows = db().select(db.classes.Starttime, orderby=db.classes.Starttime, distinct=True)
    chart_data = list()
    labels = list()
    for row in rows:
        time = unicode(row.Starttime.strftime('%H:%M')) # format it to match the dict keys
        try: # in case there's a starttime without any attendance, catch the keyerror for the dict
            chart_data.append(time_dict[time])
            labels.append(time)
        except:
            pass
    max_value = 0
    for value in chart_data:
        if value > max_value:
            max_value = value
    
    barchart = generate_barchart(title, chart_data, max_value, labels, nr_times)
    chart_title = H4(T("Attendance barchart"))
    col_titles = [ T("Class start time"), T("Attendance total") ]
    back = A(SPAN(_class="buttontext button", _title=T("Back")), SPAN(_class="icon leftarrow icon-arrow-left"), T("Back"), _class="button btn", \
        _href=URL('attendance', user_signature=True))
        
    start_title = T("Class start time")
    attendance_title = T("Attendance total")
            
    return dict(form=form, barchart=barchart, time_dict=time_dict, chart_title=chart_title, back=back, col_titles=col_titles)

@mobilize
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'stats'))
def attendance_day_year():
    try:
        year = int(request.vars['year'])
    except:
        year = datetime.date.today().year
        
    response.title = T("Statistics")
    title = T("Total by day for") + " " + unicode(year)
    response.subtitle = title
    
    form = generate_form(year)
    
    yearstart = datetime.date(year, 1, 1)
    yearend = datetime.date(year, 12, 31)
    monday = 0
    tuesday = 0
    wednesday = 0
    thursday = 0
    friday = 0
    saturday = 0
    sunday = 0
    
    result = db.executesql("""SELECT cla.id, cla.Week_day, att.count, atto.amount FROM classes cla
            LEFT JOIN
            (SELECT id, COUNT(classes_id) AS count, classes_id FROM classes_attendance WHERE ClassDate>=%s AND ClassDate <= %s group by classes_id) att 
            ON att.classes_id = cla.id
            LEFT JOIN
            (SELECT SUM(Amount) AS amount, classes_id FROM classes_attendance_override WHERE ClassDate>=%s AND ClassDate <= %s GROUP BY classes_id) atto
            ON atto.classes_id = cla.id
            ORDER BY cla.Week_day;""",
            (yearstart, yearend, yearstart, yearend))
    
    for record in result:
        weekday = int(record[1])
        if weekday == 1: # Monday
            if record[3] != None:
                monday += record[3]
            elif record[2] != None:
                monday += record[2]
        if weekday == 2: # Tuesday
            if record[3] != None:
                tuesday += record[3]
            elif record[2] != None:
                tuesday += record[2]
        if weekday == 3: # Wednesday
            if record[3] != None:
                wednesday += record[3]
            elif record[2] != None:
                wednesday += record[2]
        if weekday == 4: # Thursday
            if record[3] != None:
                thursday += record[3]
            elif record[2] != None:
                thursday += record[2]
        if weekday == 5: # Friday
            if record[3] != None:
                friday += record[3]
            elif record[2] != None:
                friday += record[2]
        if weekday == 6: # Saturday
            if record[3] != None:
                saturday += record[3]
            elif record[2] != None:
                saturday += record[2]
        if weekday == 7: # Sunday
            if record[3] != None:
                sunday += record[3]
            elif record[2] != None:
                sunday += record[2]
            
    max_value = 0
    days = [int(monday), int(tuesday), int(wednesday), int(thursday), int(friday), int(saturday), int(sunday)]
    for day in days:
        if int(day) > max_value:
            max_value = int(day)
    weekdays = [T('Monday'), T('Tuesday'), T('Wednesday'), T('Thursday'), T('Friday'), T('Saturday'), T('Sunday')]
    barchart = generate_barchart(title, days, max_value, weekdays, 7)
    
    chart_title = H4(T("Attendance barchart"))
    col_titles = [ T("Day"), T("Attendance total") ]
    back = A(SPAN(_class="buttontext button", _title=T("Back")), SPAN(_class="icon leftarrow icon-arrow-left"), T("Back"), _class="button btn", \
        _href=URL('attendance', user_signature=True))
    
    return dict(form=form, days=days, weekdays=weekdays, barchart=barchart, chart_title=chart_title, back=back, col_titles=col_titles)

@mobilize
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'stats'))
def attendance_average_year():
    try:
        year = int(request.vars['year'])
    except:
        year = datetime.date.today().year
    
    response.title = T("Statistics")
    title =  T("Attendance by week") + " " + unicode(year)
    response.subtitle = title
    
    form = generate_form(year)

    # calculate totals by week
    first_monday = get_first_monday(year)
    max_value = 0
    week = 1
    delta = datetime.timedelta(days=7)
    weekstart = first_monday
    nextyear = datetime.date(year+1, 1, 1)
    attendance_dict = dict()
    weeks = list()
    chart_data = list()
    while weekstart < nextyear:
        weekend = weekstart + delta
        result = db.executesql("""SELECT cla.id, att.count, atto.Amount FROM classes cla
            LEFT JOIN
            (SELECT id, COUNT(classes_id) AS count, classes_id FROM classes_attendance WHERE ClassDate>=%s AND ClassDate <%s group by classes_id) att 
            ON att.classes_id = cla.id
            LEFT JOIN
            (SELECT Amount, classes_id FROM classes_attendance_override WHERE ClassDate>=%s AND ClassDate <%s) atto
            ON atto.classes_id = cla.id""",
            (weekstart, weekend, weekstart, weekend))
        total = 0
        for record in result:
            if record[1] != None:
                total += record[1]
            elif record[2] != None:
                total += record[2]
        attendance_dict[unicode(week)] = total 
        chart_data.append(total)
        if total > max_value:
            max_value = total
        weeks.append(week)
        week += 1
        weekstart += delta
    
    total_attendance=0
    for number in chart_data:
        total_attendance += number
    average = total_attendance/len(weeks)
    barchart = generate_barchart(title, chart_data, max_value, weeks, week)
    
    avg_title = H4(T("General statistics"))
    col_titles = [ T("Week"), T("Attendance total") ]
    back = A(SPAN(_class="buttontext button", _title=T("Back")), SPAN(_class="icon leftarrow icon-arrow-left"), T("Back"), _class="button btn", \
        _href=URL('attendance', user_signature=True))
    
    return dict(form=form, barchart=barchart, weeks=weeks, chart_data=chart_data, maximum=max_value, average=average, back=back, 
        avg_title=avg_title, col_titles=col_titles)
    
@mobilize
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('update', 'attendance'))
def attendance():
    '''
        This function returns the attendance data for a class in a year
        specified by session.stats_attendance_year
        and should be calles as .json
    '''
    response.view = 'generic.json'
    
    year = session.stats_attendance_year
    clsID = session.stats_attendance_clsID
    
    # chart datasets for chart.js
    fillColor = "rgba(151,187,205,0.5)"
    strokeColor = "rgba(151,187,205,0.8)"
    highlightFill = "rgba(151,187,205,0.75)"
    highlightStroke = "rgba(151,187,205,1)"
    json_data = dict(attendance=dict(labels=[], 
                                datasets=[dict(label="attendance_data", 
                                           fillColor=fillColor,
                                           strokeColor=strokeColor,
                                           highlightFill=highlightFill,
                                           highlightStroke= highlightStroke,
                                           data=[])])) 
                                           
    # get the data
    rows = db(db.classes.id==clsID).select(db.classes.Week_day, 
                                           db.classes.Startdate, 
                                           db.classes.Starttime, 
                                           db.school_locations.Name, 
                                           db.school_classtypes.Name, 
                                           db.classes.school_classtypes_id,
        left = [(db.school_locations.on(db.classes.school_locations_id==\
                                        db.school_locations.id)),
                (db.school_classtypes.on(db.classes.school_classtypes_id==\
                                         db.school_classtypes.id))]
        )
    row = rows.first()
    dayofweek = int(row.classes.Week_day)
    dayofweek_name = NRtoDay(dayofweek)
    startdate = row.classes.Startdate
    starttime = unicode(row.classes.Starttime.strftime('%H:%M'))
    location = unicode(row.school_locations.Name)
    classtype = unicode(row.school_classtypes.Name)
    firstday = iso_to_gregorian(year, 1, dayofweek)
    
    date = firstday
    delta = datetime.timedelta(days=7)
    next_year = datetime.date(year+1, 1, 1)
    maximum = 0
    total = 0
    week = 1
    weeks_with_data = 0
    attendance_data = []
    attendance_labels = []
    while date < next_year:
        if date < startdate:
            attendance = 0
        else:
            attendance = count_attendance(clsID, date)
            
        if attendance > 0:
            weeks_with_data += 1
            total += attendance
            if attendance > maximum:
                maximum = attendance
            
        label = unicode(week)
            
        attendance_labels.append(label)
        attendance_data.append(attendance)
        
        json_data['attendance']['labels'].append(label)
        json_data['attendance']['datasets'][0]['data'].append(attendance)

        date += delta
        week += 1
    
    if weeks_with_data != 0:
        average = total/weeks_with_data
    else:
        average = 0
    
    title = location + " " + dayofweek_name + " " + starttime + " " + classtype + " " + unicode(year)
    response.subtitle=(title)
    
    # set other data
    chart_title = H4(T("Attendance barchart"))
    avg_title = H4(T("General statistics"))
    
    return dict(json_data=json_data)
    
def count_attendance(clsID, date):
    query = (db.classes_attendance.classes_id==clsID) & \
            (db.classes_attendance.ClassDate==date)
    attendancecount = int(db(query).count())
    
    query = (db.classes_attendance_override.classes_id==clsID) & \
            (db.classes_attendance_override.ClassDate==date)
    override_check = db(query).isempty()
    
    if not override_check: # not empty
        row = db(query).select(db.classes_attendance_override.Amount)[0]
        return row.Amount
    else:
        return attendancecount

@mobilize
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'stats'))
def list_classes():
    try:
        year = int(request.vars['year'])
    except:
        year = datetime.date.today().year
    response.title = T("Statistics")
    response.subtitle = T("Attendance by class") + " " + unicode(year)
    #response.subtitle="Attendance statistics for " + unicode(year)
        
    form = generate_form(year)
    
    #if form.accepts(request,session):
    #    response.flash = 'year changed'

    startthisyear = datetime.date(year, 01, 01)
    startnextyear = datetime.date(year+1, 01, 01)
    
    query = (db.classes.Startdate <= startnextyear) & \
            ((db.classes.Enddate >= startthisyear) | \
             (db.classes.Enddate == None))
    
    rows = db(query).select(db.classes.id, 
                            db.classes.Week_day, 
                            db.school_locations.Name, 
                            db.classes.Starttime, 
                            db.school_classtypes.Name, 
                            db.teachers.FirstName, 
                            db.teachers.LastName,
        orderby=db.classes.Week_day | \
                db.school_locations.Name | \
                db.classes.Starttime,
        left=[ 
            (db.school_locations.on(db.classes.school_locations_id==\
                                    db.school_locations.id)),
            (db.school_classtypes.on(db.classes.school_classtypes_id==\
                                     db.school_classtypes.id)),
            (db.teachers.on(db.classes.teachers_id==db.teachers.id)),
            ])
    classes = list()
    number = 0
    for row in rows:
        classname = row.classes.id, NRtoDay(row.classes.Week_day), row.classes.Week_day, \
        row.school_locations.Name, row.classes.Starttime.strftime('%H:%M'), row.school_classtypes.Name, row.teachers.FirstName, \
        row.teachers.LastName
        classes.append(classname)
        number += 1
        
    back = A(SPAN(_class="buttontext button", _title=T("Back")), SPAN(_class="icon leftarrow icon-arrow-left"), T("Back"), _class="button btn", \
        _href=URL('attendance', user_signature=True))
        
    return dict(classes=classes, form=form, year=year, back=back)
  
@mobilize
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'stats'))
def discovery():
    if request.extension == 'json':
        response.view = 'generic.json'
    
    response.title = T("Statistics")
    response.subtitle = T("Discovery")
    count = db.customers.school_discovery_id.count()
    data = list()
    json_data = list()
    labels = list()
    color = dict(red=82, green=136, blue=154)
    rows = db().select(db.school_discovery.id, 
                       db.school_discovery.Name, 
                       db.customers.school_discovery_id, 
                       count, 
                       orderby=~count, 
                       groupby=db.customers.school_discovery_id,
                       left=db.school_discovery.on(\
                                            db.customers.school_discovery_id==\
                                            db.school_discovery.id))
    for row in rows:
        if row.school_discovery.Name != None:
            label = row.school_discovery.Name
            value = row[count]
            current_color = 'rgb(' + unicode(color['red']) + ',' + \
                             unicode(color['green']) + ',' + \
                             unicode(color['blue']) + ')'
            
            data.append(value)
            labels.append(label)
            json_data.append(dict(label=label, 
                                   value=value,
                                   color=current_color,
                                   highlight="rgb(17,160,94)")
                            )
            
            color['red'] += 20
            color['green'] += 20
            color['blue'] += 20
            
    data_table = TABLE(TR(TH(T("Discovery")), TH(T("Number"))), 
        _class="os-stats_data_table")
    i = 0
    for value in data:
        data_table.append(TR(labels[i], value))
        i += 1
    
    back = get_button('back', URL('index'))
    
    return dict(back=back, data_table=data_table, json_data=json_data)
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'stats'))
def revenue():
    if request.extension == 'json':
        response.view = 'generic.json'
    
    try:
        year = int(request.vars['year'])
    except:
        year = datetime.date.today().year
    
    response.title = T("Statistics")
    response.subtitle=T("Revenue") + " " + unicode(year)
    
    form = generate_form(year)
    
    today = datetime.date.today()
    
    ### Common useful values
    labels = [T('Jan'), T('Feb'), T('Mar'), T('Apr'), T('May'), T('Jun'), 
              T('Jul'), T('Aug'), T('Sep'), T('Oct'), T('Nov'), T('Dec')]
    months = [(1,T("January")),
              (2,T("February")),
              (3,T("March")),
              (4,T("April")),
              (5,T("May")),
              (6,T("June")),
              (7,T("July")),
              (8,T("August")),
              (9,T("September")),
              (10,T("October")),
              (11,T("November")),
              (12,T("December"))]
    avg_title = H4(T("General statistics"))
    data_table_titles = [ T("Month"), T("Revenue") ]
    
    
    # chart datasets for chart.js
    fillColor = "rgba(151,187,205,0.5)"
    strokeColor = "rgba(151,187,205,0.8)"
    highlightFill = "rgba(151,187,205,0.75)"
    highlightStroke = "rgba(151,187,205,1)"
    json_data = dict(subscriptions=dict(labels=labels, 
                            datasets=[dict(label="subscriptions_data", 
                                       fillColor=fillColor,
                                       strokeColor=strokeColor,
                                       highlightFill=highlightFill,
                                       highlightStroke=highlightStroke,
                                       data=[])]), 
                      classcards=dict(labels=labels, 
                              datasets=[dict(label="classcards_data", 
                                       fillColor=fillColor,
                                       strokeColor=strokeColor,
                                       highlightFill=highlightFill,
                                       highlightStroke=highlightStroke,
                                       data=[])]), 
                      dropin=dict(labels=labels, 
                              datasets=[dict(label="dropin_data", 
                                       fillColor=fillColor,
                                       strokeColor=strokeColor,
                                       highlightFill=highlightFill,
                                       highlightStroke=highlightStroke,
                                       data=[])]),
                      trialclasses=dict(labels=labels, 
                              datasets=[dict(label="trialclasses_data", 
                                       fillColor=fillColor,
                                       strokeColor=strokeColor,
                                       highlightFill=highlightFill,
                                       highlightStroke=highlightStroke,
                                       data=[])]),
                      total=dict(labels=labels, 
                              datasets=[dict(label="total_data", 
                                       fillColor=fillColor,
                                       strokeColor=strokeColor,
                                       highlightFill=highlightFill,
                                       highlightStroke=highlightStroke,
                                       data=[])]) 
                 )

    def get_month_subscriptions(date):
        # helper function to get monthly membership revenue
        year = unicode(date.year)
        month = date.month
        firstdaythismonth = date
        firstdaynextmonth = _first_day_of_next_month(date)
        result = db.executesql("""SELECT cs.id, 
                                         cs.customerid, 
                                         cs.startdate, 
                                         cs.enddate, 
                                         mst.monthlyfee, 
                                         ap.amount 
                                   FROM customers_subscriptions cs
            LEFT JOIN 
            (SELECT id, name, monthlyfee from school_subscriptions) ssu
            ON ssu.id = ms.membershiptypeid 
            LEFT JOIN
            (SELECT customerid, paymentyear, paymentmonth, amount F
             ROM alternativepayments 
             WHERE paymentyear=%s AND 
                   paymentmonth=%s AND 
                   payment_categories_id IS NULL) ap
            ON ms.customerid = ap.customerid
            WHERE cs.startdate < %s AND 
                  (cs.enddate >= %s OR cs.enddate IS NULL)""", 
            (year, month, firstdaynextmonth, firstdaythismonth))
        paused_subscriptions = get_paused_subscriptions(date)
        total = 0
        for record in result:
            if record[0] in paused_subscriptions:
                continue
            if record[5] != None:
                total += record[5]
            else:
                total += record[4]
        if month > today.month and int(year) >= today.year:
            return 0
        else:
            return total
            
    def get_month_classcards(date):
        # helper function to get monthly class card revenue
        year = unicode(date.year)
        month = date.month
        firstdaythismonth = date
        firstdaynextmonth = _first_day_of_next_month(date)
        
        query = (db.customers_classcards.Startdate >= firstdaythismonth) & \
                (db.customers_classcards.Startdate < firstdaynextmonth)
        sum = db.customers_classcards.Price.sum()
        value = db(query).select(sum).first()[sum]
        
        if month > today.month and int(year) >= today.year:
            return 0
        else:
            if value == None:
                return 0
            else:
                return value
        
    def get_month_dropinclasses(date):
        # helper function to get monthly drop in class revenue
        year = unicode(date.year)
        month = date.month
        firstdaythismonth = date
        firstdaynextmonth = _first_day_of_next_month(date)
        
        query = (db.classes_attendance.AttendanceType==2) & \
                (db.classes_attendance.ClassDate >= firstdaythismonth) & \
                (db.classes_attendance.ClassDate < firstdaynextmonth)
        sum = db.classes.DropInPrice.sum()
        value = db(query).select(sum, 
                            left=[db.classes.on(db.classes_attendance.classes_id==\
                                                db.classes.id)]).first()[sum]
        
        if month > today.month and int(year) >= today.year:
            return 0
        else:
            if value == None:
                return 0
            else:
                return value
        
    def get_month_trialclasses(date):
        # helper function to get monthly trialclass revenue
        year = unicode(date.year)
        month = date.month
        firstdaythismonth = date
        firstdaynextmonth = _first_day_of_next_month(date)
        
        query = (db.classes_attendance.AttendanceType==1) & \
                (db.classes_attendance.ClassDate >= firstdaythismonth) & \
                (db.classes_attendance.ClassDate < firstdaynextmonth)
        sum = db.classes.TrialPrice.sum()
        value = db(query).select(sum, 
                        left=[db.classes.on(db.classes_attendance.classes_id==\
                              db.classes.id)]).first()[sum]
        
        if month > today.month and int(year) >= today.year:
            return 0
        else:
            if value == None:
                return 0
            else:
                return value
        
    def calculate_average(total):
        # calculate average
        if month > today.month and int(year) >= today.year:
            return [T("Monthly average") ,"{0:.2f}".format(total/today.month)] 
            # only show the last two decimal numbers
        else:
            return [T("Monthly average") ,"{0:.2f}".format(total/12)] 
            # only show the last two decimal numbers 
    
    ### subscriptions
    total = 0
    for month in range(1,13):
        revenue = get_month_subscriptions(datetime.date(year, month, 1))
        json_data['subscriptions']['datasets'][0]['data'].append(revenue)
        total += revenue
    
    average = calculate_average(total)
    total = [ T("Total"), total ]
    
    subscriptions_chart_title = H2(T("Subscriptions revenue"))
    
    subscriptions_avg_table = TABLE(TR(TD(total[0]), TD(total[1])), 
                                  TR(TD(average[0]), TD(average[1])))
    subscriptions_data_table = TABLE(TR(TH(data_table_titles[0]), 
                                      TH(data_table_titles[1])), 
                                   _class="os-stats_data_table")
    
    # fill the details table
    for i in range(0, 12):
        subscriptions_data_table.append(\
            TR(TD(months[i][1]), 
               TD(json_data['subscriptions']['datasets'][0]['data'][i])))
    ### subscriptions end
    
    ### classcardss begin
    classcards_chart_data = list()
    total = 0
    for month in range(1,13):
        revenue = get_month_classcards(datetime.date(year, month, 1))
        json_data['classcards']['datasets'][0]['data'].append(revenue)
        total += revenue
        
    average = calculate_average(total)
    total = [ T("Total"), total ]
    
    classcards_chart_title = H2(T("Class cards revenue"))
    
    classcards_avg_table = TABLE(TR(TD(total[0]), TD(total[1])), 
                                  TR(TD(average[0]), TD(average[1])))
    classcards_data_table = TABLE(TR(TH(data_table_titles[0]), 
                                      TH(data_table_titles[1])), 
                                   _class="os-stats_data_table")
    
    # fill the details table
    for i in range(0, 12):
        classcards_data_table.append(TR(TD(months[i][1]), 
                        TD(json_data['classcards']['datasets'][0]['data'][i])))
    
    ### classcards end
    
    ### drop in classes begin
    total = 0
    for month in range(1,13):
        revenue = get_month_dropinclasses(datetime.date(year, month, 1))
        json_data['dropin']['datasets'][0]['data'].append(revenue)
        total += revenue

            
    dropin_title = T('Drop in classes revenue for ') + unicode(year)

    average = calculate_average(total)
    total = [ T("Total"), total ]
    
    dropin_chart_title = H2(T("Drop in classes revenue"))
    
    dropin_avg_table = TABLE(TR(TD(total[0]), TD(total[1])), 
                             TR(TD(average[0]), TD(average[1])))
    dropin_data_table = TABLE(TR(TH(data_table_titles[0]), 
                                 TH(data_table_titles[1])), 
                                 _class="os-stats_data_table")
    
    # fill the details table
    for i in range(0, 12):
        dropin_data_table.append(TR(TD(months[i][1]), 
            TD(json_data['dropin']['datasets'][0]['data'][i])))

    ### drop in classes end
    
    ### trialclasses begin
    total = 0
    for month in range(1,13):
        revenue = get_month_trialclasses(datetime.date(year, month, 1))
        json_data['trialclasses']['datasets'][0]['data'].append(revenue)
        total += revenue
    
    average = calculate_average(total)
    total = [ T("Total"), total ]
    
    trialclasses_chart_title = H2(T("Trialclasses revenue"))
    
    trialclasses_avg_table = TABLE(TR(TD(total[0]), TD(total[1])), 
                                   TR(TD(average[0]), TD(average[1])))
    trialclasses_data_table = TABLE(TR(TH(data_table_titles[0]), 
                                       TH(data_table_titles[1])), 
                                       _class="os-stats_data_table")
    
    # fill the details table
    for i in range(0, 12):
        trialclasses_data_table.append(TR(TD(months[i][1]), 
            TD(json_data['trialclasses']['datasets'][0]['data'][i])))
    ### trialclasses end
    
    ### total begin
    total = 0
    for month in range(0,12):
        revenue = json_data['subscriptions']['datasets'][0]['data'][month] + \
                  json_data['classcards']['datasets'][0]['data'][month] + \
                  json_data['dropin']['datasets'][0]['data'][month] + \
                  json_data['trialclasses']['datasets'][0]['data'][month]
        json_data['total']['datasets'][0]['data'].append(revenue)
        total += revenue
    
    average = calculate_average(total)
    total = [ T("Total"), total ]
    
    total_chart_title = H3(T("Total revenue"))
    
    total_avg_table = TABLE(TR(TD(total[0]), TD(total[1])), 
                            TR(TD(average[0]), TD(average[1])))
    total_data_table = TABLE(TR(TH(data_table_titles[0]), 
                                       TH(data_table_titles[1])), 
                                       _class="os-stats_data_table")
    
    # fill the details table
    for i in range(0, 12):
        total_data_table.append(TR(TD(months[i][1]), 
            TD(json_data['total']['datasets'][0]['data'][i])))
    
    ### total end
    
    
    back = get_button('back', URL('index'))
    
    return dict(form=form, back=back, json_data=json_data,
        total_chart_title=total_chart_title,
        total_avg_table=total_avg_table,
        total_data_table=total_data_table,
        subscriptions_chart_title=subscriptions_chart_title,
        subscriptions_avg_table=subscriptions_avg_table,
        subscriptions_data_table=subscriptions_data_table,
        classcards_chart_title=classcards_chart_title,
        classcards_avg_table=classcards_avg_table,
        classcards_data_table=classcards_data_table,
        dropin_chart_title=dropin_chart_title,
        dropin_avg_table=dropin_avg_table,
        dropin_data_table=dropin_data_table,
        trialclasses_chart_title=trialclasses_chart_title,
        trialclasses_avg_table=trialclasses_avg_table,
        trialclasses_data_table=trialclasses_data_table)
        
        
