# coding: utf8
import os
import datetime
import openpyxl
from gluon.contrib.user_agent_parser import mobilize 
from gluon.tools import Crud

from general_helpers import NRtoDay
from general_helpers import get_classname
from general_helpers import get_button
from general_helpers import User_helpers

# helper functions
    
@auth.requires_login()
def _get_teachers_user_links(row):
    '''
        This function generated the drop down list in the links to manage the user account of a teacher
    '''
    account = db.auth_user(teachers_id=row.id)
    if account:
        teID = row.id
        teacher_name = account.first_name + " " + account.last_name
        reset_password = A(T("Change password"), _href=URL('user_account', args=[teID]), _class='web2py_grid_drop-down')
        delete_account = A(T("Delete account"), _href=URL('user_account_delete', args=[teID]), 
            _onclick="return confirm('" + \
                T('Do you really want to delete the OpenStudio user account for') + \
                " " + teacher_name + "?');", _class='web2py_grid_drop-down')
        
        links = [reset_password, delete_account]
        
        div = DIV( 
            A(SPAN(_class="icon user icon-user"), " " , T('Account'), " ", 
                SPAN(_class='caret'), 
                _href='#', 
                _class='btn dropdown-toggle', 
                **{'_data-toggle':"dropdown"}
                ),
            UL(*links, 
               _class='dropdown-menu' ),
            _class='btn-group'
            )
        
        return div
    else:
        return A(SPAN(_class="buttontext button", _title=T("User account")), SPAN(_class="icon user icon-user"), " " + T("Account"), _class="w2p_trap button btn", \
        _href=URL('user_account', args=[row.id]))
    

# helpers end

@mobilize
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'teachers'))
def index():
    session.teachers_back = None # unset this value in case it was set from preferences -> users
    response.title = T("Teachers")
    response.subtitle = T("")
    db.teachers.id.readable = False
    db.teachers.Education.readable = False
    db.teachers.Address.readable = False
    db.teachers.Postalcode.readable = False
    db.teachers.Country.readable = False
    db.teachers.Email.readable = False
    db.teachers.Country.readable = False
    db.teachers.Comments.readable = False
    if request.user_agent()['is_mobile']:
        db.teachers.DateOfBirth.readable=False
        db.teachers.Gender.readable=False
        db.teachers.City.readable=False
        db.teachers.Telnr.readable=False
        db.teachers.Mobnr.readable=False
    
    if auth.has_membership(group_id='admins') or auth.has_permission('update', 'teachers'):
        links = [lambda row: A(SPAN(_class="buttontext button", 
                                    _title=T("Class types")), 
                               SPAN(_class="icon edit icon-edit"), 
                               " " + T("Class types"), 
                               _class="w2p_trap button btn", 
                               _href=URL('edit_classtypes', args=[row.id])),
                 _get_teachers_user_links,
                 lambda row: get_button('edit', 
                                        URL('teacher_edit', args=[row.id]), 
                                        T("Edit this teacher")),
                 lambda row: get_button('delete_notext', 
                                        URL('check_delete_teacher', 
                                            args=[row.id]))]
    else:
        links = []
    grid = SQLFORM.grid(db.teachers, 
                        links=links, 
                        create=False, 
                        editable=False,
                        details=False, 
                        csv=False, 
                        searchable=False, 
                        deletable=False, 
                        ui = session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    
    # show back, add and export buttons above teachers list
    back = get_button('back', URL('school_properties', 'index'))
    
    add = get_button('add', URL('teacher_add'))
    export = A(SPAN(_class="buttontext button", 
                    _title=T("Export contact list")), 
               SPAN(_class="icon download icon-download"), 
               " " + T("Export contact list"), 
               _class="button btn", \
               _href=URL('export_excel'))
    console = grid.element('.web2py_console')
    console.insert(0, back)
    console.insert(1, add)
    console.insert(2, export)
        
    sidebar_content = DIV()
    if 'edit' in request.args:
        sidebar_content.append(remove)
    
    if request.user_agent()['is_mobile']:
        if 'edit' in request.args:
            left_sidebar_enabled=True
        else:
            left_sidebar_enabled=False
        right_sidebar_enabled=False
    else:
        left_sidebar_enabled=False
        if 'edit' in request.args:
            right_sidebar_enabled=True
        else:
            right_sidebar_enabled=False
    
    return dict(grid=grid, sidebar_content=sidebar_content, left_sidebar_enabled=left_sidebar_enabled, right_sidebar_enabled=right_sidebar_enabled)
    
@auth.requires_login()
def teacher_add():
    """
        This function shows an add page for a teacher
    """
    response.title = T("New teacher")
    response.subtitle = T("")
    response.view = 'general/only_content.html'
    
    return_url = URL('index')
    
    crud.messages.submit_button = T("Add")
    crud.messages.record_created = T("Added teacher")
    crud.settings.create_next = return_url
    form = crud.create(db.teachers)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires_login()
def teacher_edit():
    """
        This function shows an edit page for a teacher
        request.args[0] is expected to be the teachers_id (teID)
    """
    teID = request.args[0]
    response.title = T("Edit teacher")
    response.subtitle = T("")
    response.view = 'general/only_content.html'
    
    if session.teachers_back == 'users': # check if we're coming from preferences -> users
        return_url = URL('settings', 'users')
        session.teachers_back = None
    else:
        return_url = URL('index')
        
    user_helpers= User_helpers()
    
    crud.messages.submit_button = T("Update")
    crud.messages.record_updated = T("Updated teacher")
    crud.settings.update_next = return_url
    crud.settings.update_onaccept = [user_helpers.teacher_edit_auth_user_sync]
    crud.settings.update_deletable = False
    form = crud.update(db.teachers, teID)
    
    back = get_button('back', return_url)
    classtypes = A(SPAN(_class="buttontext button", _title=T("Class types")), 
                   SPAN(_class="icon edit icon-edit"), 
                   " " + T("Class types"), 
                   _class="w2p_trap button btn", 
                   _href=URL('edit_classtypes', args=[teID]))
    row = db.teachers(teID)
    account = _get_teachers_user_links(row)
    back = DIV(back, " ", classtypes, " ", account)
    
    return dict(content=form, back=back)
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('update', 'teachers'))
def user_account_delete():
    '''
        This function removes the user acount for a teacher
    '''
    teID = request.args[0]
    row = db.teachers[teID]
    
    user_id = db.auth_user(teachers_id = teID).id
    user_helpers = User_helpers()
    user_helpers.on_user_delete(db.auth_user, user_id)
    
    query = (db.auth_user.teachers_id == teID)
    db(query).delete()
    session.flash = T("Removed user account for " + row.FirstName + " " + row.LastName + ".")
    
    redirect(URL('index'))
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('update', 'teachers'))
def user_account():
    """
        This function is used to create a user account for a teacher or
        change the password if the account exists.
    """
    response.view = 'general/only_content.html'
    teID = request.args[0]
    
    row = db.teachers[teID]
    db.auth_user.first_name.readable = False
    db.auth_user.first_name.writable = False
    db.auth_user.last_name.readable = False
    db.auth_user.last_name.writable = False
    db.auth_user.email.writable = False
    db.auth_user.teachers_id.writable = False
    db.auth_user.first_name.default = row.FirstName
    db.auth_user.last_name.default = row.LastName
    db.auth_user.email.default = row.Email
    db.auth_user.teachers_id.default = row.id
    
    return_url = URL('index')
    
    # check if a user account exists
    account = db.auth_user(teachers_id=teID)
    if account:
        response.title = T("Change password")
        
        crud.settings.update_deletable = False
        crud.settings.update_next = return_url
        crud.messages.submit_button = T('Update')
        crud.messages.record_updated = T('Changed password for ' + row.FirstName + ' ' + row.LastName)
        form = crud.update(db.auth_user, account.id)
    else:
        response.title = T("Create user account")
        check = db.auth_user(email=db.teachers(teID).Email)
        if check is None:
            user_helpers = User_helpers()
            crud.settings.create_onaccept = [ user_helpers.on_user_create ]
            crud.settings.create_next = return_url
            crud.messages.submit_button = T('Create')
            crud.messages.record_created = T('User account created for ' + row.FirstName + ' ' + row.LastName)
            form = crud.create(db.auth_user)
        else:
            # an account already exists, so to prevent duplicate emails in db.auth_user show message instead of form
            form = DIV(BR(), T("An account could not be created, please verify the email address entered for \
                this teacher is unique and not already in use for another account."))
    response.subtitle = T("")
        
    content = DIV(form)
    
    back = get_button('back', return_url)
    
    return dict(content=content, back=back)
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('update', 'teachers'))
def check_delete_teacher():
    '''
        This function checks if a teacher can be safely deleted
        request.args[0] is expected to be the teachers_id (teID)
    '''
    response.title = T("Delete teacher")
    teID = request.args[0]
    row = db.teachers[teID]
    response.subtitle = row.FirstName + " " + row.LastName
    
    content = []
    # check for classes or subteachers entries
    query = ((db.classes.teachers_id == teID) |
             (db.classes.teachers_id2 == teID))
    cla_rows = db(query).select(db.classes.id)
    query = ((db.classes_subteachers.teachers_id == teID) | \
             (db.classes_subteachers.teachers_id2 == teID))
    sub_rows = db(query).select(db.classes_subteachers.classes_id)
    if cla_rows.first() != None or sub_rows.first() != None:
        content.append(H3(T("Unable to delete this teacher")))
    else:
        content.append(BR())
        content.append(BR())
        content.append(H3(T("Are you sure you want to delete this teacher?")))
        content.append(A(SPAN(_class="buttontext button", _title=T("Yes")), SPAN(_class=""), T("Yes"), _class="w2p_trap button btn", \
            _href=URL('delete_teacher', vars=dict(teachers_id=teID))))
        content.append(" ")
        content.append(A(SPAN(_class="buttontext button", _title=T("No")), SPAN(_class=""), B(T("No")), _class="w2p_trap button btn", \
            _href=URL('index')))
        
    if cla_rows.first() != None:
        content.append(DIV(T("Please check the teacher assignment for the following classes,"), BR(), BR()))
        d = DIV()
        for row in cla_rows:
            clsID = row.id
            classname = get_classname(clsID)
            d.append(A(classname, _href=URL('classes', 'edit_classes', args=['edit', 'classes', clsID], user_signature=True)))
            d.append(BR())
        content.append(d)
        content.append(BR())
    
    if sub_rows.first() != None:
        content.append(DIV(T("Please check the substitute teacher assignment for the following classes,"), BR(), BR()))
        d = DIV()
        for row in sub_rows:
            clsID = row.classes_id
            classname = get_classname(clsID)
            d.append(A(classname, _href=URL('classes', 'edit_classes', args=['edit', 'classes', clsID], user_signature=True)))
            d.append(BR())
        content.append(d)
        
    back = get_button('back', URL('index'))
   
    return dict(content=content, back=back)
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('update', 'teachers'))
def delete_teacher():
    teID = request.vars['teachers_id']
    query = (db.teachers.id == teID)
    row = db.teachers[teID]
    db(query).delete()
    session.flash = T("Successfully removed") + " " + row.FirstName + " " + row.LastName 
    
    redirect(URL('index'))

@auth.requires_login()
def edit_classtypes():
    # generate the form
    teachers_id = request.args[0]
    record = db.teachers[teachers_id]
    teachername = record.FirstName + " " + record.LastName
    response.title = T("Classtypes")
    response.subtitle = teachername
    response.view = 'general/only_content.html'
    
    table = TABLE()
    query = (db.teachers_classtypes.teachers_id==teachers_id)
    rows = db(query).select(db.teachers_classtypes.school_classtypes_id)
    classtypeids = list()
    for row in rows:
        classtypeids.append(unicode(row.school_classtypes_id))
    rows = db().select(db.school_classtypes.id, 
                       db.school_classtypes.Name, 
                       orderby=db.school_classtypes.Name)
    for row in rows:
        if unicode(row.id) in classtypeids:
            # check the row
            table.append(TR(INPUT(_type='checkbox', _name=row.id, _value="on", value="on"), row.Name))
        else:
            table.append(TR(INPUT(_type='checkbox', _name=row.id, _value="on"), row.Name))
    form = FORM(table, INPUT(_type='submit', _value=T('Update')))
    
    rows = db().select(db.school_classtypes.id)
    classtypeids = list()
    for row in rows:
        classtypeids.append(unicode(row.id))
        
    if form.accepts(request,session):
        #remove all current records
        db(db.teachers_classtypes.teachers_id==teachers_id).delete()
        # insert new records for teacher
        for classtypeID in classtypeids:
            if request.vars[classtypeID] == 'on':
                db.teachers_classtypes.insert(teachers_id=teachers_id, 
                                              school_classtypes_id=classtypeID)
        session.flash = T('Assigned classes changed for ' + teachername)
        redirect(URL('index'))
        
    description = T("Here you can mark which kinds of classes a teacher is able to teach.")
    content = DIV(BR(), description, BR(), form)
    
    back = get_button('back', URL('index'))
    
    return dict(content=content, back=back)
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'teachers'))
def export_excel():
    def write_cell(r, c, data, fontsize, bold=False, wrap_text=True):
        cell = ws.cell(row = r, column = c)
        cell.value = ""
        cell.style.font.bold = bold
        cell.style.font.size = fontsize
        cell.style.alignment.wrap_text = wrap_text
        cell.style.alignment.vertical = "top"
        cell.style.alignment.horizontal = "left"
        cell.value = data
    
    export_type = "contactlist"
    date = datetime.date.today()
    
    error = False

    if export_type == "contactlist":
        wb = openpyxl.workbook.Workbook()
        ws = wb.worksheets[0]
        ws.title = T("ContactList") + " " + unicode(date)
        cell = ws.cell(row = 0, column = 0)
        cell.value = T("Teachers contact list") + " " + str(date)
        cell.style.font.size = 12
        cell.style.font.bold = True
        
        # write the header
        field = [ "First name", "Last name", "Telephone", "Mobile", "Email" ]
        row = 2
        for c in range(0, len(field)):
            write_cell(row, c, field[c], 10, True, False)
        # fill the columns
        rows = db().select(db.teachers.FirstName, db.teachers.LastName, db.teachers.Telnr, db.teachers.Mobnr, db.teachers.Email, 
            orderby=db.teachers.FirstName|db.teachers.LastName)
        
        r = 3
        for row in rows:
            write_cell(r, 0, row.FirstName, 10, False, True)
            write_cell(r, 1, row.LastName, 10, False, True)
            write_cell(r, 2, row.Telnr, 10, False, True)
            write_cell(r, 3, row.Mobnr, 10, False, True)
            write_cell(r, 4, row.Email, 10, False, True)
            r += 1
            
        fname = T('Contactlist') + '.xlsx'
        filename = os.path.join(request.folder,('temp'),fname)
        wb.save(filename = filename)
        return response.stream(filename, attachment=True, filename=fname)
