# coding: utf8

from plugin_solidform import SOLIDFORM
from gluon.contrib.user_agent_parser import mobilize 

from general_helpers import get_button
from general_helpers import get_ajax_loader
from general_helpers import get_submenu
from general_helpers import Memo_links

from form_remodeller import remodel_memo_form

### helpers begin
def _generate_attendance_searchform(wsaID, name, function):
        
    form = SOLIDFORM.factory(
        Field('name', default=name, label=T("")),
        submit_button='Search',
        )
    form.element('input[name=name]')['_placeholder']='Search...'
    if function == 'manage_attendance':
        form.add_button(T('Show all'), URL(function, user_signature=True))
    else:
        form.add_button(T('Show all'), URL(function, vars=dict(wsaID=wsaID), user_signature=True))
    
    return form
    
def workshop_get_name(wsID):
    return db.workshops(wsID).Name

def _generate_activity_spaces(row):
    total = unicode(row.Spaces)
    query = ((db.workshops_registration.workshops_id==row.workshops_id) & \
             (db.workshops_registration.workshops_activities_id==None))
    workshops_registration = db(query).count()
    query = (db.workshops_registration.workshops_activities_id==row.id)
    activity_attendance = db(query).count()
    used = unicode(workshops_registration + activity_attendance)
    return used + "/" + total
    
def _activity_calculate_available_spaces(wsaID):
    # available spaces
    row = db.workshops_activities[wsaID]
    query = ((db.workshops_registration.workshops_id==row.workshops_id) & \
             (db.workshops_registration.workshops_activities_id==None))
    ws_att = db(query).count()
    query = (db.workshops_registration.workshops_activities_id==wsaID)
    act_att = db(query).count()
    return row.Spaces - (ws_att + act_att)

@auth.requires_login()
def _check_attendance_full_ws(row):
    wsID = session.wsID
    customers_id = row.id
    check = db.workshops_registration(customers_id=customers_id, 
                                      workshops_id=session.wsID, 
                                      workshops_activities_id=None)
    if check:
        return ""
    else:
        return get_button('add', URL("change_attendance",
                                     vars=dict(customers_id=row.id)))
            
@auth.requires_login()
def _check_activity_attendance(row):
    wsID = session.wsID
    wsaID = session.wsaID
    customers_id = row.id
    check = db.workshops_registration(customers_id=customers_id, 
                                      workshops_id=session.wsID, 
                                      workshops_activities_id=None)
    activity_check = db.workshops_registration(customers_id=customers_id, 
                                               workshops_activities_id=wsaID)
    if check or activity_check:
        return ""
    else:
        return A(SPAN(_class="buttontext button", _title=T("Add")), SPAN(_class="icon plus icon-plus"), T("Add"), _class="button btn", \
            _href=URL("change_activity_attendance",vars=dict(customers_id=row.id, wsaID=wsaID, remove=False, paid=True)))
    
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'workshops'))
def get_workshops_menu(page):
    
    pages = [['attendance', T('Attendance'), URL('attendance')],
             ['activity', T('Activities'), URL('activity')],
             ['memos', T('Memos'), URL('memos')],
             ['payments', T('Payments'), URL('payments')],
             ['stats', T('Quick stats'), URL('stats')],
             ['workshop_edit', T('Edit'), URL('workshop_edit')]]

    return get_submenu(pages, page)

### helpers end

@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'workshops'))
def index_get_filter_form(filter_option):
    filter_options = [('current', T('Current')), ('archived', T('Archived'))]
    
    form = FORM(_action='#', 
                _method='post', 
                _class='right', 
                _id='workshops_show_filter', 
                _style='width:300px;')
    select = SELECT(value=filter_option, 
                    _name='filter', 
                    _class='generic-widget', 
                    _onchange="this.form.submit();", 
                    _style='vertical-align:text;')
    for option in filter_options:
        selected = False
        if filter_option == option[0]:
            selected = True
        select.append(OPTION(option[1], _value=option[0], _selected=selected))
        
    submit = INPUT(_type='submit', _value=T("Go"))
    form.insert(0, select)
    form.append(submit)
        
    return form
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'workshops'))
def index():
    response.title = T("Workshops")
    response.subtitle = T("")
    response.view = 'general/one_grid_with_sidebar.html'
    
    filter_option = 'current'
    query = (db.workshops.Archived == False)
    
    if 'filter' in request.vars:
        filter_option = request.vars['filter']
        session.ws_filter_option = filter_option
        if filter_option == 'current':
            query = (db.workshops.Archived == False)
        elif filter_option == 'archived':
            query = (db.workshops.Archived == True)
    elif session.ws_filter_option == 'archived':
            query = (db.workshops.Archived == True)
    else:
        session.ws_filter_option = filter_option
    
    links = [ lambda row: A(SPAN(_class="buttontext button", _title=T("Manage")), 
                SPAN(_class="icon chevron right icon-chevron-right"), T("Manage"), 
                _class="button btn", \
                _href=URL('workshops', 'attendance', vars=dict(wsID=row.id))),
              lambda row: get_button('edit', URL('workshop_edit', args=[row.id]))
            ]
    
    if request.user_agent().is_mobile:
        fields = [ db.workshops.Name, db.workshops.Startdate ]
    else:
        fields = [ db.workshops.Name, db.workshops.Startdate, db.workshops.Teacher, db.workshops.Price ]
    maxtextlengths = {'workshops.Name' : 50}
    db.workshops.id.readable=False
    grid = SQLFORM.grid(query, fields=fields, links=links,
        maxtextlengths=maxtextlengths,
        create=False, editable=False,
        details=False, csv=False, searchable=False, 
        orderby = db.workshops.Startdate,
        ui = session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    
    add_url = URL('workshop_add')
    add = get_button('add', add_url, T("Add a new workshop"))
    # Add back button to web2py_console div at top of the grid
    console = grid.element('.web2py_console')
    console.insert(0, add)
    console.append(index_get_filter_form(session.ws_filter_option)) 
        
    return dict(grid=grid)
    
@auth.requires_login()
def workshop_add():
    """
        This function shows an add page for a workshop
    """
    response.title = T("Add workshop")
    response.subtitle = T("")
    response.view = 'general/only_content.html'
    
    return_url = URL('index')
    db.workshops.Archived.readable = False
    db.workshops.Archived.writable = False
    
    crud.messages.submit_button = T("Add")
    crud.messages.record_created = T("Added workshop")
    crud.settings.create_next = return_url
    form = crud.create(db.workshops)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires_login()
def workshop_edit():
    """
        This function shows an edit page for a workshop
        request.args[0] is expected to be the workshop ID (wsID)
    """
    try:
        wsID = request.args[0]
        return_url = URL('index')
    except:
        wsID = session.wsID
        return_url = URL('attendance')
    response.title = T("Edit workshop")
    response.subtitle = workshop_get_name(wsID)
    response.view = 'general/only_content.html'
    

    
    crud.messages.submit_button = T("Update")
    crud.messages.record_updated = T("Updated workshop")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.workshops, wsID)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('update', 'workshops'))
def activity():
    '''
        This function shows a page that lists the activities for a selected
        workshop.
    '''
    
    workshop = db.workshops(session.wsID)
    response.title = workshop.Name + ' - ' + \
                        workshop.Startdate.strftime(session.date_format)
    response.subtitle = DIV(BR(), T("Activities"))
    response.view = 'workshops/manage_workshop.html'
    
    
    db.workshops_activities.id.readable = False
    db.workshops_activities.workshops_id.default = session.wsID
    db.workshops_activities.Spaces.readable = False
    maxtextlengths = {'workshops_activities.Description' : 32,
                      'workshops_activities.Activity': 32}
    
    links = [ dict(header=T('Filled'), body=_generate_activity_spaces) ]
    if auth.has_membership(group_id='admins') or \
       auth.has_permission('update', 'workshops_activities'):
        links.append(lambda row: A(SPAN(_class="buttontext button", 
                _title=T("Attendance")), 
            SPAN(_class="icon user icon-user"), " " +  T("Attendance"), 
            _class="button btn", 
            _href=URL('activity_attendance', vars=dict(wsaID=row.id), 
                user_signature=True),
            _title=T("Attendance for this activity")))
        links.append(lambda row: get_button('duplicate', 
            URL('activity_duplicate', args=[row.id]), 
            T("Duplicate this activity")))
        links.append(lambda row: get_button('edit', 
            URL('activity_edit', args=[row.id]), 
            T("Edit this activity")))
        
    if request.user_agent().is_mobile:
        db.workshops_activities.Endtime.readable=False
        db.workshops_activities.Price.readable=False
                
    query = (db.workshops_activities.workshops_id == session.wsID)
    grid = SQLFORM.grid(query, links=links,
        maxtextlengths=maxtextlengths,
        create=False, editable=False,
        details=False, csv=False, searchable=False,
        orderby = db.workshops_activities.Activitydate,
        ui = session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    
    # add button
    add_url = URL('activity_add', args=[session.wsID])
    add = get_button('add', add_url, T("Add a new activity"))
    console = grid.element('.web2py_console')
    console.insert(0, add)
    
    content = DIV(grid)
    menu = get_workshops_menu(request.function)
    back = get_button('back', URL('index'))
    
    return dict(content=content, back=back, menu=menu,
        left_sidebar_enabled=True)
    
@auth.requires_login()
def activity_add():
    """
        This function shows an add page for a workshop activity
    """
    response.title = T("Add activity")
    response.subtitle = workshop_get_name(session.wsID)
    response.view = 'general/only_content.html'
    
    db.workshops_activities.workshops_id.default = session.wsID
    
    return_url = URL('activity')
    
    crud.messages.submit_button = T("Add")
    crud.messages.record_created = T("Added activity")
    crud.settings.create_next = return_url
    form = crud.create(db.workshops_activities)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires_login()
def activity_edit():
    """
        This function shows an edit page for an activity
        request.args[0] is expected to be the workshops_activities_id (wsaID)
    """
    wsaID = request.args[0]
    response.title = T("Edit activity")
    wsID = db.workshops_activities(wsaID).workshops_id
    response.subtitle = workshop_get_name(session.wsID)
    response.view = 'general/only_content.html'
    
    return_url = URL('activity')
    
    crud.messages.submit_button = T("Update")
    crud.messages.record_updated = T("Updated activity")
    crud.messages.record_deleted = T("Deleted activity")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = True
    
    form = crud.update(db.workshops_activities, wsaID)
    
    duplicate = get_button('duplicate', URL('activity_duplicate', args=[wsaID]), T("Duplicate this activity"))
    back = get_button('back', return_url)
    
    back = DIV(back, " ", duplicate)
    
    return dict(content=form, back=back)
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('update', 'workshops'))
def activity_attendance():
    '''
        This function shows a page which lists the attendance for a workshop activity
    '''
    if request.vars['wsaID']:
        session.wsaID = request.vars['wsaID']
    session.customers_back = 'workshops_activity_attendance'
    session.workshops_payment_back = 'activity'
    
    activity = db.workshops_activities[session.wsaID]
    wsID = activity.workshops_id
    workshop = db.workshops(wsID)
    response.title = workshop.Name + ' - ' + \
                     workshop.Startdate.strftime(session.date_format)
    response.subtitle = DIV(BR(), activity.Activity + ' - ' + \
         activity.Activitydate.strftime(session.date_format) + ' ' + \
         activity.Starttime.strftime(session.time_format))
    
    # Attendance for this activity
    activity_attendance_table = attendance_get_table(session.wsID, 
                                                     session.wsaID)
    
    # Attendance for all activities
    full_attendance_table = attendance_get_table(session.wsID)
        
    mail_teacher =  get_button('list_to_teacher', 
        URL('mail_attendance', vars=dict(wsID=session.wsID)))
    attendance_add= get_button('add', URL('attendance_add'))
    
    # putting attendance for this activity and full workshop attendange together.
    add = get_button('add', URL('activity_attendance_add'))
    mail_teacher =  get_button('list_to_teacher', 
        URL('mail_attendance', vars=dict(wsaID=session.wsaID)))
    content = DIV(H2(T("This activity")), 
                  add, ' ', 
                  mail_teacher, 
                  activity_attendance_table, 
                  H2(T("All activities")), 
                  full_attendance_table)
    
    back = get_button('back', URL('activity'))
        
    return dict(content=content, back=back)
    
def manage_activity_attendance_get_mail_delete_buttons(row, value):
    '''
        This function returns a div which holds email and remove buttons
        for the attendance lists.
    '''
    confirm_msg = T("Are you sure you want to remove this customer from \
        the attendance list?")
    
    return DIV(
        # mail button
        A(SPAN(_class="buttontext button", _title=T("Mail")), 
            SPAN(_class="icon envelope icon-envelope"), " " + T(""), \
            _class="button btn", _href='mailto:' + row.customers.Email, 
            _title=T("Mail customer")),
        # new line
        BR(), 
        # remove button
        get_button('delete_notext', 
            URL('attendance_delete', args=[row.workshops_registration.id]),
            onclick="return confirm('" + confirm_msg + "');"),
        _class='os-customer_list_buttons' )
        
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('update', 'workshops'))
def attendance_get_table(workshops_id, workshops_activities_id=None):
    '''
        This function returns the attendance table for a workshop or workshop
        activity.
        If workshops_activities_id is specified, a table for the activity will be
        returned
    '''
    if workshops_activities_id:
        query = (db.workshops_registration.workshops_activities_id == \
                                           workshops_activities_id)
    else:
        query = (db.workshops_registration.workshops_id == workshops_id) & \
                (db.workshops_registration.workshops_activities_id == None)
    rows = db(query).select(db.workshops_registration.id, db.customers.id, 
        db.workshops_payments.id,
        db.customers.ActiveMember, db.customers.ThumbSmall, 
        db.customers.FirstName, db.customers.LastName, db.customers.Email,
        db.workshops_payments.Amount, db.workshops_payments.PaymentDate, 
        left = [(db.customers.on(\
                    db.workshops_registration.customers_id==db.customers.id)),
                (db.workshops_payments.on(\
                    db.workshops_registration.id == \
                    db.workshops_payments.workshops_registration_id)),
                ], 
        orderby=db.customers.FirstName)
        
    even_row = True
    table = TABLE(_class="os-customer_table")
    for row in rows.render():
        # get payment info
        if row.workshops_payments.Amount is None:
            add_url = URL("payment_add", args=[row.workshops_registration.id])
            payment = A(T("Add"), _href=add_url)
        else:
            payment_amount = format(row.workshops_payments.Amount, '.2f')
            payment_date = \
                row.workshops_payments.PaymentDate.strftime(session.date_format)
            edit_url = URL("payment_edit", args=[row.workshops_payments.id])
            payment = A(payment_amount + ' on ' + payment_date, 
                      _href=edit_url)
        # email
        email = row.customers.Email
        
        # get attendance info form
        hidden_field_id = INPUT(_type="hidden", 
                                _name="id",
                                _value=row.workshops_registration.id)
        form = SQLFORM(db.workshops_registration, 
            row.workshops_registration.id)
        
        payment_table = DIV( form.custom.begin,
            TABLE(TR(TD(T("Payment")),
                     TD(payment)),
                  TR(TD(form.custom.label.PaymentConfirmation),
                     TD(form.custom.widget.PaymentConfirmation)),
                  TR(TD(form.custom.label.Info),
                     TD(form.custom.widget.Info)),
                  _class='os-attendance_info_table'),
            hidden_field_id,
            form.custom.end,
        _class="left")
        
        delete = SPAN(_class="icon remove icon-remove", 
                      _id=row.workshops_registration.id)

        table.append(TR(TD(row.customers.ThumbSmall, 
                           _class="os-customer_ThumbSmall"),
                        TD(SPAN(row.customers.FirstName,
                                ' ',
                                row.customers.LastName,
                                _class="os-customer_name"),
                           BR(),
                           SPAN(A(email, _href="mailto:" + email), 
                                _class="os-customer_email"),
                            _width="40%"),
                        TD(payment_table, DIV(delete, _class="right"))
                        )
                    )
        
    return table
        
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('update', 'workshops'))
def attendance():
    '''
        This function shows a page to allow editing of the full workshop
        attendance.
        request.vars['wsID'] is expected to be the workshop_id
    '''
    if 'wsID' in request.vars:
        session.wsID = request.vars['wsID']
    
    session.customers_back = 'workshops_attendance'
    session.workshops_payment_back = 'attendance'
     # unset the workshop activity ID to prevent payment link issues
    session.wsaID = None
    
    workshop = db.workshops(session.wsID)
    response.title = workshop.Name + ' - ' + \
                        workshop.Startdate.strftime(session.date_format)
    response.subtitle = DIV(BR(), T("Full workshop attendance"))
    
    table = attendance_get_table(session.wsID)
    
    mail_teacher =  get_button('list_to_teacher', 
        URL('mail_attendance', vars=dict(wsID=session.wsID)))
    attendance_add= get_button('add', URL('attendance_add'))
    
    content = DIV(attendance_add, ' ', mail_teacher, table)
    menu = get_workshops_menu(request.function)
    back = get_button('back', URL('index'))
        
    attendance_info_url = URL('attendance_change_info')
        
    return dict(content=content, back=back, menu=menu,
        attendance_info_url=attendance_info_url,
        left_sidebar_enabled=True)
        
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'workshops'))
def mail_attendance():
    '''
        This function sends the attendance list for an activity or the full 
        workshop to the teacher(s).
        If request.vars[wsID] is specified, the attendance for the full workshop
        will be sent.
        If request.vars[wsaID] is specified, the attendance for the activity 
        will be sent.
        If both are specified, the attendance for the full workshop will be sent
    '''
    wsID = request.vars['wsID']
    wsaID = request.vars['wsaID']
    
    # start the message.
    message = "<html>\n<head><STYLE TYPE='text/css'> \
            <!-- TD{font-size: 9pt; align:left;} \
            TD, TH { padding:0 20px 0 0; } \
            TH{font-size: 9pt; text-align:left;} --->  </STYLE></head> \
            <body style=font-size:10pt; font-style:normal;'>"
    message += T("Dear teacher(s)")
    message += ",<br><br>"
    
    if wsID:
        attendance_list = 'full'
        workshop = db.workshops(wsID)
        workshop_date = workshop.Startdate.strftime(session.date_format)
        return_url = URL('attendance')
        message += T("Below is the attendance list for your workshop starting on") 
        message += ' ' + workshop_date + ".<br><br>"
        
        query = (db.workshops_registration.workshops_id == wsID) & \
                (db.workshops_registration.workshops_activities_id == None)
        
        subject = T("Attendance") + ' ' + workshop.Name + ', ' + \
            T("starting on") + ' ' + workshop_date + '.'
        
    elif wsaID:
        attendance_list = 'activity'
        workshop_activity = db.workshops_activities(wsaID)
        activity_date = workshop_activity.Activitydate.strftime(session.date_format)
        wsID = workshop_activity.workshops_id
        workshop = db.workshops(wsID)
        return_url = URL('activity_attendance')
        
        message += T("Below is the attendance list for your workshop activity on") 
        message += ' ' + activity_date + ' ' + T('starting at') + ' ' 
        message += workshop_activity.Starttime.strftime(session.time_format)
        message += ".<br><br>"
        
        query = (db.workshops_registration.workshops_activities_id == wsaID) | \
                ((db.workshops_registration.workshops_id == wsID) & \
                 (db.workshops_registration.workshops_activities_id == None))
        
        subject = T("Attendance") + ' ' + workshop.Name + ', ' + \
            T("for activity on") + ' ' + activity_date + '.'
        
    attendance = []
    rows = db(query).select(db.customers.FirstName, db.customers.LastName,
        left=[db.customers.on(db.customers.id == db.workshops_registration.customers_id)],
        orderby=db.customers.FirstName)
    for row in rows:
        attendance.append([row.FirstName, row.LastName])
        
    message += unicode(TABLE(TR(TH(T("First name")), TH(T("Last name"))),
                           *[TR(*row) for row in attendance]))
    message += "<br><br>"
    
    if db.sys_properties(Property='smtp_signature'):
        smtp_signature = db.sys_properties(Property='smtp_signature').PropertyValue
    else:
        smtp_signature = ""
    message += smtp_signature    
    message += "<br><br><br><font size=2>" + T("This message is autogenerated by OpenStudio") 
    message += " - <a href='http://www.openstudioproject.com'>www.openstudioproject.com</a></font>"
    message += "</body></html>"
    
    # now send the mail
    if not workshop.TeacherEmail and not workshop.Teacher2Email:
        session.flash = T("Unable to send mail, \
            please verify the teachers' email addresses")
    else:
        to = []
        if workshop.TeacherEmail:
            to.append(workshop.TeacherEmail)
        if workshop.Teacher2Email:
            to.append(workshop.Teacher2Email)
        check = MAIL.send(to=to, subject=subject,
              # If reply_to is omitted, then mail.settings.sender is used
              reply_to=None,
              message=message)
              
        if check:
            session.flash = T("Successfully sent mail")
        else:
            session.flash = T("Unable to send mail")
    
    redirect(return_url)
        
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('update', 'workshops'))
def attendance_add():
    workshop = db.workshops(session.wsID)
    response.title = workshop.Name
    response.subtitle = T("Attendance add")
    response.view = 'general/only_content.html'
    
    session.customers_back = 'workshops_attendance_add'
    
    # check if an activity is defined, if not, redirect back
    query_count = (db.workshops_activities.workshops_id == session.wsID)
    activities_count = db(query_count).count()
    if activities_count == 0:
        session.flash = T("Please add at least one activity.")
        redirect(URL('attendance'))
    
    name = ''
    query = (db.customers.ActiveMember == True) # generic show all active customers query
    if 'name' in request.vars: # check whether a search filter is in use
        if request.vars['name'] != '':
            name = request.vars['name']
            search_name = '%' + request.vars['name'] + '%'
            query &= ((db.customers.FirstName.like(search_name)) | (db.customers.LastName.like(search_name)))
    
    form = _generate_attendance_searchform(None, name, request.function)
    
    links = [ _check_attendance_full_ws ]
    
    # create customers list and add button
    db.customers.ActiveMember.readable = False
    maxtextlengths = {'customers.Email' : 50}
    if request.user_agent().is_mobile:
        fields = [ db.customers.FirstName, db.customers.LastName ]
    else:
        fields = [ db.customers.ThumbSmall, db.customers.ActiveMember,
            db.customers.FirstName, db.customers.LastName, db.customers.Email ]
    grid = SQLFORM.grid(query, fields=fields, links=links,
        maxtextlengths=maxtextlengths,
        create=False, details=False, deletable=False, editable=False, csv=False, searchable=False, 
        orderby = ~db.customers.id,
        ui = session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
        
    content = DIV(form, BR(), grid)
        
    back = get_button('back', URL('attendance'))
    
    return dict(content=content, back=back)
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('update', 'workshops_activities'))
def attendance_delete():
    '''
        This function removed the specified attendance record
        request.vars['id'] is expected to be the 
            workshops_registration id (wsatt_id)
    '''
    response.view = 'generic.json'
    wsatt_id = request.vars['id']
    
    query = (db.workshops_registration.id == wsatt_id)
    result = db(query).delete()
    
    # result > 0 means something was removed
    if result:
        message = T("Successfully removed")
        status = "success"
    else:
        message = T("Uh oh... something went wrong...")
        status = "fail"
    
    return dict(message=message, status=status)
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('update', 'workshops_activities'))
def attendance_update_info():
    '''
        Called as JSON
        This function is meant to be called with the json extension. 
        It takes id, Info and PaymentConfirmation as variables.
    '''
    response.view = 'generic.json'
    status = "fail"
    
    if request.vars:
        wsattID = request.vars['id']
        payment_confirmation = request.vars['PaymentConfirmation']
        row = db.workshops_registration(wsattID)
        
        # info
        if 'Info' in request.vars:
            row.Info = 'T'
        else:
            row.Info = 'F'
        # payment confirmation
        if 'PaymentConfirmation' in request.vars:
            row.PaymentConfirmation = 'T'
        else:
            row.PaymentConfirmation = 'F'
            
        result = row.update_record()
        
        if result:
            status = "success"
            message = T("Updated attendance info")
        else:
            message = T("Uh oh... something went wrong...")
    else:
        message = T("Error: no data received")
        
    return dict(status=status, message=message)

@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('update', 'workshops_activities'))
def activity_duplicate():
    '''
        This function duplicates an activity and
        redirects to the duplicated activity's edit page
    '''
    wsaID = request.args[0]
    row = db.workshops_activities[wsaID]
    
    id = db.workshops_activities.insert(workshops_id=row.workshops_id, 
                                        Activity=row.Activity, 
                                        Activitydate=row.Activitydate, 
                                        Starttime=row.Starttime, 
                                        Endtime=row.Endtime, 
                                        Spaces=row.Spaces, 
                                        Price=row.Price)
        
    session.flash = T("You are now editing the duplicated activity")
    
    redirect(URL('activity_edit', args=[id]))
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('update', 'workshops_activities'))
def activity_attendance_add():
    session.customers_back = 'workshops_activity_attendance_add'
    activity = db.workshops_activities(session.wsaID)
    wsID = activity.workshops_id
    response.title = db.workshops[wsID].Name 
    response.subtitle = activity.Activity + ' ' + \
         activity.Activitydate.strftime(session.date_format)
    response.view = 'general/only_content.html'
    
    name = ''
    query = (db.customers.ActiveMember == True) # generic show all active customers query
    if 'name' in request.vars: # check whether a search filter is in use
        if request.vars['name'] != '':
            name = request.vars['name']
            search_name = '%' + request.vars['name'] + '%'
            query &= ((db.customers.FirstName.like(search_name)) | (db.customers.LastName.like(search_name)))
    
    form = _generate_attendance_searchform(session.wsaID, name, request.function)
    
    links = [ _check_activity_attendance ] 
    
    
    # create customers list and add button
    db.customers.ActiveMember.readable = False
    if request.user_agent().is_mobile:
        fields = [ db.customers.FirstName, db.customers.LastName ]
    else:
        fields = [ db.customers.ActiveMember, db.customers.ThumbSmall,
            db.customers.FirstName, db.customers.LastName, db.customers.Email ]
    grid = SQLFORM.grid(query, fields=fields, links=links,
        #maxtextlengths=maxtextlengths,
        create=False, details=False, deletable=False, editable=False, csv=False, searchable=False, 
        orderby = ~db.customers.id,
        ui = session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
        
    available_spaces = T("Available spaces") + ": " + unicode(_activity_calculate_available_spaces(session.wsaID))
    
    content = DIV(form, BR(), grid)
        
    back = get_button('back', 
        URL('activity_attendance', vars=dict(wsaID=session.wsaID)))
    
    return dict(content=content, back=back)
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('update', 'workshops_activities'))
def change_activity_attendance():
    """
    This function edits the attendance for single workshop activities
    """
    customers_id = request.vars['customers_id']
    wsaID = request.vars['wsaID']
    
    # check spaces
    total_spaces = db.workshops_activities(wsaID).Spaces
    query = (db.workshops_registration.workshops_activities_id==wsaID)
    count = db(query).count()
    if count >= total_spaces:
        session.flash = T("Unable to add, this activity is fully booked")
    else:
        db.workshops_registration.insert(customers_id=customers_id, 
                                      workshops_activities_id=wsaID,
                                      workshops_id=session.wsID)
    redirect(URL('activity_attendance_add'))
        
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('update', 'workshops'))
def change_attendance():
    """
    This function edits the attendance for the entire workshop
    """
    customers_id = request.vars['customers_id']
    
    if request.vars['remove'] == 'True':
        query = (db.workshops_registration.customers_id==customers_id) & \
                (db.workshops_registration.workshops_id==session.wsID)
        db(query).delete()
        redirect(URL('manage_attendance', user_signature=True))
    
    # check spaces
    spaces_available = True
    query = (db.workshops_activities.workshops_id==session.wsID)
    rows = db(query).select(db.workshops_activities.id, distinct=True)
    for row in rows:
        spaces = _activity_calculate_available_spaces(row.id)
        if spaces < 1:
            spaces_available = False
    
    if spaces_available == False:
        session.flash = T("Unable to add, at least one activity is fully booked")
    else:
        # first remove the customer from all seperate activities, if any
        query = (db.workshops_registration.customers_id==customers_id) & \
                (db.workshops_registration.workshops_id==session.wsID)
        db(query).delete()
        # now add the customer for the entire workshop
        db.workshops_registration.insert(customers_id=customers_id, 
                                         workshops_activities_id=None, 
                                         workshops_id=session.wsID)
    redirect(URL('attendance_add'))
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'workshops'))
def stats():
    '''
        This function shows a page with a quick view of the revenue and 
        a table listing the top 10 cities of attending customers.
    '''
    workshop = db.workshops(session.wsID)
    response.title = workshop.Name + ' - ' + \
                        workshop.Startdate.strftime(session.date_format)
    response.subtitle = DIV(BR(), T("Quick stats"))
    response.view = 'workshops/manage_workshop.html'
    
    ### top 10 cities begin
    city_top10_title = H2(T("Top 10 cities"))
    
    count = db.customers.City.count()
    query = (db.workshops_registration.workshops_id==session.wsID) & \
            ((db.customers.City!=None) & (db.customers.City!=''))
    
    rows = db(query).select(db.customers.City, count, 
        left=[(db.customers.on(db.workshops_registration.customers_id==db.customers.id))],
        groupby=db.customers.City, 
        orderby=~count|db.customers.City, limitby=(0,10))
    
    l = []
    for row in rows:
        l.append([row.customers.City, row[count]])
    
    city_top10_title = H2(T("Top 10 cities"))
    city_top10_table = TABLE(TR(TH(T("City")), TH(T("Customers"))))
    for item in l:
        city_top10_table.append(TR(*item))

    ### top 10 cities end
    
    content = TABLE(TR( TD(city_top10_title, city_top10_table, _width="250px")))
                              
    menu = get_workshops_menu(request.function)
    back = get_button('back', URL('index'))
    
    return dict(content=content, menu=menu, back=back,
        left_sidebar_enabled=True)
    
    
@auth.requires_login()
def memos():
    # reset variable if we visited the pinboard before, 
    # otherwise we might get redirected there again after adding or editing a memo.
    session.memos_back = None 
    # now continue settings things as usual
    workshop = db.workshops(session.wsID)
    response.title = workshop.Name + ' - ' + \
                        workshop.Startdate.strftime(session.date_format)
    response.subtitle = DIV(BR(), T("Memos"))
    
    response.view = 'workshops/manage_workshop.html'
    
    memo_links = Memo_links()
    
    db.memos.id.readable = False
    db.memos.workshops_id.default = session.wsID
    
    if request.user_agent()['is_mobile']:
        fields = [ db.memos.Title, db.memos.Startdate ]
        links = []
    else:
        fields = [ db.memos.Visible, db.memos.Announcement, db.memos.Title, db.memos.Msg, 
            db.memos.Duedate, db.memos.Startdate, db.memos.Enddate, db.memos.Priority ]
        if 'edit' in request.args:
            links = []
        else:
            links = [ dict(header=T("Priority"), body=memo_links.memo_link_priority), 
                dict(header=T("Visible"), body=memo_links.memo_link_visible), 
                dict(header=T("Top"), body=memo_links.memo_link_announcement) ]
    
    maxtextlengths = {'memos.Msg' : 30}
    query = (db.memos.workshops_id == session.wsID)
    grid = SQLFORM.grid(query, fields=fields, links=links, details=False, csv=False, searchable=False, 
        create=False,
        maxtextlengths=maxtextlengths, ui=session.grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    
    add_url = URL('memo_add')
    add = get_button('add', add_url, T("Add a new memo"))
                
    if 'edit' not in request.args:
        console = grid.element('.web2py_console')
        console.insert(0, add)
    else:
        redirect(URL('memo_edit', args=[request.args[2]]))
        
    back = get_button('back', URL('index'))
    menu = get_workshops_menu(request.function)
    
    return dict(content=grid, menu=menu, back=back, left_sidebar_enabled=True)
    
@auth.requires_login()
def memo_add():
    """
        This function shows an add page for a memo
        The workshops_id is supplied by session.wsID
    """
    response.title = T("New memo")
    response.view = 'general/only_content.html'
    db.memos.workshops_id.default = session.wsID
    
    workshop = db.workshops(session.wsID)
    response.subtitle = workshop.Name
        
    return_url = URL('memos')
    
    crud.messages.submit_button = T("Create")
    crud.messages.record_created = T("Added memo")
    crud.settings.create_next = return_url
    form = crud.create(db.memos)
    
    content = remodel_memo_form(form)
    
    back = get_button('back', return_url)
    
    return dict(content=content, back=back)
    
@auth.requires_login()
def memo_edit():
    """
        This function shows an edit page for a memo
        request.args[0] is expected to be the memoID
    """
    response.view = 'general/only_content.html'
    response.title = T("Edit memo")
    memoID = request.args[0]
    db.memos.workshops_id.default = session.wsID
    
    workshop = db.workshops(session.wsID)
    response.subtitle = workshop.Name
    
    return_url = URL('memos')
    
    crud.messages.submit_button = T("Update")
    crud.messages.record_updated = T("Updated memo")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.memos, memoID)
    
    content = remodel_memo_form(form)
    
    back = get_button('back', return_url)
    
    return dict(content=content, back=back)
    
@auth.requires(auth.has_membership(group_id='admins') or \
                auth.has_permission('read', 'workshops'))
def payments():
    '''
        This function shows a page with an overview of the payments for a 
        workshop.
    '''
    
    session.workshops_payment_back = None
    workshop = db.workshops(session.wsID)
    response.title = workshop.Name + ' - ' + \
                        workshop.Startdate.strftime(session.date_format)
    response.subtitle = DIV(BR(), T("Payments"))
    response.view = 'workshops/manage_workshop.html'
    
    db.workshops_payments.Amount.represent = \
        lambda value, row: format(value, '.2f')
    
    query = (db.workshops_registration.workshops_id == session.wsID)
    
    db.workshops_payments.id.readable = False
    fields = [ db.workshops_payments.id,
               db.workshops_payments.PaymentDate,
               db.workshops_payments.Amount,
               db.workshops_payments.payment_methods_id,
               db.customers.FirstName,
               db.customers.LastName,
               db.workshops_registration.workshops_activities_id ]
    
    links = [ lambda row: get_button('edit', 
                           URL('payment_edit', args=[row.workshops_payments.id]), 
                           T("Edit the name of this location")) ]
    grid = SQLFORM.grid(query, fields=fields, links=links,
        create=False,
        editable=False,
        details=False, 
        searchable=False, 
        csv=False, 
        ui=session.grid_ui,
        left=[db.workshops_registration.on(db.workshops_registration.id==\
                         db.workshops_payments.workshops_registration_id),
              db.customers.on(db.customers.id==\
                                    db.workshops_registration.customers_id)],
        orderby=db.workshops_payments.PaymentDate
        )
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button
    
    payments_title = H3(T("Payments"))
    
    # revenue begin
    query = (db.workshops_registration.workshops_id == session.wsID)
    rows = db(query).select(db.workshops_registration.workshops_id,
                       db.workshops_registration.workshops_activities_id,
                       db.workshops_payments.Amount,
        left=[db.workshops_registration.on(db.workshops_registration.id==\
                                db.workshops_payments.workshops_registration_id)]
        )
    
    full_workshop = 0
    separate_activities = 0
    
    for row in rows:
        amount = row.workshops_payments.Amount 
        try:
            if row.workshops_registration.workshops_activities_id:
                separate_activities += amount
            else:
                full_workshop += amount
        except TypeError:
            # catch the None error when amount is None
            pass
    
    revenue_total = format((full_workshop + separate_activities), '.2f')
    full_workshop = format(full_workshop, '.2f')
    separate_activities = format(separate_activities, '.2f')
    
    revenue_table = TABLE(TR(TD(T("Full workshop")), 
                             TD(full_workshop),
                          TR(TD(T("Separate activities")),
                             TD(separate_activities))),
                          TR(TD(HR()), TD(HR())),
                          TR(TD(T("Total")),
                             TD(revenue_total))
                        )
    revenue_title = H3(T("Revenue"))
    
    # revenue end
    
    content = DIV(payments_title, grid, revenue_title, revenue_table)
    
    menu = get_workshops_menu(request.function)
    back = get_button('back', URL('index'))
    
    return dict(content=content, back=back, menu=menu,
        left_sidebar_enabled=True)
        
@auth.requires_login()
def payment_add():
    """
        This function shows an add page for a payment
        request.args[0] is expected to be the workshops_registrationID (wsattID)
    """
    response.title = T("New payment")
    response.subtitle = T('')
    response.view = 'general/only_content.html'
    
    ''' if the activity id is not set it means full workshop activity, so 
    redirect to workshops page instead of activity page'''
    if session.workshops_payment_back == 'activity':
        return_url = URL('activity_attendance')
    elif session.workshops_payment_back == 'customer':
        return_url = URL('customers', 'workshops')
    else:
        return_url = URL('attendance')
    
    wsattID = request.args[0]
    db.workshops_payments.workshops_registration_id.default = wsattID
    
    wsatt = db.workshops_registration(wsattID)
    wsaID = wsatt.workshops_activities_id
    wsID = wsatt.workshops_id
    if wsaID:
        activity = db.workshops_activities(wsaID)
        price = activity.Price
    else:
        workshop = db.workshops(wsID)
        price = workshop.Price
        
    if not price is None:
        db.workshops_payments.Amount.default = price
    
    crud.messages.submit_button = T("Add")
    crud.messages.record_created = T("Added payment")
    crud.settings.create_next = return_url
    form = crud.create(db.workshops_payments)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)
    
@auth.requires_login()
def payment_edit():
    """
        This function shows an edit page for a payment
        request.args[0] is expected to be the workshops_payment ID (wspID)
    """
    response.title = T("Edit payment")
    response.subtitle = T('')
    response.view = 'general/only_content.html'
    
    wspID = request.args[0]
    
    ''' if the activity id is not set it means full workshop activity, so 
    redirect to workshops page instead of activity page'''
    if session.workshops_payment_back == 'activity':
        return_url = URL('activity_attendance')
    elif session.workshops_payment_back == 'attendance':
        return_url = URL('attendance')
    elif session.workshops_payment_back == 'customer':
        return_url = URL('customers', 'workshops')
    else:
        return_url = URL('payments')
    
    crud.messages.submit_button = T("Update")
    crud.messages.record_updated = T("Updated payment")
    crud.messages.record_deleted = T("Deleted payment")
    crud.settings.update_next = return_url
    form = crud.update(db.workshops_payments, wspID)
    
    back = get_button('back', return_url)
    
    return dict(content=form, back=back)

