# coding: utf8
import datetime

from smarthumb import SMARTHUMB

from general_helpers import NRtoDay
from general_helpers import NRtoMonth
from general_helpers import NRtoPriority
from general_helpers import represent_date
from general_helpers import represent_datetime
from general_helpers import get_months_list
from general_helpers import get_last_day_month
from general_helpers import get_group_id

from general_helpers import create_teachers_dict
from general_helpers import create_locations_dict
from general_helpers import create_classtypes_dict

# helper functions

def set_genders():
    return [ ['F', T('Female')],
             ['M', T('Male')],
             ['X', T('Other') ] ]
    
def set_class_status():
    session.class_status = dict(subteacher=T('Subteacher'),
                                open_class=T('Open'),
                                cancelled=T('Cancelled')) 
    
    
def set_dateformat():
    session.date_formats = [('%Y-%m-%d','yyyy-mm-dd'),('%m-%d-%Y','mm-dd-yyyy'),('%d-%m-%Y','dd-mm-yyyy')]
    if db.sys_properties(Property='DateFormat'):
        date_format = db.sys_properties(Property='DateFormat').PropertyValue
    else:
        date_format = None
    if date_format == None:
        session.date_format = '%Y-%m-%d'
    else:
        session.date_format = date_format
    session.time_format = '%H:%M'
    session.datetime_format = session.date_format + ' ' + session.time_format
    
def _represent_customer_Image(value, row):
    cls = 'customer_image'
    active = False
    active_marker = ''
    first_name = ''
    last_name = ''
    if 'ActiveMember' in row:
        first_name = row.FirstName
        last_name = row.LastName
        thumb = row.ThumbSmall
        cu_id = row.id
        if row.ActiveMember:
            active = True
    elif 'ActiveMember' in row.customers:
        first_name = row.customers.FirstName
        last_name = row.customers.LastName
        thumb = row.customers.ThumbSmall
        cu_id = row.customers.id
        if row.customers.ActiveMember:
            active = True
            
    if active:
        marker_class = 'customer_active_marker'
        active_marker = DIV(_class=marker_class)
    
    alt = first_name + " " + last_name
    if thumb is None:
        return DIV(active_marker,
                   A(IMG(_src=URL('static', 'images/person.png'), _alt=alt),
                   _href=URL('customers', 'edit', args=[cu_id])),
                   _class=cls)
    else:
        return DIV(active_marker,
                   A(IMG(_src=URL('default', 'download', args=value), _alt=alt),
                   _href=URL('customers', 'edit', args=[cu_id])),
                   _class=cls)
                   
def represent_gender(value, row):
    '''
        Helper to represent genders
    '''
    return_value = ''
    for gender in GENDERS:
        if value == gender[0]:
            return_value = gender[1]
    
    return return_value
               
def represent_subscription_as_membershiptype_name(value, row):
    subscription = db.customers_subscriptions(row.customers_subscriptions_id)
    id = subscription.school_subscriptions_id
    return db.school_subscriptions(id).Name
    
def represent_time(value, row):
    if value is None:
        return value
    else:
        return value.strftime(session.time_format)
        
def represent_birthday(value, row):
    if value is None:
        return value
    else:
        return value.strftime("%B %d")
        
def represent_float_as_amount(value, row):
    '''
        Takes value and rounds it to a 2 decimal number.
    '''
    if value is None:
        return ''
    else:
        return format(value, '.2f')
    
def _set_birthday(row):
    if row.DateOfBirth != None:
        return datetime.date(1900, row.DateOfBirth.month, row.DateOfBirth.day)
    else:
        return None
            
def set_show_location():
    show_location=False
    row = db.sys_properties(Property='Customer_ShowLocation')
    if row:
        if row.PropertyValue == "on":
            show_location=True
    session.show_location = show_location
    
def create_languages_dict():
    rows = db().select(db.school_languages.id, db.school_languages.Name)
    d = dict()
    for row in rows:
        d[row.id] = row.Name
    d[None] = ""
    return d
def create_customers_dict():
    rows = db().select(db.customers.id, 
                       db.customers.FirstName, 
                       db.customers.LastName)
    d = dict()
    for row in rows:
        d[row.id] = row.FirstName + " " + row.LastName
    d[None] = ""
    return d
def create_discovery_dict():
    rows = db().select(db.school_discovery.id, db.school_discovery.Name)
    d = dict()
    for row in rows:
        d[row.id] = row.Name
    d[None] = ""
    return d
def create_mstypes_dict():
    rows = db().select(db.school_subscriptions.id, db.school_subscriptions.Name)
    d = dict()
    for row in rows:
        d[row.id] = row.Name
    d[None] = ""
    return d
def create_school_levels_dict():
    rows = db().select(db.school_levels.id, db.school_levels.Name)
    d = dict()
    for row in rows:
        d[row.id] = row.Name
    d[None] = XML('&nbsp;')
    return d
def create_payment_categories_dict():
    rows = db().select(db.payment_categories.id, 
                       db.payment_categories.Name)
    d = dict()
    for row in rows:
        d[row.id] = row.Name
    d[None] = T("Default")
    return d
def create_payment_methods_dict():
    rows = db().select(db.payment_methods.id, db.payment_methods.Name)
    d = dict()
    for row in rows:
        d[row.id] = row.Name
    d[None] = ""
    d[0] = ""
    return d
def create_classes_dict():
    rows = db().select(db.classes.id, 
                       db.classes.school_locations_id, 
                       db.classes.school_classtypes_id, 
                       db.classes.Starttime, 
                       db.classes.Week_day)
    d = dict()
    for row in rows:
        location = locations_dict.get(row.school_locations_id)
        classtype = classtypes_dict.get(row.school_classtypes_id)
        
        d[row.id] = location + " "  + \
                    NRtoDay(row.Week_day) + " " + \
                    row.Starttime.strftime('%H:%M') + " " + \
                    classtype
    d[None] = ""
    return d
def create_workshops_activities_dict():
    rows = db().select(db.workshops_activities.id, 
                       db.workshops_activities.Activity)
    d = dict()
    for row in rows:
        d[row.id] = row.Activity
    d[None] = T("Full workshop")
    d[0] = T("Full workshop")
    
    return d

# the tables below here need to be defined first because of dependencies

# set global values for tables

permissions = ['create', 'read', 'update', 'delete', 'select']

# end global values for tables

def define_sys_properties():
    db.define_table('sys_properties',
        Field('Property', required=True, 
            requires=IS_NOT_EMPTY(), 
            label=T("Property")),
        Field('PropertyValue', 'text', required=True, 
            requires=IS_NOT_EMPTY(), 
            label=T("Value")),
        format='%(Property)s') 
        
def define_sys_api_users():
    db.define_table('sys_api_users',
        Field('ActiveUser', 'boolean', required=True,
            writable=False,
            default=True,
            label=T("Active")),
        Field('Username', required=True,
            requires=IS_NOT_IN_DB(db, 'sys_api_users.Username'),
            label=T("Username")),
        Field('Description', required=True,
            requires=IS_NOT_EMPTY(),
            label=T("Description")),
        Field('APIKey', required=True,
            writable=False,
            requires=IS_NOT_EMPTY(),
            label=T("Key")),
        )
        
def define_payment_methods():
    db.define_table('payment_methods',
        Field('Name', required=True, 
            requires=IS_NOT_EMPTY(), 
            label=T("Name")),
        format='%(Name)s') 
        
def define_memos():
    priorities = [(3,T("Low")),(2,T("Normal")),(1,T("High"))]
    
    db.define_table('memos',
        Field('customers_id', db.customers, readable=False, writable=False,
            requires=IS_EMPTY_OR(\
                IS_IN_DB(db,'customers.id','%(FirstName)s %(LastName)s')), 
            represent=lambda value, row: customers_dict.get(value, None), 
            label=T("Customer")),
        Field('workshops_id', db.workshops, readable=False, writable=False,
            requires=IS_EMPTY_OR(IS_IN_DB(db,'workshops.id','%(Name)s'))),
        Field('Announcement', 'boolean', readable=False, required=True, 
            default=False, 
            label=T("Show at top")),
        Field('Visible', 'boolean', readable=False, required=True, 
            default=True, 
            label=T("Visible")),
        Field('Title', required=True, 
            requires=IS_NOT_EMPTY(), 
            label=T("Title")),
        Field('Msg', 'text', required=True, 
            requires=IS_NOT_EMPTY(), 
            label=T("Message")),
        Field('Duedate', 'date', required=False, 
            requires=IS_EMPTY_OR(IS_DATE_IN_RANGE(format=session.date_format,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1))), 
            represent=represent_date, 
            label=T("Due")),
        Field('Startdate', 'date', required=True, 
            requires=IS_DATE_IN_RANGE(format=session.date_format,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)), 
            default=datetime.date.today(), 
            label=T("Notification from")),
        Field('Enddate', 'date', required=False, 
            requires=IS_EMPTY_OR(IS_DATE_IN_RANGE(format=session.date_format,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1))), 
            represent=represent_date, 
            label=T("Notification until")),
        Field('Priority', 'integer', readable=False, 
            requires=IS_IN_SET(priorities, zero=None), 
            default=2,
            represent=NRtoPriority, 
            label=T("Priority")),
        )

def define_school_languages():
    db.define_table('school_languages',
        Field('Name', required=True, 
            requires=IS_NOT_EMPTY(), 
            label=T("Name")),
        format='%(Name)s') 

def define_school_classtypes():
    db.define_table('school_classtypes',
        Field('Name', required=True, 
            requires=IS_NOT_EMPTY(), 
            label=T("Name")),
        format='%(Name)s') 
    
def define_school_discovery():
    db.define_table('school_discovery',
        Field('Name', required=True, 
            requires=IS_NOT_EMPTY(), 
            label=T("Name")),
        format='%(Name)s')
    
def define_school_locations():
    db.define_table('school_locations',
        Field('Name', required=True, 
            requires=IS_NOT_EMPTY(), 
            label=T("Name")),
        Field('AllowAPI', 'boolean', required=True,
            default=False,
            label=T("Allow over API"),
            comment=T("When the API is in use, this checkbox defines whether a \
                location is available over the API.")),
        format='%(Name)s')
    
def define_school_subscriptions():
    db.define_table('school_subscriptions',
        Field('Archived', 'boolean', 
            default=False, 
            label=T("Archived")),
        Field('Name', required=True, 
            requires=IS_NOT_EMPTY(), 
            label=T("Name")),
        Field('MonthlyFee', 'float', required=True, 
            requires=IS_FLOAT_IN_RANGE(0,99999999, dot='.', \
                error_message=T('Too small or too large')), 
            represent = lambda value, row: format(value, '.2f'),
            label=T("Monthly Fee")),
        Field('NRofClasses', 'integer', required=False, 
            represent=lambda value, row:  value or T('Unlimited'), 
            label=T("Weekly classes")),
        format='%(Name)s')
        
def define_school_subscriptions_price():
    db.define_table('school_subscriptions_price',
        Field('school_subscriptions_id', db.school_subscriptions, required=True),
        Field('Startdate', 'date', required=True, 
            requires=IS_DATE_IN_RANGE(format=session.date_format,
                       minimum=datetime.date(2000,1,1),
                       maximum=datetime.date(2999,12,31))),
        Field('Enddate', 'date', 
            requires=IS_EMPTY_OR(IS_DATE_IN_RANGE(format=session.date_format,
                       minimum=datetime.date(2000,1,1),
                       maximum=datetime.date(2999,12,31)))),
        Field('Price', 'float', required=True, 
            requires=IS_FLOAT_IN_RANGE(0,99999999, dot='.', \
                error_message=T('Too small or too large')), 
            represent = lambda value, row: format(value, '.2f'),
            label=T("Monthly Fee")),
        )

def define_payment_categories():
    types_dict = {0:T("Collection"),1:T("Payment")}
    db.define_table('payment_categories',
        Field('Name', required=True, requires=IS_NOT_EMPTY(), label=T("Name")),
        Field('CategoryType', 'integer', required=True, requires=IS_IN_SET(types_dict, zero=T("Please select...")), \
            represent=lambda value, row: types_dict.get(value, ""), label=T("Type")),
        format='%(Name)s')
    
def define_school_levels():
    db.define_table('school_levels',
        Field('Name', required=True, requires=IS_NOT_EMPTY(), label=T("Name")),
        format='%(Name)s')  
    
def define_teachers():
    db.define_table('teachers',
        Field('FirstName', required=True, 
            requires=IS_NOT_EMPTY(), 
            label=T("First name")),
        Field('LastName', required=True, 
            requires=IS_NOT_EMPTY(), 
            label=T("Last name")),
        Field('Gender', 
            requires=IS_EMPTY_OR(IS_IN_SET(GENDERS)), 
            represent=represent_gender, 
            label=T("Gender")),
        Field('DateOfBirth', 'date', required=False, 
            requires=IS_EMPTY_OR(IS_DATE_IN_RANGE(format=session.date_format,
                                 minimum=datetime.date(1900,1,1),
                                 maximum=datetime.date(2999,1,1))), 
            represent=represent_date, label=T("Date of birth")),
        Field('Birthday', 'date', compute=_set_birthday, readable=False, writable=False, requires=IS_EMPTY_OR(IS_DATE(format=session.date_format)),
            represent=lambda value, row: value.strftime("%B %d")),
        Field('Education', 'text', required=False, label=T("Education")),
        Field('Address', required=False, label=T("Address")),
        Field('Postalcode', required=False, label=T("Postal code")),
        Field('City', required=False, represent=lambda value, row:  value or "", label=T("City")),
        Field('Country', required=False, represent=lambda value, row:  value or "", label=T("Country")),
        Field('Email', required=True, requires = IS_EMAIL(error_message=T('Please enter a valid email address')), label=T("Email")),
        Field('Telnr', required=False, represent=lambda value, row:  value or "", label=T("Telephone")),
        Field('Mobnr', required=False, represent=lambda value, row:  value or "", label=T("Mobile")),
        Field('Comments', 'text', required=False, label=T("Comments")),
        format='%(FirstName)s %(LastName)s')

def define_customers():
    db.define_table('customers',
        Field('ActiveMember', 'boolean', default=True, label=T("Active")),
        Field('FirstName', required=True, 
            requires=IS_NOT_EMPTY(), 
            label=T("First name")),
        Field('LastName', required=True, 
            requires=IS_NOT_EMPTY(), 
            label=T("Last name")),
        Field('Gender', 
            requires=IS_EMPTY_OR(IS_IN_SET(GENDERS)), 
            represent=represent_gender, 
            label=T("Gender")),
        Field('DateOfBirth', 'date', required=False, 
            requires=IS_EMPTY_OR(IS_DATE_IN_RANGE(format=session.date_format,
                                 minimum=datetime.date(1900,1,1),
                                 maximum=datetime.date(2999,1,1))), 
            represent=represent_date, 
            label=T("Date of birth")),
        Field('Birthday', 'date', compute=_set_birthday, 
            readable=False, 
            writable=False, 
            requires=IS_EMPTY_OR(IS_DATE(format=session.date_format)), 
            represent=represent_birthday),
        Field('Address', required=False, 
            label=T("Address")),
        Field('PostalCode', required=False, 
            label=T("Postal code")),
        Field('City', required=False, 
            represent=lambda value, row: value or "", 
            label=T("City")),
        Field('Country', required=False, 
            represent=lambda value, row: value or "", 
            label=T("Country")),
        Field('Email', required=True, 
            requires=[ IS_EMAIL(error_message=T('Invalid email address')), 
                       IS_NOT_IN_DB(db, 'customers.Email',
                                    error_message=T('Email already registered'))
                     ],
            label=T("Email")),
        Field('Newsletter', 'boolean', default=True, 
            label=T("Newsletter")),
        Field('Telnr', required=False, 
            label=T("Telephone")),
        Field('Mobnr', required=False, 
            label=T("Mobile")),
        Field('Company', required=False,
            label=T("Company")),
        Field('Keynr', required=False, 
            represent=lambda value, row: value or "", 
            label=T("Key Number")),
        Field('school_discovery_id', db.school_discovery, 
            requires=IS_EMPTY_OR(IS_IN_DB(db,'school_discovery.id','%(Name)s')), 
            represent=lambda value, row: discovery_dict.get(value, None), 
            label=T("Discovery")),
        Field('school_levels_id', db.school_levels, 
            requires=IS_EMPTY_OR(IS_IN_DB(db,'school_levels.id','%(Name)s')), 
            represent=lambda value, row: levels_dict.get(value, None), 
            label=T("Level")),
        Field('school_locations_id', db.school_locations, 
            requires=IS_EMPTY_OR(IS_IN_DB(db,'school_locations.id','%(Name)s')), 
            represent=lambda value, row: locations_dict.get(value, None), 
            label=T("Location")),
        Field('school_languages_id', db.school_languages, 
            requires=IS_EMPTY_OR(IS_IN_DB(db,'school_languages.id','%(Name)s')), 
            represent=lambda value, row: languages_dict.get(value, None), 
            label=T("Language")),
        Field('Comments', 'text', 
            label=T("Comments")),
        Field('Picture', 'upload', autodelete=True,
            requires=IS_EMPTY_OR([IS_IMAGE(extensions=('jpeg', 'png'), 
                                           maxsize=(1280, 1280)), 
                                           IS_LENGTH(maxsize=524288)]),
            label=T("Customer")),
        Field('ThumbSmall', 'upload', # generate 50*50 for list view
            autodelete=True, writable=False, 
            compute = lambda row: SMARTHUMB(row.Picture, 
                                            (50, 50), 
                                             name="Small"),
            represent = _represent_customer_Image,
            label=T("Customer")),  
        Field('ThumbLarge', 'upload', # generate 230*230 for edit view
            autodelete=True, writable=False,
            compute = lambda row: SMARTHUMB(row.Picture, 
                                             (300, 300), 
                                             name="Large")), 
        format='%(FirstName)s %(LastName)s',
        singular=T("Customer"), plural=T("Customers")
        )
        
    # make school_locations_id required when customers are separated by location
    if session.show_location:
        db.customers.school_locations_id.requires = \
            IS_IN_DB(db,'school_locations.id','%(Name)s', zero=T("Please select..."))
    
def define_alternativepayments():
    months = get_months_list()
    
    db.define_table('alternativepayments',
        Field('customers_id', db.customers, readable=False, writable=False, required=True, label=T("CustomerID")),
        Field('PaymentYear', 'integer', default=datetime.date.today().year, required=True, requires=IS_NOT_EMPTY(), label=T("Year")),
        Field('PaymentMonth', 'integer', requires=IS_IN_SET(months, zero=T("Please select...")),
            default=datetime.date.today().month, required=True, represent=NRtoMonth, label=T("Month")),
        Field('Amount', 'double', required=True, 
            requires=IS_NOT_EMPTY(), 
            represent = lambda value, row: format(value, '.2f'),
            label=T("Amount")),
        Field('payment_categories_id', db.payment_categories, \
            requires=IS_EMPTY_OR(IS_IN_DB(db,
                                          'payment_categories.id', 
                                          '%(Name)s', 
                                          '%(CategoryType)s',
                                          zero=T('Default'))), \
            represent=lambda value, row: paycat_dict.get(value, ""), 
            label=T("Category")),
        Field('Description', 'text', required=False, 
            requires=IS_NOT_EMPTY(),
            label=T("Description")),
        singular=T("Alternative payment"), plural=T("Alternative payments")
        )

def define_overduepayments():
    db.define_table('overduepayments',
        Field('customers_id', db.customers, required=True, 
            readable=False, 
            writable=False, 
            label=T("CustomerID")),
        Field('Paid', 'boolean', required=True, 
        label=T("Paid")),
        Field('PaymentDate', 'date', required=True,
            default=datetime.date.today(), 
            requires=IS_DATE_IN_RANGE(format=session.date_format,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)), 
            represent=represent_date, label=T("Due date")),
        Field('Amount', 'double', required=True, 
            requires=IS_NOT_EMPTY(), 
            represent = lambda value, row: format(value, '.2f'),
            label=T("Amount")),
        Field('Description', 'text', required=False, 
            label=T("Description")),
        singular=T("Overdue payment"), plural=T("Overdue payments")
        )

def define_classes():
    weekdays = [('1',T('Monday')),
                ('2',T('Tuesday')),
                ('3',T('Wednesday')),
                ('4',T('Thusday')),
                ('5',T('Friday')),
                ('6',T('Saturday')),
                ('7',T('Sunday'))]
    db.define_table('classes', 
        Field('school_locations_id', db.school_locations, required=True, 
            label=T("Location")),
        Field('school_classtypes_id', db.school_classtypes, required=True, 
            label=T("Type")),
        Field('school_levels_id', db.school_levels, required=False,
            requires=IS_EMPTY_OR(IS_IN_DB(db,'school_levels.id','%(Name)s',
                                          zero=T(""))),
            represent=lambda value, row: levels_dict.get(value, T("No level")), 
            label=T("Level")),
        Field('Week_day', required=True, 
            requires=IS_IN_SET(weekdays), 
            represent=NRtoDay, 
            label=T("Weekday")),
        Field('Starttime', 'time', required=True, 
            requires=IS_TIME(error_message='must be HH:MM:SS'), 
            represent=lambda value, row: value.strftime('%H:%M'), 
            label=T("Start")),
        Field('Endtime', 'time', required=True, 
            requires=IS_TIME(error_message='must be HH:MM:SS'), 
            represent=lambda value, row: value.strftime('%H:%M'), 
            label=T("End")),
        Field('Startdate', 'date', required=True, 
            requires=IS_DATE_IN_RANGE(format=session.date_format,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)), 
            represent=represent_date,
            label=T("Start date")),
        Field('Enddate', 'date', required=False, 
            requires=IS_EMPTY_OR(IS_DATE_IN_RANGE(format=session.date_format,
                                 minimum=datetime.date(1900,1,1),
                                 maximum=datetime.date(2999,1,1))), 
            represent=represent_date, label=T("End date")),
        Field('teachers_id', db.teachers, 
            requires=IS_IN_DB(db,'teachers.id','%(FirstName)s %(LastName)s',
                              zero=(T('Select teacher...'))), \
            represent=lambda value, row: teachers_dict.get(value, None), 
            label=T("Teacher")),
        Field('teachers_id2', db.teachers, 
            requires=IS_EMPTY_OR(IS_IN_DB(db,'teachers.id','%(FirstName)s %(LastName)s')), \
            represent=lambda value, row: teachers_dict.get(value, None), 
            label=T("Teacher 2")),
        Field('Maxstudents', 'integer', required=True, 
            requires=IS_NOT_EMPTY(), 
            label=T("Spaces")),
        Field('DropInPrice', 'double', required=False, 
            represent=represent_float_as_amount, 
            label=T("Drop-in")),
        Field('TrialPrice', 'double', required=False, 
            represent=represent_float_as_amount,
            default=0, 
            label=T("Trialclass")),
        Field('AllowAPI', 'boolean', required=True,
            default=False,
            label=T("Allow over API"),
            comment=T("When the API is in use, this checkbox defines whether a \
                class is shown on the website.")),
        )
        
def define_classes_open():
    db.define_table('classes_open',
        Field('classes_id', db.classes, required=True),
        Field('ClassDate', 'date', required=True, 
            requires=IS_DATE_IN_RANGE(format=session.date_format,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)))
        )
        
def define_classes_cancelled():
    db.define_table('classes_cancelled',
        Field('classes_id', db.classes, required=True),
        Field('ClassDate', 'date', required=True, 
            requires=IS_DATE_IN_RANGE(format=session.date_format,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)))
        )

def define_customers_classcards():
    db.define_table('customers_classcards',
        Field('customers_id', db.customers, required=True,
            readable=False, 
            writable=False, 
            label=T("CustomerID")),
        Field('Startdate', 'date', required=True, 
            requires=IS_DATE_IN_RANGE(format=session.date_format,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            represent=represent_date, label=T("Start")),
        Field('Enddate', 'date', required=False, 
            requires=IS_EMPTY_OR(IS_DATE_IN_RANGE(format=session.date_format,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1))),
            represent=represent_date, 
            label=T("Expiration")),
        Field('Classes', 'integer', readable=False, required=True, 
            requires=IS_NOT_EMPTY(), 
            label=T("Number of classes")),
        Field('Price', 'double', required=True, 
            requires=IS_NOT_EMPTY(), 
            label=T("Price")),
        singular=T("Classcard"), plural=T("Classcards")
        )

def define_classes_attendance():
    types = [ (1,T("Trialclass")),
              (2,T("Drop In")),
              (3,T("Class card")) ]
    session.att_types_dict = { 1:T("Trialclass"),
                               2:T("Drop In"),
                               3:T("Class card") }
    db.define_table('classes_attendance',
        Field('customers_id', db.customers, required=True, 
            label=T("CustomerID")),
        Field('classes_id', db.classes, required=True, 
            represent=lambda value, row: classes_dict.get(value, None), 
            label=T("Class")),
        Field('ClassDate', 'date', required=True, 
            requires=IS_DATE_IN_RANGE(format=session.date_format,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)), 
            represent=represent_date, label=T("Class date")),
        Field('AttendanceType', 'integer', required=True, 
            requires=IS_IN_SET(types), 
            represent=lambda value, row: session.att_types_dict.get(value, ""), 
            label=T("Type")),
        Field('customers_classcards_id', db.customers_classcards, 
            represent=lambda value, row: value or "", 
            label=T("Pass nr")),
        )
    
def define_classes_attendance_override():
    db.define_table('classes_attendance_override',
        Field('classes_id', db.classes, required=True, 
            readable=False, 
            writable=False, 
            label=T("ClassID")),
        Field('ClassDate', 'date', required=True, 
            readable=False, 
            writable=False, 
                        requires=IS_DATE_IN_RANGE(format=session.date_format,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            label=T("Class date")),
        Field('Amount', 'integer', required=True, 
            label=T("")),
        )
    
def define_customers_payment_info():
    db.define_table('customers_payment_info',
        Field('customers_id', db.customers, required=True,
            readable=False, 
            writable=False, 
            label=T("CustomerID")),
        Field('payment_methods_id', db.payment_methods, required=True, 
            requires=IS_EMPTY_OR(IS_IN_DB(db,'payment_methods.id','%(Name)s',
                                          zero=T("Please select..."))),
            represent=lambda value, row: payment_methods_dict.get(value), 
            label=T("Payment method")),
        Field('AccountNumber', 
            represent=lambda value, row: value or "", 
            label=T("Account number")),
        Field('AccountHolder', 
            represent=lambda value, row: value or "", 
            label=T("Account holder")),
        Field('BIC', 
            represent=lambda value, row: value or "", 
            label=T("BIC")),
        Field('MandateSignatureDate', 'date', 
            requires=IS_EMPTY_OR(IS_DATE_IN_RANGE(format=session.date_format,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1))), 
            label=T("Mandate signature date")),
        Field('BankName', 
            represent=lambda value, row: value or "", 
            label=T("Bank name")),
        Field('BankLocation', 
            represent=lambda value, row: value or "", 
            label=T("Bank location")),
        singular=T("Bank details"), plural=T("Bank details")
        )
        
    sorted_payment_methods = [dict(id=None, Name=T("Please select..."))]
    payment_methods = db(db.payment_methods).select(orderby=db.payment_methods.id)
    for method in payment_methods:
        sorted_payment_methods.append(dict(Name=method.Name, id=method.id))
    
    db.customers_payment_info.payment_methods_id.widget = lambda f, v: \
        SELECT([OPTION(i['Name'], 
            _value=i['id']) for i in sorted_payment_methods], 
            _name=f.name, _id="%s_%s" % (f._tablename, f.name), 
            _value=v, 
            value=v)
    

def define_customers_subscriptions():
    subscriptions_query = (db.school_subscriptions.Archived == False)
    db.define_table('customers_subscriptions',
        Field('customers_id', db.customers, readable=False, writable=False, 
            required=True, 
            requires=IS_NOT_EMPTY(), 
            label=T("CustomerID")),
        Field('school_subscriptions_id', db.school_subscriptions, 
            requires = IS_IN_DB(db(subscriptions_query), 
                'school_subscriptions.id','%(Name)s', 
                zero=T("Please select...")), 
            represent=lambda value, row: mstypes_dict.get(value, None), 
            label=T("Subscription")),
        Field('Startdate', 'date', required=True, 
            requires=IS_DATE_IN_RANGE(format=session.date_format,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)), 
            represent=represent_date, 
            label=T("Start")),
        Field('Enddate', 'date', required=False, 
            requires=IS_EMPTY_OR(IS_DATE_IN_RANGE(format=session.date_format,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1))),
            represent=represent_date, label=T("End")),
        singular=T("Subscription"), plural=T("Subscriptions"))
        
def define_customers_subscriptions_paused():
    today = datetime.date.today()
    startdate_default = datetime.date(today.year, today.month, 1)
    enddate_default = get_last_day_month(startdate_default)
    
    db.define_table('customers_subscriptions_paused',
        Field('customers_subscriptions_id', db.customers_subscriptions, required=True, 
            represent=represent_subscription_as_membershiptype_name, 
            label=T("Subscription")),
        Field('Startdate', 'date', required=True, 
            requires=IS_DATE_IN_RANGE(format=session.date_format,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            default=startdate_default, 
            label=T("Pause from")),
        Field('Enddate', 'date', required=True, 
            requires=IS_DATE_IN_RANGE(format=session.date_format,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            default=enddate_default, 
            label=T("Pause until")), 
        Field('Description', 
            represent=lambda value, row: value or "",
            label=T("Description")),
        )
        
def define_paymentsummary():
    db.define_table('paymentsummary',
        Field('customers_id', db.customers, required=True, 
            readable=False, 
            writable=False, 
            requires=IS_NOT_EMPTY(), 
            label=T("CustomerID")),
        Field('PaymentYear', 'integer', required=True, default=datetime.date.today().year, label=T("Year")),
        Field('M01', 'boolean', required=True, label=T("January")),
        Field('M02', 'boolean', required=True, label=T("February")),
        Field('M03', 'boolean', required=True, label=T("March")),
        Field('M04', 'boolean', required=True, label=T("April")),
        Field('M05', 'boolean', required=True, label=T("May")),
        Field('M06', 'boolean', required=True, label=T("June")),
        Field('M07', 'boolean', required=True, label=T("July")),
        Field('M08', 'boolean', required=True, label=T("August")),
        Field('M09', 'boolean', required=True, label=T("September")),
        Field('M10', 'boolean', required=True, label=T("October")),
        Field('M11', 'boolean', required=True, label=T("November")),
        Field('M12', 'boolean', required=True, label=T("December")),
        )
    
def define_customers_documents():
    db.define_table('customers_documents',
        Field('customers_id', db.customers, required=False,
            readable=False,
            writable=False,
            requires=IS_EMPTY_OR(\
                IS_IN_DB(db,'customers.id','%(FirstName)s %(LastName)s')),
            label=T("CustomerID")),
        Field('Description', required=False),
        Field('DocumentFile', 'upload', autodelete=True,
            requires=IS_LENGTH(maxsize=4153344,
                error_message=T("Maximum size is 4MB")),
            represent=lambda value, row: A(T("Download"), 
                _href=URL('default', 'download', args=value)),
            label=T("File (max 4MB)")),
        Field('UploadDateTime', 'datetime', required=True,
            readable=False,
            writable=False,
            default=datetime.datetime.now(),
            represent=represent_datetime,
            label=T("Uploaded on"))
        )
    
def define_teachers_documents():
    db.define_table('teachers_documents',
        Field('teachers_id', db.teachers, required=False,
            requires=IS_EMPTY_OR(\
                IS_IN_DB(db,'teachers.id','%(FirstName)s %(LastName)s')),
            label=T("TeacherID")),
        Field('Description', required=False),
        Field('DocumentFile', 'upload', autodelete=True)
        )
    
def define_teachers_classtypes():
    db.define_table('teachers_classtypes',
        Field('teachers_id', db.teachers, required=True, 
            label=T("Teacher")),
        Field('school_classtypes_id', db.school_classtypes, required=True, 
            label=T("Class type")),
        )
        
def define_classes_subteachers():
    db.define_table('classes_subteachers',
        Field('classes_id', db.classes, required=True, 
            readable=False, 
            writable=False, 
            label=T("ClassID")),
        Field('ClassDate', 'date', required=True, 
            readable=False, 
            writable=False, 
            requires=IS_DATE_IN_RANGE(format=session.date_format,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            label=T("Class date")),
        Field('teachers_id', db.teachers, 
            requires=IS_IN_DB(db,'teachers.id','%(FirstName)s %(LastName)s',
                              zero=(T('Select teacher...'))), \
            represent=lambda value, row: teachers_dict.get(value, None), 
            label=T("Teacher")),
        Field('teachers_id2', db.teachers, 
            requires=IS_EMPTY_OR(\
                IS_IN_DB(db,'teachers.id','%(FirstName)s %(LastName)s')), 
            represent=lambda value, row: teachers_dict.get(value, None), 
            label=T("Teacher 2")),
        )
        
def define_classes_registration():
    db.define_table('classes_registration',
        Field('customers_id', db.customers, 
            label=T("Customer")),
        Field('classes_id', db.classes, 
            writable=False, 
            represent=lambda value, row: classes_dict.get(value, None), 
            label=T("Class")),
        )
        
def define_classes_waitinglist():
    db.define_table('classes_waitinglist',
        Field('customers_id', db.customers, 
            label=T("Customer")),
        Field('classes_id', db.classes, 
            writable=False, 
            represent=lambda value, row: classes_dict.get(value, None), 
            label=T("Class")),
        )
        
def define_workshops():
    db.define_table('workshops',
        Field('Archived', 'boolean', 
            default=False, 
            label=T("Archived")),
        Field('Name', 
            requires=IS_NOT_EMPTY(), 
            label=T("Workshop title")),
        Field('Startdate', 'date', required=True, 
            requires=IS_DATE_IN_RANGE(format=session.date_format,
                              minimum=datetime.date(1900,1,1),
                              maximum=datetime.date(2999,1,1)),
            label=T("Start date")),
        Field('Teacher', 
            requires=IS_NOT_EMPTY(), 
            label=T("Teacher")),
        Field('TeacherEmail', 
            requires=IS_EMPTY_OR(IS_EMAIL()), 
            label=T("Teacher email")),
        Field('Teacher2', 
            represent=lambda value, row: value or "", 
            label=T("Teacher 2")),
        Field('Teacher2Email', 
            requires=IS_EMPTY_OR(IS_EMAIL()), 
            label=T("Teacher2 email")),
        Field('Price', 'double', 
            represent=lambda value, row: value or "", 
            label=T("Price")),
        )
        
def define_workshops_activities():
    db.define_table('workshops_activities',
        Field('workshops_id', db.workshops, readable=False, writable=False, 
            label=T("Workshop")),
        Field('Activity', 
            requires=IS_NOT_EMPTY(), 
            label=T("Name")),
        Field('Activitydate', 'date', required=True, 
            requires=IS_DATE_IN_RANGE(format=session.date_format,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            label=T("Date")),
        Field('Starttime', 'time', requires=IS_TIME(), 
            represent=represent_time, 
            label=T("Start time")),
        Field('Endtime', 'time', requires=IS_TIME(), 
            represent=represent_time, 
            label=T("End time")),
        Field('Spaces', 'integer', 
            requires=IS_NOT_EMPTY(), 
            label=T("Spaces")),
        Field('Price', 'double', 
            represent=lambda value, row: value or "", 
            label=T("Price")),
        )
        
def define_workshops_registration():
    db.define_table('workshops_registration',
        Field('workshops_id', db.workshops, 
            label=T("Workshop")),
        Field('workshops_activities_id', db.workshops_activities,
            requires=IS_EMPTY_OR(IS_IN_DB(db, 'workshops_activities.id', 
                                          '%(Activity)s')), 
            represent = lambda value, row: \
                workshops_activities_dict.get(value, T("Not Found")),
            label=T("Workshop Activity")),
        Field('customers_id', db.customers, 
            label=T("Customer")),
        Field('Paid', 'boolean', 
            default=False, 
            label=T("Payment received")), 
        Field('PaymentConfirmation', 'boolean', 
            default=False, 
            label=T("Payment confirmation")), 
        Field('Info', 'boolean', 
            default=False, 
            label=T("Workshop information")), 
        )
        
def define_workshops_payments():
    db.define_table('workshops_payments',
        Field('workshops_registration_id',  db.workshops_registration, 
            readable=False, writable=False, 
            label=T("Workshop Attendance ID")),
        Field('PaymentDate', 'date', required=True, 
            default=datetime.date.today(),
            requires=IS_DATE_IN_RANGE(format=session.date_format,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            label=T("Date paid")),
        Field('Amount', 'double', required=True, 
            requires=IS_NOT_EMPTY(), 
            label=T("Amount")),
        Field('payment_methods_id', db.payment_methods, required=True, 
            label=T("Payment Method")),
        )
        
def define_school_holidays():
    db.define_table('school_holidays',
        Field('Description', required=True,
            requires=IS_NOT_EMPTY(),
            label=T('Description')),
        Field('Startdate', 'date', required=True, 
            requires=IS_DATE_IN_RANGE(format=session.date_format,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)), 
            represent=represent_date,
            label=T("Start date")),
        Field('Enddate', 'date', required=True, 
            requires=IS_DATE_IN_RANGE(format=session.date_format,
                                 minimum=datetime.date(1900,1,1),
                                 maximum=datetime.date(2999,1,1)), 
            represent=represent_date, label=T("End date")),
        Field('Classes', 'boolean', required=True,
            label=T('Classes')),
            )
        
def configure_mail():
    if db.sys_properties(Property='from_name'):
        from_name = db.sys_properties(Property='from_name').PropertyValue
    else:
        from_name = ""
    if db.sys_properties(Property='from_address'):
        from_address = db.sys_properties(Property='from_address').PropertyValue
    else:
        from_address = ""
    if db.sys_properties(Property='smtp_server'):
        smtp_server = db.sys_properties(Property='smtp_server').PropertyValue
    else:
        smtp_server = ""
    if db.sys_properties(Property='smtp_port'):
        smtp_port = db.sys_properties(Property='smtp_port').PropertyValue
    else:
        smtp_port = ""
    if db.sys_properties(Property='smtp_tls'):
        smtp_tls = db.sys_properties(Property='smtp_tls').PropertyValue
        if smtp_tls == "on":
            smtp_tls = True
        else:
            smtp_tls = False
    else:
        smtp_tls = False
    if db.sys_properties(Property='smtp_username'):
        smtp_username = db.sys_properties(Property='smtp_username').PropertyValue
    else:
        smtp_username = ""
    if db.sys_properties(Property='smtp_password'):
        smtp_password = db.sys_properties(Property='smtp_password').PropertyValue
    else:
        smtp_password = ""

    if smtp_tls == False:
        login = None
    else:
        login = smtp_username + ':' + smtp_password

    mail = auth.settings.mailer
    mail.settings.server = smtp_server +  ':' + smtp_port
    mail.settings.sender = from_name + " <" + from_address + ">"
    mail.settings.tls = False
    mail.settings.login = login
    
    return mail
    
def set_static_payment_methods():
    '''
        This function adds the following to the paymentmethods table
        1. Cash
        2. Wire transfer
        3. Direct debit 
    '''
    methods = [T('Cash'), T('Wire transfer'), T('Direct debit')]
    i = 1
    for method in methods:
        row = db.payment_methods(1)
        if not row is None:
            query = (db.payment_methods.id == i)
            db(query).delete()
        db.payment_methods.insert(id=i, 
                                  Name=method)
        i += 1
        
def set_default_storage_space():
    '''
        Sets 5GB (5000MB) as default storage space
    '''
    allowed_space = '5000' # In MB
    db.sys_properties.insert(Property='storage_allowed_space', 
                             PropertyValue=allowed_space)
        
def set_preferences_permissions():
    '''
        Set the additional required permissions for the groups with the read 
        permission for preferences.
        Currently create, read, update, delete and select will be set for
        payment_methods and payment_categories.
    '''
    if not auth.user is None:
        user_id = auth.user.id
        check = auth.has_permission('read', 'preferences', 0, user_id)
        if check:
            tables = ['payment_categories', 'payment_methods']
            group_id = get_group_id()
            for table in tables:
                for permission in permissions:
                    auth.add_permission(group_id, permission, table, 0)
        
    
def create_admin_user_and_group():
    row = db.auth_user(1)
    if row is None:
        password = db.auth_user.password.validate('admin')[0]
        db.auth_user.insert(id=1, 
                            first_name='admin', 
                            last_name='admin', 
                            email='admin@openstudioproject.com', 
                            password=password) 
    row = db.auth_group(1)
    if row is None:
        db.auth_group.insert(id=1, 
                             role='Admins', 
                             description='This group has full access')
        auth.add_membership(1, 1) # add the admin user to the admins group
        set_permissions_for_admin_group()
        
def set_permissions_for_admin_group():
    for table in db.tables:
        for permission in permissions:
            auth.add_permission(1, permission, table, 0)
    
def set_version():
    row = db.sys_properties(Property='Version')
    version = '2.10'
    if not row:
        db.sys_properties.insert(Property='Version', PropertyValue=version)
    else:
        row.PropertyValue = version
        row.update_record()
        
def setup():
    """
        This function runs when running OpenStudio for the first time.
        To check whether setup has run, use the Property "setup_complete" in 
        the sys_properties table. If the value is "T", it has run.
    """
    setup_complete = 'F'
    row = db.sys_properties(Property="setup_complete")
    if not row is None:
        setup_complete = row.PropertyValue
        
    if setup_complete != 'T':
        set_static_payment_methods()
        set_default_storage_space()
        db.sys_properties.insert(Property="setup_complete", 
                                 PropertyValue="T")
    
def upgrade():
    """
        This function executes commands needed for upgrades to new versions
    """
    if db.sys_properties(Property='Version'):
        version = float(db.sys_properties(Property='Version').PropertyValue)
        
        #if version < 2.10:
            #query = (db.customers.Gender == 'Male')
            #db(query).update(Gender='M')
            #query = (db.customers.Gender == 'Female')
            #db(query).update(Gender='F')
            
        # always renew permissions for admin group after update
        set_permissions_for_admin_group()
            
# set some global variables
GENDERS = set_genders()
            
# call the define table functions
define_sys_properties()
define_sys_api_users()
set_show_location()
set_dateformat()
set_class_status()
define_payment_methods()
payment_methods_dict = create_payment_methods_dict()

define_school_languages()
languages_dict = create_languages_dict()
define_school_classtypes()
classtypes_dict = create_classtypes_dict()
define_school_discovery()
discovery_dict = create_discovery_dict()
define_school_locations()
locations_dict = create_locations_dict()
define_school_subscriptions()
mstypes_dict = create_mstypes_dict()
define_payment_categories()
paycat_dict = create_payment_categories_dict()
define_school_levels()
levels_dict = create_school_levels_dict()
define_teachers()
teachers_dict = create_teachers_dict()
define_teachers_documents()
define_customers()
customers_dict = create_customers_dict()
define_customers_documents()
define_customers_payment_info()
define_alternativepayments()
define_overduepayments()
define_paymentsummary()
define_classes()
classes_dict = create_classes_dict()
define_classes_open()
define_classes_cancelled()
define_customers_classcards()
define_classes_registration()
define_classes_waitinglist()
define_classes_attendance()
define_classes_attendance_override()
define_customers_subscriptions()
define_customers_subscriptions_paused()
define_teachers_classtypes()
define_classes_subteachers()
define_workshops()
define_workshops_activities()
workshops_activities_dict = create_workshops_activities_dict()
define_workshops_registration()
define_workshops_payments()
define_memos()
define_school_holidays()

set_preferences_permissions()

# some system maintendance
create_admin_user_and_group()
setup()
upgrade()
set_version()

# set up email
MAIL = configure_mail()
