# -*- coding: utf-8 -*-
import datetime
import calendar

from gluon import *

def get_months_list():
    '''
        This function returns a list of tuples [0] is the number of the month and [1] is the name in English
    '''
    return [(1,current.T("January")),
            (2,current.T("February")),
            (3,current.T("March")),
            (4,current.T("April")),
            (5,current.T("May")),
            (6,current.T("June")),
            (7,current.T("July")),
            (8,current.T("August")),
            (9,current.T("September")),
            (10,current.T("October")),
            (11,current.T("November")),
            (12,current.T("December"))]

def get_last_day_month(date):
    '''
        This function returns the last day of the month as a datetime.date object
    '''
    return datetime.date(date.year, 
                         date.month, 
                         calendar.monthrange(date.year,date.month)[1])
                         
def get_paused_subscriptions(date):
    '''
        This helper function returns the paused subscriptions the month of date
    '''
    lastdaythismonth = get_last_day_month(date)
    firstdaythismonth = datetime.date(date.year, date.month, 1)
    
    dba = current.globalenv['db']
    query = (dba.customers_subscriptions_paused.Startdate <= \
             lastdaythismonth) & \
            ((dba.customers_subscriptions_paused.Enddate >= \
              firstdaythismonth) | \
            (dba.customers_subscriptions_paused.Enddate == None))
    rows = dba(query).select(dba.customers_subscriptions_paused.ALL)
    paused_subscriptions = dict()
    for row in rows:
        paused_subscriptions[row.customers_subscriptions_id] = True
    return paused_subscriptions

def get_classname(clsID):
    dba = current.globalenv['db']
    record = dba.classes[clsID]
    location = dba.school_locations[record.school_locations_id].Name
    classtype = dba.school_classtypes[record.school_classtypes_id].Name
    Weekday = NRtoDay(record.Week_day)
    return Weekday + " " + \
           location + " " + \
           record.Starttime.strftime('%H:%M') + " " + \
           classtype
    
def get_submenu(pages, page, horizontal=False, _id='', _class=''):
    '''
        This function returns a submenu based on the list pages which 
        is in the following format:
        [ page, title, url ]
        when page matches pages[n][0] the menu whill receive an extra tag to 
        indicate it's active 
    '''
    submenu_class = "os-submenu"
    if horizontal:
        submenu_class = "os-submenu-horizontal"
    
    menu = []
    for p in pages:
        active = False
        if p[0] == page:
            active = True
        menu.append([p[1], active, p[2]])
    
    return MENU(menu, li_first='', li_last='', _class=submenu_class, _id=_id)
    
def get_group_id():
    '''
        This function returns the group id of the currently logged in user
    '''
    dba = current.globalenv['db']
    w2p_auth = current.globalenv['auth']
    row = dba(dba.auth_membership.user_id == w2p_auth.user.id).select(dba.auth_membership.group_id).first()
    return row.group_id
    
    
def get_button(button_type, 
               url, 
               tooltip="", 
               title='', 
               _class='', 
               onclick=None):
    """
        This function returns a button of type "button_type" and redirects to url "url".
        See below for supported button types
        The tooltip argument can be used to specify text shown when the mouse hovers over the button
    """
    if button_type == 'add':
        title = current.T("Add")
        icon = "icon plus icon-plus"
    elif button_type == 'add_class':
        title = current.T("Add class")
        icon = "icon plus icon-plus"
    elif button_type == 'edit':
        title = current.T("Edit")
        icon = "icon pencil icon-pencil"
    elif button_type == 'delete':
        title = current.T("Delete")
        icon = "icon remove icon-remove"
    elif button_type == 'delete_notext':
        title = current.T("")
        icon = "icon remove icon-remove"
    elif button_type == 'delete_notext':
        title = current.T("")
        icon = "icon remove icon-remove"
    elif button_type == 'back':
        title = current.T("Back")
        icon = "icon leftarrow icon-arrow-left"
    elif button_type == 'duplicate':
        title = current.T("Duplicate")
        icon = 'icon retweet icon-retweet'
    elif button_type == 'list_to_teacher':
        title = current.T("List to teacher")
        icon = 'icon envelope icon-envelope'
    elif button_type == 'next_no_text':
        title = ''
        icon = 'icon arrow right icon-arrow-right'
    elif button_type == 'previous_no_text':
        title = ''
        icon = 'icon arrow left icon-arrow-left'
    else:
        title = current.T("Invalid button type, please check...")
        icon = ''
    
    if _class:
        _class = "button btn" + ' ' + _class
    else:
        _class = "button btn"
    
    return A(SPAN(_class="buttontext button", _title=tooltip), 
                SPAN(_class=icon), 
                title, 
                _class=_class,
                _href=url, 
                _onclick=onclick,
                _title=tooltip)

def get_priorities():
    """
    This function returns the priorities list of tuples.
    """
    return [(3,T("Low")),(2,T("Normal")),(1,T("High"))]
    
# date related functions

def datestr_to_python(date_format, datestr):
    """
    This function takes date string and converts it into a python datetime.date object
    first argument: date_format
    second argument: date string
    """
    if date_format == '%d-%m-%Y':
        day = int(datestr.split("-")[0])
        month = int(datestr.split("-")[1])
        year = int(datestr.split("-")[2])
    elif date_format == '%m-%d-%Y':
        month = int(datestr.split("-")[0])
        day = int(datestr.split("-")[1])
        year = int(datestr.split("-")[2])
    elif date_format == '%Y-%m-%d':
        year = int(datestr.split("-")[0])
        month = int(datestr.split("-")[1])
        day = int(datestr.split("-")[2])
    return datetime.date(year, month, day)
    
def represent_date(date, row=None):
    if date is None:
        return ""
    else:
        return date.strftime(current.session.date_format)
        
def represent_datetime(datetime, row=None):
    if datetime is None:
        return ""
    else:
        return datetime.strftime(current.session.datetime_format)
    
def NRtoDay(value, row=None):
    """
    This function takes a value and row as arguments and converts the value (that's within 1 and 7) to a week day.
    The row value is ignored, but present so it can be used within web2py's database represent function for Field definitions.
    """
    value = int(value)
    if value == 1:
        day = "Monday"
    elif value == 2:
        day = "Tuesday"
    elif value == 3:
        day = "Wednesday"
    elif value == 4:
        day = "Thursday"
    elif value == 5:
        day = "Friday"
    elif value == 6:
        day = "Saturday"
    elif value == 7:
        day = "Sunday"
    return current.T(day)
    
def NRtoMonth(value, row=None):
    value = int(value)
    months = [(1,"January"),(2,"February"),(3,"March"),(4,"April"),(5,"May"),(6,"June"),(7,"July"),\
    (8,"August"),(9,"September"),(10,"October"),(11,"November"),(12,"December")]
    for m in months:
        if m[0] == value:
            return current.T(m[1])
            
def iso_year_start(iso_year):
    "The gregorian calendar date of the first day of the given ISO year"
    fourth_jan = datetime.date(iso_year, 1, 4)
    delta = datetime.timedelta(fourth_jan.isoweekday()-1)
    return fourth_jan - delta 
    
def iso_to_gregorian(iso_year, iso_week, iso_day):
    "Gregorian calendar date for the given ISO year, week and day"
    iso_year = int(iso_year)
    iso_week = int(iso_week)
    year_start = iso_year_start(iso_year)
    return year_start + datetime.timedelta(days=iso_day-1, weeks=iso_week-1)
            
# date functions end
            
def NRtoPriority(value, row=None):
    value = int(value)
    priorities = _get_priorities()
    for p in priorities:
        if p[0] == value:
            return p[1]
            
            
def create_teachers_dict():
    dba = current.globalenv['db']
    rows = dba().select(dba.teachers.id, dba.teachers.FirstName, dba.teachers.LastName)
    d = dict()
    for row in rows:
        d[row.id] = row.FirstName + " "  + row.LastName
    d[None] = ""
    return d
    
def create_locations_dict():
    dba = current.globalenv['db']
    rows = dba().select(dba.school_locations.id, dba.school_locations.Name)
    d = dict()
    for row in rows:
        d[row.id] = row.Name
    d[None] = ""
    return d
    
def create_classtypes_dict():
    dba = current.globalenv['db']
    rows = dba().select(dba.school_classtypes.id, dba.school_classtypes.Name)
    d = dict()
    for row in rows:
        d[row.id] = row.Name
    d[None] = ""
    return d
    
def highlight_submenu(menu_link_title):
    """
    This function is used to make menu items bold
    """
    return (SPAN(menu_link_title, _style="font-weight:900;"))
    
def get_ajax_loader(msg='', big=False):
    '''
        This function returns a div with a message and a loading gif
    '''
    if big:
        loader = IMG(_src=URL('static', 'plugin_os-layout/images/ajax-loader-big.gif'))
    else:
        loader = IMG(_src=URL('static', 'plugin_os-layout/images/ajax-loader.gif'))
    return DIV(msg, BR(), loader)
    
class User_helpers():
    def __init__(self):
        self.auth = current.globalenv['auth']
        self.dba = current.globalenv['db']
    def on_user_create(self, form):
        '''
            This function creates a new group for a user. We're using this function
            because we're manually adding users and the web2py auth functions 
            aren't called at those times.
        '''
        user_id = form.vars.id
        group_name = "user_{0}".format(user_id)
        group_id = self.auth.add_group(group_name, group_name)
        self.auth.add_membership(group_id, user_id)
        
        row = self.dba.auth_user(user_id)
        if not row.teachers_id is None: 
            # it's a teacher! Give permission to edit their own teacher entry
            self.auth.add_permission(group_id, 
                                     'update', 
                                     'teachers', 
                                     row.teachers_id)
    
    def on_user_delete(self, table, record_id):
        '''
            This function creates a removed the group created for user and
            removes the membership form the group.
        '''
        user_id = record_id
        group_name = "user_{0}".format(user_id)
        group_id = self.auth.id_group(group_name)
        self.auth.del_membership(group_id, user_id)
        self.auth.del_group(group_id)
        
    def teacher_edit_auth_user_sync(self, form):
        """
        This function is executed when a teacher is updated. 
        It moves the firstname, lastname and email from the teacher to the auth_user table
        to keep the values in the teachers and auth_user table in sync.
        """
        teID = form.vars['id']
        fname = form.vars['FirstName']
        lname = form.vars['LastName']
        email = form.vars['Email']
        
        dba = current.globalenv['db']
        
        row = dba.auth_user(teachers_id = teID)
        try:
            row.first_name = fname
            row.last_name = lname
            row.email = email
            row.update_record()
        except AttributeError:
            # there's no row to work with, so pass
            pass
            
    def check_read_permission(self, item, user_id):
        '''
            This function checks the permission for a menu link
        '''
        if self.auth.has_permission('read', item, 0, user_id) or self.auth.has_membership(1, user_id):
            return True
        else:
            return False
            
# memo functions begin
class Memo_links():
    def memo_link_visible(self, row):
        visible = False
        try:
            visible = row.Visible
            row_id = row.id
        except:
            try:
                visible = row.memos.Visible
                row_id = row.memos.id
            except:
                pass
        
        if visible:
            icon = 'icon ok icon-ok'
        else:
            icon = 'icon remove icon-remove'
        return A(SPAN(_class="buttontext button", _title=current.T("")), 
                SPAN(_class=icon), current.T(""), _class="button btn", \
                _href=URL('memo_change_visibility', 
                    vars=dict(reminderID=row_id)),
                _title=current.T("Show/Hide this memo"))
                
    def memo_link_announcement(self, row):
        announcement = False
        try:
            announcement = row.Announcement
            row_id = row.id
        except:
            try:
                announcement = row.memos.Announcement
                row_id = row.memos.id
            except:
                pass
        
        if announcement:
            icon = 'icon ok icon-ok'
        else:
            icon = 'icon remove icon-remove'
        return A(SPAN(_class="buttontext button", _title=current.T("")),
                SPAN(_class=icon), current.T(""), _class="button btn", \
                _href=URL('memo_change_announcement',
                    vars=dict(reminderID=row_id)),
                _title=current.T("Show at top"))
            
    def memo_link_priority(self, row):
        priority = None
        try:
            priority = row.Priority
            row_id = row.id
        except:
            try:
                priority = row.memos.Priority
                row_id = row.memos.id
            except:
                pass
        
        if priority:
            if priority == 1:
                title = current.T("High")
            elif priority == 2:
                title = current.T("Normal")
            else:
                title = current.T("Low")
            return A(title, _href=URL('memo_change_priority', 
                vars=dict(reminderID=row_id)),
                _title=current.T("Change the priority of this memo"))
        else:
            return current.T("Error fetching priority")

# memo funcions end

def max_string_length(string, length):
    '''
        Cuts string to desired length, if longer, cuts and replaces last 3 
        characters with "..."
    ''' 
    if len(string) > length:
        return_value = string[:length-3] + "..."
    else:
        return_value = string
        
    return return_value
