/**

	Created by Edwin van de Ven (edwin@openstudioproject.com)
	License: GPLv2 or v3

**/

// global variable for canvas id
var canvas_id = "attendance-chart-area"
var canvas_selector = "#"+canvas_id;

$(document).ready(function() {
	// Chart begin
	drawChart();

	$('span#previous').click(function() {
		setAttendanceYear(-1);
	});
	
	$('span#next').click(function() {
		setAttendanceYear(1);
	});
	// Chart end
	
	// Attendance list begin
	var tools_selector = 'div.os-customer_tools';
	$(tools_selector).each(function() {
		$(this).hide(); // hide the customer tools
	});

	$('table.os-customer_table tr').each(function(index) {
		$(this).hover(function() {
				$(this).find(tools_selector).show();
			}, // end handlerIn
			function() {
				$(this).find(tools_selector).hide();
			} // end handlerOut
		);
	}); // end settup up hover handler for table rows
	
	$('span.os-customer_tools_remove').each(function() {
	$(this).click(function(event) {
			event.preventDefault(); // prevent jumping to top of page after confirmation
			$row = $(this).closest('tr'); // save the table row in a variable to fade it out when removing
			if(confirm("Are you sure you want to remove this customer from the attendance list?")){	
				var $id = $(this).closest('tr').attr('id')
				var data = {};
				data['cuID'] = $id.split('_')[1];
				data['clsID'] = $id.split('_')[2];
				data['date'] = $id.split('_')[3];
				var url = '/classes/attendance_remove.json';
			
				var post = $.post(url, data, function(json) {
					console.log("Attendance remove posted to " + url + 
								", data: " + JSON.stringify(data));
				}, "json");
			
				// success
				post.done(function(json) {
					$('div.flash').html(json.message + '<span id="closeflash"> × </span>');
					$('div.flash').show();
					setTimeout(function() {
						$('div.flash').fadeOut();	
					}, 2500 ); 
					if (json.status == 'success') { // fade out row in table
						$row.fadeOut().slideUp();	
					}
					console.log("Attendance remove done, result: " + JSON.stringify(json));
				});
			
				// fail
				post.fail(function(data) {
					var msg = "Uh oh... something went wrong while removing the customer from the list..."
					$('div.flash').html(msg + '<span id="closeflash"> × </span>');
					$('div.flash').show();
					setTimeout(function() {
						$('div.flash').fadeOut();	
					}, 2500 );
					console.log("Attendance remove failed. Status: " + data.status + " " +
							 	"Status Text: " + data.statusText);
				});
			} // close if for confirmation
		}); // close click event
	}); // close loop for customer tools span
	
	// Attendance list end
	
	// Expected attendance list begin
	$('td.add_attendance').each(function() {
		$(this).click(function(event) {
			event.preventDefault(); // prevent jumping to top of page after confirmation
			var $id = $(this).closest('tr').attr('id').split('_')[1];
			// redirect to attendance_process to add the customer, later use AJAJ
			window.location.href = '/classes/attendance_process?cuID=' + $id
			
			/* Maybe later when we'll actually use AJAJ
			$(this).children('span.icon-arrow-right').hide();
			$(this).unbind('click');
			$(this).removeClass('os-clickable');
			*/
		});
	});
	// Expected attendance list end
});
	
// Chart functions begin

function getContext() {
	canvas = getCanvas();
	return canvas.getContext("2d");
}

function getCanvas() {
	return $(canvas_selector).get(0);
}

function resetCanvas() {
	var canvas = getCanvas();
	var $container = $('div.os-barchart')
	$container.fadeOut();
	canvas.remove();
	$container.append('<canvas id="' + canvas_id + '" width="870" height="290"></canvas>');
	$container.fadeIn();
}

function drawChart() {
	var chart_opts = { 
		  scaleFontSize: 9,
          barValueSpacing: 2,
	};
	var url = "/stats/attendance.json"
	var get = $.getJSON(url, function(json) {
		console.log("Requested json data from  " + url); 
	});
	
	// success
	get.done( function(json) {
		// city chart
		var	data = json.json_data.attendance; 
		var ctx = getContext();
		var chart = new Chart(ctx).Bar(data, chart_opts);
	});
}
	
function setAttendanceYear(year) {
		var url = '/classes/attendance_set_chart_year.json';
		data = {};
		data['year'] = year;
		var post = $.post(url, data, function(json) {
			console.log("Posted year info to " + url + 
						", data: " + JSON.stringify(data)); 
		}, "json");

		// Success
		post.done(function(json) {
			if (json.status == 'success') {
				title = json.chart_title.title;
				setChartTitle(title);
				resetCanvas();
			} else {
				// show what went wrong
				$('div.flash').html(json.message + '<span id="closeflash"> × </span>');
				$('div.flash').show();
				setTimeout(function() {
					$('div.flash').fadeOut();	
				}, 2500 );
			}
			// always draw the chart again
			drawChart();
			
			console.log("Success setting year, result: " + JSON.stringify(json));
		});
		
		// fail
		post.fail(function(data) {
			var msg = "Uh oh... something went wrong while changing years..."
			$('div.flash').html(msg + '<span id="closeflash"> × </span>');
			$('div.flash').show();
			setTimeout(function() {
				$('div.flash').fadeOut();	
			}, 2500 );
			console.log("Failed setting year. Status: " + data.status + " " +
					 	"Status Text: " + data.statusText);
		});
}

function setChartTitle(title) {
	$chart_title = $('div.os-attendance_chart_title div span h3')
	$chart_title.html(title)
}

// Chart functions end

