#!/usr/bin/env python

'''py.test test cases to test OpenStudio.

These tests run based on webclient and need web2py server running.
'''

from gluon.contrib.populate import populate
from populate_os_tables import populate_customers

    
def test_memo_add_exists(client):
    """
        test if the add memos page shows
    """
    client.get('/customers/memo_add/None')
    assert 'name="Visible"' in client.text
    assert 'name="Startdate"' in client.text
    
def test_memo_add_from_default(client, web2py):
    '''
        Created a memo?
    '''
    client.get('/default/pinboard') # first get this url to set the session variable for redirection
    assert client.status == 200
    assert 'Welcome' in client.text # the welcome message should be shown on the pinboard
    
    url = '/customers/memo_add/None'
    client.get(url)
    assert client.status == 200
    data = dict(Title='Bananas',
                Msg='message text',
                Startdate='2014-01-01',
                Priority='2',
                Visible='on',
                )
    client.post(url, data=data)
    assert client.status == 200
    assert "board" in client.text # check is we're at the pin board page
    
    assert web2py.db(web2py.db.memos).count() == 1
    assert data['Title'] in client.text
    
    assert web2py.db(web2py.db.memos.Title == data['Title']).count() == 1
    assert web2py.db(web2py.db.memos.customers_id == None).select(web2py.db.memos.ALL) != None
    
def test_memo_edit_from_default(client, web2py):
    '''
        Can we edit a memo
    '''
    web2py.db.memos.insert(customers_id=None, Visible=True, Title="Strawberries", Msg="I Like them!!", Startdate='2014-01-01', Priority=2)
    web2py.db.commit()
    
    client.get('/default/pinboard') # first get this url to set the session variable for redirection
    assert client.status == 200
    
    url = '/customers/memo_edit/1'
    client.get(url)
    assert client.status == 200
    data = dict(Title='Bananas',
                Msg='message text',
                Startdate='2014-01-01',
                Priority='2',
                Visible='on',
                )
    client.post(url, data=data)
    assert client.status == 200
    assert "board" in client.text # check is we're at the pin board page
    
    assert data['Title'] in client.text
    
    assert web2py.db(web2py.db.memos.Title == data['Title']).count() == 1
    assert web2py.db(web2py.db.memos.customers_id == None).select(web2py.db.memos.ALL) != None

def test_memos_filter(client, web2py):
    """
        test memos filter 
    """
    populate(web2py.db.school_discovery, 3)
    populate(web2py.db.school_locations, 2)
    populate(web2py.db.school_levels, 3)
    populate(web2py.db.school_languages, 2)
    web2py.db.commit()
    populate(web2py.db.customers, 10) # insert 10 customers with random data
    web2py.db.commit()

    client.get('/default/memos')
    assert client.status == 200
    client.get('/customers/memo_add/None')
    assert client.status == 200
    data = dict(Title='strawberries',
                Msg='message text',
                Startdate='2014-01-01',
                Priority='2',
                Visible='on',
                )
    client.post('/customers/memo_add/None', data=data)
    assert client.status == 200
    assert data['Title'] in client.text
    
    url = '/default/memos'
    
    data_filter1 = dict(name=data['Title'],
                        date_from="2014-01-01",
                        date_until="2014-01-02")
    client.post(url, data=data_filter1)
    assert client.status == 200
    assert data_filter1['name'] in client.text
    
    #data_filter2 = dict(name="apples", 
                        #date_from="2014-01-01",
                        #date_until="2014-01-02")
    #client.post(url, data=data_filter2)
    #assert client.status == 200
    #assert """No records found""" in client.text
    
    ## continue to test the show_all button and function
    #client.get("/default/_memos_show_all")
    #assert client.status == 200
    #assert data['Title'] in client.text
    
def test_subscriptions_overview(client, web2py):
    '''
        Test the overview of subscriptions
    '''
    monthly_fee = 40123456
    mst_name = '1 class a week'
    
    populate(web2py.db.school_discovery, 3)
    populate(web2py.db.school_locations, 2)
    populate(web2py.db.school_levels, 3)
    populate(web2py.db.school_languages, 2)
    populate(web2py.db.customers, 10) # insert 10 customers with random data
    web2py.db.school_subscriptions.insert(Name=mst_name,
                                          MonthlyFee=monthly_fee,
                                          NRofClasses=1,
                                          Archived=False)
    web2py.db.customers_subscriptions.insert(school_subscriptions_id=1,
                                             Startdate='2014-01-01',
                                             Enddate=None,
                                             customers_id=1)
    web2py.db.customers_subscriptions.insert(school_subscriptions_id=1,
                                             Startdate='2014-01-01',
                                             Enddate=None,
                                             customers_id=2)
    web2py.db.commit()
    assert web2py.db(web2py.db.customers_subscriptions).count() == 2
    
    url = '/subscriptions_overview'
    client.get(url)
    assert client.status == 200
    
    data = dict(month=1,
                year=2014,
                _formname='form_select_data')
    client.post(url, data=data)
    assert client.status == 200
    
    assert mst_name in client.text
    assert unicode(monthly_fee*2) in client.text # check whether total works
    
    
    
    
    
