#!/usr/bin/env python

'''py.test test cases to test OpenStudio.

These tests run based on webclient and need web2py server running.
'''

from gluon.contrib.populate import populate

def test_teachers_index(client, web2py):
    '''
        Is the index page showing?
    '''
    url = '/teachers/index'
    client.get(url)
    assert client.status == 200
    assert 'Teachers' in client.text

def test_teacher_add(client, web2py):
    '''
        Can we add a teacher?
    '''
    url = '/teachers/teacher_add'
    client.get(url)
    assert client.status == 200
    assert 'New teacher' in client.text
    
    data = dict(FirstName='Edwin',
                 LastName='van de Ven',
                 Email='edwin@openstudioproject.com')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Teachers' in client.text # verify redirection
    assert data['FirstName'] in client.text
    
    assert web2py.db(web2py.db.teachers).count() == 1

def test_teacher_edit(client, web2py):
    '''
        Can we edit a teacher?
    '''
    populate(web2py.db.teachers, 1)
    assert web2py.db(web2py.db.teachers).count() == 1
    
    url = '/teachers/teacher_edit/1'
    client.get(url)
    assert client.status == 200
    assert 'Edit teacher' in client.text
    
    data = dict(FirstName='Edwin',
                 LastName='van de Ven',
                 Email='edwin@openstudioproject.com')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Teachers' in client.text # verify redirection
    assert data['FirstName'] in client.text
    
    assert web2py.db(web2py.db.teachers).count() > 0
    
def test_teacher_account_auth_user_sync(client, web2py):
    '''
        If a teacher has a user account, the firsname, lastname and email should sync from 
        teachers to auth_user. Do they?
    '''
    web2py.db.teachers.insert(FirstName='bla', 
                              LastName='bla', 
                              Gender='Male', 
                              Email='a@a.nl')
    password = web2py.db.auth_user.password.validate('admin')[0]
    ''' here you would expect teachers_id = 1, but as for some reasons these tests don't actually update the records, but 
        insert another one, 2 is the appropriate ID for this test. '''
    web2py.db.auth_user.insert(id=2, 
                               first_name='bla', 
                               last_name='bla', 
                               email='a@a.nl', 
                               password=password, 
                               teachers_id='2') 
    web2py.db.commit()
    
    assert web2py.db(web2py.db.teachers).count() == 1
    assert web2py.db(web2py.db.auth_user).count() == 1
    
    url = '/teachers/teacher_edit/1'
    client.get(url)
    assert client.status == 200
    assert web2py.db(web2py.db.auth_user).count() == 2 # we got an url, so modules/tables.py has created the admin user.
    
    data = dict(FirstName='Edwin',
                 LastName='van de Ven',
                 Email='edwin@openstudioproject.com')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Teachers' in client.text # verify redirection
    assert data['FirstName'] in client.text
    
    assert web2py.db.auth_user(2).first_name == data['FirstName']
    assert web2py.db.auth_user(2).last_name == data['LastName']
    assert web2py.db.auth_user(2).email == data['Email']
    
def test_teacher_create_user_account(client, web2py):
    '''
        Is the user account being created as it should?
    '''
    populate(web2py.db.teachers, 1)
    assert web2py.db(web2py.db.teachers).count() == 1
    
    url = '/teachers/user_account/1'
    client.get(url)
    assert client.status == 200
    assert 'Create' in client.text
    
    data = dict(password = "Hello")
    client.post(url, data=data)
    assert client.status == 200
    assert 'Teachers' in client.text # check redirection
    
    assert web2py.db(web2py.db.auth_user).count() == 2 # the account for the teacher + auto created admin user
    
    # check if a group has been created and the user added
    # and permissions set
    teacher_id = 1
    user_id = 2
    group_id = web2py.auth.user_group(user_id)
    assert not group_id is None # see if a group was created
    assert web2py.auth.has_membership(group_id, user_id) # see if the created user is a member of the created group
    assert web2py.auth.has_permission('update', 'teachers', teacher_id, user_id)
