#!/usr/bin/env python

'''py.test test cases to test OpenStudio.

These tests run based on webclient and need web2py server running.
'''

from gluon.contrib.populate import populate

def test_school_subscriptions_index(client, web2py):
    '''
        Is the index page showing?
    '''
    url = '/school_properties/subscriptions'
    client.get(url)
    assert client.status == 200
    assert 'Subscriptions' in client.text
    
def test_subscription_add(client, web2py):
    '''
        Can we add a subscription?
    '''
    url = '/school_properties/subscription_add'
    client.get(url)
    assert client.status == 200
    assert 'New subscription' in client.text
    
    data = dict(Name='1 Class a week',
                 MonthlyFee='40',
                 NRofClasses='1')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Subscriptions' in client.text # verify redirection
    assert data['Name'] in client.text
    
    assert web2py.db(web2py.db.school_subscriptions).count() == 1
    
def test_subsription_edit(client, web2py):
    '''
        Can we edit a membership type?
    '''
    web2py.db.school_subscriptions.insert(Name='banana class', 
                                          MonthlyFee='500', 
                                          NRofClasses='15')
    web2py.db.commit()
    assert web2py.db(web2py.db.school_subscriptions).count() == 1
    
    url = '/school_properties/subscription_edit/1'
    client.get(url)
    assert client.status == 200
    assert 'Edit subscription' in client.text
    
    data = dict(Name='1 Class a week',
                 MonthlyFee='40',
                 NRofClasses='1')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Subscription' in client.text # verify redirection
    assert data['Name'] in client.text
    
    assert web2py.db(web2py.db.school_subscriptions).count() > 0
    
def test_school_levels_index(client, web2py):
    '''
        Is the index page showing?
    '''
    url = '/school_properties/school_levels'
    client.get(url)
    assert client.status == 200
    assert 'Practice levels' in client.text
    
def test_school_level_add(client, web2py):
    '''
        Can we add a school_level?
    '''
    url = '/school_properties/level_add'
    client.get(url)
    assert client.status == 200
    assert 'New practice level' in client.text
    
    data = dict(Name='Beginner')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Practice levels' in client.text # verify redirection
    assert data['Name'] in client.text
    
    assert web2py.db(web2py.db.school_levels).count() == 1
    
def test_school_level_edit(client, web2py):
    '''
        Can we edit a school_level?
    '''
    populate(web2py.db.school_levels, 1)
    assert web2py.db(web2py.db.school_levels).count() == 1
    
    url = '/school_properties/level_edit/1'
    client.get(url)
    assert client.status == 200
    assert 'Edit practice level' in client.text
    
    data = dict(Name='Beginner')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Practice levels' in client.text # verify redirection
    assert data['Name'] in client.text
    
    assert web2py.db(web2py.db.school_levels).count() > 0
    
def test_discovery_index(client, web2py):
    '''
        Is the index page showing?
    '''
    url = '/school_properties/discovery'
    client.get(url)
    assert client.status == 200
    assert 'Discovery' in client.text
    
def test_discovery_add(client, web2py):
    '''
        Can we add a way of discovery?
    '''
    url = '/school_properties/discovery_add'
    client.get(url)
    assert client.status == 200
    assert 'New way of discovery' in client.text
    
    data = dict(Name='Google')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Discovery' in client.text # verify redirection
    assert data['Name'] in client.text
    
    assert web2py.db(web2py.db.school_discovery).count() == 1
    
def test_discovery_edit(client, web2py):
    '''
        Can we edit a way of discovery?
    '''
    populate(web2py.db.school_discovery, 1)
    assert web2py.db(web2py.db.school_discovery).count() == 1
    
    url = '/school_properties/discovery_edit/1'
    client.get(url)
    assert client.status == 200
    assert 'Edit way of discovery' in client.text
    
    data = dict(Name='Google')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Discovery' in client.text # verify redirection
    assert data['Name'] in client.text
    
    assert web2py.db(web2py.db.school_discovery).count() > 0
    
def test_classtypes_index(client, web2py):
    '''
        Is the index page showing?
    '''
    url = '/school_properties/classtypes'
    client.get(url)
    assert client.status == 200
    assert 'Class types' in client.text
    
def test_classtype_add(client, web2py):
    '''
        Can we add a classtype?
    '''
    url = '/school_properties/classtype_add'
    client.get(url)
    assert client.status == 200
    assert 'New class type' in client.text
    
    data = dict(Name='Mysore style')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Class types' in client.text # verify redirection
    assert data['Name'] in client.text
    
    assert web2py.db(web2py.db.school_classtypes).count() == 1
    
def test_classtype_edit(client, web2py):
    '''
        Can we edit a classtype?
    '''
    populate(web2py.db.school_classtypes, 1)
    assert web2py.db(web2py.db.school_classtypes).count() == 1
    
    url = '/school_properties/classtype_edit/1'
    client.get(url)
    assert client.status == 200
    assert 'Edit class type' in client.text
    
    data = dict(Name='Mysore style')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Class types' in client.text # verify redirection
    assert data['Name'] in client.text
    
    assert web2py.db(web2py.db.school_classtypes).count() > 0
    
def test_locations_index(client, web2py):
    '''
        Is the index page showing?
    '''
    url = '/school_properties/locations'
    client.get(url)
    assert client.status == 200
    assert 'Locations' in client.text
    
def test_location_add(client, web2py):
    '''
        Can we add a location?
    '''
    url = '/school_properties/location_add'
    client.get(url)
    assert client.status == 200
    assert 'New location' in client.text
    
    data = dict(Name='Maastricht')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Locations' in client.text # verify redirection
    assert data['Name'] in client.text
    
    assert web2py.db(web2py.db.school_locations).count() == 1
    
def test_location_edit(client, web2py):
    '''
        Can we edit a location?
    '''
    populate(web2py.db.school_locations, 1)
    assert web2py.db(web2py.db.school_locations).count() == 1
    
    url = '/school_properties/location_edit/1'
    client.get(url)
    assert client.status == 200
    assert 'Edit location' in client.text
    
    data = dict(Name='Maastricht')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Locations' in client.text # verify redirection
    assert data['Name'] in client.text
    
    assert web2py.db(web2py.db.school_locations).count() > 0
    
def test_language_index(client, web2py):
    '''
        Is the index page showing?
    '''
    url = '/school_properties/languages'
    client.get(url)
    assert client.status == 200
    assert 'Languages' in client.text
    
def test_languages_add(client, web2py):
    '''
        Can we add a language?
    '''
    url = '/school_properties/language_add'
    client.get(url)
    assert client.status == 200
    assert 'New language' in client.text
    
    data = dict(Name='Dutch')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Languages' in client.text # verify redirection
    assert data['Name'] in client.text
    
    assert web2py.db(web2py.db.school_languages).count() == 1
    
def test_language_edit(client, web2py):
    '''
        Can we edit a language?
    '''
    web2py.db.school_languages.insert(Name="Dutch")
    web2py.db.commit()
    assert web2py.db(web2py.db.school_languages).count() == 1
    
    url = '/school_properties/language_edit/1'
    client.get(url)
    assert client.status == 200
    assert 'Edit language' in client.text
    
    data = dict(Name='English')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Languages' in client.text # verify redirection
    assert data['Name'] in client.text
    
    assert web2py.db(web2py.db.school_languages).count() > 0
